@@ -15,12 +15,10 @@
 #ifndef CEPH_KEYSSERVER_H
 #define CEPH_KEYSSERVER_H
 
-#include "common/config.h"
-
 #include "auth/KeyRing.h"
 #include "CephxProtocol.h"
-
-#include "common/Timer.h"
+#include "CephxKeyServer.h"
+#include "common/Mutex.h"
 
 class CephContext;
 
@@ -35,7 +33,7 @@ struct KeyServerData {
   version_t rotating_ver;
   map<uint32_t, RotatingSecrets> rotating_secrets;
 
-  KeyServerData(KeyRing *extra)
+  explicit KeyServerData(KeyRing *extra)
     : version(0),
       extra_secrets(extra),
       rotating_ver(0) {}
@@ -180,8 +178,8 @@ struct KeyServerData {
   }
 
 };
-WRITE_CLASS_ENCODER(KeyServerData);
-WRITE_CLASS_ENCODER(KeyServerData::Incremental);
+WRITE_CLASS_ENCODER(KeyServerData)
+WRITE_CLASS_ENCODER(KeyServerData::Incremental)
 
 
 
@@ -231,17 +229,24 @@ public:
     ::decode(data, bl);
   }
   bool contains(const EntityName& name) const;
-  void list_secrets(stringstream& ss) const;
+  int encode_secrets(Formatter *f, stringstream *ds) const;
+  void encode_formatted(string label, Formatter *f, bufferlist &bl);
+  void encode_plaintext(bufferlist &bl);
+  int list_secrets(stringstream& ds) const {
+    return encode_secrets(NULL, &ds);
+  }
   version_t get_ver() const {
     Mutex::Locker l(lock);
     return data.version;    
   }
 
   void clear_secrets() {
+    Mutex::Locker l(lock);
     data.clear_secrets();
   }
 
   void apply_data_incremental(KeyServerData::Incremental& inc) {
+    Mutex::Locker l(lock);
     data.apply_incremental(inc);
   }
   void set_ver(version_t ver) {
@@ -263,17 +268,17 @@ public:
     map<EntityName, EntityAuth>::const_iterator b = data.secrets_begin();
     return (b != data.secrets_end());
   }
-
-  /*void add_rotating_secret(uint32_t service_id, ExpiringCryptoKey& key) {
+  int get_num_secrets() {
     Mutex::Locker l(lock);
-    data.add_rotating_secret(service_id, key);
+    return data.secrets.size();
   }
-  */
+
   void clone_to(KeyServerData& dst) const {
     Mutex::Locker l(lock);
     dst = data;
   }
   void export_keyring(KeyRing& keyring) {
+    Mutex::Locker l(lock);
     for (map<EntityName, EntityAuth>::iterator p = data.secrets.begin();
 	 p != data.secrets.end();
 	 ++p) {
@@ -289,8 +294,12 @@ public:
   bool get_service_caps(const EntityName& name, uint32_t service_id,
 			AuthCapsInfo& caps) const;
 
+  map<EntityName, EntityAuth>::iterator secrets_begin()
+  { return data.secrets_begin(); }
+  map<EntityName, EntityAuth>::iterator secrets_end()
+  { return data.secrets_end(); }
 };
-WRITE_CLASS_ENCODER(KeyServer);
+WRITE_CLASS_ENCODER(KeyServer)
 
 
 #endif
