@@ -15,7 +15,7 @@
 #ifndef CEPH_AUTHUNKNOWNAUTHORIZEHANDLER_H
 #define CEPH_AUTHUNKNOWNAUTHORIZEHANDLER_H
 
-#include "../AuthAuthorizeHandler.h"
+#include "auth/AuthAuthorizeHandler.h"
 
 class CephContext;
 
@@ -23,9 +23,8 @@ struct AuthUnknownAuthorizeHandler : public AuthAuthorizeHandler {
   bool verify_authorizer(CephContext *cct, KeyStore *keys,
 			 bufferlist& authorizer_data, bufferlist& authorizer_reply,
                          EntityName& entity_name, uint64_t& global_id,
-			 AuthCapsInfo& caps_info, CryptoKey& session_key, uint64_t *auid,
-			 std::unique_ptr<AuthAuthorizerChallenge> *challenge) override;
-  int authorizer_session_crypto() override;
+			 AuthCapsInfo& caps_info, CryptoKey& session_key, uint64_t *auid=NULL);
+  int authorizer_session_crypto();
 };
 
 
