@@ -7,7 +7,28 @@
 #include "Dir.h"
 #include "MetaSession.h"
 #include "ClientSnapRealm.h"
-#include "UserGroups.h"
+
+#include "mds/flock.h"
+
+Inode::~Inode()
+{
+  cap_item.remove_myself();
+  snaprealm_item.remove_myself();
+
+  if (snapdir_parent) {
+    snapdir_parent->flags &= ~I_SNAPDIR_OPEN;
+    snapdir_parent.reset();
+  }
+
+  if (!oset.objects.empty()) {
+    lsubdout(client->cct, client, 0) << __func__ << ": leftover objects on inode 0x"
+      << std::hex << ino << std::dec << dendl;
+    assert(oset.objects.empty());
+  }
+
+  delete fcntl_locks;
+  delete flock_locks;
+}
 
 ostream& operator<<(ostream &out, const Inode &in)
 {
@@ -23,11 +44,10 @@ ostream& operator<<(ostream &out, const Inode &in)
       << " caps=" << ccap_string(in.caps_issued());
   if (!in.caps.empty()) {
     out << "(";
-    bool first = true;
-    for (const auto &pair : in.caps) {
-      if (!first)
+    for (auto p = in.caps.begin(); p != in.caps.end(); ++p) {
+      if (p != in.caps.begin())
         out << ',';
-      out << pair.first << '=' << ccap_string(pair.second->issued);
+      out << p->first << '=' << ccap_string(p->second->issued);
     }
     out << ")";
   }
@@ -172,12 +192,6 @@ int Inode::caps_issued(int *implemented) const
       c |= it->second->issued;
       i |= it->second->implemented;
     }
-  // exclude caps issued by non-auth MDS, but are been revoking by
-  // the auth MDS. The non-auth MDS should be revoking/exporting
-  // these caps, but the message is delayed.
-  if (auth_cap)
-    c &= ~auth_cap->implemented | auth_cap->issued;
-
   if (implemented)
     *implemented = i;
   return c;
@@ -260,11 +274,40 @@ int Inode::caps_wanted()
   return want;
 }
 
+int Inode::caps_mds_wanted()
+{
+  int want = 0;
+  for (auto it = caps.begin(); it != caps.end(); ++it)
+    want |= it->second->wanted;
+  return want;
+}
+
 int Inode::caps_dirty()
 {
   return dirty_caps | flushing_caps;
 }
 
+const UserPerm* Inode::get_best_perms()
+{
+  const UserPerm *perms = NULL;
+  for (const auto ci : caps) {
+    const UserPerm& iperm = ci.second->latest_perms;
+    if (!perms) { // we don't have any, take what's present
+      perms = &iperm;
+    } else if (iperm.uid() == uid) {
+      if (iperm.gid() == gid) { // we have the best possible, return
+	return &iperm;
+      }
+      if (perms->uid() != uid) { // take uid > gid every time
+	perms = &iperm;
+      }
+    } else if (perms->uid() != uid && iperm.gid() == gid) {
+      perms = &iperm; // a matching gid is better than nothing
+    }
+  }
+  return perms;
+}
+
 bool Inode::have_valid_size()
 {
   // RD+RDCACHE or WR+WRBUFFER => valid size
@@ -287,12 +330,12 @@ Dir *Inode::open_dir()
   return dir;
 }
 
-bool Inode::check_mode(uid_t ruid, UserGroups& groups, unsigned want)
+bool Inode::check_mode(const UserPerm& perms, unsigned want)
 {
-  if (uid == ruid) {
+  if (uid == perms.uid()) {
     // if uid is owner, owner entry determines access
     want = want << 6;
-  } else if (groups.is_in(gid)) {
+  } else if (perms.gid_in_groups(gid)) {
     // if a gid or sgid matches the owning group, group entry determines access
     want = want << 3;
   }
@@ -323,6 +366,7 @@ void Inode::dump(Formatter *f) const
   if (rdev)
     f->dump_unsigned("rdev", rdev);
   f->dump_stream("ctime") << ctime;
+  f->dump_stream("btime") << btime;
   f->dump_stream("mode") << '0' << std::oct << mode << std::dec;
   f->dump_unsigned("uid", uid);
   f->dump_unsigned("gid", gid);
@@ -335,6 +379,7 @@ void Inode::dump(Formatter *f) const
   f->dump_stream("mtime") << mtime;
   f->dump_stream("atime") << atime;
   f->dump_int("time_warp_seq", time_warp_seq);
+  f->dump_int("change_attr", change_attr);
 
   f->dump_object("layout", layout);
   if (is_dir()) {
@@ -342,6 +387,9 @@ void Inode::dump(Formatter *f) const
     ::dump(dir_layout, f);
     f->close_section();
 
+    f->dump_bool("complete", flags & I_COMPLETE);
+    f->dump_bool("ordered", flags & I_DIR_ORDERED);
+
     /* FIXME when wip-mds-encoding is merged ***
     f->open_object_section("dir_stat");
     dirstat.dump(f);
