@@ -12,8 +12,6 @@
  * 
  */
 
-#define FUSE_USE_VERSION 30
-
 #include <sys/file.h>
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -35,9 +33,9 @@
 #include "common/config.h"
 #include "include/assert.h"
 
+#include "fuse_ll.h"
 #include <fuse.h>
 #include <fuse_lowlevel.h>
-#include "fuse_ll.h"
 
 #define FINO_INO(x) ((x) & ((1ull<<48)-1ull))
 #define FINO_STAG(x) ((x) >> 48)
@@ -99,6 +97,48 @@ public:
   struct fuse_args args;
 };
 
+static int getgroups(fuse_req_t req, gid_t **sgids)
+{
+#if FUSE_VERSION >= FUSE_MAKE_VERSION(2, 8)
+  assert(sgids);
+  int c = fuse_req_getgroups(req, 0, NULL);
+  if (c < 0) {
+    return c;
+  }
+  if (c == 0) {
+    return 0;
+  }
+
+  *sgids = (gid_t*)malloc(c*sizeof(**sgids));
+  if (!*sgids) {
+    return -ENOMEM;
+  }
+  c = fuse_req_getgroups(req, c, *sgids);
+  if (c < 0) {
+    free(*sgids);
+    return c;
+  }
+  return c;
+#endif
+  return -ENOSYS;
+}
+
+static int getgroups_cb(void *handle, gid_t **sgids)
+{
+  CephFuse::Handle *cfuse = (CephFuse::Handle *) handle;
+  fuse_req_t req = cfuse->get_fuse_req();
+  return getgroups(req, sgids);
+}
+
+#define GET_GROUPS(perms, req)	{				\
+  if (cfuse->client->cct->_conf->fuse_set_user_groups) {	\
+    gid_t *gids = NULL;						\
+    int count = getgroups(req, &gids);				\
+    perms.init_gids(gids, count);				\
+    perms.take_gids();						\
+  } }
+
+
 static CephFuse::Handle *fuse_ll_req_prepare(fuse_req_t req)
 {
   CephFuse::Handle *cfuse = (CephFuse::Handle *)fuse_req_userdata(req);
@@ -113,9 +153,11 @@ static void fuse_ll_lookup(fuse_req_t req, fuse_ino_t parent, const char *name)
   struct fuse_entry_param fe;
   Inode *i2, *i1 = cfuse->iget(parent); // see below
   int r;
+  UserPerm perms(ctx->uid, ctx->gid);
+  GET_GROUPS(perms, req);
 
   memset(&fe, 0, sizeof(fe));
-  r = cfuse->client->ll_lookup(i1, name, &fe.attr, &i2, ctx->uid, ctx->gid);
+  r = cfuse->client->ll_lookup(i1, name, &fe.attr, &i2, perms);
   if (r >= 0) {
     fe.ino = cfuse->make_fake_ino(fe.attr.st_ino, fe.attr.st_dev);
     fe.attr.st_rdev = new_encode_dev(fe.attr.st_rdev);
@@ -144,10 +186,12 @@ static void fuse_ll_getattr(fuse_req_t req, fuse_ino_t ino,
   const struct fuse_ctx *ctx = fuse_req_ctx(req);
   Inode *in = cfuse->iget(ino);
   struct stat stbuf;
+  UserPerm perms(ctx->uid, ctx->gid);
+  GET_GROUPS(perms, req);
   
   (void) fi; // XXX
 
-  if (cfuse->client->ll_getattr(in, &stbuf, ctx->uid, ctx->gid)
+  if (cfuse->client->ll_getattr(in, &stbuf, perms)
       == 0) {
     stbuf.st_ino = cfuse->make_fake_ino(stbuf.st_ino, stbuf.st_dev);
     stbuf.st_rdev = new_encode_dev(stbuf.st_rdev);
@@ -164,6 +208,8 @@ static void fuse_ll_setattr(fuse_req_t req, fuse_ino_t ino, struct stat *attr,
   CephFuse::Handle *cfuse = fuse_ll_req_prepare(req);
   const struct fuse_ctx *ctx = fuse_req_ctx(req);
   Inode *in = cfuse->iget(ino);
+  UserPerm perms(ctx->uid, ctx->gid);
+  GET_GROUPS(perms, req);
 
   int mask = 0;
   if (to_set & FUSE_SET_ATTR_MODE) mask |= CEPH_SETATTR_MODE;
@@ -172,10 +218,12 @@ static void fuse_ll_setattr(fuse_req_t req, fuse_ino_t ino, struct stat *attr,
   if (to_set & FUSE_SET_ATTR_MTIME) mask |= CEPH_SETATTR_MTIME;
   if (to_set & FUSE_SET_ATTR_ATIME) mask |= CEPH_SETATTR_ATIME;
   if (to_set & FUSE_SET_ATTR_SIZE) mask |= CEPH_SETATTR_SIZE;
+#if !defined(DARWIN)
   if (to_set & FUSE_SET_ATTR_MTIME_NOW) mask |= CEPH_SETATTR_MTIME_NOW;
   if (to_set & FUSE_SET_ATTR_ATIME_NOW) mask |= CEPH_SETATTR_ATIME_NOW;
+#endif
 
-  int r = cfuse->client->ll_setattr(in, attr, mask, ctx->uid, ctx->gid);
+  int r = cfuse->client->ll_setattr(in, attr, mask, perms);
   if (r == 0)
     fuse_reply_attr(req, attr, 0);
   else
@@ -197,9 +245,10 @@ static void fuse_ll_setxattr(fuse_req_t req, fuse_ino_t ino, const char *name,
   CephFuse::Handle *cfuse = fuse_ll_req_prepare(req);
   const struct fuse_ctx *ctx = fuse_req_ctx(req);
   Inode *in = cfuse->iget(ino);
+  UserPerm perms(ctx->uid, ctx->gid);
+  GET_GROUPS(perms, req);
 
-  int r = cfuse->client->ll_setxattr(in, name, value, size, flags, ctx->uid,
-				     ctx->gid);
+  int r = cfuse->client->ll_setxattr(in, name, value, size, flags, perms);
   fuse_reply_err(req, -r);
 
   cfuse->iput(in); // iput required
@@ -211,8 +260,10 @@ static void fuse_ll_listxattr(fuse_req_t req, fuse_ino_t ino, size_t size)
   const struct fuse_ctx *ctx = fuse_req_ctx(req);
   Inode *in = cfuse->iget(ino);
   char buf[size];
+  UserPerm perms(ctx->uid, ctx->gid);
+  GET_GROUPS(perms, req);
 
-  int r = cfuse->client->ll_listxattr(in, buf, size, ctx->uid, ctx->gid);
+  int r = cfuse->client->ll_listxattr(in, buf, size, perms);
   if (size == 0 && r >= 0)
     fuse_reply_xattr(req, r);
   else if (r >= 0) 
@@ -234,8 +285,10 @@ static void fuse_ll_getxattr(fuse_req_t req, fuse_ino_t ino, const char *name,
   const struct fuse_ctx *ctx = fuse_req_ctx(req);
   Inode *in = cfuse->iget(ino);
   char buf[size];
+  UserPerm perms(ctx->uid, ctx->gid);
+  GET_GROUPS(perms, req);
 
-  int r = cfuse->client->ll_getxattr(in, name, buf, size, ctx->uid, ctx->gid);
+  int r = cfuse->client->ll_getxattr(in, name, buf, size, perms);
   if (size == 0 && r >= 0)
     fuse_reply_xattr(req, r);
   else if (r >= 0)
@@ -252,9 +305,10 @@ static void fuse_ll_removexattr(fuse_req_t req, fuse_ino_t ino,
   CephFuse::Handle *cfuse = fuse_ll_req_prepare(req);
   const struct fuse_ctx *ctx = fuse_req_ctx(req);
   Inode *in = cfuse->iget(ino);
+  UserPerm perms(ctx->uid, ctx->gid);
+  GET_GROUPS(perms, req);
 
-  int r = cfuse->client->ll_removexattr(in, name, ctx->uid,
-					ctx->gid);
+  int r = cfuse->client->ll_removexattr(in, name, perms);
   fuse_reply_err(req, -r);
 
   cfuse->iput(in); // iput required
@@ -268,10 +322,13 @@ static void fuse_ll_opendir(fuse_req_t req, fuse_ino_t ino,
   Inode *in = cfuse->iget(ino);
   void *dirp;
 
+  UserPerm perms(ctx->uid, ctx->gid);
+  GET_GROUPS(perms, req);
+
   int r = cfuse->client->ll_opendir(in, fi->flags, (dir_result_t **)&dirp,
-				    ctx->uid, ctx->gid);
+				    perms);
   if (r >= 0) {
-    fi->fh = (long)dirp;
+    fi->fh = (uint64_t)dirp;
     fuse_reply_open(req, fi);
   } else {
     fuse_reply_err(req, -r);
@@ -286,8 +343,10 @@ static void fuse_ll_readlink(fuse_req_t req, fuse_ino_t ino)
   const struct fuse_ctx *ctx = fuse_req_ctx(req);
   Inode *in = cfuse->iget(ino);
   char buf[PATH_MAX + 1];  // leave room for a null terminator
+  UserPerm perms(ctx->uid, ctx->gid);
+  GET_GROUPS(perms, req);
 
-  int r = cfuse->client->ll_readlink(in, buf, sizeof(buf) - 1, ctx->uid, ctx->gid);
+  int r = cfuse->client->ll_readlink(in, buf, sizeof(buf) - 1, perms);
   if (r >= 0) {
     buf[r] = '\0';
     fuse_reply_readlink(req, buf);
@@ -305,11 +364,13 @@ static void fuse_ll_mknod(fuse_req_t req, fuse_ino_t parent, const char *name,
   const struct fuse_ctx *ctx = fuse_req_ctx(req);
   Inode *i2, *i1 = cfuse->iget(parent);
   struct fuse_entry_param fe;
+  UserPerm perms(ctx->uid, ctx->gid);
+  GET_GROUPS(perms, req);
 
   memset(&fe, 0, sizeof(fe));
 
   int r = cfuse->client->ll_mknod(i1, name, mode, new_decode_dev(rdev),
-				  &fe.attr, &i2, ctx->uid, ctx->gid);
+				  &fe.attr, &i2, perms);
   if (r == 0) {
     fe.ino = cfuse->make_fake_ino(fe.attr.st_ino, fe.attr.st_dev);
     fe.attr.st_rdev = new_encode_dev(fe.attr.st_rdev);
@@ -332,7 +393,8 @@ static void fuse_ll_mkdir(fuse_req_t req, fuse_ino_t parent, const char *name,
   struct fuse_entry_param fe;
 
   memset(&fe, 0, sizeof(fe));
-
+  UserPerm perm(ctx->uid, ctx->gid);
+  GET_GROUPS(perm, req);
 #ifdef HAVE_SYS_SYNCFS
   if (cfuse->fino_snap(parent) == CEPH_SNAPDIR &&
       cfuse->client->cct->_conf->fuse_multithreaded &&
@@ -340,11 +402,11 @@ static void fuse_ll_mkdir(fuse_req_t req, fuse_ino_t parent, const char *name,
     int err = 0;
     int fd = ::open(cfuse->mountpoint, O_RDONLY | O_DIRECTORY);
     if (fd < 0) {
-      err = -errno;
+      err = errno;
     } else {
       int r = ::syncfs(fd);
       if (r < 0)
-	err = -errno;
+	err = errno;
       ::close(fd);
     }
     if (err) {
@@ -355,8 +417,7 @@ static void fuse_ll_mkdir(fuse_req_t req, fuse_ino_t parent, const char *name,
 #endif
 
   i1 = cfuse->iget(parent);
-  int r = cfuse->client->ll_mkdir(i1, name, mode, &fe.attr, &i2, ctx->uid,
-				  ctx->gid);
+  int r = cfuse->client->ll_mkdir(i1, name, mode, &fe.attr, &i2, perm);
   if (r == 0) {
     fe.ino = cfuse->make_fake_ino(fe.attr.st_ino, fe.attr.st_dev);
     fe.attr.st_rdev = new_encode_dev(fe.attr.st_rdev);
@@ -375,8 +436,10 @@ static void fuse_ll_unlink(fuse_req_t req, fuse_ino_t parent, const char *name)
   CephFuse::Handle *cfuse = fuse_ll_req_prepare(req);
   const struct fuse_ctx *ctx = fuse_req_ctx(req);
   Inode *in = cfuse->iget(parent);
+  UserPerm perm(ctx->uid, ctx->gid);
+  GET_GROUPS(perm, req);
 
-  int r = cfuse->client->ll_unlink(in, name, ctx->uid, ctx->gid);
+  int r = cfuse->client->ll_unlink(in, name, perm);
   fuse_reply_err(req, -r);
 
   cfuse->iput(in); // iput required
@@ -387,8 +450,10 @@ static void fuse_ll_rmdir(fuse_req_t req, fuse_ino_t parent, const char *name)
   CephFuse::Handle *cfuse = fuse_ll_req_prepare(req);
   const struct fuse_ctx *ctx = fuse_req_ctx(req);
   Inode *in = cfuse->iget(parent);
+  UserPerm perms(ctx->uid, ctx->gid);
+  GET_GROUPS(perms, req);
 
-  int r = cfuse->client->ll_rmdir(in, name, ctx->uid, ctx->gid);
+  int r = cfuse->client->ll_rmdir(in, name, perms);
   fuse_reply_err(req, -r);
 
   cfuse->iput(in); // iput required
@@ -401,11 +466,12 @@ static void fuse_ll_symlink(fuse_req_t req, const char *existing,
   const struct fuse_ctx *ctx = fuse_req_ctx(req);
   Inode *i2, *i1 = cfuse->iget(parent);
   struct fuse_entry_param fe;
+  UserPerm perms(ctx->uid, ctx->gid);
+  GET_GROUPS(perms, req);
 
   memset(&fe, 0, sizeof(fe));
 
-  int r = cfuse->client->ll_symlink(i1, name, existing, &fe.attr, &i2, ctx->uid,
-				    ctx->gid);
+  int r = cfuse->client->ll_symlink(i1, name, existing, &fe.attr, &i2, perms);
   if (r == 0) {
     fe.ino = cfuse->make_fake_ino(fe.attr.st_ino, fe.attr.st_dev);
     fe.attr.st_rdev = new_encode_dev(fe.attr.st_rdev);
@@ -426,8 +492,10 @@ static void fuse_ll_rename(fuse_req_t req, fuse_ino_t parent, const char *name,
   const struct fuse_ctx *ctx = fuse_req_ctx(req);
   Inode *in = cfuse->iget(parent);
   Inode *nin = cfuse->iget(newparent);
+  UserPerm perm(ctx->uid, ctx->gid);
+  GET_GROUPS(perm, req);
 
-  int r = cfuse->client->ll_rename(in, name, nin, newname, ctx->uid, ctx->gid);
+  int r = cfuse->client->ll_rename(in, name, nin, newname, perm);
   fuse_reply_err(req, -r);
 
   cfuse->iput(in); // iputs required
@@ -444,9 +512,10 @@ static void fuse_ll_link(fuse_req_t req, fuse_ino_t ino, fuse_ino_t newparent,
   struct fuse_entry_param fe;
 
   memset(&fe, 0, sizeof(fe));
+  UserPerm perm(ctx->uid, ctx->gid);
+  GET_GROUPS(perm, req);
   
-  int r = cfuse->client->ll_link(in, nin, newname, &fe.attr, ctx->uid,
-				 ctx->gid);
+  int r = cfuse->client->ll_link(in, nin, newname, &fe.attr, perm);
   if (r == 0) {
     fe.ino = cfuse->make_fake_ino(fe.attr.st_ino, fe.attr.st_dev);
     fe.attr.st_rdev = new_encode_dev(fe.attr.st_rdev);
@@ -466,12 +535,16 @@ static void fuse_ll_open(fuse_req_t req, fuse_ino_t ino,
   const struct fuse_ctx *ctx = fuse_req_ctx(req);
   Inode *in = cfuse->iget(ino);
   Fh *fh = NULL;
+  UserPerm perms(ctx->uid, ctx->gid);
+  GET_GROUPS(perms, req);
 
-  int r = cfuse->client->ll_open(in, fi->flags, &fh, ctx->uid, ctx->gid);
+  int r = cfuse->client->ll_open(in, fi->flags, &fh, perms);
   if (r == 0) {
-    fi->fh = (long)fh;
+    fi->fh = (uint64_t)fh;
 #if FUSE_VERSION >= FUSE_MAKE_VERSION(2, 8)
-    if (cfuse->client->cct->_conf->fuse_use_invalidate_cb)
+    if (cfuse->client->cct->_conf->fuse_disable_pagecache)
+      fi->direct_io = 1;
+    else if (cfuse->client->cct->_conf->fuse_use_invalidate_cb)
       fi->keep_cache = 1;
 #endif
     fuse_reply_open(req, fi);
@@ -664,15 +737,23 @@ static void fuse_ll_create(fuse_req_t req, fuse_ino_t parent, const char *name,
   Inode *i1 = cfuse->iget(parent), *i2;
   struct fuse_entry_param fe;
   Fh *fh = NULL;
+  UserPerm perms(ctx->uid, ctx->gid);
+  GET_GROUPS(perms, req);
 
   memset(&fe, 0, sizeof(fe));
 
   // pass &i2 for the created inode so that ll_create takes an initial ll_ref
   int r = cfuse->client->ll_create(i1, name, mode, fi->flags, &fe.attr, &i2,
-				   &fh, ctx->uid, ctx->gid);
+				   &fh, perms);
   if (r == 0) {
-    fi->fh = (long)fh;
+    fi->fh = (uint64_t)fh;
     fe.ino = cfuse->make_fake_ino(fe.attr.st_ino, fe.attr.st_dev);
+#if FUSE_VERSION >= FUSE_MAKE_VERSION(2, 8)
+    if (cfuse->client->cct->_conf->fuse_disable_pagecache)
+      fi->direct_io = 1;
+    else if (cfuse->client->cct->_conf->fuse_use_invalidate_cb)
+      fi->keep_cache = 1;
+#endif
     fuse_reply_create(req, &fe, fi);
   } else
     fuse_reply_err(req, -r);
@@ -686,8 +767,11 @@ static void fuse_ll_statfs(fuse_req_t req, fuse_ino_t ino)
   struct statvfs stbuf;
   CephFuse::Handle *cfuse = fuse_ll_req_prepare(req);
   Inode *in = cfuse->iget(ino);
+  const struct fuse_ctx *ctx = fuse_req_ctx(req);
+  UserPerm perms(ctx->uid, ctx->gid);
+  GET_GROUPS(perms, req);
 
-  int r = cfuse->client->ll_statfs(in, &stbuf);
+  int r = cfuse->client->ll_statfs(in, &stbuf, perms);
   if (r == 0)
     fuse_reply_statfs(req, &stbuf);
   else
@@ -762,36 +846,7 @@ static void fuse_ll_flock(fuse_req_t req, fuse_ino_t ino,
 }
 #endif
 
-static int getgroups_cb(void *handle, gid_t **sgids)
-{
-#if FUSE_VERSION >= FUSE_MAKE_VERSION(2, 8)
-  CephFuse::Handle *cfuse = (CephFuse::Handle *)handle;
-  fuse_req_t req = cfuse->get_fuse_req();
-
-  assert(sgids);
-  int c = fuse_req_getgroups(req, 0, NULL);
-  if (c < 0) {
-    return c;
-  }
-  if (c == 0) {
-    return 0;
-  }
-
-  *sgids = (gid_t*)malloc(c*sizeof(**sgids));
-  if (!*sgids) {
-    return -ENOMEM;
-  }
-  c = fuse_req_getgroups(req, c, *sgids);
-  if (c < 0) {
-    free(*sgids);
-    return c;
-  }
-  return c;
-#else
-  return -ENOSYS;
-#endif
-}
-
+#if !defined(DARWIN)
 static mode_t umask_cb(void *handle)
 {
   CephFuse::Handle *cfuse = (CephFuse::Handle *)handle;
@@ -799,6 +854,7 @@ static mode_t umask_cb(void *handle)
   const struct fuse_ctx *ctx = fuse_req_ctx(req);
   return ctx->umask;
 }
+#endif
 
 static void ino_invalidate_cb(void *handle, vinodeno_t vino, int64_t off,
 			      int64_t len)
@@ -845,12 +901,14 @@ static void do_init(void *data, fuse_conn_info *conn)
   CephFuse::Handle *cfuse = (CephFuse::Handle *)data;
   Client *client = cfuse->client;
 
+#if !defined(DARWIN)
   if (!client->cct->_conf->fuse_default_permissions &&
       client->ll_handle_umask()) {
     // apply umask in userspace if posix acl is enabled
     if(conn->capable & FUSE_CAP_DONT_MASK)
       conn->want |= FUSE_CAP_DONT_MASK;
   }
+#endif
 
   if (cfuse->fd_on_success) {
     //cout << "fuse init signaling on fd " << fd_on_success << std::endl;
@@ -1049,7 +1107,9 @@ int CephFuse::Handle::start()
     remount_cb: remount_cb,
 #endif
     getgroups_cb: getgroups_cb,
+#if !defined(DARWIN)
     umask_cb: umask_cb,
+#endif
   };
   client->ll_register_callbacks(&args);
 
