@@ -35,7 +35,8 @@ namespace librbd {
                                     std::map<rados::cls::lock::locker_id_t,
                                              rados::cls::lock::locker_info_t> *lockers,
                                     bool *exclusive_lock, std::string *lock_tag,
-                                    ::SnapContext *snapc, parent_info *parent);
+				    ::SnapContext *snapc, parent_info *parent,
+				    cls::rbd::GroupSpec *uplink);
     int get_mutable_metadata(librados::IoCtx *ioctx, const std::string &oid,
 			     bool read_only, uint64_t *size, uint64_t *features,
 			     uint64_t *incompatible_features,
@@ -44,14 +45,19 @@ namespace librbd {
 			     bool *exclusive_lock,
 			     std::string *lock_tag,
 			     ::SnapContext *snapc,
-			     parent_info *parent);
+			     parent_info *parent,
+			     cls::rbd::GroupSpec *uplink);
 
     // low-level interface (mainly for testing)
+    void create_image(librados::ObjectWriteOperation *op, uint64_t size, uint8_t order,
+		      uint64_t features, const std::string &object_prefix);
     int create_image(librados::IoCtx *ioctx, const std::string &oid,
 		     uint64_t size, uint8_t order, uint64_t features,
 		     const std::string &object_prefix);
     int get_features(librados::IoCtx *ioctx, const std::string &oid,
 		     snapid_t snap_id, uint64_t *features);
+    void set_features(librados::ObjectWriteOperation *op, uint64_t features,
+                     uint64_t mask);
     int set_features(librados::IoCtx *ioctx, const std::string &oid,
                      uint64_t features, uint64_t mask);
     int get_object_prefix(librados::IoCtx *ioctx, const std::string &oid,
@@ -116,12 +122,6 @@ namespace librbd {
 		      std::vector<parent_info> *parents,
 		      std::vector<uint8_t> *protection_statuses);
 
-    void get_all_features_start(librados::ObjectReadOperation *op);
-    int get_all_features_finish(bufferlist::iterator *it,
-                                uint64_t *all_features);
-    int get_all_features(librados::IoCtx *ioctx, const std::string &oid,
-                         uint64_t *all_features);
-
     int copyup(librados::IoCtx *ioctx, const std::string &oid,
 	       bufferlist data);
     int get_protection_status(librados::IoCtx *ioctx, const std::string &oid,
@@ -130,6 +130,10 @@ namespace librbd {
 			      snapid_t snap_id, uint8_t protection_status);
     void set_protection_status(librados::ObjectWriteOperation *op,
                                snapid_t snap_id, uint8_t protection_status);
+    int snapshot_get_limit(librados::IoCtx *ioctx, const std::string &oid,
+			   uint64_t *limit);
+    void snapshot_set_limit(librados::ObjectWriteOperation *op,
+			    uint64_t limit);
 
     void get_stripe_unit_count_start(librados::ObjectReadOperation *op);
     int get_stripe_unit_count_finish(bufferlist::iterator *it,
@@ -138,6 +142,8 @@ namespace librbd {
     int get_stripe_unit_count(librados::IoCtx *ioctx, const std::string &oid,
 			      uint64_t *stripe_unit, uint64_t *stripe_count);
 
+    void set_stripe_unit_count(librados::ObjectWriteOperation *op,
+			       uint64_t stripe_unit, uint64_t stripe_count);
     int set_stripe_unit_count(librados::IoCtx *ioctx, const std::string &oid,
 			      uint64_t stripe_unit, uint64_t stripe_count);
     int metadata_list(librados::IoCtx *ioctx, const std::string &oid,
@@ -147,8 +153,12 @@ namespace librbd {
                              const std::string &start, uint64_t max_return);
     int metadata_list_finish(bufferlist::iterator *it,
                              std::map<std::string, bufferlist> *pairs);
+    void metadata_set(librados::ObjectWriteOperation *op,
+                      const map<std::string, bufferlist> &data);
     int metadata_set(librados::IoCtx *ioctx, const std::string &oid,
                      const map<std::string, bufferlist> &data);
+    void metadata_remove(librados::ObjectWriteOperation *op,
+                         const std::string &key);
     int metadata_remove(librados::IoCtx *ioctx, const std::string &oid,
                         const std::string &key);
     int metadata_get(librados::IoCtx *ioctx, const std::string &oid,
@@ -159,6 +169,7 @@ namespace librbd {
     int get_id_finish(bufferlist::iterator *it, std::string *id);
     int get_id(librados::IoCtx *ioctx, const std::string &oid, std::string *id);
 
+    void set_id(librados::ObjectWriteOperation *op, std::string id);
     int set_id(librados::IoCtx *ioctx, const std::string &oid, std::string id);
 
     // operations on rbd_directory objects
@@ -172,10 +183,14 @@ namespace librbd {
     int dir_list(librados::IoCtx *ioctx, const std::string &oid,
 		 const std::string &start, uint64_t max_return,
 		 map<string, string> *images);
+    void dir_add_image(librados::ObjectWriteOperation *op,
+		       const std::string &name, const std::string &id);
     int dir_add_image(librados::IoCtx *ioctx, const std::string &oid,
 		      const std::string &name, const std::string &id);
     int dir_remove_image(librados::IoCtx *ioctx, const std::string &oid,
 			 const std::string &name, const std::string &id);
+    void dir_remove_image(librados::ObjectWriteOperation *op,
+			  const std::string &name, const std::string &id);
     // atomic remove and add
     void dir_rename_image(librados::ObjectWriteOperation *op,
 			  const std::string &src, const std::string &dest,
@@ -221,6 +236,9 @@ namespace librbd {
     // operations on the rbd_mirroring object
     int mirror_uuid_get(librados::IoCtx *ioctx, std::string *uuid);
     int mirror_uuid_set(librados::IoCtx *ioctx, const std::string &uuid);
+    void mirror_mode_get_start(librados::ObjectReadOperation *op);
+    int mirror_mode_get_finish(bufferlist::iterator *it,
+			       cls::rbd::MirrorMode *mirror_mode);
     int mirror_mode_get(librados::IoCtx *ioctx,
                         cls::rbd::MirrorMode *mirror_mode);
     int mirror_mode_set(librados::IoCtx *ioctx,
@@ -255,8 +273,13 @@ namespace librbd {
                                 const std::string &image_id);
     int mirror_image_get_finish(bufferlist::iterator *iter,
 			        cls::rbd::MirrorImage *mirror_image);
+    void mirror_image_set(librados::ObjectWriteOperation *op,
+			  const std::string &image_id,
+			  const cls::rbd::MirrorImage &mirror_image);
     int mirror_image_set(librados::IoCtx *ioctx, const std::string &image_id,
 			 const cls::rbd::MirrorImage &mirror_image);
+    void mirror_image_remove(librados::ObjectWriteOperation *op,
+			     const std::string &image_id);
     int mirror_image_remove(librados::IoCtx *ioctx,
 			    const std::string &image_id);
     int mirror_image_status_set(librados::IoCtx *ioctx,
@@ -294,6 +317,30 @@ namespace librbd {
     int mirror_image_status_remove_down(librados::IoCtx *ioctx);
     void mirror_image_status_remove_down(librados::ObjectWriteOperation *op);
 
+    // Consistency groups functions
+    int group_create(librados::IoCtx *ioctx, const std::string &oid);
+    int group_dir_list(librados::IoCtx *ioctx, const std::string &oid,
+		    const std::string &start, uint64_t max_return,
+		    map<string, string> *groups);
+    int group_dir_add(librados::IoCtx *ioctx, const std::string &oid,
+	           const std::string &name, const std::string &id);
+    int group_dir_remove(librados::IoCtx *ioctx, const std::string &oid,
+		      const std::string &name, const std::string &id);
+    int group_image_remove(librados::IoCtx *ioctx, const std::string &oid,
+			   const cls::rbd::GroupImageSpec &spec);
+    int group_image_list(librados::IoCtx *ioctx, const std::string &oid,
+			 const cls::rbd::GroupImageSpec &start,
+			 uint64_t max_return,
+			 std::vector<cls::rbd::GroupImageStatus>& images);
+    int group_image_set(librados::IoCtx *ioctx, const std::string &oid,
+			const cls::rbd::GroupImageStatus &st);
+    int image_add_group(librados::IoCtx *ioctx, const std::string &oid,
+	                const cls::rbd::GroupSpec &group_spec);
+    int image_remove_group(librados::IoCtx *ioctx, const std::string &oid,
+			   const cls::rbd::GroupSpec &group_spec);
+    int image_get_group(librados::IoCtx *ioctx, const std::string &oid,
+			cls::rbd::GroupSpec &s);
+
   } // namespace cls_client
 } // namespace librbd
 #endif // CEPH_LIBRBD_CLS_RBD_CLIENT_H
