@@ -8,14 +8,12 @@
 
 #include "include/stringify.h"
 
-
-
 // ----
 // LogEntryKey
 
-void LogEntryKey::encode(bufferlist& bl) const
+void LogEntryKey::encode(bufferlist& bl, uint64_t features) const
 {
-  ::encode(who, bl);
+  ::encode(who, bl, features);
   ::encode(stamp, bl);
   ::encode(seq, bl);
 }
@@ -184,11 +182,11 @@ void LogEntry::log_to_syslog(string level, string facility)
   }
 }
 
-void LogEntry::encode(bufferlist& bl) const
+void LogEntry::encode(bufferlist& bl, uint64_t features) const
 {
   ENCODE_START(3, 2, bl);
   __u16 t = prio;
-  ::encode(who, bl);
+  ::encode(who, bl, features);
   ::encode(stamp, bl);
   ::encode(seq, bl);
   ::encode(t, bl);
@@ -236,11 +234,11 @@ void LogEntry::generate_test_instances(list<LogEntry*>& o)
 
 // -----
 
-void LogSummary::encode(bufferlist& bl) const
+void LogSummary::encode(bufferlist& bl, uint64_t features) const
 {
   ENCODE_START(2, 2, bl);
   ::encode(version, bl);
-  ::encode(tail, bl);
+  ::encode(tail, bl, features);
   ENCODE_FINISH(bl);
 }
 
