@@ -13,6 +13,22 @@
 
 namespace journal {
 
+struct JournalTrimmer::C_RemoveSet : public Context {
+  JournalTrimmer *journal_trimmer;
+  uint64_t object_set;
+  Mutex lock;
+  uint32_t refs;
+  int return_value;
+
+  C_RemoveSet(JournalTrimmer *_journal_trimmer, uint64_t _object_set,
+              uint8_t _splay_width);
+  virtual void complete(int r);
+  virtual void finish(int r) {
+    journal_trimmer->handle_set_removed(r, object_set);
+    journal_trimmer->m_async_op_tracker.finish_op();
+  }
+};
+
 JournalTrimmer::JournalTrimmer(librados::IoCtx &ioctx,
                                const std::string &object_oid_prefix,
                                const JournalMetadataPtr &journal_metadata)
@@ -47,38 +63,35 @@ void JournalTrimmer::shut_down(Context *on_finish) {
   m_journal_metadata->flush_commit_position(on_finish);
 }
 
-int JournalTrimmer::remove_objects(bool force) {
+void JournalTrimmer::remove_objects(bool force, Context *on_finish) {
   ldout(m_cct, 20) << __func__ << dendl;
-  C_SaferCond ops_ctx;
-  m_async_op_tracker.wait_for_ops(&ops_ctx);
-  ops_ctx.wait();
 
-  C_SaferCond ctx;
-  {
-    Mutex::Locker locker(m_lock);
+  on_finish = new FunctionContext([this, force, on_finish](int r) {
+      Mutex::Locker locker(m_lock);
 
-    if (m_remove_set_pending) {
-      return -EBUSY;
-    }
-
-    if (!force) {
-      JournalMetadata::RegisteredClients registered_clients;
-      m_journal_metadata->get_registered_clients(&registered_clients);
-
-      if (registered_clients.size() == 0) {
-	return -EINVAL;
-      } else if (registered_clients.size() > 1) {
-	return -EBUSY;
+      if (m_remove_set_pending) {
+        on_finish->complete(-EBUSY);
       }
-    }
 
-    m_remove_set = std::numeric_limits<uint64_t>::max();
-    m_remove_set_pending = true;
-    m_remove_set_ctx = &ctx;
+      if (!force) {
+        JournalMetadata::RegisteredClients registered_clients;
+        m_journal_metadata->get_registered_clients(&registered_clients);
 
-    remove_set(m_journal_metadata->get_minimum_set());
-  }
-  return ctx.wait();
+        if (registered_clients.size() == 0) {
+          on_finish->complete(-EINVAL);
+        } else if (registered_clients.size() > 1) {
+          on_finish->complete(-EBUSY);
+        }
+      }
+
+      m_remove_set = std::numeric_limits<uint64_t>::max();
+      m_remove_set_pending = true;
+      m_remove_set_ctx = on_finish;
+
+      remove_set(m_journal_metadata->get_minimum_set());
+    });
+
+  m_async_op_tracker.wait_for_ops(on_finish);
 }
 
 void JournalTrimmer::committed(uint64_t commit_tid) {
