@@ -12,10 +12,6 @@
 #include <memory>
 #include <boost/scoped_ptr.hpp>
 #include "rocksdb/write_batch.h"
-#include "rocksdb/perf_context.h"
-#include "rocksdb/iostats_context.h"
-#include "rocksdb/statistics.h"
-#include "rocksdb/table.h"
 #include <errno.h>
 #include "common/errno.h"
 #include "common/dout.h"
@@ -30,6 +26,7 @@ enum {
   l_rocksdb_first = 34300,
   l_rocksdb_gets,
   l_rocksdb_txns,
+  l_rocksdb_txns_sync,
   l_rocksdb_get_latency,
   l_rocksdb_submit_latency,
   l_rocksdb_submit_sync_latency,
@@ -51,7 +48,6 @@ namespace rocksdb{
   class Iterator;
   class Logger;
   struct Options;
-  struct BlockBasedTableOptions;
 }
 
 extern rocksdb::Logger *create_rocksdb_ceph_logger();
@@ -66,10 +62,7 @@ class RocksDBStore : public KeyValueDB {
   void *priv;
   rocksdb::DB *db;
   rocksdb::Env *env;
-  std::shared_ptr<rocksdb::Statistics> dbstats;
-  rocksdb::BlockBasedTableOptions bbt_opts;
   string options_str;
-
   int do_open(ostream &out, bool create_if_missing);
 
   // manage async compactions
@@ -126,7 +119,6 @@ public:
     priv(p),
     db(NULL),
     env(static_cast<rocksdb::Env*>(p)),
-    dbstats(NULL),
     compact_queue_lock("RocksDBStore::compact_thread_lock"),
     compact_queue_stop(false),
     compact_thread(this),
@@ -145,11 +137,6 @@ public:
   int create_and_open(ostream &out);
 
   void close();
-
-  void split(const std::string &s, char delim, std::vector<std::string> &elems);
-  std::vector<std::string> split(const std::string &s, char delim);
-  void get_statistics(Formatter *f);
-
   struct  RocksWBHandler: public rocksdb::WriteBatch::Handler {
     std::string seen ;
     int num_seen = 0;
@@ -251,13 +238,20 @@ public:
     void set(
       const string &prefix,
       const string &k,
-      const bufferlist &bl);
+      const bufferlist &bl) override;
     void rmkey(
       const string &prefix,
-      const string &k);
+      const string &k) override;
+    void rm_single_key(
+      const string &prefix,
+      const string &k) override;
     void rmkeys_by_prefix(
       const string &prefix
-      );
+      ) override;
+    void merge(
+      const string& prefix,
+      const string& k,
+      const bufferlist &bl) override;
   };
 
   KeyValueDB::Transaction get_transaction() {
@@ -321,6 +315,12 @@ public:
   static bufferlist to_bufferlist(rocksdb::Slice in);
   static string past_prefix(const string &prefix);
 
+  class MergeOperatorRouter;
+  friend class MergeOperatorRouter;
+  virtual int set_merge_operator(const std::string& prefix,
+				 std::shared_ptr<KeyValueDB::MergeOperator> mop);
+  string assoc_name; ///< Name of associative operator
+
   virtual uint64_t get_estimated_size(map<string,uint64_t> &extra) {
     DIR *store_dir = opendir(path.c_str());
     if (!store_dir) {
@@ -395,4 +395,6 @@ protected:
 
 };
 
+
+
 #endif
