@@ -46,9 +46,6 @@ struct librados::IoCtxImpl {
   xlist<AioCompletionImpl*> aio_write_list;
   map<ceph_tid_t, std::list<AioCompletionImpl*> > aio_write_waiters;
 
-  Mutex cached_pool_names_lock;
-  std::list<std::string> cached_pool_names;
-
   Objecter *objecter;
 
   IoCtxImpl();
@@ -90,7 +87,7 @@ struct librados::IoCtxImpl {
     return poolid;
   }
 
-  const string& get_cached_pool_name();
+  string get_cached_pool_name();
 
   int get_object_hash_position(const std::string& oid, uint32_t *hash_postion);
   int get_object_pg_hash_position(const std::string& oid, uint32_t *pg_hash_position);
@@ -127,6 +124,8 @@ struct librados::IoCtxImpl {
   int write(const object_t& oid, bufferlist& bl, size_t len, uint64_t off);
   int append(const object_t& oid, bufferlist& bl, size_t len);
   int write_full(const object_t& oid, bufferlist& bl);
+  int writesame(const object_t& oid, bufferlist& bl,
+		size_t write_len, uint64_t offset);
   int clone_range(const object_t& dst_oid, uint64_t dst_offset,
                   const object_t& src_oid, uint64_t src_offset, uint64_t len);
   int read(const object_t& oid, bufferlist& bl, size_t len, uint64_t off);
@@ -201,6 +200,8 @@ struct librados::IoCtxImpl {
 		 const bufferlist& bl, size_t len);
   int aio_write_full(const object_t &oid, AioCompletionImpl *c,
 		     const bufferlist& bl);
+  int aio_writesame(const object_t &oid, AioCompletionImpl *c,
+		    const bufferlist& bl, size_t write_len, uint64_t off);
   int aio_remove(const object_t &oid, AioCompletionImpl *c);
   int aio_exec(const object_t& oid, AioCompletionImpl *c, const char *cls,
 	       const char *method, bufferlist& inbl, bufferlist *outbl);
@@ -232,9 +233,10 @@ struct librados::IoCtxImpl {
 
   void set_sync_op_version(version_t ver);
   int watch(const object_t& oid, uint64_t *cookie, librados::WatchCtx *ctx,
-	    librados::WatchCtx2 *ctx2);
+	    librados::WatchCtx2 *ctx2, bool internal = false);
   int aio_watch(const object_t& oid, AioCompletionImpl *c, uint64_t *cookie,
-                librados::WatchCtx *ctx, librados::WatchCtx2 *ctx2);
+                librados::WatchCtx *ctx, librados::WatchCtx2 *ctx2,
+                bool internal = false);
   int watch_check(uint64_t cookie);
   int unwatch(uint64_t cookie);
   int aio_unwatch(uint64_t cookie, AioCompletionImpl *c);
@@ -248,7 +250,8 @@ struct librados::IoCtxImpl {
 
   int set_alloc_hint(const object_t& oid,
                      uint64_t expected_object_size,
-                     uint64_t expected_write_size);
+                     uint64_t expected_write_size,
+		     uint32_t flags);
 
   version_t last_version();
   void set_assert_version(uint64_t ver);
