@@ -140,7 +140,6 @@ private:
   bool m_sparse;
   int m_op_flags;
   ceph::bufferlist m_read_data;
-  AioCompletion *m_parent_completion;
   ExtentMap m_ext_map;
 
   /**
@@ -169,7 +168,7 @@ private:
 
   void send_copyup();
 
-  void read_from_parent(const Extents& image_extents);
+  void read_from_parent(Extents&& image_extents);
 };
 
 class AbstractAioObjectWrite : public AioObjectRequest<> {
@@ -294,7 +293,6 @@ public:
 protected:
   virtual void add_write_ops(librados::ObjectWriteOperation *wr) {
     if (has_parent()) {
-      wr->create(false);
       wr->truncate(0);
     } else {
       wr->remove();
@@ -303,7 +301,7 @@ protected:
 
   virtual const char* get_write_type() const {
     if (has_parent()) {
-      return "remove (create+trunc)";
+      return "remove (trunc)";
     }
     return "remove";
   }
@@ -332,14 +330,11 @@ private:
 
 class AioObjectTrim : public AbstractAioObjectWrite {
 public:
-  // we'd need to only conditionally specify if a post object map
-  // update is needed. pre update is decided as usual (by checking
-  // the state of the object in the map).
   AioObjectTrim(ImageCtx *ictx, const std::string &oid, uint64_t object_no,
-                const ::SnapContext &snapc, Context *completion,
-                bool post_object_map_update)
+                const ::SnapContext &snapc, Context *completion)
     : AbstractAioObjectWrite(ictx, oid, object_no, 0, 0, snapc, completion,
-                             true), m_post_object_map_update(post_object_map_update) { }
+                             true) {
+  }
 
 protected:
   virtual void add_write_ops(librados::ObjectWriteOperation *wr) {
@@ -355,11 +350,8 @@ protected:
   }
 
   virtual bool post_object_map_update() {
-    return m_post_object_map_update;
+    return true;
   }
-
-private:
-  bool m_post_object_map_update;
 };
 
 class AioObjectTruncate : public AbstractAioObjectWrite {
