@@ -82,7 +82,7 @@ bool ExclusiveLock<I>::accept_requests(int *ret_val) const {
   Mutex::Locker locker(m_lock);
 
   bool accept_requests = (!is_shutdown() && m_state == STATE_LOCKED &&
-                          !m_request_blocked);
+                          m_request_blocked_count == 0);
   *ret_val = m_request_blocked_ret_val;
 
   ldout(m_image_ctx.cct, 20) << this << " " << __func__ << "="
@@ -93,9 +93,10 @@ bool ExclusiveLock<I>::accept_requests(int *ret_val) const {
 template <typename I>
 void ExclusiveLock<I>::block_requests(int r) {
   Mutex::Locker locker(m_lock);
-  assert(!m_request_blocked);
-  m_request_blocked = true;
-  m_request_blocked_ret_val = r;
+  m_request_blocked_count++;
+  if (m_request_blocked_ret_val == 0) {
+    m_request_blocked_ret_val = r;
+  }
 
   ldout(m_image_ctx.cct, 20) << this << " " << __func__ << dendl;
 }
@@ -103,9 +104,11 @@ void ExclusiveLock<I>::block_requests(int r) {
 template <typename I>
 void ExclusiveLock<I>::unblock_requests() {
   Mutex::Locker locker(m_lock);
-  assert(m_request_blocked);
-  m_request_blocked = false;
-  m_request_blocked_ret_val = 0;
+  assert(m_request_blocked_count > 0);
+  m_request_blocked_count--;
+  if (m_request_blocked_count == 0) {
+    m_request_blocked_ret_val = 0;
+  }
 
   ldout(m_image_ctx.cct, 20) << this << " " << __func__ << dendl;
 }
@@ -121,13 +124,10 @@ void ExclusiveLock<I>::init(uint64_t features, Context *on_init) {
     m_state = STATE_INITIALIZING;
   }
 
-  if (m_image_ctx.clone_copy_on_read ||
-      (features & RBD_FEATURE_JOURNALING) != 0) {
-    m_image_ctx.aio_work_queue->set_require_lock(AIO_DIRECTION_BOTH, true);
-  } else {
-    m_image_ctx.aio_work_queue->set_require_lock(AIO_DIRECTION_WRITE, true);
-  }
   m_image_ctx.aio_work_queue->block_writes(new C_InitComplete(this, on_init));
+  if ((features & RBD_FEATURE_JOURNALING) != 0) {
+    m_image_ctx.aio_work_queue->set_require_lock_on_read();
+  }
 }
 
 template <typename I>
@@ -141,7 +141,7 @@ void ExclusiveLock<I>::shut_down(Context *on_shut_down) {
   }
 
   // if stalled in request state machine -- abort
-  handle_peer_notification(0);
+  handle_peer_notification();
 }
 
 template <typename I>
@@ -233,62 +233,22 @@ void ExclusiveLock<I>::reacquire_lock(Context *on_reacquired) {
 }
 
 template <typename I>
-void ExclusiveLock<I>::handle_peer_notification(int r) {
-  {
-    Mutex::Locker locker(m_lock);
-    if (m_state != STATE_WAITING_FOR_PEER) {
-      return;
-    }
-
-    ldout(m_image_ctx.cct, 10) << this << " " << __func__ << dendl;
-    assert(get_active_action() == ACTION_REQUEST_LOCK);
-    m_state = STATE_ACQUIRING;
-
-    if (r >= 0) {
-      execute_next_action();
-      return;
-    }
+void ExclusiveLock<I>::handle_peer_notification() {
+  Mutex::Locker locker(m_lock);
+  if (m_state != STATE_WAITING_FOR_PEER) {
+    return;
   }
 
-  handle_acquire_lock(r);
+  ldout(m_image_ctx.cct, 10) << this << " " << __func__ << dendl;
+  assert(get_active_action() == ACTION_REQUEST_LOCK);
+  execute_next_action();
 }
 
 template <typename I>
-int ExclusiveLock<I>::assert_header_locked() {
-  CephContext *cct = m_image_ctx.cct;
-  ldout(cct, 10) << this << " " << __func__ << dendl;
-
-  librados::ObjectReadOperation op;
-  {
-    Mutex::Locker locker(m_lock);
-    rados::cls::lock::assert_locked(&op, RBD_LOCK_NAME, LOCK_EXCLUSIVE,
-                                    m_cookie, WATCHER_LOCK_TAG);
-  }
-
-  int r = m_image_ctx.md_ctx.operate(m_image_ctx.header_oid, &op, nullptr);
-  if (r < 0) {
-    if (r == -EBLACKLISTED) {
-      ldout(cct, 5) << this << " " << __func__ << ": "
-		    << "client is not lock owner -- client blacklisted"
-                    << dendl;
-    } else if (r == -ENOENT) {
-      ldout(cct, 5) << this << " " << __func__ << ": "
-		    << "client is not lock owner -- no lock detected"
-                    << dendl;
-    } else if (r == -EBUSY) {
-      ldout(cct, 5) << this << " " << __func__ << ": "
-		    << "client is not lock owner -- owned by different client"
-                    << dendl;
-    } else {
-      lderr(cct) << this << " " << __func__ << ": "
-		 << "failed to verify lock ownership: " << cpp_strerror(r)
-                 << dendl;
-    }
-
-    return r;
-  }
-
-  return 0;
+void ExclusiveLock<I>::assert_header_locked(librados::ObjectWriteOperation *op) {
+  Mutex::Locker locker(m_lock);
+  rados::cls::lock::assert_locked(op, RBD_LOCK_NAME, LOCK_EXCLUSIVE,
+                                  m_cookie, WATCHER_LOCK_TAG);
 }
 
 template <typename I>
@@ -492,13 +452,12 @@ void ExclusiveLock<I>::handle_acquire_lock(int r) {
   {
     m_lock.Lock();
     assert(m_state == STATE_ACQUIRING ||
-           m_state == STATE_POST_ACQUIRING ||
-           m_state == STATE_WAITING_FOR_PEER);
+           m_state == STATE_POST_ACQUIRING);
 
     Action action = get_active_action();
     assert(action == ACTION_TRY_LOCK || action == ACTION_REQUEST_LOCK);
     if (action == ACTION_REQUEST_LOCK && r < 0 && r != -EBLACKLISTED &&
-        r != -EPERM && r != -EROFS) {
+        r != -EPERM) {
       m_state = STATE_WAITING_FOR_PEER;
       m_lock.Unlock();
 
@@ -516,7 +475,7 @@ void ExclusiveLock<I>::handle_acquire_lock(int r) {
 
   if (next_state == STATE_LOCKED) {
     m_image_ctx.image_watcher->notify_acquired_lock();
-    m_image_ctx.aio_work_queue->set_require_lock(AIO_DIRECTION_BOTH, false);
+    m_image_ctx.aio_work_queue->clear_require_lock_on_read();
     m_image_ctx.aio_work_queue->unblock_writes();
   }
 
@@ -606,10 +565,6 @@ void ExclusiveLock<I>::handle_reacquire_lock(int r) {
     }
   } else {
     m_cookie = m_new_cookie;
-
-    m_lock.Unlock();
-    m_image_ctx.image_watcher->notify_acquired_lock();
-    m_lock.Lock();
   }
 
   complete_active_action(STATE_LOCKED, 0);
@@ -650,21 +605,31 @@ void ExclusiveLock<I>::handle_releasing_lock(int r) {
 
 template <typename I>
 void ExclusiveLock<I>::handle_release_lock(int r) {
-  Mutex::Locker locker(m_lock);
-  ldout(m_image_ctx.cct, 10) << this << " " << __func__ << ": r=" << r
-                             << dendl;
+  bool lock_request_needed = false;
+  {
+    Mutex::Locker locker(m_lock);
+    ldout(m_image_ctx.cct, 10) << this << " " << __func__ << ": r=" << r
+                               << dendl;
 
-  assert(m_state == STATE_PRE_RELEASING ||
-         m_state == STATE_RELEASING);
-  if (r >= 0) {
-    m_lock.Unlock();
-    m_image_ctx.image_watcher->notify_released_lock();
-    m_lock.Lock();
+    assert(m_state == STATE_PRE_RELEASING ||
+           m_state == STATE_RELEASING);
+    if (r >= 0) {
+      m_lock.Unlock();
+      m_image_ctx.image_watcher->notify_released_lock();
+      lock_request_needed = m_image_ctx.aio_work_queue->is_lock_request_needed();
+      m_lock.Lock();
 
-    m_cookie = "";
-    m_watch_handle = 0;
+      m_cookie = "";
+      m_watch_handle = 0;
+    }
+    complete_active_action(r < 0 ? STATE_LOCKED : STATE_UNLOCKED, r);
+  }
+
+  if (r >= 0 && lock_request_needed) {
+    // if we have blocked IO -- re-request the lock
+    RWLock::RLocker owner_locker(m_image_ctx.owner_lock);
+    request_lock(nullptr);
   }
-  complete_active_action(r < 0 ? STATE_LOCKED : STATE_UNLOCKED, r);
 }
 
 template <typename I>
@@ -722,6 +687,7 @@ void ExclusiveLock<I>::handle_shutdown_released(int r) {
 
   {
     RWLock::WLocker owner_locker(m_image_ctx.owner_lock);
+    m_image_ctx.aio_work_queue->clear_require_lock_on_read();
     m_image_ctx.exclusive_lock = nullptr;
   }
 
@@ -729,7 +695,6 @@ void ExclusiveLock<I>::handle_shutdown_released(int r) {
     lderr(cct) << "failed to shut down exclusive lock: " << cpp_strerror(r)
                << dendl;
   } else {
-    m_image_ctx.aio_work_queue->set_require_lock(AIO_DIRECTION_BOTH, false);
     m_image_ctx.aio_work_queue->unblock_writes();
   }
 
@@ -744,10 +709,10 @@ void ExclusiveLock<I>::handle_shutdown(int r) {
 
   {
     RWLock::WLocker owner_locker(m_image_ctx.owner_lock);
+    m_image_ctx.aio_work_queue->clear_require_lock_on_read();
     m_image_ctx.exclusive_lock = nullptr;
   }
 
-  m_image_ctx.aio_work_queue->set_require_lock(AIO_DIRECTION_BOTH, false);
   m_image_ctx.aio_work_queue->unblock_writes();
   m_image_ctx.image_watcher->flush(util::create_context_callback<
     ExclusiveLock<I>, &ExclusiveLock<I>::complete_shutdown>(this));
