@@ -43,9 +43,9 @@ public:
 
   void reacquire_lock(Context *on_reacquired = nullptr);
 
-  int assert_header_locked();
+  void handle_peer_notification();
 
-  void handle_peer_notification(int r);
+  void assert_header_locked(librados::ObjectWriteOperation *op);
 
   static bool decode_lock_cookie(const std::string &cookie, uint64_t *handle);
 
@@ -151,7 +151,7 @@ private:
 
   ActionsContexts m_actions_contexts;
 
-  bool m_request_blocked = false;
+  uint32_t m_request_blocked_count = 0;
   int m_request_blocked_ret_val = 0;
 
   std::string encode_lock_cookie() const;
