@@ -9,6 +9,7 @@
 #include "common/errno.h"
 #include "common/perf_counters.h"
 #include "common/WorkQueue.h"
+#include "common/Timer.h"
 
 #include "librbd/AioImageRequestWQ.h"
 #include "librbd/AioCompletion.h"
@@ -63,6 +64,21 @@ public:
   }
 };
 
+class SafeTimerSingleton : public SafeTimer {
+public:
+  Mutex lock;
+
+  explicit SafeTimerSingleton(CephContext *cct)
+      : SafeTimer(cct, lock, true),
+        lock("librbd::Journal::SafeTimerSingleton::lock") {
+    init();
+  }
+  virtual ~SafeTimerSingleton() {
+    Mutex::Locker locker(lock);
+    shutdown();
+  }
+};
+
 struct C_FlushCache : public Context {
   ImageCtx *image_ctx;
   Context *on_safe;
@@ -72,7 +88,6 @@ struct C_FlushCache : public Context {
   }
   virtual void finish(int r) {
     // successful cache flush indicates all IO is now safe
-    assert(image_ctx->owner_lock.is_locked());
     image_ctx->flush_cache(on_safe);
   }
 };
@@ -93,42 +108,44 @@ struct C_ShutDownCache : public Context {
 struct C_InvalidateCache : public Context {
   ImageCtx *image_ctx;
   bool purge_on_error;
+  bool reentrant_safe;
   Context *on_finish;
 
-  C_InvalidateCache(ImageCtx *_image_ctx, bool _purge_on_error, Context *_on_finish)
-    : image_ctx(_image_ctx), purge_on_error(_purge_on_error), on_finish(_on_finish) {
+  C_InvalidateCache(ImageCtx *_image_ctx, bool _purge_on_error,
+                    bool _reentrant_safe, Context *_on_finish)
+    : image_ctx(_image_ctx), purge_on_error(_purge_on_error),
+      reentrant_safe(_reentrant_safe), on_finish(_on_finish) {
   }
   virtual void finish(int r) {
-    {
-      RWLock::RLocker owner_Locker(image_ctx->owner_lock);
-      Mutex::Locker cache_locker(image_ctx->cache_lock);
-      CephContext *cct = image_ctx->cct;
+    assert(image_ctx->cache_lock.is_locked());
+    CephContext *cct = image_ctx->cct;
 
-      if (r == -EBLACKLISTED) {
-	lderr(cct) << "Blacklisted during flush!  Purging cache..." << dendl;
-	image_ctx->object_cacher->purge_set(image_ctx->object_set);
-      } else if (r != 0 && purge_on_error) {
-	lderr(cct) << "invalidate cache encountered error "
-		   << cpp_strerror(r) << " !Purging cache..." << dendl;
-	image_ctx->object_cacher->purge_set(image_ctx->object_set);
-      } else if (r != 0) {
-	lderr(cct) << "flush_cache returned " << r << dendl;
-      }
-
-      loff_t unclean = image_ctx->object_cacher->release_set(
-	image_ctx->object_set);
-      if (unclean == 0) {
-	r = 0;
-      } else {
-	lderr(cct) << "could not release all objects from cache: "
-		   << unclean << " bytes remain" << dendl;
-	if (r == 0) {
-	  r = -EBUSY;
-	}
-      }
+    if (r == -EBLACKLISTED) {
+      lderr(cct) << "Blacklisted during flush!  Purging cache..." << dendl;
+      image_ctx->object_cacher->purge_set(image_ctx->object_set);
+    } else if (r != 0 && purge_on_error) {
+      lderr(cct) << "invalidate cache encountered error "
+                 << cpp_strerror(r) << " !Purging cache..." << dendl;
+      image_ctx->object_cacher->purge_set(image_ctx->object_set);
+    } else if (r != 0) {
+      lderr(cct) << "flush_cache returned " << r << dendl;
     }
 
-    on_finish->complete(r);
+    loff_t unclean = image_ctx->object_cacher->release_set(
+      image_ctx->object_set);
+    if (unclean == 0) {
+      r = 0;
+    } else {
+      lderr(cct) << "could not release all objects from cache: "
+                 << unclean << " bytes remain" << dendl;
+      r = -EBUSY;
+    }
+
+    if (reentrant_safe) {
+      on_finish->complete(r);
+    } else {
+      image_ctx->op_work_queue->queue(on_finish, r);
+    }
   }
 
 };
@@ -180,12 +197,10 @@ struct C_InvalidateCache : public Context {
 
     memset(&header, 0, sizeof(header));
 
-    ThreadPoolSingleton *thread_pool_singleton;
-    cct->lookup_or_create_singleton_object<ThreadPoolSingleton>(
-      thread_pool_singleton, "librbd::thread_pool");
-    aio_work_queue = new AioImageRequestWQ<>(this, "librbd::aio_work_queue",
-					     cct->_conf->rbd_op_thread_timeout,
-					     thread_pool_singleton);
+    ThreadPool *thread_pool_singleton = get_thread_pool_instance(cct);
+    aio_work_queue = new AioImageRequestWQ(this, "librbd::aio_work_queue",
+                                           cct->_conf->rbd_op_thread_timeout,
+                                           thread_pool_singleton);
     op_work_queue = new ContextWQ("librbd::op_work_queue",
                                   cct->_conf->rbd_op_thread_timeout,
                                   thread_pool_singleton);
@@ -195,7 +210,7 @@ struct C_InvalidateCache : public Context {
     } else {
       exclusive_lock_policy = new exclusive_lock::StandardPolicy(this);
     }
-    journal_policy = new journal::StandardPolicy(this);
+    journal_policy = new journal::StandardPolicy<ImageCtx>(this);
   }
 
   ImageCtx::~ImageCtx() {
@@ -723,7 +738,6 @@ struct C_InvalidateCache : public Context {
   }
 
   void ImageCtx::flush_cache(Context *onfinish) {
-    assert(owner_lock.is_locked());
     cache_lock.Lock();
     object_cacher->flush_set(object_set, onfinish);
     cache_lock.Unlock();
@@ -735,15 +749,12 @@ struct C_InvalidateCache : public Context {
       return;
     }
 
-    RWLock::RLocker owner_locker(owner_lock);
     cache_lock.Lock();
     object_cacher->release_set(object_set);
     cache_lock.Unlock();
 
-    Context *shut_down = util::create_async_context_callback(
-      *this, new C_ShutDownCache(this, on_finish));
-    flush_cache(util::create_async_context_callback(
-      *this, new C_InvalidateCache(this, true, shut_down)));
+    C_ShutDownCache *shut_down = new C_ShutDownCache(this, on_finish);
+    flush_cache(new C_InvalidateCache(this, true, false, shut_down));
   }
 
   int ImageCtx::invalidate_cache(bool purge_on_error) {
@@ -757,14 +768,13 @@ struct C_InvalidateCache : public Context {
     cache_lock.Unlock();
 
     C_SaferCond ctx;
-    flush_cache(util::create_async_context_callback(
-      *this, new C_InvalidateCache(this, purge_on_error, &ctx)));
+    flush_cache(new C_InvalidateCache(this, purge_on_error, true, &ctx));
 
     int result = ctx.wait();
     return result;
   }
 
-  void ImageCtx::invalidate_cache(bool purge_on_error, Context *on_finish) {
+  void ImageCtx::invalidate_cache(Context *on_finish) {
     if (object_cacher == NULL) {
       op_work_queue->queue(on_finish, 0);
       return;
@@ -774,9 +784,7 @@ struct C_InvalidateCache : public Context {
     object_cacher->release_set(object_set);
     cache_lock.Unlock();
 
-    on_finish = util::create_async_context_callback(*this, on_finish);
-    flush_cache(util::create_async_context_callback(
-      *this, new C_InvalidateCache(this, purge_on_error, on_finish)));
+    flush_cache(new C_InvalidateCache(this, false, false, on_finish));
   }
 
   void ImageCtx::clear_nonexistence_cache() {
@@ -786,11 +794,6 @@ struct C_InvalidateCache : public Context {
     object_cacher->clear_nonexistence(object_set);
   }
 
-  bool ImageCtx::is_cache_empty() {
-    Mutex::Locker locker(cache_lock);
-    return object_cacher->set_is_empty(object_set);
-  }
-
   void ImageCtx::register_watch(Context *on_finish) {
     assert(image_watcher == NULL);
     image_watcher = new ImageWatcher<>(*this);
@@ -848,7 +851,6 @@ struct C_InvalidateCache : public Context {
     // ensure no locks are held when flush is complete
     on_safe = util::create_async_context_callback(*this, on_safe);
 
-    assert(owner_lock.is_locked());
     if (object_cacher != NULL) {
       // flush cache after completing all in-flight AIO ops
       on_safe = new C_FlushCache(this, on_safe);
@@ -1008,8 +1010,8 @@ struct C_InvalidateCache : public Context {
     return new ExclusiveLock<ImageCtx>(*this);
   }
 
-  ObjectMap<ImageCtx> *ImageCtx::create_object_map(uint64_t snap_id) {
-    return new ObjectMap<ImageCtx>(*this, snap_id);
+  ObjectMap *ImageCtx::create_object_map(uint64_t snap_id) {
+    return new ObjectMap(*this, snap_id);
   }
 
   Journal<ImageCtx> *ImageCtx::create_journal() {
@@ -1061,4 +1063,20 @@ struct C_InvalidateCache : public Context {
     delete journal_policy;
     journal_policy = policy;
   }
+
+  ThreadPool *ImageCtx::get_thread_pool_instance(CephContext *cct) {
+    ThreadPoolSingleton *thread_pool_singleton;
+    cct->lookup_or_create_singleton_object<ThreadPoolSingleton>(
+      thread_pool_singleton, "librbd::thread_pool");
+    return thread_pool_singleton;
+  }
+
+  void ImageCtx::get_timer_instance(CephContext *cct, SafeTimer **timer,
+                                    Mutex **timer_lock) {
+    SafeTimerSingleton *safe_timer_singleton;
+    cct->lookup_or_create_singleton_object<SafeTimerSingleton>(
+      safe_timer_singleton, "librbd::journal::safe_timer");
+    *timer = safe_timer_singleton;
+    *timer_lock = &safe_timer_singleton->lock;
+  }
 }
