@@ -32,12 +32,14 @@ class CephContext;
 class ContextWQ;
 class Finisher;
 class PerfCounters;
+class ThreadPool;
+class SafeTimer;
 
 namespace librbd {
 
   struct ImageCtx;
   class AioCompletion;
-  template <typename> class AioImageRequestWQ;
+  class AioImageRequestWQ;
   class AsyncOperation;
   class CopyupRequest;
   template <typename> class ExclusiveLock;
@@ -45,10 +47,11 @@ namespace librbd {
   template <typename> class ImageWatcher;
   template <typename> class Journal;
   class LibrbdAdminSocketHook;
-  template <typename> class ObjectMap;
+  class ObjectMap;
   template <typename> class Operations;
   class LibrbdWriteback;
 
+  namespace cache { struct ImageCache; }
   namespace exclusive_lock { struct Policy; }
   namespace journal { struct Policy; }
 
@@ -119,11 +122,13 @@ namespace librbd {
     std::string id; // only used for new-format images
     parent_info parent_md;
     ImageCtx *parent;
+    cls::rbd::GroupSpec group_spec;
     uint64_t stripe_unit, stripe_count;
     uint64_t flags;
 
     file_layout_t layout;
 
+    cache::ImageCache *image_cache = nullptr;
     ObjectCacher *object_cacher;
     LibrbdWriteback *writeback_handler;
     ObjectCacher::ObjectSet *object_set;
@@ -141,11 +146,11 @@ namespace librbd {
     Operations<ImageCtx> *operations;
 
     ExclusiveLock<ImageCtx> *exclusive_lock;
-    ObjectMap<ImageCtx> *object_map;
+    ObjectMap *object_map;
 
     xlist<operation::ResizeRequest<ImageCtx>*> resize_reqs;
 
-    AioImageRequestWQ<ImageCtx> *aio_work_queue;
+    AioImageRequestWQ *aio_work_queue;
     xlist<AioCompletion*> completed_reqs;
     EventSocket event_socket;
 
@@ -268,10 +273,9 @@ namespace librbd {
     void user_flushed();
     void flush_cache(Context *onfinish);
     void shut_down_cache(Context *on_finish);
-    int invalidate_cache(bool purge_on_error);
-    void invalidate_cache(bool purge_on_error, Context *on_finish);
+    int invalidate_cache(bool purge_on_error=false);
+    void invalidate_cache(Context *on_finish);
     void clear_nonexistence_cache();
-    bool is_cache_empty();
     void register_watch(Context *on_finish);
     uint64_t prune_parent_extents(vector<pair<uint64_t,uint64_t> >& objectx,
 				  uint64_t overlap);
@@ -288,7 +292,7 @@ namespace librbd {
     void apply_metadata(const std::map<std::string, bufferlist> &meta);
 
     ExclusiveLock<ImageCtx> *create_exclusive_lock();
-    ObjectMap<ImageCtx> *create_object_map(uint64_t snap_id);
+    ObjectMap *create_object_map(uint64_t snap_id);
     Journal<ImageCtx> *create_journal();
 
     void clear_pending_completions();
@@ -303,6 +307,10 @@ namespace librbd {
 
     journal::Policy *get_journal_policy() const;
     void set_journal_policy(journal::Policy *policy);
+
+    static ThreadPool *get_thread_pool_instance(CephContext *cct);
+    static void get_timer_instance(CephContext *cct, SafeTimer **timer,
+                                   Mutex **timer_lock);
   };
 }
 
