@@ -16,6 +16,8 @@
 #include "common/Timer.h"
 #include "common/WorkQueue.h"
 #include "include/rados/librados.hpp"
+#include "librbd/journal/RemoveRequest.h"
+#include "librbd/journal/CreateRequest.h"
 
 #include <boost/scope_exit.hpp>
 #include <utility>
@@ -155,28 +157,12 @@ public:
   }
 };
 
-class SafeTimerSingleton : public SafeTimer {
-public:
-  Mutex lock;
-
-  explicit SafeTimerSingleton(CephContext *cct)
-      : SafeTimer(cct, lock, true),
-        lock("librbd::Journal::SafeTimerSingleton::lock") {
-    init();
-  }
-  virtual ~SafeTimerSingleton() {
-    Mutex::Locker locker(lock);
-    shutdown();
-  }
-};
-
 template <typename I>
 struct C_IsTagOwner : public Context {
   I *image_ctx;
   bool *is_tag_owner;
   Context *on_finish;
 
-  typedef ::journal::Journaler Journaler;
   Journaler *journaler;
   cls::journal::Client client;
   journal::ImageClientMeta client_meta;
@@ -479,12 +465,7 @@ Journal<I>::Journal(I &image_ctx)
   m_work_queue = new ContextWQ("librbd::journal::work_queue",
                                cct->_conf->rbd_op_thread_timeout,
                                thread_pool_singleton);
-
-  SafeTimerSingleton *safe_timer_singleton;
-  cct->lookup_or_create_singleton_object<SafeTimerSingleton>(
-    safe_timer_singleton, "librbd::journal::safe_timer");
-  m_timer = safe_timer_singleton;
-  m_timer_lock = &safe_timer_singleton->lock;
+  ImageCtx::get_timer_instance(cct, &m_timer, &m_timer_lock);
 }
 
 template <typename I>
@@ -509,58 +490,25 @@ bool Journal<I>::is_journal_supported(I &image_ctx) {
 
 template <typename I>
 int Journal<I>::create(librados::IoCtx &io_ctx, const std::string &image_id,
-		       uint8_t order, uint8_t splay_width,
-		       const std::string &object_pool, bool non_primary,
-                       const std::string &primary_mirror_uuid) {
+                       uint8_t order, uint8_t splay_width,
+                       const std::string &object_pool) {
   CephContext *cct = reinterpret_cast<CephContext *>(io_ctx.cct());
   ldout(cct, 5) << __func__ << ": image=" << image_id << dendl;
 
-  librados::Rados rados(io_ctx);
-  int64_t pool_id = -1;
-  if (!object_pool.empty()) {
-    IoCtx data_io_ctx;
-    int r = rados.ioctx_create(object_pool.c_str(), data_io_ctx);
-    if (r != 0) {
-      lderr(cct) << __func__ << ": "
-                 << "failed to create journal: "
-		 << "error opening journal objects pool '" << object_pool
-		 << "': " << cpp_strerror(r) << dendl;
-      return r;
-    }
-    pool_id = data_io_ctx.get_id();
-  }
+  C_SaferCond cond;
+  journal::TagData tag_data(LOCAL_MIRROR_UUID);
+  ContextWQ op_work_queue("librbd::op_work_queue",
+                          cct->_conf->rbd_op_thread_timeout,
+                          ImageCtx::get_thread_pool_instance(cct));
+  journal::CreateRequest<I> *req = journal::CreateRequest<I>::create(
+    io_ctx, image_id, order, splay_width, object_pool, cls::journal::Tag::TAG_CLASS_NEW,
+    tag_data, IMAGE_CLIENT_ID, &op_work_queue, &cond);
+  req->send();
 
-  Journaler journaler(io_ctx, image_id, IMAGE_CLIENT_ID, {});
+  int r = cond.wait();
+  op_work_queue.drain();
 
-  int r = journaler.create(order, splay_width, pool_id);
-  if (r < 0) {
-    lderr(cct) << __func__ << ": "
-               << "failed to create journal: " << cpp_strerror(r) << dendl;
-    return r;
-  }
-
-  cls::journal::Client client;
-  cls::journal::Tag tag;
-
-  assert(non_primary ^ primary_mirror_uuid.empty());
-  std::string mirror_uuid = (non_primary ? primary_mirror_uuid :
-                                           LOCAL_MIRROR_UUID);
-  journal::TagPredecessor predecessor;
-  r = allocate_journaler_tag(cct, &journaler, client,
-			     cls::journal::Tag::TAG_CLASS_NEW,
-                             predecessor, mirror_uuid, &tag);
-
-  bufferlist client_data;
-  ::encode(journal::ClientData{journal::ImageClientMeta{tag.tag_class}},
-           client_data);
-
-  r = journaler.register_client(client_data);
-  if (r < 0) {
-    lderr(cct) << __func__ << ": "
-               << "failed to register client: " << cpp_strerror(r) << dendl;
-    return r;
-  }
-  return 0;
+  return r;
 }
 
 template <typename I>
@@ -568,40 +516,18 @@ int Journal<I>::remove(librados::IoCtx &io_ctx, const std::string &image_id) {
   CephContext *cct = reinterpret_cast<CephContext *>(io_ctx.cct());
   ldout(cct, 5) << __func__ << ": image=" << image_id << dendl;
 
-  Journaler journaler(io_ctx, image_id, IMAGE_CLIENT_ID, {});
-
-  bool journal_exists;
-  int r = journaler.exists(&journal_exists);
-  if (r < 0) {
-    lderr(cct) << __func__ << ": "
-               << "failed to stat journal header: " << cpp_strerror(r) << dendl;
-    return r;
-  } else if (!journal_exists) {
-    return 0;
-  }
-
   C_SaferCond cond;
-  journaler.init(&cond);
-  BOOST_SCOPE_EXIT_ALL(&journaler) {
-    journaler.shut_down();
-  };
+  ContextWQ op_work_queue("librbd::op_work_queue",
+                          cct->_conf->rbd_op_thread_timeout,
+                          ImageCtx::get_thread_pool_instance(cct));
+  journal::RemoveRequest<I> *req = journal::RemoveRequest<I>::create(
+    io_ctx, image_id, IMAGE_CLIENT_ID, &op_work_queue, &cond);
+  req->send();
 
-  r = cond.wait();
-  if (r == -ENOENT) {
-    return 0;
-  } else if (r < 0) {
-    lderr(cct) << __func__ << ": "
-               << "failed to initialize journal: " << cpp_strerror(r) << dendl;
-    return r;
-  }
+  int r = cond.wait();
+  op_work_queue.drain();
 
-  r = journaler.remove(true);
-  if (r < 0) {
-    lderr(cct) << __func__ << ": "
-               << "failed to remove journal: " << cpp_strerror(r) << dendl;
-    return r;
-  }
-  return 0;
+  return r;
 }
 
 template <typename I>
@@ -641,14 +567,16 @@ int Journal<I>::reset(librados::IoCtx &io_ctx, const std::string &image_id) {
     }
   }
 
-  r = journaler.remove(true);
+  C_SaferCond ctx1;
+  journaler.remove(true, &ctx1);
+  r = ctx1.wait();
   if (r < 0) {
     lderr(cct) << __func__ << ": "
                << "failed to reset journal: " << cpp_strerror(r) << dendl;
     return r;
   }
 
-  r = create(io_ctx, image_id, order, splay_width, pool_name, false, "");
+  r = create(io_ctx, image_id, order, splay_width, pool_name);
   if (r < 0) {
     lderr(cct) << __func__ << ": "
                << "failed to create journal: " << cpp_strerror(r) << dendl;
@@ -797,40 +725,6 @@ int Journal<I>::promote(I *image_ctx) {
     return r;
   }
 
-  journaler.start_append(0, 0, 0);
-  BOOST_SCOPE_EXIT_ALL(&journaler) {
-    C_SaferCond stop_append_ctx;
-    journaler.stop_append(&stop_append_ctx);
-    stop_append_ctx.wait();
-  };
-
-  journal::EventEntry event_entry{journal::DemotePromoteEvent{}};
-  bufferlist event_entry_bl;
-  ::encode(event_entry, event_entry_bl);
-  Future future = journaler.append(new_tag.tid, event_entry_bl);
-
-  C_SaferCond flush_ctx;
-  future.flush(&flush_ctx);
-  r = flush_ctx.wait();
-  if (r < 0) {
-    lderr(cct) << __func__ << ": "
-               << "failed to append promotion journal event: " << cpp_strerror(r)
-               << dendl;
-    return r;
-  }
-
-  journaler.committed(future);
-
-  C_SaferCond flush_commit_ctx;
-  journaler.flush_commit_position(&flush_commit_ctx);
-  r = flush_commit_ctx.wait();
-  if (r < 0) {
-    lderr(cct) << __func__ << ": "
-               << "failed to flush promotion commit position: "
-               << cpp_strerror(r) << dendl;
-    return r;
-  }
-
   return 0;
 }
 
@@ -996,7 +890,7 @@ int Journal<I>::demote() {
       return r;
     }
 
-    journal::EventEntry event_entry{journal::DemotePromoteEvent{}};
+    journal::EventEntry event_entry{journal::DemoteEvent{}};
     bufferlist event_entry_bl;
     ::encode(event_entry, event_entry_bl);
 
@@ -1103,8 +997,6 @@ uint64_t Journal<I>::append_write_event(uint64_t offset, size_t length,
                                         const bufferlist &bl,
                                         const AioObjectRequests &requests,
                                         bool flush_entry) {
-  assert(m_image_ctx.owner_lock.is_locked());
-
   assert(m_max_append_size > journal::AioWriteEvent::get_fixed_size());
   uint64_t max_write_data_size =
     m_max_append_size - journal::AioWriteEvent::get_fixed_size();
@@ -1138,8 +1030,6 @@ uint64_t Journal<I>::append_io_event(journal::EventEntry &&event_entry,
                                      const AioObjectRequests &requests,
                                      uint64_t offset, size_t length,
                                      bool flush_entry) {
-  assert(m_image_ctx.owner_lock.is_locked());
-
   bufferlist bl;
   ::encode(event_entry, bl);
   return append_io_events(event_entry.get_event_type(), {bl}, requests, offset,
@@ -1152,7 +1042,6 @@ uint64_t Journal<I>::append_io_events(journal::EventType event_type,
                                       const AioObjectRequests &requests,
                                       uint64_t offset, size_t length,
                                       bool flush_entry) {
-  assert(m_image_ctx.owner_lock.is_locked());
   assert(!bufferlists.empty());
 
   uint64_t tid;
@@ -1712,10 +1601,6 @@ void Journal<I>::handle_replay_complete(int r) {
         handle_flushing_replay();
       }
     });
-  ctx = new FunctionContext([this, ctx](int r) {
-      // ensure the commit position is flushed to disk
-      m_journaler->flush_commit_position(ctx);
-    });
   ctx = new FunctionContext([this, cct, cancel_ops, ctx](int r) {
       ldout(cct, 20) << this << " handle_replay_complete: "
                      << "shut down replay" << dendl;
@@ -1760,13 +1645,7 @@ void Journal<I>::handle_replay_process_safe(ReplayEntry replay_entry, int r) {
       m_lock.Unlock();
 
       // stop replay, shut down, and restart
-      Context* ctx = create_context_callback<
-        Journal<I>, &Journal<I>::handle_flushing_restart>(this);
-      ctx = new FunctionContext([this, ctx](int r) {
-          // ensure the commit position is flushed to disk
-          m_journaler->flush_commit_position(ctx);
-        });
-      ctx = new FunctionContext([this, cct, ctx](int r) {
+      Context *ctx = new FunctionContext([this, cct](int r) {
           ldout(cct, 20) << this << " handle_replay_process_safe: "
                          << "shut down replay" << dendl;
           {
@@ -1774,7 +1653,8 @@ void Journal<I>::handle_replay_process_safe(ReplayEntry replay_entry, int r) {
             assert(m_state == STATE_FLUSHING_RESTART);
           }
 
-          m_journal_replay->shut_down(true, ctx);
+          m_journal_replay->shut_down(true, create_context_callback<
+            Journal<I>, &Journal<I>::handle_flushing_restart>(this));
         });
       m_journaler->stop_replay(ctx);
       return;
@@ -1916,7 +1796,6 @@ void Journal<I>::handle_io_event_safe(int r, uint64_t tid) {
       (*it)->complete(r);
     } else {
       // send any waiting aio requests now that journal entry is safe
-      RWLock::RLocker owner_locker(m_image_ctx.owner_lock);
       (*it)->send();
     }
   }
