@@ -11,7 +11,6 @@
 #include "common/Cond.h"
 #include "common/Mutex.h"
 #include "common/Cond.h"
-#include "common/WorkQueue.h"
 #include "journal/Future.h"
 #include "journal/JournalMetadataListener.h"
 #include "journal/ReplayEntry.h"
@@ -24,6 +23,7 @@
 #include <string>
 #include <unordered_map>
 
+class ContextWQ;
 class SafeTimer;
 namespace journal {
 class Journaler;
@@ -97,9 +97,8 @@ public:
 
   static bool is_journal_supported(ImageCtxT &image_ctx);
   static int create(librados::IoCtx &io_ctx, const std::string &image_id,
-		    uint8_t order, uint8_t splay_width,
-		    const std::string &object_pool, bool non_primary,
-                    const std::string &primary_mirror_uuid);
+                    uint8_t order, uint8_t splay_width,
+                    const std::string &object_pool);
   static int remove(librados::IoCtx &io_ctx, const std::string &image_id);
   static int reset(librados::IoCtx &io_ctx, const std::string &image_id);
 
