@@ -28,9 +28,8 @@ using util::create_context_callback;
 
 template <typename I>
 RefreshRequest<I>::RefreshRequest(I &image_ctx, bool acquiring_lock,
-                                  bool skip_open_parent, Context *on_finish)
+                                  Context *on_finish)
   : m_image_ctx(image_ctx), m_acquiring_lock(acquiring_lock),
-    m_skip_open_parent_image(skip_open_parent),
     m_on_finish(create_async_context_callback(m_image_ctx, on_finish)),
     m_error_result(0), m_flush_aio(false), m_exclusive_lock(nullptr),
     m_object_map(nullptr), m_journal(nullptr), m_refresh_parent(nullptr) {
@@ -251,7 +250,8 @@ Context *RefreshRequest<I>::handle_v2_get_mutable_metadata(int *result) {
                                                       &m_lockers,
                                                       &m_exclusive_locked,
                                                       &m_lock_tag, &m_snapc,
-                                                      &m_parent_md);
+                                                      &m_parent_md,
+						      &m_group_spec);
   }
   if (*result < 0) {
     lderr(cct) << "failed to retrieve mutable metadata: "
@@ -398,8 +398,8 @@ void RefreshRequest<I>::send_v2_refresh_parent() {
 
     parent_info parent_md;
     int r = get_parent_info(m_image_ctx.snap_id, &parent_md);
-    if (!m_skip_open_parent_image && (r < 0 ||
-        RefreshParentRequest<I>::is_refresh_required(m_image_ctx, parent_md))) {
+    if (r < 0 ||
+        RefreshParentRequest<I>::is_refresh_required(m_image_ctx, parent_md)) {
       CephContext *cct = m_image_ctx.cct;
       ldout(cct, 10) << this << " " << __func__ << dendl;
 
@@ -499,7 +499,7 @@ void RefreshRequest<I>::send_v2_open_journal() {
         !journal_disabled_by_policy &&
         m_image_ctx.exclusive_lock != nullptr &&
         m_image_ctx.journal == nullptr) {
-      m_image_ctx.aio_work_queue->set_require_lock(AIO_DIRECTION_BOTH, true);
+      m_image_ctx.aio_work_queue->set_require_lock_on_read();
     }
     send_v2_block_writes();
     return;
@@ -877,6 +877,8 @@ void RefreshRequest<I>::apply() {
       m_image_ctx.parent_md = m_parent_md;
     }
 
+     m_image_ctx.group_spec = m_group_spec;
+
     for (size_t i = 0; i < m_snapc.snaps.size(); ++i) {
       std::vector<librados::snap_t>::const_iterator it = std::find(
         m_image_ctx.snaps.begin(), m_image_ctx.snaps.end(),
@@ -936,8 +938,8 @@ void RefreshRequest<I>::apply() {
       }
       if (!m_image_ctx.test_features(RBD_FEATURE_JOURNALING,
                                      m_image_ctx.snap_lock)) {
-        if (!m_image_ctx.clone_copy_on_read && m_image_ctx.journal != nullptr) {
-          m_image_ctx.aio_work_queue->set_require_lock(AIO_DIRECTION_READ, false);
+        if (m_image_ctx.journal != nullptr) {
+          m_image_ctx.aio_work_queue->clear_require_lock_on_read();
         }
         std::swap(m_journal, m_image_ctx.journal);
       } else if (m_journal != nullptr) {
