@@ -24,6 +24,8 @@
 #include "include/compat.h"
 #include "include/on_exit.h"
 #include "include/uuid.h"
+#include "Entry.h"
+#include "SubsystemMap.h"
 
 #define DEFAULT_MAX_NEW    100
 #define DEFAULT_MAX_RECENT 10000
@@ -281,7 +283,7 @@ void Log::flush()
 
   // trim
   while (m_recent.m_len > m_max_recent) {
-    m_recent.dequeue()->destroy();
+    delete m_recent.dequeue();
   }
 
   m_flush_mutex_holder = 0;
@@ -474,10 +476,5 @@ void Log::inject_segv()
   m_inject_segv = true;
 }
 
-void Log::reset_segv()
-{
-  m_inject_segv = false;
-}
-
 } // ceph::log::
 } // ceph::
