@@ -8,14 +8,14 @@
 
 #include <pthread.h>
 
-#include "Entry.h"
 #include "EntryQueue.h"
-#include "SubsystemMap.h"
 
 namespace ceph {
 namespace log {
 
 class Graylog;
+class SubsystemMap;
+class Entry;
 
 class Log : private Thread
 {
@@ -96,7 +96,6 @@ public:
 
   /// induce a segv on the next log event
   void inject_segv();
-  void reset_segv();
 };
 
 }
