@@ -301,7 +301,8 @@ CDentry *CDir::lookup(const string& name, snapid_t snap)
   if (iter == items.end())
     return 0;
   if (iter->second->name == name &&
-      iter->second->first <= snap) {
+      iter->second->first <= snap &&
+      iter->second->last >= snap) {
     dout(20) << "  hit -> " << iter->first << dendl;
     return iter->second;
   }
@@ -560,10 +561,10 @@ void CDir::link_inode_work( CDentry *dn, CInode *in)
 
 void CDir::unlink_inode(CDentry *dn)
 {
-  if (dn->get_linkage()->is_remote()) {
-    dout(12) << "unlink_inode " << *dn << dendl;
-  } else {
+  if (dn->get_linkage()->is_primary()) {
     dout(12) << "unlink_inode " << *dn << " " << *dn->get_linkage()->get_inode() << dendl;
+  } else {
+    dout(12) << "unlink_inode " << *dn << dendl;
   }
 
   unlink_inode_work(dn);
@@ -611,10 +612,8 @@ void CDir::unlink_inode_work( CDentry *dn )
       dn->unlink_remote(dn->get_linkage());
 
     dn->get_linkage()->set_remote(0, 0);
-  } else {
+  } else if (dn->get_linkage()->is_primary()) {
     // primary
-    assert(dn->get_linkage()->is_primary());
- 
     // unpin dentry?
     if (in->get_num_ref())
       dn->put(CDentry::PIN_INODEPIN);
@@ -626,6 +625,8 @@ void CDir::unlink_inode_work( CDentry *dn )
     // detach inode
     in->remove_primary_parent(dn);
     dn->get_linkage()->inode = 0;
+  } else {
+    assert(!dn->get_linkage()->is_null());
   }
 }
 
@@ -938,10 +939,8 @@ void CDir::split(int bits, list<CDir*>& subs, list<MDSInternalContextBase*>& wai
   frag_info_t fragstatdiff;
   if (fnode.accounted_rstat.version == rstat_version)
     rstatdiff.add_delta(fnode.accounted_rstat, fnode.rstat);
-  if (fnode.accounted_fragstat.version == dirstat_version) {
-    bool touched_mtime;
-    fragstatdiff.add_delta(fnode.accounted_fragstat, fnode.fragstat, touched_mtime);
-  }
+  if (fnode.accounted_fragstat.version == dirstat_version)
+    fragstatdiff.add_delta(fnode.accounted_fragstat, fnode.fragstat);
   dout(10) << " rstatdiff " << rstatdiff << " fragstatdiff " << fragstatdiff << dendl;
 
   prepare_old_fragment(replay);
@@ -1019,7 +1018,7 @@ void CDir::merge(list<CDir*>& subs, list<MDSInternalContextBase*>& waiters, bool
 
   nest_info_t rstatdiff;
   frag_info_t fragstatdiff;
-  bool touched_mtime;
+  bool touched_mtime, touched_chattr;
   version_t rstat_version = inode->get_projected_inode()->rstat.version;
   version_t dirstat_version = inode->get_projected_inode()->dirstat.version;
 
@@ -1032,7 +1031,7 @@ void CDir::merge(list<CDir*>& subs, list<MDSInternalContextBase*>& waiters, bool
       rstatdiff.add_delta(dir->fnode.accounted_rstat, dir->fnode.rstat);
     if (dir->fnode.accounted_fragstat.version == dirstat_version)
       fragstatdiff.add_delta(dir->fnode.accounted_fragstat, dir->fnode.fragstat,
-			     touched_mtime);
+			     &touched_mtime, &touched_chattr);
 
     dir->prepare_old_fragment(replay);
 
@@ -1358,14 +1357,24 @@ void CDir::mark_clean()
   }
 }
 
+
+struct C_Dir_Dirty : public CDirContext {
+  version_t pv;
+  LogSegment *ls;
+  C_Dir_Dirty(CDir *d, version_t p, LogSegment *l) : CDirContext(d), pv(p), ls(l) {}
+  void finish(int r) {
+    dir->mark_dirty(pv, ls);
+    dir->auth_unpin(dir);
+  }
+};
+
 // caller should hold auth pin of this
 void CDir::log_mark_dirty()
 {
-  if (is_dirty() || projected_version > get_version())
-    return; // noop if it is already dirty or will be dirty
-
+  MDLog *mdlog = inode->mdcache->mds->mdlog;
   version_t pv = pre_dirty();
-  mark_dirty(pv, cache->mds->mdlog->get_current_segment());
+  mdlog->flush();
+  mdlog->wait_for_safe(new C_Dir_Dirty(this, pv, mdlog->get_current_segment()));
 }
 
 void CDir::mark_complete() {
@@ -1433,6 +1442,7 @@ void CDir::fetch(MDSInternalContextBase *c, const string& want_dn, bool ignore_a
   }
 
   if (c) add_waiter(WAIT_COMPLETE, c);
+  if (!want_dn.empty()) wanted_items.insert(want_dn);
   
   // already fetching?
   if (state_test(CDir::STATE_FETCHING)) {
@@ -1445,93 +1455,76 @@ void CDir::fetch(MDSInternalContextBase *c, const string& want_dn, bool ignore_a
 
   if (cache->mds->logger) cache->mds->logger->inc(l_mds_dir_fetch);
 
-  _omap_fetch(want_dn);
+  std::set<dentry_key_t> empty;
+  _omap_fetch(NULL, empty);
 }
 
-class C_IO_Dir_TMAP_Fetched : public CDirIOContext {
- protected:
-  string want_dn;
- public:
-  bufferlist bl;
-
-  C_IO_Dir_TMAP_Fetched(CDir *d, const string& w) : CDirIOContext(d), want_dn(w) { }
-  void finish(int r) {
-    dir->_tmap_fetched(bl, want_dn, r);
-  }
-};
-
-void CDir::_tmap_fetch(const string& want_dn)
+void CDir::fetch(MDSInternalContextBase *c, const std::set<dentry_key_t>& keys)
 {
-  // start by reading the first hunk of it
-  C_IO_Dir_TMAP_Fetched *fin = new C_IO_Dir_TMAP_Fetched(this, want_dn);
-  object_t oid = get_ondisk_object();
-  object_locator_t oloc(cache->mds->mdsmap->get_metadata_pool());
-  ObjectOperation rd;
-  rd.tmap_get(&fin->bl, NULL);
-  cache->mds->objecter->read(oid, oloc, rd, CEPH_NOSNAP, NULL, 0,
-			     new C_OnFinisher(fin, cache->mds->finisher));
-}
+  dout(10) << "fetch " << keys.size() << " keys on " << *this << dendl;
 
-void CDir::_tmap_fetched(bufferlist& bl, const string& want_dn, int r)
-{
-  LogChannelRef clog = cache->mds->clog;
-  dout(10) << "_tmap_fetched " << bl.length()  << " bytes for " << *this
-	   << " want_dn=" << want_dn << dendl;
-
-  assert(r == 0 || r == -ENOENT);
   assert(is_auth());
-  assert(!is_frozen());
+  assert(!is_complete());
 
-  bufferlist header;
-  map<string, bufferlist> omap;
-
-  if (bl.length() == 0) {
-    r = -ENODATA;
-  } else {
-    bufferlist::iterator p = bl.begin();
-    ::decode(header, p);
-    ::decode(omap, p);
-
-    if (!p.end()) {
-      clog->warn() << "tmap buffer of dir " << dirfrag() << " has "
-		  << bl.length() - p.get_off() << " extra bytes\n";
-    }
-    bl.clear();
+  if (!can_auth_pin()) {
+    dout(7) << "fetch keys waiting for authpinnable" << dendl;
+    add_waiter(WAIT_UNFREEZE, c);
+    return;
+  }
+  if (state_test(CDir::STATE_FETCHING)) {
+    dout(7) << "fetch keys waiting for full fetch" << dendl;
+    add_waiter(WAIT_COMPLETE, c);
+    return;
   }
 
-  _omap_fetched(header, omap, want_dn, r);
+  auth_pin(this);
+  if (cache->mds->logger) cache->mds->logger->inc(l_mds_dir_fetch);
+
+  _omap_fetch(c, keys);
 }
 
 class C_IO_Dir_OMAP_Fetched : public CDirIOContext {
- protected:
-  string want_dn;
- public:
+  MDSInternalContextBase *fin;
+public:
   bufferlist hdrbl;
   map<string, bufferlist> omap;
   bufferlist btbl;
   int ret1, ret2, ret3;
 
-  C_IO_Dir_OMAP_Fetched(CDir *d, const string& w) : 
-    CDirIOContext(d), want_dn(w),
-    ret1(0), ret2(0), ret3(0) {}
+  C_IO_Dir_OMAP_Fetched(CDir *d, MDSInternalContextBase *f) :
+    CDirIOContext(d), fin(f), ret1(0), ret2(0), ret3(0) { }
   void finish(int r) {
     // check the correctness of backtrace
     if (r >= 0 && ret3 != -ECANCELED)
       dir->inode->verify_diri_backtrace(btbl, ret3);
     if (r >= 0) r = ret1;
     if (r >= 0) r = ret2;
-    dir->_omap_fetched(hdrbl, omap, want_dn, r);
+    dir->_omap_fetched(hdrbl, omap, !fin, r);
+    if (fin)
+      fin->complete(r);
   }
 };
 
-void CDir::_omap_fetch(const string& want_dn)
+void CDir::_omap_fetch(MDSInternalContextBase *c, const std::set<dentry_key_t>& keys)
 {
-  C_IO_Dir_OMAP_Fetched *fin = new C_IO_Dir_OMAP_Fetched(this, want_dn);
+  C_IO_Dir_OMAP_Fetched *fin = new C_IO_Dir_OMAP_Fetched(this, c);
   object_t oid = get_ondisk_object();
   object_locator_t oloc(cache->mds->mdsmap->get_metadata_pool());
   ObjectOperation rd;
   rd.omap_get_header(&fin->hdrbl, &fin->ret1);
-  rd.omap_get_vals("", "", (uint64_t)-1, &fin->omap, &fin->ret2);
+  if (keys.empty()) {
+    assert(!c);
+    rd.omap_get_vals("", "", (uint64_t)-1, &fin->omap, &fin->ret2);
+  } else {
+    assert(c);
+    std::set<std::string> str_keys;
+    for (auto p = keys.begin(); p != keys.end(); ++p) {
+      string str;
+      p->encode(str);
+      str_keys.insert(str);
+    }
+    rd.omap_get_vals_by_keys(str_keys, &fin->omap, &fin->ret2);
+  }
   // check the correctness of backtrace
   if (g_conf->mds_verify_backtrace > 0 && frag == frag_t()) {
     rd.getxattr("parent", &fin->btbl, &fin->ret3);
@@ -1551,7 +1544,8 @@ CDentry *CDir::_load_dentry(
     bufferlist &bl,
     const int pos,
     const std::set<snapid_t> *snaps,
-    bool *force_dirty)
+    bool *force_dirty,
+    list<CInode*> *undef_inodes)
 {
   bufferlist::iterator q = bl.begin();
 
@@ -1603,16 +1597,10 @@ CDentry *CDir::_load_dentry(
     }
 
     if (dn) {
-      CDentry::linkage_t *dnl = dn->get_linkage();
-      dout(12) << "_fetched had " << (dnl->is_null() ? "NEG" : "") << " dentry " << *dn << dendl;
-      if (committed_version == 0 &&
-	  dnl->is_remote() &&
-	  dn->is_dirty() &&
-	  ino == dnl->get_remote_ino() &&
-	  d_type == dnl->get_remote_d_type()) {
-	// see comment below
-	dout(10) << "_fetched  had underwater dentry " << *dn << ", marking clean" << dendl;
-	dn->mark_clean();
+      if (dn->get_linkage()->get_inode() == 0) {
+        dout(12) << "_fetched  had NEG dentry " << *dn << dendl;
+      } else {
+        dout(12) << "_fetched  had dentry " << *dn << dendl;
       }
     } else {
       // (remote) link
@@ -1645,35 +1633,15 @@ CDentry *CDir::_load_dentry(
 
     bool undef_inode = false;
     if (dn) {
-      CDentry::linkage_t *dnl = dn->get_linkage();
-      dout(12) << "_fetched had " << (dnl->is_null() ? "NEG" : "") << " dentry " << *dn << dendl;
-
-      if (dnl->is_primary()) {
-	CInode *in = dnl->get_inode();
-	if (in->state_test(CInode::STATE_REJOINUNDEF)) {
-	  undef_inode = true;
-	} else if (committed_version == 0 &&
-		   dn->is_dirty() &&
-		   inode_data.inode.ino == in->ino() &&
-		   inode_data.inode.version == in->get_version()) {
-	  /* clean underwater item?
-	   * Underwater item is something that is dirty in our cache from
-	   * journal replay, but was previously flushed to disk before the
-	   * mds failed.
-	   *
-	   * We only do this is committed_version == 0. that implies either
-	   * - this is a fetch after from a clean/empty CDir is created
-	   *   (and has no effect, since the dn won't exist); or
-	   * - this is a fetch after _recovery_, which is what we're worried
-	   *   about.  Items that are marked dirty from the journal should be
-	   *   marked clean if they appear on disk.
-	   */
-	  dout(10) << "_fetched  had underwater dentry " << *dn << ", marking clean" << dendl;
-	  dn->mark_clean();
-	  dout(10) << "_fetched  had underwater inode " << *dnl->get_inode() << ", marking clean" << dendl;
-	  in->mark_clean();
-	}
-      }
+      CInode *in = dn->get_linkage()->get_inode();
+      if (in) {
+        dout(12) << "_fetched  had dentry " << *dn << dendl;
+        if (in->state_test(CInode::STATE_REJOINUNDEF)) {
+          undef_inodes->push_back(in);
+          undef_inode = true;
+        }
+      } else
+        dout(12) << "_fetched  had NEG dentry " << *dn << dendl;
     }
 
     if (!dn || undef_inode) {
@@ -1714,9 +1682,7 @@ CDentry *CDir::_load_dentry(
           in->mark_dirty_rstat();
 
         if (inode->is_stray()) {
-          dn->state_set(CDentry::STATE_STRAY);
-          if (in->inode.nlink == 0)
-            in->state_set(CInode::STATE_ORPHAN);
+	  cache->notify_stray_loaded(dn);
         }
 
         //in->hack_accessed = false;
@@ -1747,30 +1713,23 @@ CDentry *CDir::_load_dentry(
 }
 
 void CDir::_omap_fetched(bufferlist& hdrbl, map<string, bufferlist>& omap,
-			 const string& want_dn, int r)
+			 bool complete, int r)
 {
   LogChannelRef clog = cache->mds->clog;
   dout(10) << "_fetched header " << hdrbl.length() << " bytes "
-	   << omap.size() << " keys for " << *this
-	   << " want_dn=" << want_dn << dendl;
+	   << omap.size() << " keys for " << *this << dendl;
 
   assert(r == 0 || r == -ENOENT || r == -ENODATA);
   assert(is_auth());
   assert(!is_frozen());
 
   if (hdrbl.length() == 0) {
-    if (r != -ENODATA) { // called by _tmap_fetched() ?
-      dout(10) << "_fetched 0 byte from omap, retry tmap" << dendl;
-      _tmap_fetch(want_dn);
-      return;
-    }
-
     dout(0) << "_fetched missing object for " << *this << dendl;
 
     clog->error() << "dir " << dirfrag() << " object missing on disk; some "
                      "files may be lost (" << get_path() << ")";
 
-    go_bad();
+    go_bad(complete);
     return;
   }
 
@@ -1784,14 +1743,14 @@ void CDir::_omap_fetched(bufferlist& hdrbl, map<string, bufferlist>& omap,
         << ": " << err << dendl;
       clog->warn() << "Corrupt fnode header in " << dirfrag() << ": "
 		  << err << " (" << get_path() << ")";
-      go_bad();
+      go_bad(complete);
       return;
     }
     if (!p.end()) {
       clog->warn() << "header buffer of dir " << dirfrag() << " has "
 		  << hdrbl.length() - p.get_off() << " extra bytes ("
                   << get_path() << ")";
-      go_bad();
+      go_bad(complete);
       return;
     }
   }
@@ -1845,7 +1804,7 @@ void CDir::_omap_fetched(bufferlist& hdrbl, map<string, bufferlist>& omap,
     try {
       dn = _load_dentry(
             p->first, dname, last, p->second, pos, snaps,
-            &force_dirty);
+            &force_dirty, &undef_inodes);
     } catch (const buffer::error &err) {
       cache->mds->clog->warn() << "Corrupt dentry '" << dname << "' in "
                                   "dir frag " << dirfrag() << ": "
@@ -1864,28 +1823,50 @@ void CDir::_omap_fetched(bufferlist& hdrbl, map<string, bufferlist>& omap,
       continue;
     }
 
-    if (!dn)
-      continue;
-
-    CDentry::linkage_t *dnl = dn->get_linkage();
-    if (dnl->is_primary() && dnl->get_inode()->state_test(CInode::STATE_REJOINUNDEF))
-      undef_inodes.push_back(dnl->get_inode());
-
-    if (dn && want_dn.length() && want_dn == dname) {
+    if (dn && (wanted_items.count(dname) > 0 || !complete)) {
       dout(10) << " touching wanted dn " << *dn << dendl;
       inode->mdcache->touch_dentry(dn);
     }
+
+    /** clean underwater item?
+     * Underwater item is something that is dirty in our cache from
+     * journal replay, but was previously flushed to disk before the
+     * mds failed.
+     *
+     * We only do this is committed_version == 0. that implies either
+     * - this is a fetch after from a clean/empty CDir is created
+     *   (and has no effect, since the dn won't exist); or
+     * - this is a fetch after _recovery_, which is what we're worried 
+     *   about.  Items that are marked dirty from the journal should be
+     *   marked clean if they appear on disk.
+     */
+    if (committed_version == 0 &&     
+	dn &&
+	dn->get_version() <= got_fnode.version &&
+	dn->is_dirty()) {
+      dout(10) << "_fetched  had underwater dentry " << *dn << ", marking clean" << dendl;
+      dn->mark_clean();
+
+      if (dn->get_linkage()->is_primary()) {
+	assert(dn->get_linkage()->get_inode()->get_version() <= got_fnode.version);
+	dout(10) << "_fetched  had underwater inode " << *dn->get_linkage()->get_inode() << ", marking clean" << dendl;
+	dn->get_linkage()->get_inode()->mark_clean();
+      }
+    }
   }
 
   //cache->mds->logger->inc("newin", num_new_inodes_loaded);
 
   // mark complete, !fetching
-  mark_complete();
-  state_clear(STATE_FETCHING);
+  if (complete) {
+    wanted_items.clear();
+    mark_complete();
+    state_clear(STATE_FETCHING);
 
-  if (scrub_infop && scrub_infop->need_scrub_local) {
-    scrub_infop->need_scrub_local = false;
-    scrub_local();
+    if (scrub_infop && scrub_infop->need_scrub_local) {
+      scrub_infop->need_scrub_local = false;
+      scrub_local();
+    }
   }
 
   // open & force frags
@@ -1897,13 +1878,15 @@ void CDir::_omap_fetched(bufferlist& hdrbl, map<string, bufferlist>& omap,
   }
 
   // dirty myself to remove stale snap dentries
-  if (force_dirty && !inode->mdcache->is_readonly())
+  if (force_dirty && !is_dirty() && !inode->mdcache->is_readonly())
     log_mark_dirty();
+  else
+    auth_unpin(this);
 
-  auth_unpin(this);
-
-  // kick waiters
-  finish_waiting(WAIT_COMPLETE, 0);
+  if (complete) {
+    // kick waiters
+    finish_waiting(WAIT_COMPLETE, 0);
+  }
 }
 
 void CDir::_go_bad()
@@ -1923,23 +1906,25 @@ void CDir::_go_bad()
 void CDir::go_bad_dentry(snapid_t last, const std::string &dname)
 {
   const bool fatal = cache->mds->damage_table.notify_dentry(
-      inode->ino(), frag, last, dname, get_path() + "/" + dname);
+      inode->ino(), frag, last, dname);
   if (fatal) {
     cache->mds->damaged();
     assert(0);  // unreachable, damaged() respawns us
   }
 }
 
-void CDir::go_bad()
+void CDir::go_bad(bool complete)
 {
-  const bool fatal = cache->mds->damage_table.notify_dirfrag(
-      inode->ino(), frag, get_path());
+  const bool fatal = cache->mds->damage_table.notify_dirfrag(inode->ino(), frag);
   if (fatal) {
     cache->mds->damaged();
     assert(0);  // unreachable, damaged() respawns us
   }
 
-  _go_bad();
+  if (complete)
+    _go_bad();
+  else
+    auth_unpin(this);
 }
 
 // -----------------------
@@ -2082,8 +2067,6 @@ void CDir::_omap_commit(int op_prio)
       if (!is_new() && !state_test(CDir::STATE_FRAGMENTING))
 	op.stat(NULL, (ceph::real_time*) NULL, NULL);
 
-      op.tmap_to_omap(true); // convert tmap to omap
-
       if (!to_set.empty())
 	op.omap_set(to_set);
       if (!to_remove.empty())
@@ -2106,8 +2089,6 @@ void CDir::_omap_commit(int op_prio)
   if (!is_new() && !state_test(CDir::STATE_FRAGMENTING))
     op.stat(NULL, (ceph::real_time*)NULL, NULL);
 
-  op.tmap_to_omap(true); // convert tmap to omap
-
   /*
    * save the header at the last moment.. If we were to send it off before other
    * updates, but die before sending them all, we'd think that the on-disk state
@@ -2150,7 +2131,7 @@ void CDir::_encode_dentry(CDentry *dn, bufferlist& bl,
     bl.append('L');         // remote link
     ::encode(ino, bl);
     ::encode(d_type, bl);
-  } else {
+  } else if (dn->linkage.is_primary()) {
     // primary link
     CInode *in = dn->linkage.get_inode();
     assert(in);
@@ -2172,6 +2153,8 @@ void CDir::_encode_dentry(CDentry *dn, bufferlist& bl,
     bufferlist snap_blob;
     in->encode_snap_blob(snap_blob);
     in->encode_bare(bl, cache->mds->mdsmap->get_up_features(), &snap_blob);
+  } else {
+    assert(!dn->linkage.is_null());
   }
 }
 
@@ -3062,7 +3045,7 @@ int CDir::scrub_dentry_next(MDSInternalContext *cb, CDentry **dnout)
     dout(20) << __func__ << " inserted to directories scrubbing: "
       << *dnout << dendl;
     scrub_infop->directories_scrubbing.insert((*dnout)->key());
-  } else if (rval < 0 || rval == EAGAIN) {
+  } else if (rval == EAGAIN) {
     // we don't need to do anything else
   } else { // we emptied out the directory scrub set
     assert(rval == ENOENT);
@@ -3152,7 +3135,7 @@ bool CDir::scrub_local()
 std::string CDir::get_path() const
 {
   std::string path;
-  get_inode()->make_path_string(path, true);
+  get_inode()->make_path_string_projected(path);
   return path;
 }
 
