@@ -274,7 +274,7 @@ void FSMap::encode(bufferlist& bl, uint64_t features) const
       // mds_info with the standbys to get a pre-jewel-style mon MDSMap.
       MDSMap full_mdsmap = fs->mds_map;
       full_mdsmap.epoch = epoch;
-      for (const auto p : standby_daemons) {
+      for (const auto &p : standby_daemons) {
         full_mdsmap.mds_info[p.first] = p.second;
       }
 
@@ -421,9 +421,6 @@ void FSMap::decode(bufferlist::iterator& p)
       migrate_fs->mds_map.epoch = epoch;
       filesystems[migrate_fs->fscid] = migrate_fs;
 
-      // List of GIDs that had invalid states
-      std::set<mds_gid_t> drop_gids;
-
       // Construct mds_roles, standby_daemons, and remove
       // standbys from the MDSMap in the Filesystem.
       for (auto &p : migrate_fs->mds_map.mds_info) {
@@ -433,27 +430,14 @@ void FSMap::decode(bufferlist::iterator& p)
           p.second.rank = p.second.standby_for_rank;
         }
         if (p.second.rank == MDS_RANK_NONE) {
-          if (p.second.state != MDSMap::STATE_STANDBY) {
-            // Old MDSMaps can have down:dne here, which
-            // is invalid in an FSMap (#17837)
-            drop_gids.insert(p.first);
-          } else {
-            insert(p.second); // into standby_daemons
-          }
+          insert(p.second); // into standby_daemons
         } else {
           mds_roles[p.first] = migrate_fs->fscid;
         }
       }
       for (const auto &p : standby_daemons) {
-        // Erase from this Filesystem's MDSMap, because it has
-        // been copied into FSMap::Standby_daemons above
         migrate_fs->mds_map.mds_info.erase(p.first);
       }
-      for (const auto &gid : drop_gids) {
-        // Throw away all info for this MDS because it was identified
-        // as having invalid state above.
-        migrate_fs->mds_map.mds_info.erase(gid);
-      }
 
       legacy_client_fscid = migrate_fs->fscid;
     } else {
@@ -513,7 +497,7 @@ int FSMap::parse_filesystem(
   std::string ns_err;
   fs_cluster_id_t fscid = strict_strtol(ns_str.c_str(), 10, &ns_err);
   if (!ns_err.empty() || filesystems.count(fscid) == 0) {
-    for (auto &fs : filesystems) {
+    for (auto fs : filesystems) {
       if (fs.second->mds_map.fs_name == ns_str) {
         *result = std::const_pointer_cast<const Filesystem>(fs.second);
         return 0;
@@ -557,7 +541,14 @@ mds_gid_t FSMap::find_standby_for(mds_role_t role, const std::string& name) cons
       continue;
     }
 
-    if ((info.standby_for_rank == role.rank && info.standby_for_fscid == role.fscid)
+    // The mds_info_t may or may not tell us exactly which filesystem
+    // the standby_for_rank refers to: lookup via legacy_client_fscid
+    mds_role_t target_role = {
+      info.standby_for_fscid == FS_CLUSTER_ID_NONE ?
+        legacy_client_fscid : info.standby_for_fscid,
+      info.standby_for_rank};
+
+    if ((target_role.rank == role.rank && target_role.fscid == role.fscid)
         || (name.length() && info.standby_for_name == name)) {
       // It's a named standby for *me*, use it.
       return gid;
@@ -746,7 +737,7 @@ void FSMap::erase(mds_gid_t who, epoch_t blacklist_epoch)
     standby_daemons.erase(who);
     standby_epochs.erase(who);
   } else {
-    auto &fs = filesystems.at(mds_roles.at(who));
+    auto fs = filesystems.at(mds_roles.at(who));
     const auto &info = fs->mds_map.mds_info.at(who);
     if (info.state != MDSMap::STATE_STANDBY_REPLAY) {
       if (info.state == MDSMap::STATE_CREATING) {
@@ -803,12 +794,14 @@ bool FSMap::undamaged(const fs_cluster_id_t fscid, const mds_rank_t rank)
 
 void FSMap::insert(const MDSMap::mds_info_t &new_info)
 {
+  assert(new_info.state == MDSMap::STATE_STANDBY);
+  assert(new_info.rank == MDS_RANK_NONE);
   mds_roles[new_info.global_id] = FS_CLUSTER_ID_NONE;
   standby_daemons[new_info.global_id] = new_info;
   standby_epochs[new_info.global_id] = epoch;
 }
 
-void FSMap::stop(mds_gid_t who)
+std::list<mds_gid_t> FSMap::stop(mds_gid_t who)
 {
   assert(mds_roles.at(who) != FS_CLUSTER_ID_NONE);
   auto fs = filesystems.at(mds_roles.at(who));
@@ -817,10 +810,24 @@ void FSMap::stop(mds_gid_t who)
   fs->mds_map.in.erase(info.rank);
   fs->mds_map.stopped.insert(info.rank);
 
+  // Also drop any standby replays that were following this rank
+  std::list<mds_gid_t> standbys;
+  for (const auto &i : fs->mds_map.mds_info) {
+    const auto &other_gid = i.first;
+    const auto &other_info = i.second;
+    if (other_info.rank == info.rank
+        && other_info.state == MDSMap::STATE_STANDBY_REPLAY) {
+      standbys.push_back(other_gid);
+      erase(other_gid, 0);
+    }
+  }
+
   fs->mds_map.mds_info.erase(who);
   mds_roles.erase(who);
 
   fs->mds_map.epoch = epoch;
+
+  return standbys;
 }
 
 
@@ -838,41 +845,66 @@ int FSMap::parse_role(
     mds_role_t *role,
     std::ostream &ss) const
 {
-  size_t colon_pos = role_str.find(":");
-  size_t rank_pos;
-  std::shared_ptr<const Filesystem> fs;
-  if (colon_pos == std::string::npos) {
+  auto colon_pos = role_str.find(":");
+
+  if (colon_pos != std::string::npos && colon_pos != role_str.size()) {
+    auto fs_part = role_str.substr(0, colon_pos);
+    auto rank_part = role_str.substr(colon_pos + 1);
+
+    std::string err;
+    fs_cluster_id_t fs_id = FS_CLUSTER_ID_NONE;
+    long fs_id_i = strict_strtol(fs_part.c_str(), 10, &err);
+    if (fs_id_i < 0 || !err.empty()) {
+      // Try resolving as name
+      auto fs = get_filesystem(fs_part);
+      if (fs == nullptr) {
+        ss << "Unknown filesystem name '" << fs_part << "'";
+        return -EINVAL;
+      } else {
+        fs_id = fs->fscid;
+      }
+    } else {
+      fs_id = fs_id_i;
+    }
+
+    mds_rank_t rank;
+    long rank_i = strict_strtol(rank_part.c_str(), 10, &err);
+    if (rank_i < 0 || !err.empty()) {
+      ss << "Invalid rank '" << rank_part << "'";
+      return -EINVAL;
+    } else {
+      rank = rank_i;
+    }
+
+    *role = {fs_id, rank};
+  } else {
+    std::string err;
+    long who_i = strict_strtol(role_str.c_str(), 10, &err);
+    if (who_i < 0 || !err.empty()) {
+      ss << "Invalid rank '" << role_str << "'";
+      return -EINVAL;
+    }
+
     if (legacy_client_fscid == FS_CLUSTER_ID_NONE) {
       ss << "No filesystem selected";
       return -ENOENT;
+    } else {
+      *role = mds_role_t(legacy_client_fscid, who_i);
     }
-    fs = get_filesystem(legacy_client_fscid);
-    rank_pos = 0;
-  } else {
-    if (parse_filesystem(role_str.substr(0, colon_pos), &fs) < 0) {
-      ss << "Invalid filesystem";
-      return -ENOENT;
-    }
-    rank_pos = colon_pos+1;
   }
 
-  mds_rank_t rank;
-  std::string err;
-  std::string rank_str = role_str.substr(rank_pos);
-  long rank_i = strict_strtol(rank_str.c_str(), 10, &err);
-  if (rank_i < 0 || !err.empty()) {
-    ss << "Invalid rank '" << rank_str << "'";
-    return -EINVAL;
-  } else {
-    rank = rank_i;
-  }
-
-  if (fs->mds_map.in.count(rank) == 0) {
-    ss << "Rank '" << rank << "' not found";
+  // Now check that the role actually exists
+  if (get_filesystem(role->fscid) == nullptr) {
+    ss << "Filesystem with ID '" << role->fscid << "' not found";
     return -ENOENT;
   }
 
-  *role = {fs->fscid, rank};
+  auto fs = get_filesystem(role->fscid);
+  if (fs->mds_map.in.count(role->rank) == 0) {
+    ss << "Rank '" << role->rank << "' not found";
+    return -ENOENT;
+  }
 
   return 0;
 }
+
