@@ -75,6 +75,8 @@ void MDLog::create_logger()
   plb.add_u64(l_mdl_rdpos, "rdpos", "Journaler  read position");
   plb.add_u64(l_mdl_jlat, "jlat", "Journaler flush latency");
 
+  plb.add_u64_counter(l_mdl_replayed, "replayed", "Events replayed");
+
   // logger
   logger = plb.create_perf_counters();
   g_ceph_context->get_perfcounters_collection()->add(logger);
@@ -848,6 +850,7 @@ void MDLog::replay(MDSInternalContextBase *c)
   // empty?
   if (journaler->get_read_pos() == journaler->get_write_pos()) {
     dout(10) << "replay - journal empty, done." << dendl;
+    mds->mdcache->trim(-1);
     if (c) {
       c->complete(0);
     }
@@ -928,6 +931,10 @@ void MDLog::_recovery_thread(MDSInternalContextBase *completion)
     if (mds->is_standby_replay()) {
       dout(1) << "Journal " << jp.front << " is being rewritten, "
         << "cannot replay in standby until an active MDS completes rewrite" << dendl;
+      Mutex::Locker l(mds->mds_lock);
+      if (mds->is_daemon_stopping()) {
+        return;
+      }
       completion->complete(-EAGAIN);
       return;
     }
@@ -1377,6 +1384,7 @@ void MDLog::_replay_thread()
         if (mds->is_daemon_stopping()) {
           return;
         }
+        logger->inc(l_mdl_replayed);
         le->replay(mds);
       }
     }
@@ -1416,20 +1424,9 @@ void MDLog::standby_trim_segments()
   bool removed_segment = false;
   while (have_any_segments()) {
     LogSegment *seg = get_oldest_segment();
-    dout(10) << " segment seq=" << seg->seq << " " << seg->offset <<
-      "~" << seg->end - seg->offset << dendl;
-
-    if (seg->end > expire_pos) {
-      dout(10) << " won't remove, not expired!" << dendl;
+    if (seg->end > expire_pos)
       break;
-    }
-
-    if (segments.size() == 1) {
-      dout(10) << " won't remove, last segment!" << dendl;
-      break;
-    }
-
-    dout(10) << " removing segment" << dendl;
+    dout(10) << " removing segment " << seg->seq << "/" << seg->offset << dendl;
     mds->mdcache->standby_trim_segment(seg);
     remove_oldest_segment();
     removed_segment = true;
@@ -1438,9 +1435,8 @@ void MDLog::standby_trim_segments()
   if (removed_segment) {
     dout(20) << " calling mdcache->trim!" << dendl;
     mds->mdcache->trim(-1);
-  } else {
+  } else
     dout(20) << " removed no segments!" << dendl;
-  }
 }
 
 void MDLog::dump_replay_status(Formatter *f) const
