@@ -114,12 +114,12 @@ public:
 /**
  * No-op for callers expecting MDSInternalContextBase
  */
-class C_MDSInternalNoop final : public MDSInternalContextBase
+class C_MDSInternalNoop : public MDSInternalContextBase
 {
   virtual MDSRank* get_mds() {assert(0);}
 public:
   void finish(int r) {}
-  void complete(int r) { delete this; }
+  void complete(int r) {}
 };
 
 
@@ -135,8 +135,17 @@ public:
   C_IO_Wrapper(MDSRank *mds_, MDSInternalContextBase *wrapped_) : MDSIOContext(mds_), wrapped(wrapped_) {
     assert(wrapped != NULL);
   }
+
+  ~C_IO_Wrapper() {
+    if (wrapped != nullptr) {
+      delete wrapped;
+      wrapped = nullptr;
+    }
+  }
+
   virtual void finish(int r) {
     wrapped->complete(r);
+    wrapped = nullptr;
   }
 };
 
