@@ -32,6 +32,7 @@
 #include "common/TrackedOp.h"
 #include "common/Finisher.h"
 #include "common/cmdparse.h"
+#include "mgr/MgrClient.h"
 
 #include "MDSRank.h"
 #include "MDSMap.h"
@@ -41,29 +42,17 @@
 
 #define CEPH_MDS_PROTOCOL    28 /* cluster internal */
 
-class filepath;
-
 class MonClient;
 
-class Objecter;
-class Filer;
-
 class Server;
 class Locker;
 class MDCache;
 class MDBalancer;
 class MDSInternalContextBase;
 
-class CInode;
-class CDir;
-class CDentry;
-
 class Messenger;
 class Message;
 
-class MMDSBeacon;
-
-class InoTable;
 class SnapServer;
 class SnapClient;
 
@@ -93,8 +82,8 @@ class MDSDaemon : public Dispatcher, public md_config_obs_t {
 
   Messenger    *messenger;
   MonClient    *monc;
+  MgrClient     mgrc;
   MDSMap       *mdsmap;
-  Objecter     *objecter;
   LogClient    log_client;
   LogChannelRef clog;
 
@@ -125,18 +114,8 @@ class MDSDaemon : public Dispatcher, public md_config_obs_t {
 				  const std::set <std::string> &changed);
  protected:
   // tick and other timer fun
-  class C_MDS_Tick : public Context {
-    protected:
-      MDSDaemon *mds_daemon;
-  public:
-    explicit C_MDS_Tick(MDSDaemon *m) : mds_daemon(m) {}
-    void finish(int r) {
-      assert(mds_daemon->mds_lock.is_locked_by_me());
-
-      mds_daemon->tick_event = 0;
-      mds_daemon->tick();
-    }
-  } *tick_event;
+  class C_MDS_Tick;
+  C_MDS_Tick *tick_event;
   void     reset_tick();
 
   void wait_for_omap_osds();
@@ -146,12 +125,12 @@ class MDSDaemon : public Dispatcher, public md_config_obs_t {
   bool ms_get_authorizer(int dest_type, AuthAuthorizer **authorizer, bool force_new);
   bool ms_verify_authorizer(Connection *con, int peer_type,
 			       int protocol, bufferlist& authorizer_data, bufferlist& authorizer_reply,
-			    bool& isvalid, CryptoKey& session_key,
-			    std::unique_ptr<AuthAuthorizerChallenge> *challenge) override;
-  void ms_handle_accept(Connection *con) override;
-  void ms_handle_connect(Connection *con) override;
-  bool ms_handle_reset(Connection *con) override;
-  void ms_handle_remote_reset(Connection *con) override;
+			       bool& isvalid, CryptoKey& session_key);
+  void ms_handle_accept(Connection *con);
+  void ms_handle_connect(Connection *con);
+  bool ms_handle_reset(Connection *con);
+  void ms_handle_remote_reset(Connection *con);
+  bool ms_handle_refused(Connection *con);
 
  protected:
   // admin socket handling
