@@ -127,7 +127,7 @@ class MDSRank {
 
   public:
     mds_rank_t get_nodeid() const { return whoami; }
-    uint64_t get_metadata_pool();
+    int64_t get_metadata_pool();
 
     // Reference to global MDS::mds_lock, so that users of MDSRank don't
     // carry around references to the outer MDS, and we can substitute
@@ -170,7 +170,6 @@ class MDSRank {
     Session *get_session(client_t client) {
       return sessionmap.get_session(entity_name_t::CLIENT(client.v));
     }
-    Session *get_session(Message *m);
 
     PerfCounters       *logger, *mlogger;
     OpTracker    op_tracker;
@@ -226,6 +225,7 @@ class MDSRank {
     bool _dispatch(Message *m, bool new_msg);
 
     ceph::heartbeat_handle_d *hb;  // Heartbeat for threads using mds_lock
+    void heartbeat_reset();
 
     bool is_stale_message(Message *m);
 
@@ -248,6 +248,8 @@ class MDSRank {
      * Emit clog warnings for any ops reported as warnings by optracker
      */
     void check_ops_in_flight();
+  
+    int mds_slow_req_count;
 
     /**
      * Share MDSMap with clients
@@ -276,7 +278,6 @@ class MDSRank {
         MDSMap *& mdsmap_,
         Messenger *msgr,
         MonClient *monc_,
-        Objecter *objecter_,
         Context *respawn_hook_,
         Context *suicide_hook_);
     ~MDSRank();
@@ -291,12 +292,6 @@ class MDSRank {
     void respawn();
     // <<<
 
-    /**
-     * Call this periodically if inside a potentially long running piece
-     * of code while holding the mds_lock
-     */
-    void heartbeat_reset();
-
     /**
      * Report state DAMAGED to the mon, and then pass on to respawn().  Call
      * this when an unrecoverable error is encountered while attempting
@@ -359,6 +354,7 @@ class MDSRank {
 
     void set_osd_epoch_barrier(epoch_t e);
     epoch_t get_osd_epoch_barrier() const {return osd_epoch_barrier;}
+    epoch_t get_osd_epoch() const;
 
     ceph_tid_t issue_tid() { return ++last_tid; }
 
@@ -367,6 +363,8 @@ class MDSRank {
     MDSMap *get_mds_map() { return mdsmap; }
 
     int get_req_rate() { return logger->get(l_mds_request); }
+  
+    int get_mds_slow_req_count() const { return mds_slow_req_count; }
 
     void dump_status(Formatter *f) const;
 
@@ -453,8 +451,6 @@ class MDSRank {
     void active_start();
     void stopping_start();
     void stopping_done();
-
-    void validate_sessions();
     // <<<
     
     // >>>
@@ -496,7 +492,7 @@ public:
                            Formatter *f, std::ostream& ss);
   void handle_mds_map(MMDSMap *m, MDSMap *oldmap);
   void handle_osd_map();
-  bool kill_session(int64_t session_id, bool wait, std::stringstream& ss);
+  bool kill_session(int64_t session_id);
   void update_log_config();
   bool handle_command_legacy(std::vector<std::string> args);
 
@@ -523,7 +519,6 @@ public:
       MDSMap *& mdsmap_,
       Messenger *msgr,
       MonClient *monc_,
-      Objecter *objecter_,
       Context *respawn_hook_,
       Context *suicide_hook_);
 };
