@@ -194,17 +194,17 @@ MDRequestImpl::More* MDRequestImpl::more()
 
 bool MDRequestImpl::has_more()
 {
-  return _more;
+  return _more != nullptr;
 }
 
 bool MDRequestImpl::has_witnesses()
 {
-  return _more && !_more->witnessed.empty();
+  return (_more != nullptr) && (!_more->witnessed.empty());
 }
 
 bool MDRequestImpl::slave_did_prepare()
 {
-  return more()->slave_commit;
+  return has_more() && more()->slave_commit;
 }
 
 bool MDRequestImpl::did_ino_allocation()
@@ -303,7 +303,7 @@ void MDRequestImpl::set_filepath2(const filepath& fp)
   more()->filepath2 = fp;
 }
 
-void MDRequestImpl::print(ostream &out)
+void MDRequestImpl::print(ostream &out) const
 {
   out << "request(" << reqid;
   //if (request) out << " " << *request;
@@ -314,11 +314,6 @@ void MDRequestImpl::print(ostream &out)
 }
 
 void MDRequestImpl::dump(Formatter *f) const
-{
-  _dump(ceph_clock_now(g_ceph_context), f);
-}
-
-void MDRequestImpl::_dump(utime_t now, Formatter *f) const
 {
   f->dump_string("flag_point", state_string());
   f->dump_stream("reqid") << reqid;
