@@ -70,16 +70,13 @@ private:
 public:
   void encode_payload(uint64_t features) {
     ::encode(tid, payload);
-    ::encode(client, payload);
+    ::encode(client, payload, features);
     ::encode(client_caps, payload, features);
     // Encode client message with intersection of target and source
     // features.  This could matter if the semantics of the encoded
     // message are changed when reencoding with more features than the
     // client had originally.  That should never happen, but we may as
     // well be defensive here.
-    if (con_features != features) {
-      msg->clear_payload();
-    }
     encode_message(msg, features & con_features, payload);
     ::encode(con_features, payload);
     ::encode(entity_name, payload);
