@@ -21,31 +21,10 @@
 #include "include/ceph_features.h"
 
 class MMDSMap : public Message {
- public:
-  /*
-  map<epoch_t, bufferlist> maps;
-  map<epoch_t, bufferlist> incremental_maps;
+  static const int HEAD_VERSION = 1;
+  static const int COMPAT_VERSION = 1;
+public:
 
-  epoch_t get_first() {
-    epoch_t e = 0;
-    map<epoch_t, bufferlist>::iterator i = maps.begin();
-    if (i != maps.end())  e = i->first;
-    i = incremental_maps.begin();    
-    if (i != incremental_maps.end() &&
-        (e == 0 || i->first < e)) e = i->first;
-    return e;
-  }
-  epoch_t get_last() {
-    epoch_t e = 0;
-    map<epoch_t, bufferlist>::reverse_iterator i = maps.rbegin();
-    if (i != maps.rend())  e = i->first;
-    i = incremental_maps.rbegin();    
-    if (i != incremental_maps.rend() &&
-        (e == 0 || i->first > e)) e = i->first;
-    return e;
-  }
-  */
-  
   uuid_d fsid;
   epoch_t epoch;
   bufferlist encoded;
@@ -54,9 +33,9 @@ class MMDSMap : public Message {
   bufferlist& get_encoded() { return encoded; }
 
   MMDSMap() : 
-    Message(CEPH_MSG_MDS_MAP) {}
+    Message(CEPH_MSG_MDS_MAP, HEAD_VERSION, COMPAT_VERSION) {}
   MMDSMap(const uuid_d &f, MDSMap *mm) :
-    Message(CEPH_MSG_MDS_MAP),
+    Message(CEPH_MSG_MDS_MAP, HEAD_VERSION, COMPAT_VERSION),
     fsid(f) {
     epoch = mm->get_epoch();
     mm->encode(encoded, -1);  // we will reencode with fewer features as necessary
