@@ -40,11 +40,14 @@ class MOSDPGNotify : public Message {
   vector<pair<pg_notify_t,pg_interval_map_t> >& get_pg_list() { return pg_list; }
 
   MOSDPGNotify()
-    : Message(MSG_OSD_PG_NOTIFY, HEAD_VERSION, COMPAT_VERSION) { }
+    : Message(MSG_OSD_PG_NOTIFY, HEAD_VERSION, COMPAT_VERSION) { 
+    set_priority(CEPH_MSG_PRIO_HIGH); 
+  }
   MOSDPGNotify(epoch_t e, vector<pair<pg_notify_t,pg_interval_map_t> >& l)
     : Message(MSG_OSD_PG_NOTIFY, HEAD_VERSION, COMPAT_VERSION),
       epoch(e) {
     pg_list.swap(l);
+    set_priority(CEPH_MSG_PRIO_HIGH);
   }
 private:
   ~MOSDPGNotify() {}
