@@ -22,21 +22,11 @@
 #include "messages/MAuth.h"
 #include "messages/MAuthReply.h"
 #include "messages/MMonGlobalID.h"
-
-#include "common/Timer.h"
-#include "common/config.h"
-#include "common/cmdparse.h"
+#include "msg/Messenger.h"
 
 #include "auth/AuthServiceHandler.h"
 #include "auth/KeyRing.h"
-
-#include "osd/osd_types.h"
-
 #include "include/assert.h"
-#include "include/str_list.h"
-
-#include "mds/MDSAuthCaps.h"
-#include "osd/OSDCap.h"
 
 #define dout_subsys ceph_subsys_mon
 #undef dout_prefix
@@ -131,7 +121,7 @@ void AuthMonitor::update_from_paxos(bool *need_bootstrap)
   version_t keys_ver = mon->key_server.get_ver();
   if (version == keys_ver)
     return;
-  assert(version >= keys_ver);
+  assert(version > keys_ver);
 
   version_t latest_full = get_version_latest_full();
 
@@ -403,7 +393,8 @@ bool AuthMonitor::prep_auth(MonOpRequestRef op, bool paxos_writable)
     if (!m->get_connection()->has_feature(CEPH_FEATURE_MSG_AUTH)) {
       if (entity_name.get_type() == CEPH_ENTITY_TYPE_MON ||
 	  entity_name.get_type() == CEPH_ENTITY_TYPE_OSD ||
-	  entity_name.get_type() == CEPH_ENTITY_TYPE_MDS) {
+	  entity_name.get_type() == CEPH_ENTITY_TYPE_MDS ||
+	  entity_name.get_type() == CEPH_ENTITY_TYPE_MGR) {
 	if (g_conf->cephx_cluster_require_signatures ||
 	    g_conf->cephx_require_signatures) {
 	  dout(1) << m->get_source_inst()
@@ -422,34 +413,13 @@ bool AuthMonitor::prep_auth(MonOpRequestRef op, bool paxos_writable)
 	  supported.erase(CEPH_AUTH_CEPHX);
 	}
       }
-    } else if (!m->get_connection()->has_feature(CEPH_FEATURE_CEPHX_V2)) {
-      if (entity_name.get_type() == CEPH_ENTITY_TYPE_MON ||
-	  entity_name.get_type() == CEPH_ENTITY_TYPE_OSD ||
-	  entity_name.get_type() == CEPH_ENTITY_TYPE_MDS) {
-	if (g_conf->cephx_cluster_require_version >= 2 ||
-	    g_conf->cephx_require_version >= 2) {
-	  dout(1) << m->get_source_inst()
-                  << " supports cephx but not v2 and"
-                  << " 'cephx [cluster] require version >= 2';"
-                  << " disallowing cephx" << dendl;
-	  supported.erase(CEPH_AUTH_CEPHX);
-	}
-      } else {
-	if (g_conf->cephx_service_require_version >= 2 ||
-	    g_conf->cephx_require_version >= 2) {
-	  dout(1) << m->get_source_inst()
-                  << " supports cephx but not v2 and"
-                  << " 'cephx [service] require version >= 2';"
-                  << " disallowing cephx" << dendl;
-	  supported.erase(CEPH_AUTH_CEPHX);
-	}
-      }
     }
 
     int type;
     if (entity_name.get_type() == CEPH_ENTITY_TYPE_MON ||
 	entity_name.get_type() == CEPH_ENTITY_TYPE_OSD ||
-	entity_name.get_type() == CEPH_ENTITY_TYPE_MDS)
+	entity_name.get_type() == CEPH_ENTITY_TYPE_MDS ||
+	entity_name.get_type() == CEPH_ENTITY_TYPE_MGR)
       type = mon->auth_cluster_required.pick(supported);
     else
       type = mon->auth_service_required.pick(supported);
@@ -677,15 +647,11 @@ void AuthMonitor::export_keyring(KeyRing& keyring)
   mon->key_server.export_keyring(keyring);
 }
 
-int AuthMonitor::import_keyring(KeyRing& keyring)
+void AuthMonitor::import_keyring(KeyRing& keyring)
 {
   for (map<EntityName, EntityAuth>::iterator p = keyring.get_keys().begin();
        p != keyring.get_keys().end();
        ++p) {
-    if (p->second.caps.empty()) {
-      dout(0) << "import: no caps supplied" << dendl;
-      return -EINVAL;
-    }
     KeyServerData::Incremental auth_inc;
     auth_inc.name = p->first;
     auth_inc.auth = p->second;
@@ -694,38 +660,6 @@ int AuthMonitor::import_keyring(KeyRing& keyring)
     dout(30) << "    " << auth_inc.auth << dendl;
     push_cephx_inc(auth_inc);
   }
-  return 0;
-}
-
-bool AuthMonitor::valid_caps(const vector<string>& caps, ostream *out)
-{
-  for (vector<string>::const_iterator p = caps.begin();
-       p != caps.end(); p += 2) {
-    if ((p+1) == caps.end()) {
-      *out << "cap '" << *p << "' has no value";
-      return false;
-    }
-    if (*p == "mon") {
-      MonCap tmp;
-      if (!tmp.parse(*(p+1), out)) {
-	return false;
-      }
-    } else if (*p == "osd") {
-      OSDCap ocap;
-      if (!ocap.parse(*(p+1), out)) {
-	return false;
-      }
-    } else if (*p == "mds") {
-      MDSAuthCaps mdscap;
-      if (!mdscap.parse(g_ceph_context, *(p+1), out)) {
-	return false;
-      }
-    } else {
-      *out << "unknown cap type '" << *p << "'";
-      return false;
-    }
-  }
-  return true;
 }
 
 bool AuthMonitor::prepare_command(MonOpRequestRef op)
@@ -789,16 +723,10 @@ bool AuthMonitor::prepare_command(MonOpRequestRef op)
       ::decode(keyring, iter);
     } catch (const buffer::error &ex) {
       ss << "error decoding keyring" << " " << ex.what();
-      rs = err;
+      err = -EINVAL;
       goto done;
     }
-    err = import_keyring(keyring);
-    if (err < 0) {
-      ss << "auth import: no caps supplied";
-      getline(ss, rs);
-      mon->reply_command(op, -EINVAL, rs, get_last_committed());
-      return true;
-    }
+    import_keyring(keyring);
     ss << "imported keyring";
     getline(ss, rs);
     err = 0;
@@ -829,11 +757,6 @@ bool AuthMonitor::prepare_command(MonOpRequestRef op)
       }
     }
 
-    if (!valid_caps(caps_vec, &ss)) {
-      err = -EINVAL;
-      goto done;
-    }
-
     // are we about to have it?
     for (vector<Incremental>::iterator p = pending_auth.begin();
         p != pending_auth.end();
@@ -934,7 +857,7 @@ bool AuthMonitor::prepare_command(MonOpRequestRef op)
 						   get_last_committed() + 1));
     return true;
   } else if ((prefix == "auth get-or-create-key" ||
-	      prefix == "auth get-or-create") &&
+	     prefix == "auth get-or-create") &&
 	     !entity_name.empty()) {
     // auth get-or-create <name> [mon osdcapa osd osdcapb ...]
 
@@ -1063,7 +986,7 @@ bool AuthMonitor::prepare_command(MonOpRequestRef op)
     wait_for_finished_proposal(op, new Monitor::C_Command(mon, op, 0, rs,
 					      get_last_committed() + 1));
     return true;
-  } else if ((prefix == "auth del" || prefix == "auto rm") &&
+  } else if ((prefix == "auth del" || prefix == "auth rm") &&
              !entity_name.empty()) {
     KeyServerData::Incremental auth_inc;
     auth_inc.name = entity;
