@@ -17,7 +17,6 @@
 
 #include "common/Timer.h"
 #include "MonitorDBStore.h"
-#include "MonmapMonitor.h"
 #include "messages/MMonElection.h"
 
 #include "common/config.h"
@@ -43,7 +42,8 @@ void Elector::init()
 
 void Elector::shutdown()
 {
-  cancel_timer();
+  if (expire_event)
+    mon->timer.cancel_event(expire_event);
 }
 
 void Elector::bump_epoch(epoch_t e) 
@@ -126,8 +126,6 @@ void Elector::defer(int who)
 
 void Elector::reset_timer(double plus)
 {
-  // set the timer
-  cancel_timer();
   /**
    * This class is used as the callback when the expire_event timer fires up.
    *
@@ -141,9 +139,17 @@ void Elector::reset_timer(double plus)
    * as far as we know, we may even be dead); so, just propose ourselves as the
    * Leader.
    */
-  expire_event = new C_MonContext(mon, [this](int) {
-      expire();
-    });
+  class C_ElectionExpire : public Context {
+    Elector *elector;
+  public:
+    explicit C_ElectionExpire(Elector *e) : elector(e) { }
+    void finish(int r) {
+      elector->expire();
+    }
+  };
+  // set the timer
+  cancel_timer();
+  expire_event = new C_ElectionExpire(this);
   mon->timer.add_event_after(g_conf->mon_election_timeout + plus,
 			     expire_event);
 }
@@ -444,10 +450,10 @@ void Elector::dispatch(MonOpRequestRef op)
 	return;
       }
 
-      MonMap *peermap = new MonMap;
-      peermap->decode(em->monmap_bl);
-      if (peermap->epoch > mon->monmap->epoch) {
-	dout(0) << em->get_source_inst() << " has newer monmap epoch " << peermap->epoch
+      MonMap peermap;
+      peermap.decode(em->monmap_bl);
+      if (peermap.epoch > mon->monmap->epoch) {
+	dout(0) << em->get_source_inst() << " has newer monmap epoch " << peermap.epoch
 		<< " > my epoch " << mon->monmap->epoch 
 		<< ", taking it"
 		<< dendl;
@@ -459,15 +465,13 @@ void Elector::dispatch(MonOpRequestRef op)
 	//mon->monmon()->paxos->stash_latest(mon->monmap->epoch, em->monmap_bl);
 	cancel_timer();
 	mon->bootstrap();
-	delete peermap;
 	return;
       }
-      if (peermap->epoch < mon->monmap->epoch) {
-	dout(0) << em->get_source_inst() << " has older monmap epoch " << peermap->epoch
+      if (peermap.epoch < mon->monmap->epoch) {
+	dout(0) << em->get_source_inst() << " has older monmap epoch " << peermap.epoch
 		<< " < my epoch " << mon->monmap->epoch 
 		<< dendl;
       } 
-      delete peermap;
 
       switch (em->op) {
       case MMonElection::OP_PROPOSE:
