@@ -12,7 +12,6 @@
  * 
  */
 
-#include "msg/Messenger.h"
 #include "messages/MMonGetMap.h"
 #include "messages/MMonGetVersion.h"
 #include "messages/MMonGetVersionReply.h"
@@ -26,8 +25,6 @@
 
 #include "messages/MMonSubscribe.h"
 #include "messages/MMonSubscribeAck.h"
-#include "common/ConfUtils.h"
-#include "common/ceph_argparse.h"
 #include "common/errno.h"
 #include "common/LogClient.h"
 
@@ -37,11 +34,7 @@
 #include "auth/Auth.h"
 #include "auth/KeyRing.h"
 #include "auth/AuthMethodList.h"
-
-#include "include/str_list.h"
-#include "include/addr_parsing.h"
-
-#include "common/config.h"
+#include "auth/RotatingKeyRing.h"
 
 
 #define dout_subsys ceph_subsys_monc
@@ -56,7 +49,6 @@ MonClient::MonClient(CephContext *cct_) :
   rng(getpid()),
   monc_lock("MonClient::monc_lock"),
   timer(cct_, monc_lock), finisher(cct_),
-  authorize_handler_registry(NULL),
   initialized(false),
   no_keyring_disabled_cephx(false),
   log_client(NULL),
@@ -310,10 +302,10 @@ bool MonClient::ms_dispatch(Message *m)
   return true;
 }
 
-void MonClient::send_log(bool flush)
+void MonClient::send_log()
 {
   if (log_client) {
-    Message *lm = log_client->get_mon_log_message(flush);
+    Message *lm = log_client->get_mon_log_message();
     if (lm)
       _send_mon_message(lm);
     more_log_pending = log_client->are_pending();
@@ -434,6 +426,7 @@ void MonClient::shutdown()
   monc_lock.Unlock();
 
   if (initialized) {
+    finisher.wait_for_empty();
     finisher.stop();
   }
   monc_lock.Lock();
@@ -539,9 +532,13 @@ void MonClient::handle_auth(MAuthReply *m)
 	_send_mon_message(waiting_for_session.front());
 	waiting_for_session.pop_front();
       }
+
       _resend_mon_commands();
 
-      send_log(true);
+      if (log_client) {
+	log_client->reset_session();
+	send_log();
+      }
       if (session_established_context) {
         cb = session_established_context;
         session_established_context = NULL;
@@ -747,6 +744,14 @@ void MonClient::tick()
 
 void MonClient::schedule_tick()
 {
+  struct C_Tick : public Context {
+    MonClient *monc;
+    explicit C_Tick(MonClient *m) : monc(m) {}
+    void finish(int r) {
+      monc->tick();
+    }
+  };
+
   if (hunting)
     timer.add_event_after(cct->_conf->mon_client_hunt_interval
                           * reopen_interval_multiplier, new C_Tick(this));
@@ -1013,6 +1018,16 @@ int MonClient::start_mon_command(const vector<string>& cmd,
   r->prs = outs;
   r->onfinish = onfinish;
   if (cct->_conf->rados_mon_op_timeout > 0) {
+    class C_CancelMonCommand : public Context
+    {
+      uint64_t tid;
+      MonClient *monc;
+      public:
+      C_CancelMonCommand(uint64_t tid, MonClient *monc) : tid(tid), monc(monc) {}
+      void finish(int r) {
+	monc->_cancel_mon_command(tid, -ETIMEDOUT);
+      }
+    };
     r->ontimeout = new C_CancelMonCommand(r->tid, this);
     timer.add_event_after(cct->_conf->rados_mon_op_timeout, r->ontimeout);
   }
