@@ -33,29 +33,16 @@
 #include "Paxos.h"
 #include "Session.h"
 
-#include "osd/OSDMap.h"
-
 #include "common/LogClient.h"
-#include "common/SimpleRNG.h"
-#include "common/cmdparse.h"
-
 #include "auth/cephx/CephxKeyServer.h"
 #include "auth/AuthMethodList.h"
 #include "auth/KeyRing.h"
-
-#include "perfglue/heap_profiler.h"
-
 #include "messages/MMonCommand.h"
-#include "messages/MPing.h"
 #include "mon/MonitorDBStore.h"
-
-#include <memory>
 #include "include/memory.h"
-#include "include/str_map.h"
 #include <errno.h>
 #include <cmath>
 
-#include "common/TrackedOp.h"
 #include "mon/MonOpRequest.h"
 
 
@@ -125,14 +112,6 @@ struct MonCommand;
 
 #define COMPAT_SET_LOC "feature_set"
 
-class C_MonContext final : public FunctionContext {
-  const Monitor *mon;
-public:
-  explicit C_MonContext(Monitor *m, boost::function<void(int)>&& callback)
-    : FunctionContext(std::move(callback)), mon(m) {}
-  void finish(int r) override;
-};
-
 class Monitor : public Dispatcher,
                 public md_config_obs_t {
 public:
@@ -176,6 +155,7 @@ public:
 
 private:
   void new_tick();
+  friend class C_Mon_Tick;
 
   // -- local storage --
 public:
@@ -343,6 +323,14 @@ private:
    */
   version_t sync_last_committed_floor;
 
+  struct C_SyncTimeout : public Context {
+    Monitor *mon;
+    explicit C_SyncTimeout(Monitor *m) : mon(m) {}
+    void finish(int r) {
+      mon->sync_timeout();
+    }
+  };
+
   /**
    * Obtain the synchronization target prefixes in set form.
    *
@@ -502,6 +490,14 @@ private:
    */
   Context *timecheck_event;
 
+  struct C_TimeCheck : public Context {
+    Monitor *mon;
+    explicit C_TimeCheck(Monitor *m) : mon(m) { }
+    void finish(int r) {
+      mon->timecheck_start_round();
+    }
+  };
+
   void timecheck_start();
   void timecheck_finish();
   void timecheck_start_round();
@@ -560,7 +556,15 @@ private:
    */
   void handle_ping(MonOpRequestRef op);
 
-  Context *probe_timeout_event = nullptr;  // for probing
+  Context *probe_timeout_event;  // for probing
+
+  struct C_ProbeTimeout : public Context {
+    Monitor *mon;
+    explicit C_ProbeTimeout(Monitor *m) : mon(m) {}
+    void finish(int r) {
+      mon->probe_timeout(r);
+    }
+  };
 
   void reset_probe_timeout();
   void cancel_probe_timeout();
@@ -645,6 +649,10 @@ public:
     return (class LogMonitor*) paxos_service[PAXOS_LOG];
   }
 
+  class MgrMonitor *mgrmon() {
+    return (class MgrMonitor*) paxos_service[PAXOS_MGR];
+  }
+
   friend class Paxos;
   friend class OSDMonitor;
   friend class MDSMonitor;
@@ -659,6 +667,9 @@ public:
   MonSessionMap session_map;
   AdminSocketHook *admin_hook;
 
+  void check_subs();
+  void check_sub(Subscription *sub);
+
   void send_latest_monmap(Connection *con);
 
   // messages
@@ -702,8 +713,29 @@ public:
     }
   } health_status_cache;
 
-  Context *health_tick_event = nullptr;
-  Context *health_interval_event = nullptr;
+  struct C_HealthToClogTick : public Context {
+    Monitor *mon;
+    explicit C_HealthToClogTick(Monitor *m) : mon(m) { }
+    void finish(int r) {
+      if (r < 0)
+        return;
+      mon->do_health_to_clog();
+      mon->health_tick_start();
+    }
+  };
+
+  struct C_HealthToClogInterval : public Context {
+    Monitor *mon;
+    explicit C_HealthToClogInterval(Monitor *m) : mon(m) { }
+    void finish(int r) {
+      if (r < 0)
+        return;
+      mon->do_health_to_clog_interval();
+    }
+  };
+
+  Context *health_tick_event;
+  Context *health_interval_event;
 
   void health_tick_start();
   void health_tick_stop();
@@ -845,7 +877,7 @@ public:
   //ms_dispatch handles a lot of logic and we want to reuse it
   //on forwarded messages, so we create a non-locking version for this class
   void _ms_dispatch(Message *m);
-  bool ms_dispatch(Message *m) {
+  bool ms_dispatch(Message *m) override {
     lock.Lock();
     _ms_dispatch(m);
     lock.Unlock();
@@ -857,10 +889,10 @@ public:
   bool ms_get_authorizer(int dest_type, AuthAuthorizer **authorizer, bool force_new);
   bool ms_verify_authorizer(Connection *con, int peer_type,
 			    int protocol, bufferlist& authorizer_data, bufferlist& authorizer_reply,
-			    bool& isvalid, CryptoKey& session_key,
-			    std::unique_ptr<AuthAuthorizerChallenge> *challenge) override;
+			    bool& isvalid, CryptoKey& session_key);
   bool ms_handle_reset(Connection *con) override;
   void ms_handle_remote_reset(Connection *con) override {}
+  bool ms_handle_refused(Connection *con) override;
 
   int write_default_keyring(bufferlist& bl);
   void extract_save_mon_key(KeyRing& keyring);
@@ -887,9 +919,9 @@ public:
   static int check_features(MonitorDBStore *store);
 
   // config observer
-  virtual const char** get_tracked_conf_keys() const;
-  virtual void handle_conf_change(const struct md_config_t *conf,
-                                  const std::set<std::string> &changed);
+  const char** get_tracked_conf_keys() const override;
+  void handle_conf_change(const struct md_config_t *conf,
+                          const std::set<std::string> &changed) override;
 
   void update_log_clients();
   int sanitize_options();
