@@ -30,17 +30,15 @@ using namespace std;
 #include "include/utime.h"
 #include "common/histogram.h"
 #include "msg/Messenger.h"
-#include "common/config.h"
 #include "mon/MonitorDBStore.h"
 
-#include "messages/MPGStats.h"
-#include "messages/MPGStatsAck.h"
 class MStatfs;
 class MMonCommand;
 class MGetPoolStats;
 
 class RatioMonitor;
 class TextTable;
+class MPGStats;
 
 class PGMonitor : public PaxosService {
 public:
@@ -80,26 +78,7 @@ private:
   bool prepare_pg_stats(MonOpRequestRef op);
   void _updated_stats(MonOpRequestRef op, MonOpRequestRef ack_op);
 
-  struct C_Stats : public C_MonOp {
-    PGMonitor *pgmon;
-    MonOpRequestRef stats_op_ack;
-    entity_inst_t who;
-    C_Stats(PGMonitor *p,
-            MonOpRequestRef op,
-            MonOpRequestRef op_ack)
-      : C_MonOp(op), pgmon(p), stats_op_ack(op_ack) {}
-    void _finish(int r) {
-      if (r >= 0) {
-	pgmon->_updated_stats(op, stats_op_ack);
-      } else if (r == -ECANCELED) {
-        return;
-      } else if (r == -EAGAIN) {
-	pgmon->dispatch(op);
-      } else {
-	assert(0 == "bad C_Stats return value");
-      }
-    }    
-  };
+  struct C_Stats;
 
   void handle_statfs(MonOpRequestRef op);
   bool preprocess_getpoolstats(MonOpRequestRef op);
@@ -112,22 +91,7 @@ private:
   // when we last received PG stats from each osd
   map<int,utime_t> last_osd_report;
 
-  void register_pg(OSDMap *osdmap, pg_t pgid,
-		   epoch_t epoch, bool new_pool);
-
-  /**
-   * check latest osdmap for new pgs to register
-   *
-   * @return true if we updated pending_inc (and should propose)
-   */
-  bool register_new_pgs();
-
-  /**
-   * recalculate creating pg mappings
-   *
-   * @return true if we updated pending_inc
-   */
-  bool map_pg_creates();
+  void register_new_pgs();
 
   void send_pg_creates();
   epoch_t send_pg_creates(int osd, Connection *con, epoch_t next);
@@ -138,9 +102,8 @@ private:
    * clears need_check_down_pgs
    * clears need_check_down_pg_osds
    *
-   * @return true if we updated pending_inc (and should propose)
    */
-  bool check_down_pgs();
+  void check_down_pgs();
   void _try_mark_pg_stale(const OSDMap *osdmap, pg_t pgid,
 			  const pg_stat_t& cur_stat);
 
@@ -154,14 +117,6 @@ private:
 			  int threshold,
 			  vector<string>& args) const;
 
-  static void dump_object_stat_sum(TextTable &tbl, Formatter *f,
-				   object_stat_sum_t &sum,
-				   uint64_t avail,
-				   float raw_used_rate,
-				   bool verbose, const pg_pool_t *pool);
-
-  int64_t get_rule_avail(OSDMap& osdmap, int ruleno) const;
-
 public:
   PGMonitor(Monitor *mn, Paxos *p, const string& service_name)
     : PaxosService(mn, p, service_name),
@@ -198,9 +153,6 @@ public:
 
   void check_osd_map(epoch_t epoch);
 
-  void dump_pool_stats(stringstream &ss, Formatter *f, bool verbose);
-  void dump_fs_stats(stringstream &ss, Formatter *f, bool verbose) const;
-
   void dump_info(Formatter *f) const;
 
   int _warn_slow_request_histogram(const pow2_hist_t& h, string suffix,
