@@ -15,14 +15,13 @@
 #include <sstream>
 #include "Paxos.h"
 #include "Monitor.h"
-#include "MonitorDBStore.h"
-
 #include "messages/MMonPaxos.h"
 
 #include "common/config.h"
 #include "include/assert.h"
 #include "include/stringify.h"
-#include "common/Formatter.h"
+#include "common/Timer.h"
+#include "messages/PaxosServiceMessage.h"
 
 #define dout_subsys ceph_subsys_paxos
 #undef dout_prefix
@@ -38,11 +37,66 @@ static ostream& _prefix(std::ostream *_dout, Monitor *mon, const string& name,
 		<< ") ";
 }
 
+class Paxos::C_CollectTimeout : public Context {
+  Paxos *paxos;
+public:
+  explicit C_CollectTimeout(Paxos *p) : paxos(p) {}
+  void finish(int r) {
+    if (r == -ECANCELED)
+      return;
+    paxos->collect_timeout();
+  }
+};
+
+class Paxos::C_AcceptTimeout : public Context {
+  Paxos *paxos;
+public:
+  explicit C_AcceptTimeout(Paxos *p) : paxos(p) {}
+  void finish(int r) {
+    if (r == -ECANCELED)
+      return;
+    paxos->accept_timeout();
+  }
+};
+
+class Paxos::C_LeaseAckTimeout : public Context {
+  Paxos *paxos;
+public:
+  explicit C_LeaseAckTimeout(Paxos *p) : paxos(p) {}
+  void finish(int r) {
+    if (r == -ECANCELED)
+      return;
+    paxos->lease_ack_timeout();
+  }
+};
+
+class Paxos::C_LeaseTimeout : public Context {
+  Paxos *paxos;
+public:
+  explicit C_LeaseTimeout(Paxos *p) : paxos(p) {}
+  void finish(int r) {
+    if (r == -ECANCELED)
+      return;
+    paxos->lease_timeout();
+  }
+};
+
+class Paxos::C_LeaseRenew : public Context {
+  Paxos *paxos;
+public:
+  explicit C_LeaseRenew(Paxos *p) : paxos(p) {}
+  void finish(int r) {
+    if (r == -ECANCELED)
+      return;
+    paxos->lease_renew_timeout();
+  }
+};
+
 class Paxos::C_Trimmed : public Context {
   Paxos *paxos;
 public:
   explicit C_Trimmed(Paxos *p) : paxos(p) { }
-  void finish(int r) override {
+  void finish(int r) {
     paxos->trimming = false;
   }
 };
@@ -195,11 +249,7 @@ void Paxos::collect(version_t oldpn)
   }
 
   // set timeout event
-  collect_timeout_event = new C_MonContext(mon, [this](int r) {
-	if (r == -ECANCELED)
-	  return;
-	collect_timeout();
-    });
+  collect_timeout_event = new C_CollectTimeout(this);
   mon->timer.add_event_after(g_conf->mon_accept_timeout_factor *
 			     g_conf->mon_lease,
 			     collect_timeout_event);
@@ -370,7 +420,7 @@ bool Paxos::store_state(MMonPaxos *m)
     // ignore everything if values start in the future.
     dout(10) << "store_state ignoring all values, they start at " << start->first
 	     << " > last_committed+1" << dendl;
-    start = m->values.end();
+    return false;
   }
 
   // push forward the start position on the message's values iterator, up until
@@ -683,11 +733,7 @@ void Paxos::begin(bufferlist& v)
   }
 
   // set timeout event
-  accept_timeout_event = new C_MonContext(mon, [this](int r) {
-      if (r == -ECANCELED)
-	return;
-      accept_timeout();
-    });
+  accept_timeout_event = new C_AcceptTimeout(this);
   mon->timer.add_event_after(g_conf->mon_accept_timeout_factor *
 			     g_conf->mon_lease,
 			     accept_timeout_event);
@@ -973,22 +1019,14 @@ void Paxos::extend_lease()
   // set timeout event.
   //  if old timeout is still in place, leave it.
   if (!lease_ack_timeout_event) {
-    lease_ack_timeout_event = new C_MonContext(mon, [this](int r) {
-	if (r == -ECANCELED)
-	  return;
-	lease_ack_timeout();
-      });
+    lease_ack_timeout_event = new C_LeaseAckTimeout(this);
     mon->timer.add_event_after(g_conf->mon_lease_ack_timeout_factor *
 			       g_conf->mon_lease,
 			       lease_ack_timeout_event);
   }
 
   // set renew event
-  lease_renew_event = new C_MonContext(mon, [this](int r) {
-      if (r == -ECANCELED)
-	return;
-      lease_renew_timeout();
-    });
+  lease_renew_event = new C_LeaseRenew(this);
   utime_t at = lease_expire;
   at -= g_conf->mon_lease;
   at += g_conf->mon_lease_renew_interval_factor * g_conf->mon_lease;
@@ -1042,9 +1080,7 @@ void Paxos::commit_proposal()
   assert(mon->is_leader());
   assert(is_refresh());
 
-  list<Context*> ls;
-  ls.swap(committing_finishers);
-  finish_contexts(g_ceph_context, ls);
+  finish_contexts(g_ceph_context, committing_finishers);
 }
 
 void Paxos::finish_round()
@@ -1173,11 +1209,7 @@ void Paxos::reset_lease_timeout()
   dout(20) << "reset_lease_timeout - setting timeout event" << dendl;
   if (lease_timeout_event)
     mon->timer.cancel_event(lease_timeout_event);
-  lease_timeout_event = new C_MonContext(mon, [this](int r) {
-      if (r == -ECANCELED)
-	return;
-      lease_timeout();
-    });
+  lease_timeout_event = new C_LeaseTimeout(this);
   mon->timer.add_event_after(g_conf->mon_lease_ack_timeout_factor *
 			     g_conf->mon_lease,
 			     lease_timeout_event);
@@ -1454,9 +1486,7 @@ bool Paxos::is_readable(version_t v)
     ret =
       (mon->is_peon() || mon->is_leader()) &&
       (is_active() || is_updating() || is_writing()) &&
-      last_committed > 0 &&           // must have a value
-      (mon->get_quorum().size() == 1 ||  // alone, or
-       is_lease_valid()); // have lease
+      last_committed > 0 && is_lease_valid(); // must have a value alone, or have lease
   dout(5) << __func__ << " = " << (int)ret
 	  << " - now=" << ceph_clock_now(g_ceph_context)
 	  << " lease_expire=" << lease_expire
