@@ -17,7 +17,6 @@
 #define CEPH_DISPATCHER_H
 
 #include "include/assert.h"
-#include <memory>
 #include "include/buffer_fwd.h"
 #include "include/assert.h"
 
@@ -27,7 +26,6 @@ class Connection;
 class AuthAuthorizer;
 class CryptoKey;
 class CephContext;
-class AuthAuthorizerChallenge;
 
 class Dispatcher {
 public:
@@ -158,6 +156,16 @@ public:
    */
   virtual void ms_handle_remote_reset(Connection *con) = 0;
   
+  /**
+   * This indicates that the connection is both broken and further
+   * connection attempts are failing because other side refuses
+   * it.
+   *
+   * @param con The Connection which broke. You are not granted
+   * a reference to it.
+   */
+  virtual bool ms_handle_refused(Connection *con) = 0;
+
   /**
    * @defgroup Authentication
    * @{
@@ -196,10 +204,7 @@ public:
 				    ceph::bufferlist& authorizer,
 				    ceph::bufferlist& authorizer_reply,
 				    bool& isvalid,
-				    CryptoKey& session_key,
-				    std::unique_ptr<AuthAuthorizerChallenge> *challenge) {
-    return false;
-  }
+				    CryptoKey& session_key) { return false; }
   /**
    * @} //Authentication
    */
