@@ -90,6 +90,7 @@ using namespace std;
 #include "messages/MMonGetVersion.h"
 #include "messages/MMonGetVersionReply.h"
 #include "messages/MMonHealth.h"
+#include "messages/MMonMetadata.h"
 #include "messages/MDataPing.h"
 #include "messages/MAuth.h"
 #include "messages/MAuthReply.h"
@@ -110,6 +111,8 @@ using namespace std;
 #include "messages/MMDSSlaveRequest.h"
 
 #include "messages/MMDSMap.h"
+#include "messages/MFSMap.h"
+#include "messages/MFSMapUser.h"
 #include "messages/MMDSBeacon.h"
 #include "messages/MMDSLoadTargets.h"
 #include "messages/MMDSResolve.h"
@@ -153,6 +156,13 @@ using namespace std;
 #include "messages/MCacheExpire.h"
 #include "messages/MInodeFileCaps.h"
 
+#include "messages/MMgrBeacon.h"
+#include "messages/MMgrMap.h"
+#include "messages/MMgrDigest.h"
+#include "messages/MMgrReport.h"
+#include "messages/MMgrOpen.h"
+#include "messages/MMgrConfigure.h"
+
 #include "messages/MLock.h"
 
 #include "messages/MWatchNotify.h"
@@ -169,6 +179,9 @@ using namespace std;
 #include "messages/MOSDECSubOpRead.h"
 #include "messages/MOSDECSubOpReadReply.h"
 
+#include "messages/MOSDPGUpdateLogMissing.h"
+#include "messages/MOSDPGUpdateLogMissingReply.h"
+
 #define DEBUGLVL  10    // debug level of output
 
 #define dout_subsys ceph_subsys_ms
@@ -396,6 +409,9 @@ Message *decode_message(CephContext *cct, int crcflags,
   case CEPH_MSG_MON_GET_VERSION_REPLY:
     m = new MMonGetVersionReply();
     break;
+  case CEPH_MSG_MON_METADATA:
+    m = new MMonMetadata();
+    break;
 
   case MSG_OSD_BOOT:
     m = new MOSDBoot();
@@ -433,6 +449,12 @@ Message *decode_message(CephContext *cct, int crcflags,
   case MSG_OSD_REPOPREPLY:
     m = new MOSDRepOpReply();
     break;
+  case MSG_OSD_PG_UPDATE_LOG_MISSING:
+    m = new MOSDPGUpdateLogMissing();
+    break;
+  case MSG_OSD_PG_UPDATE_LOG_MISSING_REPLY:
+    m = new MOSDPGUpdateLogMissingReply();
+    break;
 
   case CEPH_MSG_OSD_MAP:
     m = new MOSDMap;
@@ -561,6 +583,12 @@ Message *decode_message(CephContext *cct, int crcflags,
   case CEPH_MSG_MDS_MAP:
     m = new MMDSMap;
     break;
+  case CEPH_MSG_FS_MAP:
+    m = new MFSMap;
+    break;
+  case CEPH_MSG_FS_MAP_USER:
+    m = new MFSMapUser;
+    break;
   case MSG_MDS_BEACON:
     m = new MMDSBeacon;
     break;
@@ -576,12 +604,7 @@ Message *decode_message(CephContext *cct, int crcflags,
   case MSG_MDS_CACHEREJOIN:
     m = new MMDSCacheRejoin;
 	break;
-	/*
-  case MSG_MDS_CACHEREJOINACK:
-	m = new MMDSCacheRejoinAck;
-	break;
-	*/
-
+  
   case MSG_MDS_DIRUPDATE:
     m = new MDirUpdate();
     break;
@@ -690,6 +713,30 @@ Message *decode_message(CephContext *cct, int crcflags,
     m = new MLock();
     break;
 
+  case MSG_MGR_BEACON:
+    m = new MMgrBeacon();
+    break;
+
+  case MSG_MGR_MAP:
+    m = new MMgrMap();
+    break;
+
+  case MSG_MGR_DIGEST:
+    m = new MMgrDigest();
+    break;
+
+  case MSG_MGR_OPEN:
+    m = new MMgrOpen();
+    break;
+
+  case MSG_MGR_REPORT:
+    m = new MMgrReport();
+    break;
+
+  case MSG_MGR_CONFIGURE:
+    m = new MMgrConfigure();
+    break;
+
   case MSG_TIMECHECK:
     m = new MTimeCheck();
     break;
