@@ -30,72 +30,15 @@ using namespace std;
 #include "include/atomic.h"
 #include "common/Cond.h"
 #include "common/Thread.h"
-#include "common/Throttle.h"
 
 #include "msg/SimplePolicyMessenger.h"
+#include "msg/DispatchQueue.h"
 #include "include/assert.h"
 #include "AsyncConnection.h"
 #include "Event.h"
-#include "common/simple_spin.h"
 
 
 class AsyncMessenger;
-class WorkerPool;
-
-enum {
-  l_msgr_first = 94000,
-  l_msgr_recv_messages,
-  l_msgr_send_messages,
-  l_msgr_send_messages_inline,
-  l_msgr_recv_bytes,
-  l_msgr_send_bytes,
-  l_msgr_created_connections,
-  l_msgr_active_connections,
-  l_msgr_last,
-};
-
-
-class Worker : public Thread {
-  static const uint64_t InitEventNumber = 5000;
-  static const uint64_t EventMaxWaitUs = 30000000;
-  CephContext *cct;
-  WorkerPool *pool;
-  bool done;
-  int id;
-  PerfCounters *perf_logger;
-
- public:
-  EventCenter center;
-  std::atomic_uint references;
-  Worker(CephContext *c, WorkerPool *p, int i)
-    : cct(c), pool(p), done(false), id(i), perf_logger(NULL), center(c), references(0) {
-    center.init(InitEventNumber);
-    char name[128];
-    sprintf(name, "AsyncMessenger::Worker-%d", id);
-    // initialize perf_logger
-    PerfCountersBuilder plb(cct, name, l_msgr_first, l_msgr_last);
-
-    plb.add_u64_counter(l_msgr_recv_messages, "msgr_recv_messages", "Network received messages");
-    plb.add_u64_counter(l_msgr_send_messages, "msgr_send_messages", "Network sent messages");
-    plb.add_u64_counter(l_msgr_send_messages_inline, "msgr_send_messages_inline", "Network sent inline messages");
-    plb.add_u64_counter(l_msgr_recv_bytes, "msgr_recv_bytes", "Network received bytes");
-    plb.add_u64_counter(l_msgr_send_bytes, "msgr_send_bytes", "Network received bytes");
-    plb.add_u64_counter(l_msgr_created_connections, "msgr_created_connections", "Created connection number");
-    plb.add_u64_counter(l_msgr_active_connections, "msgr_active_connections", "Active connection number");
-
-    perf_logger = plb.create_perf_counters();
-    cct->get_perfcounters_collection()->add(perf_logger);
-  }
-  ~Worker() {
-    if (perf_logger) {
-      cct->get_perfcounters_collection()->remove(perf_logger);
-      delete perf_logger;
-    }
-  }
-  void *entry();
-  void stop();
-  PerfCounters *get_perf_counter() { return perf_logger; }
-};
 
 /**
  * If the Messenger binds to a specific address, the Processor runs
@@ -105,71 +48,21 @@ class Processor {
   AsyncMessenger *msgr;
   NetHandler net;
   Worker *worker;
-  int listen_sd;
+  ServerSocket listen_socket;
   uint64_t nonce;
   EventCallbackRef listen_handler;
 
-  class C_processor_accept : public EventCallback {
-    Processor *pro;
-
-   public:
-    explicit C_processor_accept(Processor *p): pro(p) {}
-    void do_request(int id) {
-      pro->accept();
-    }
-  };
+  class C_processor_accept;
 
  public:
-  Processor(AsyncMessenger *r, CephContext *c, uint64_t n)
-          : msgr(r), net(c), worker(NULL), listen_sd(-1), nonce(n), listen_handler(new C_processor_accept(this)) {}
+  Processor(AsyncMessenger *r, Worker *w, CephContext *c, uint64_t n);
   ~Processor() { delete listen_handler; };
 
   void stop();
   int bind(const entity_addr_t &bind_addr, const set<int>& avoid_ports);
   int rebind(const set<int>& avoid_port);
-  int start(Worker *w);
-  void accept();
-};
-
-class WorkerPool {
-  WorkerPool(const WorkerPool &);
-  WorkerPool& operator=(const WorkerPool &);
-  CephContext *cct;
-  vector<Worker*> workers;
-  vector<int> coreids;
-  // Used to indicate whether thread started
-  bool started;
-  Mutex barrier_lock;
-  Cond barrier_cond;
-  atomic_t barrier_count;
-  simple_spinlock_t pool_spin = SIMPLE_SPINLOCK_INITIALIZER;
-
-  class C_barrier : public EventCallback {
-    WorkerPool *pool;
-   public:
-    explicit C_barrier(WorkerPool *p): pool(p) {}
-    void do_request(int id) {
-      Mutex::Locker l(pool->barrier_lock);
-      pool->barrier_count.dec();
-      pool->barrier_cond.Signal();
-      delete this;
-    }
-  };
-  friend class C_barrier;
- public:
-  explicit WorkerPool(CephContext *c);
-  virtual ~WorkerPool();
   void start();
-  Worker *get_worker();
-  void release_worker(EventCenter* c);
-  int get_cpuid(int id) {
-    if (coreids.empty())
-      return -1;
-    return coreids[id % coreids.size()];
-  }
-  void barrier();
-  // uniq name for CephContext to distinguish differnt object
-  static const string name;
+  void accept();
 };
 
 /*
@@ -202,14 +95,14 @@ public:
   /** @defgroup Accessors
    * @{
    */
-  void set_addr_unknowns(entity_addr_t& addr);
+  void set_addr_unknowns(const entity_addr_t &addr) override;
 
   int get_dispatch_queue_len() {
-    return 0;
+    return dispatch_queue.get_queue_len();
   }
 
   double get_dispatch_queue_max_age(utime_t now) {
-    return 0;
+    return dispatch_queue.get_max_age(now);
   }
   /** @} Accessors */
 
@@ -231,9 +124,9 @@ public:
    * @defgroup Startup/Shutdown
    * @{
    */
-  virtual int start();
-  virtual void wait();
-  virtual int shutdown();
+  int start() override;
+  void wait() override;
+  int shutdown() override;
 
   /** @} // Startup/Shutdown */
 
@@ -241,8 +134,8 @@ public:
    * @defgroup Messaging
    * @{
    */
-  virtual int send_message(Message *m, const entity_inst_t& dest) {
-          Mutex::Locker l(lock);
+  int send_message(Message *m, const entity_inst_t& dest) override {
+    Mutex::Locker l(lock);
 
     return _send_message(m, dest);
   }
@@ -253,11 +146,13 @@ public:
    * @defgroup Connection Management
    * @{
    */
-  virtual ConnectionRef get_connection(const entity_inst_t& dest);
-  virtual ConnectionRef get_loopback_connection();
+  ConnectionRef get_connection(const entity_inst_t& dest) override;
+  ConnectionRef get_loopback_connection() override;
   int send_keepalive(Connection *con);
-  virtual void mark_down(const entity_addr_t& addr);
-  virtual void mark_down_all();
+  virtual void mark_down(const entity_addr_t& addr) override;
+  virtual void mark_down_all() override {
+    shutdown_connections(true);
+  }
   /** @} // Connection Management */
 
   /**
@@ -265,12 +160,6 @@ public:
    * @{
    */
 
-  Connection *create_anon_connection() {
-    Mutex::Locker l(lock);
-    Worker *w = pool->get_worker();
-    return new AsyncConnection(cct, this, &w->center, w->get_perf_counter());
-  }
-
   /**
    * @} // Inner classes
    */
@@ -283,7 +172,7 @@ protected:
   /**
    * Start up the DispatchQueue thread once we have somebody to dispatch to.
    */
-  virtual void ready();
+  void ready() override;
   /** @} // Messenger Interfaces */
 
 private:
@@ -326,21 +215,11 @@ private:
  private:
   static const uint64_t ReapDeadConnectionThreshold = 5;
 
-  WorkerPool *pool;
-
-  Processor processor;
+  NetworkStack *stack;
+  std::vector<Processor*> processors;
   friend class Processor;
+  DispatchQueue dispatch_queue;
 
-  class C_handle_reap : public EventCallback {
-    AsyncMessenger *msgr;
-
-   public:
-    explicit C_handle_reap(AsyncMessenger *m): msgr(m) {}
-    void do_request(int id) {
-      // judge whether is a time event
-      msgr->reap_dead();
-    }
-  };
   // the worker run messenger's cron jobs
   Worker *local_worker;
 
@@ -431,6 +310,8 @@ private:
     ms_deliver_handle_fast_connect(local_connection.get());
   }
 
+  void shutdown_connections(bool queue_reset);
+
 public:
 
   /// con used for sending messages to ourselves
@@ -459,8 +340,6 @@ public:
       // If conn already in, we will return 0
       Mutex::Locker l(deleted_lock);
       if (deleted_conns.erase(existing)) {
-        existing->get_perf_counter()->dec(l_msgr_active_connections);
-        conns.erase(it);
       } else if (conn != existing) {
         return -1;
       }
@@ -472,7 +351,7 @@ public:
   }
 
   void learned_addr(const entity_addr_t &peer_addr_for_me);
-  AsyncConnectionRef add_accept(int sd);
+  AsyncConnectionRef add_accept(Worker *w, ConnectedSocket cli_socket, entity_addr_t &addr);
 
   /**
    * This wraps ms_deliver_get_authorizer. We use it for AsyncConnection.
@@ -485,10 +364,9 @@ public:
    * This wraps ms_deliver_verify_authorizer; we use it for AsyncConnection.
    */
   bool verify_authorizer(Connection *con, int peer_type, int protocol, bufferlist& auth, bufferlist& auth_reply,
-                         bool& isvalid, CryptoKey& session_key,
-			 std::unique_ptr<AuthAuthorizerChallenge> *challenge) {
+                         bool& isvalid, CryptoKey& session_key) {
     return ms_deliver_verify_authorizer(con, peer_type, protocol, auth,
-                                        auth_reply, isvalid, session_key, challenge);
+                                        auth_reply, isvalid, session_key);
   }
   /**
    * Increment the global sequence for this AsyncMessenger and return it.
@@ -510,7 +388,7 @@ public:
    * a peer protocol (if it matches our own), the protocol version for the
    * peer (if we're connecting), or our protocol version (if we're accepting).
    */
-  int get_proto_version(int peer_type, bool connect);
+  int get_proto_version(int peer_type, bool connect) const;
 
   /**
    * Fill in the address and peer type for the local connection, which
@@ -528,7 +406,6 @@ public:
    */
   void unregister_conn(AsyncConnectionRef conn) {
     Mutex::Locker l(deleted_lock);
-    conn->release_worker();
     deleted_conns.insert(conn);
 
     if (deleted_conns.size() >= ReapDeadConnectionThreshold) {
@@ -544,10 +421,6 @@ public:
    * See "deleted_conns"
    */
   int reap_dead();
-  
-  void release_worker(EventCenter* c) {
-    pool->release_worker(c);
-  }
 
   /**
    * @} // AsyncMessenger Internals
