@@ -17,9 +17,6 @@
 #ifndef CEPH_MSG_EVENTSELECT_H
 #define CEPH_MSG_EVENTSELECT_H
 
-#include <unistd.h>
-#include <sys/select.h>
-
 #include "Event.h"
 
 class SelectDriver : public EventDriver {
@@ -34,11 +31,12 @@ class SelectDriver : public EventDriver {
   explicit SelectDriver(CephContext *c): max_fd(0), cct(c) {}
   virtual ~SelectDriver() {}
 
-  int init(int nevent);
-  int add_event(int fd, int cur_mask, int add_mask);
-  int del_event(int fd, int cur_mask, int del_mask);
-  int resize_events(int newsize);
-  int event_wait(vector<FiredFileEvent> &fired_events, struct timeval *tp);
+  int init(int nevent) override;
+  int add_event(int fd, int cur_mask, int add_mask) override;
+  int del_event(int fd, int cur_mask, int del_mask) override;
+  int resize_events(int newsize) override;
+  int event_wait(vector<FiredFileEvent> &fired_events,
+		 struct timeval *tp) override;
 };
 
 #endif
