@@ -35,7 +35,7 @@ using namespace std;
 #include "msg/Message.h"
 #include "include/assert.h"
 
-#include "DispatchQueue.h"
+#include "msg/DispatchQueue.h"
 #include "Pipe.h"
 #include "Accepter.h"
 
@@ -93,7 +93,7 @@ public:
   /** @defgroup Accessors
    * @{
    */
-  void set_addr_unknowns(entity_addr_t& addr);
+  void set_addr_unknowns(const entity_addr_t& addr) override;
 
   int get_dispatch_queue_len() {
     return dispatch_queue.get_queue_len();
@@ -305,8 +305,8 @@ private:
   /// internal cluster protocol version, if any, for talking to entities of the same type.
   int cluster_protocol;
 
-  /// Throttle preventing us from building up a big backlog waiting for dispatch
-  Throttle dispatch_throttler;
+  Cond  stop_cond;
+  bool stopped = true;
 
   bool reaper_started, reaper_stop;
   Cond reaper_cond;
@@ -346,10 +346,8 @@ public:
   /**
    * This wraps ms_deliver_verify_authorizer; we use it for Pipe.
    */
-  bool verify_authorizer(Connection *con, int peer_type, int protocol, bufferlist& auth,
-			 bufferlist& auth_reply,
-                         bool& isvalid,CryptoKey& session_key,
-			 std::unique_ptr<AuthAuthorizerChallenge> *challenge);
+  bool verify_authorizer(Connection *con, int peer_type, int protocol, bufferlist& auth, bufferlist& auth_reply,
+                         bool& isvalid,CryptoKey& session_key);
   /**
    * Increment the global sequence for this SimpleMessenger and return it.
    * This is for the connect protocol, although it doesn't hurt if somebody
@@ -385,13 +383,6 @@ public:
    */
   void learned_addr(const entity_addr_t& peer_addr_for_me);
 
-  /**
-   * Release memory accounting back to the dispatch throttler.
-   *
-   * @param msize The amount of memory to release.
-   */
-  void dispatch_throttle_release(uint64_t msize);
-
   /**
    * This function is used by the reaper thread. As long as nobody
    * has set reaper_stop, it calls the reaper function, then
