@@ -28,8 +28,8 @@ extern "C" {
 #include "include/atomic.h"
 #include "auth/AuthSessionHandler.h"
 
-#define XIO_ALL_FEATURES (CEPH_FEATURES_ALL & \
-			  ~CEPH_FEATURE_MSGR_KEEPALIVE2)
+#define XIO_ALL_FEATURES (CEPH_FEATURES_ALL)
+
 
 #define XIO_NOP_TAG_MARKDOWN 0x0001
 
@@ -37,7 +37,7 @@ namespace bi = boost::intrusive;
 
 class XioPortal;
 class XioMessenger;
-class XioMsg;
+class XioSend;
 
 class XioConnection : public Connection
 {
@@ -195,8 +195,13 @@ private:
 
   // message submission queue
   struct SendQ {
+    bool keepalive;
+    bool ack;
+    utime_t ack_time;
     Message::Queue mqueue; // deferred
     XioSubmit::Queue requeue;
+
+    SendQ():keepalive(false), ack(false){}
   } outgoing;
 
   // conns_entity_map comparison functor
@@ -231,12 +236,12 @@ private:
   friend class XioMessenger;
   friend class XioDispatchHook;
   friend class XioMarkDownHook;
-  friend class XioMsg;
+  friend class XioSend;
 
   int on_disconnect_event() {
     connected.set(false);
     pthread_spin_lock(&sp);
-    discard_input_queue(CState::OP_FLAG_LOCKED);
+    discard_out_queues(CState::OP_FLAG_LOCKED);
     pthread_spin_unlock(&sp);
     return 0;
   }
@@ -271,7 +276,8 @@ public:
   bool is_connected() override { return connected.read(); }
 
   int send_message(Message *m) override;
-  void send_keepalive() override {}
+  void send_keepalive() override {send_keepalive_or_ack();}
+  void send_keepalive_or_ack(bool ack = false, const utime_t *tp = nullptr);
   void mark_down() override;
   int _mark_down(uint32_t flags);
   void mark_disposable() override;
@@ -311,16 +317,20 @@ public:
 
   int passive_setup(); /* XXX */
 
+  int handle_data_msg(struct xio_session *session, struct xio_msg *msg,
+		 int more_in_batch, void *cb_user_context);
   int on_msg(struct xio_session *session, struct xio_msg *msg,
 		 int more_in_batch, void *cb_user_context);
   int on_ow_msg_send_complete(struct xio_session *session, struct xio_msg *msg,
 			      void *conn_user_context);
   int on_msg_error(struct xio_session *session, enum xio_status error,
 		   struct xio_msg  *msg, void *conn_user_context);
-  void msg_send_fail(XioMsg *xmsg, int code);
+  void msg_send_fail(XioSend *xsend, int code);
   void msg_release_fail(struct xio_msg *msg, int code);
+private:
+  void send_keepalive_or_ack_internal(bool ack = false, const utime_t *tp = nullptr);
   int flush_out_queues(uint32_t flags);
-  int discard_input_queue(uint32_t flags);
+  int discard_out_queues(uint32_t flags);
   int adjust_clru(uint32_t flags);
 };
 
@@ -346,7 +356,7 @@ public:
   bool is_connected() override { return true; }
 
   int send_message(Message *m) override;
-  void send_keepalive() override {}
+  void send_keepalive() override;
   void mark_down() override {}
   void mark_disposable() override {}
 
