@@ -24,7 +24,6 @@
 #define dout_subsys ceph_subsys_filestore
 
 const string HashIndex::SUBDIR_ATTR = "contents";
-const string HashIndex::SETTINGS_ATTR = "settings";
 const string HashIndex::IN_PROGRESS_OP_TAG = "in_progress_op";
 
 /// hex digit to integer value
@@ -316,8 +315,6 @@ int HashIndex::split_dirs(const vector<string> &path) {
   }
 
   if (must_split(info)) {
-    dout(1) << __func__ << " " << path << " has " << info.objs
-            << " objects, starting split." << dendl;
     r = initiate_split(path, info);
     if (r < 0) {
       dout(10) << "error initiating split on " << path << ": "
@@ -326,8 +323,6 @@ int HashIndex::split_dirs(const vector<string> &path) {
     }
 
     r = complete_split(path, info);
-    dout(1) << __func__ << " " << path << " split completed."
-            << dendl;
     if (r < 0) {
       dout(10) << "error completing split on " << path << ": "
 	       << cpp_strerror(r) << dendl;
@@ -358,50 +353,14 @@ int HashIndex::split_dirs(const vector<string> &path) {
 int HashIndex::apply_layout_settings() {
   vector<string> path;
   dout(10) << __func__ << " split multiple = " << split_multiplier
-	   << " merge threshold = " << merge_threshold
-	   << " split rand factor = " << g_conf->filestore_split_rand_factor
-	   << dendl;
-  int r = write_settings();
-  if (r < 0)
-    return r;
+	   << " merge threshold = " << merge_threshold << dendl;
   return split_dirs(path);
 }
 
 int HashIndex::_init() {
   subdir_info_s info;
   vector<string> path;
-  int r = set_info(path, info);
-  if (r < 0)
-    return r;
-  return write_settings();
-}
-
-int HashIndex::write_settings() {
-  if (g_conf->filestore_split_rand_factor > 0) {
-    settings.split_rand_factor = rand() % g_conf->filestore_split_rand_factor;
-  } else {
-    settings.split_rand_factor = 0;
-  }
-  vector<string> path;
-  bufferlist bl;
-  settings.encode(bl);
-  return add_attr_path(path, SETTINGS_ATTR, bl);
-}
-
-int HashIndex::read_settings() {
-  vector<string> path;
-  bufferlist bl;
-  int r = get_attr_path(path, SETTINGS_ATTR, bl);
-  if (r == -ENODATA)
-    return 0;
-  if (r < 0) {
-    derr << __func__ << " error reading settings: " << cpp_strerror(r) << dendl;
-    return r;
-  }
-  bufferlist::iterator it = bl.begin();
-  settings.decode(it);
-  dout(20) << __func__ << " split_rand_factor = " << settings.split_rand_factor << dendl;
-  return 0;
+  return set_info(path, info);
 }
 
 /* LFNIndex virtual method implementations */
@@ -419,15 +378,10 @@ int HashIndex::_created(const vector<string> &path,
     return r;
 
   if (must_split(info)) {
-    dout(1) << __func__ << " " << path << " has " << info.objs
-            << " objects, starting split." << dendl;
     int r = initiate_split(path, info);
     if (r < 0)
       return r;
-    r = complete_split(path, info);
-    dout(1) << __func__ << " " << path << " split completed."
-            << dendl;
-    return r;
+    return complete_split(path, info);
   } else {
     return 0;
   }
@@ -533,7 +487,7 @@ int HashIndex::pre_split_folder(uint32_t pg_num, uint64_t expected_num_objs)
 
   // Calculate the number of leaf folders (which actually store files)
   // need to be created
-  const uint64_t objs_per_folder = ((uint64_t)(abs(merge_threshold)) * (uint64_t)split_multiplier + settings.split_rand_factor) * 16;
+  const uint64_t objs_per_folder = (uint64_t)(abs(merge_threshold)) * (uint64_t)split_multiplier * 16;
   uint64_t leavies = expected_num_objs / objs_per_folder ;
   // No need to split
   if (leavies == 0 || expected_num_objs == objs_per_folder)
@@ -742,7 +696,7 @@ bool HashIndex::must_merge(const subdir_info_s &info) {
 
 bool HashIndex::must_split(const subdir_info_s &info) {
   return (info.hash_level < (unsigned)MAX_HASH_LEVEL &&
-	  info.objs > ((unsigned)(abs(merge_threshold) * split_multiplier + settings.split_rand_factor) * 16));
+	  info.objs > ((unsigned)(abs(merge_threshold)) * 16 * split_multiplier));
 
 }
 
@@ -1111,7 +1065,7 @@ int HashIndex::list_by_hash_bitwise(
 	}
 	if (cmp_bitwise(j->second, end) >= 0) {
 	  if (next)
-	    *next = j->second;
+	    *next = ghobject_t::get_max();
 	  return 0;
 	}
 	if (!next || cmp_bitwise(j->second, *next) >= 0) {
@@ -1177,7 +1131,7 @@ int HashIndex::list_by_hash_nibblewise(
 	}
 	if (cmp_nibblewise(j->second, end) >= 0) {
 	  if (next)
-	    *next = j->second;
+	    *next = ghobject_t::get_max();
 	  return 0;
 	}
 	if (!next || cmp_nibblewise(j->second, *next) >= 0) {
