@@ -74,7 +74,7 @@ IndexManager::~IndexManager() {
 
 
 int IndexManager::init_index(coll_t c, const char *path, uint32_t version) {
-  Mutex::Locker l(lock);
+  RWLock::WLocker l(lock);
   int r = set_version(path, version);
   if (r < 0)
     return r;
@@ -82,10 +82,7 @@ int IndexManager::init_index(coll_t c, const char *path, uint32_t version) {
 		  g_conf->filestore_split_multiple,
 		  version,
 		  g_conf->filestore_index_retry_probability);
-  r = index.init();
-  if (r < 0)
-    return r;
-  return index.read_settings();
+  return index.init();
 }
 
 int IndexManager::build_index(coll_t c, const char *path, CollectionIndex **index) {
@@ -105,7 +102,7 @@ int IndexManager::build_index(coll_t c, const char *path, CollectionIndex **inde
       // Must be a HashIndex
       *index = new HashIndex(c, path, g_conf->filestore_merge_threshold,
 				   g_conf->filestore_split_multiple, version);
-      return (*index)->read_settings();
+      return 0;
     }
     default: assert(0);
     }
@@ -116,13 +113,23 @@ int IndexManager::build_index(coll_t c, const char *path, CollectionIndex **inde
 				 g_conf->filestore_split_multiple,
 				 CollectionIndex::HOBJECT_WITH_POOL,
 				 g_conf->filestore_index_retry_probability);
-    return (*index)->read_settings();
+    return 0;
   }
 }
 
-int IndexManager::get_index(coll_t c, const string& baseDir, Index *index) {
+bool IndexManager::get_index_optimistic(coll_t c, Index *index) {
+  RWLock::RLocker l(lock);
+  ceph::unordered_map<coll_t, CollectionIndex* > ::iterator it = col_indices.find(c);
+  if (it == col_indices.end()) 
+    return false;
+  index->index = it->second;
+  return true;
+}
 
-  Mutex::Locker l(lock);
+int IndexManager::get_index(coll_t c, const string& baseDir, Index *index) {
+  if (get_index_optimistic(c, index))
+    return 0;
+  RWLock::WLocker l(lock);
   ceph::unordered_map<coll_t, CollectionIndex* > ::iterator it = col_indices.find(c);
   if (it == col_indices.end()) {
     char path[PATH_MAX];
