@@ -21,7 +21,6 @@
 #include "include/assert.h" 
 #include "osd_types.h"
 #include "os/ObjectStore.h"
-#include "common/ceph_context.h"
 #include <list>
 using namespace std;
 
@@ -30,6 +29,8 @@ using namespace std;
 #define PGLOG_INDEXED_EXTRA_CALLER_OPS (1 << 2)
 #define PGLOG_INDEXED_ALL              (PGLOG_INDEXED_OBJECTS | PGLOG_INDEXED_CALLER_OPS | PGLOG_INDEXED_EXTRA_CALLER_OPS)
 
+class CephContext;
+
 struct PGLog : DoutPrefixProvider {
   DoutPrefixProvider *prefix_provider;
   string gen_prefix() const {
@@ -50,7 +51,7 @@ struct PGLog : DoutPrefixProvider {
     virtual void remove(
       const hobject_t &hoid) = 0;
     virtual void try_stash(
-      const hobject_t &entry,
+      const hobject_t &hoid,
       version_t v) = 0;
     virtual void trim(
       const pg_log_entry_t &entry) = 0;
@@ -58,10 +59,10 @@ struct PGLog : DoutPrefixProvider {
   };
 
   /* Exceptions */
-  class read_log_error : public buffer::error {
+  class read_log_and_missing_error : public buffer::error {
   public:
-    explicit read_log_error(const char *what) {
-      snprintf(buf, sizeof(buf), "read_log_error: %s", what);
+    explicit read_log_and_missing_error(const char *what) {
+      snprintf(buf, sizeof(buf), "read_log_and_missing_error: %s", what);
     }
     const char *what() const throw () {
       return buf;
@@ -165,9 +166,11 @@ struct PGLog : DoutPrefixProvider {
     bool get_request(
       const osd_reqid_t &r,
       eversion_t *replay_version,
-      version_t *user_version) const {
+      version_t *user_version,
+      int *return_code) const {
       assert(replay_version);
       assert(user_version);
+      assert(return_code);
       ceph::unordered_map<osd_reqid_t,pg_log_entry_t*>::const_iterator p;
       if (!(indexed_data & PGLOG_INDEXED_CALLER_OPS)) {
         index_caller_ops();
@@ -176,6 +179,7 @@ struct PGLog : DoutPrefixProvider {
       if (p != caller_ops.end()) {
 	*replay_version = p->second->version;
 	*user_version = p->second->user_version;
+	*return_code = p->second->return_code;
 	return true;
       }
 
@@ -193,6 +197,7 @@ struct PGLog : DoutPrefixProvider {
 	  if (i->first == r) {
 	    *replay_version = p->second->version;
 	    *user_version = i->second;
+	    *return_code = p->second->return_code;
 	    return true;
 	  }
 	}
@@ -228,13 +233,6 @@ struct PGLog : DoutPrefixProvider {
       }
     }
     
-    void reset_riter() {
-      rollback_info_trimmed_to_riter = log.rbegin();
-      while (rollback_info_trimmed_to_riter != log.rend() &&
-        rollback_info_trimmed_to_riter->version > rollback_info_trimmed_to)
-        ++rollback_info_trimmed_to_riter;
-    }
-
     void reset_rollback_info_trimmed_to_riter() {
       rollback_info_trimmed_to_riter = log.rbegin();
       while (rollback_info_trimmed_to_riter != log.rend() &&
@@ -250,9 +248,10 @@ struct PGLog : DoutPrefixProvider {
       for (list<pg_log_entry_t>::iterator i = log.begin();
              i != log.end();
              ++i) {
-               
-        objects[i->soid] = &(*i);
-        
+	if (i->object_is_indexed()) {
+	  objects[i->soid] = &(*i);
+	}
+
         if (i->reqid_is_indexed()) {
         //assert(caller_ops.count(i->reqid) == 0);  // divergent merge_log indexes new before unindexing old
           caller_ops[i->reqid] = &(*i);
@@ -266,7 +265,6 @@ struct PGLog : DoutPrefixProvider {
         }
       }
         
-      reset_riter();
       indexed_data = PGLOG_INDEXED_ALL;
       reset_rollback_info_trimmed_to_riter();
     }
@@ -276,7 +274,9 @@ struct PGLog : DoutPrefixProvider {
       for (list<pg_log_entry_t>::const_iterator i = log.begin();
             i != log.end();
             ++i) {
-         objects[i->soid] = const_cast<pg_log_entry_t*>(&(*i));
+	if (i->object_is_indexed()) {
+	  objects[i->soid] = const_cast<pg_log_entry_t*>(&(*i));
+	}
        }
  
       indexed_data |= PGLOG_INDEXED_OBJECTS;
@@ -315,8 +315,8 @@ struct PGLog : DoutPrefixProvider {
     }
 
     void index(pg_log_entry_t& e) {
-      if (indexed_data & PGLOG_INDEXED_OBJECTS) {
-        if (objects.count(e.soid) == 0 || 
+      if ((indexed_data & PGLOG_INDEXED_OBJECTS) && e.object_is_indexed()) {
+        if (objects.count(e.soid) == 0 ||
             objects[e.soid]->version < e.version)
           objects[e.soid] = &e;
       }
@@ -392,7 +392,7 @@ struct PGLog : DoutPrefixProvider {
       head = e.version;
 
       // to our index
-      if (indexed_data & PGLOG_INDEXED_OBJECTS) {
+      if ((indexed_data & PGLOG_INDEXED_OBJECTS) && e.object_is_indexed()) {
         objects[e.soid] = &(log.back());
       }
       if (indexed_data & PGLOG_INDEXED_CALLER_OPS) {
@@ -425,8 +425,7 @@ struct PGLog : DoutPrefixProvider {
 protected:
   //////////////////// data members ////////////////////
 
-  map<eversion_t, hobject_t> divergent_priors;
-  pg_missing_t     missing;
+  pg_missing_tracker_t missing;
   IndexedLog  log;
 
   eversion_t dirty_to;         ///< must clear/writeout all keys <= dirty_to
@@ -437,16 +436,8 @@ protected:
   bool pg_log_debug;
   /// Log is clean on [dirty_to, dirty_from)
   bool touched_log;
-  bool dirty_divergent_priors;
+  bool clear_divergent_priors;
 
-  bool is_dirty() const {
-    return !touched_log ||
-      (dirty_to != eversion_t()) ||
-      (dirty_from != eversion_t::max()) ||
-      dirty_divergent_priors ||
-      (writeout_from != eversion_t::max()) ||
-      !(trimmed.empty());
-  }
   void mark_dirty_to(eversion_t to) {
     if (to > dirty_to)
       dirty_to = to;
@@ -459,11 +450,15 @@ protected:
     if (from < writeout_from)
       writeout_from = from;
   }
-  void add_divergent_prior(eversion_t version, hobject_t obj) {
-    divergent_priors.insert(make_pair(version, obj));
-    dirty_divergent_priors = true;
-  }
 public:
+  bool is_dirty() const {
+    return !touched_log ||
+      (dirty_to != eversion_t()) ||
+      (dirty_from != eversion_t::max()) ||
+      (writeout_from != eversion_t::max()) ||
+      !(trimmed.empty()) ||
+      !missing.is_clean();
+  }
   void mark_log_for_rewrite() {
     mark_dirty_to(eversion_t::max());
     mark_dirty_from(eversion_t());
@@ -492,11 +487,11 @@ protected:
   void undirty() {
     dirty_to = eversion_t();
     dirty_from = eversion_t::max();
-    dirty_divergent_priors = false;
     touched_log = true;
     trimmed.clear();
     writeout_from = eversion_t::max();
     check();
+    missing.flush();
   }
 public:
   // cppcheck-suppress noExplicitConstructor
@@ -506,7 +501,8 @@ public:
     writeout_from(eversion_t::max()), 
     cct(cct), 
     pg_log_debug(!(cct && !(cct->_conf->osd_debug_pg_log_writeout))),
-    touched_log(false), dirty_divergent_priors(false) {}
+    touched_log(false),
+    clear_divergent_priors(false) {}
 
 
   void reset_backfill();
@@ -515,16 +511,11 @@ public:
 
   //////////////////// get or set missing ////////////////////
 
-  const pg_missing_t& get_missing() const { return missing; }
+  const pg_missing_tracker_t& get_missing() const { return missing; }
   void resort_missing(bool sort_bitwise) {
     missing.resort(sort_bitwise);
   }
 
-  void missing_got(map<hobject_t, pg_missing_t::item, hobject_t::ComparatorWithDefault>::const_iterator m) {
-    map<hobject_t, pg_missing_t::item, hobject_t::ComparatorWithDefault>::iterator p = missing.missing.find(m->first);
-    missing.got(p);
-  }
-
   void revise_have(hobject_t oid, eversion_t have) {
     missing.revise_have(oid, have);
   }
@@ -537,11 +528,6 @@ public:
     missing.add(oid, need, have);
   }
 
-  void missing_rm(map<hobject_t, pg_missing_t::item, hobject_t::ComparatorWithDefault>::const_iterator m) {
-    map<hobject_t, pg_missing_t::item, hobject_t::ComparatorWithDefault>::iterator p = missing.missing.find(m->first);
-    missing.rm(p);
-  }
-
   void missing_add_event(const pg_log_entry_t &e) {
     missing.add_next_event(e);
   }
@@ -605,7 +591,7 @@ public:
 
   //////////////////// get or set log & missing ////////////////////
 
-  void claim_log_and_clear_rollback_info(const pg_log_t &o, LogEntryHandler *h) {
+  void reset_backfill_claim_log(const pg_log_t &o, LogEntryHandler *h) {
     log.can_rollback_to = log.head;
     log.advance_rollback_info_trimmed_to(log.head, h);
     log.claim_log_and_clear_rollback_info(o);
@@ -621,19 +607,6 @@ public:
     missing.split_into(child_pgid, split_bits, &(opg_log->missing));
     opg_log->mark_dirty_to(eversion_t::max());
     mark_dirty_to(eversion_t::max());
-
-    unsigned mask = ~((~0)<<split_bits);
-    for (map<eversion_t, hobject_t>::iterator i = divergent_priors.begin();
-	 i != divergent_priors.end();
-	 ) {
-      if ((i->second.get_hash() & mask) == child_pgid.m_seed) {
-	opg_log->add_divergent_prior(i->first, i->second);
-	divergent_priors.erase(i++);
-	dirty_divergent_priors = true;
-      } else {
-	++i;
-      }
-    }
   }
 
   void recover_got(hobject_t oid, eversion_t v, pg_info_t &info) {
@@ -641,13 +614,14 @@ public:
       missing.got(oid, v);
       
       // raise last_complete?
-      if (missing.missing.empty()) {
+      if (missing.get_items().empty()) {
 	log.complete_to = log.log.end();
 	info.last_complete = info.last_update;
       }
       while (log.complete_to != log.log.end()) {
-	if (missing.missing[missing.rmissing.begin()->second].need <=
-	    log.complete_to->version)
+	if (missing.get_items().at(
+	      missing.get_rmissing().begin()->second
+	      ).need <= log.complete_to->version)
 	  break;
 	if (info.last_complete < log.complete_to->version)
 	  info.last_complete = log.complete_to->version;
@@ -662,7 +636,9 @@ public:
   void activate_not_complete(pg_info_t &info) {
     log.complete_to = log.log.begin();
     while (log.complete_to->version <
-	   missing.missing[missing.rmissing.begin()->second].need)
+	   missing.get_items().at(
+	     missing.get_rmissing().begin()->second
+	     ).need)
       ++log.complete_to;
     assert(log.complete_to != log.log.end());
     if (log.complete_to == log.log.begin()) {
@@ -689,30 +665,196 @@ protected:
   }
 
   /**
-   * Merge complete list of divergent entries for an object
+   * _merge_object_divergent_entries
    *
-   * @param new_divergent_prior [out] filled out for a new divergent prior
+   * There are 5 distinct cases:
+   * 1) There is a more recent update: in this case we assume we adjusted the
+   *    store and missing during merge_log
+   * 2) The first entry in the divergent sequence is a create.  This might
+   *    either be because the object is a clone or because prior_version is
+   *    eversion_t().  In this case the object does not exist and we must
+   *    adjust missing and the store to match.
+   * 3) We are currently missing the object.  In this case, we adjust the
+   *    missing to our prior_version taking care to add a divergent_prior
+   *    if necessary
+   * 4) We can rollback all of the entries.  In this case, we do so using
+   *    the rollbacker and return -- the object does not go into missing.
+   * 5) We cannot rollback at least 1 of the entries.  In this case, we
+   *    clear the object out of the store and add a missing entry at
+   *    prior_version taking care to add a divergent_prior if
+   *    necessary.
    */
+  template <typename missing_type>
   static void _merge_object_divergent_entries(
     const IndexedLog &log,               ///< [in] log to merge against
     const hobject_t &hoid,               ///< [in] object we are merging
     const list<pg_log_entry_t> &entries, ///< [in] entries for hoid to merge
-    const pg_info_t &oinfo,              ///< [in] info for merging entries
+    const pg_info_t &info,              ///< [in] info for merging entries
     eversion_t olog_can_rollback_to,     ///< [in] rollback boundary
-    pg_missing_t &omissing,              ///< [in,out] missing to adjust, use
-    boost::optional<pair<eversion_t, hobject_t> > *new_divergent_prior,
+    missing_type &missing,              ///< [in,out] missing to adjust, use
     LogEntryHandler *rollbacker,         ///< [in] optional rollbacker object
     const DoutPrefixProvider *dpp        ///< [in] logging provider
-    );
+    ) {
+    ldpp_dout(dpp, 20) << __func__ << ": merging hoid " << hoid
+		       << " entries: " << entries << dendl;
+
+    if (cmp(hoid, info.last_backfill, info.last_backfill_bitwise) > 0) {
+      ldpp_dout(dpp, 10) << __func__ << ": hoid " << hoid << " after last_backfill"
+			 << dendl;
+      return;
+    }
+
+    // entries is non-empty
+    assert(!entries.empty());
+    eversion_t last;
+    for (list<pg_log_entry_t>::const_iterator i = entries.begin();
+	 i != entries.end();
+	 ++i) {
+      // all entries are on hoid
+      assert(i->soid == hoid);
+      if (i != entries.begin() && i->prior_version != eversion_t()) {
+	// in increasing order of version
+	assert(i->version > last);
+	// prior_version correct
+	assert(i->prior_version == last);
+      }
+      last = i->version;
+
+      if (rollbacker)
+	rollbacker->trim(*i);
+    }
+
+    const eversion_t prior_version = entries.begin()->prior_version;
+    const eversion_t first_divergent_update = entries.begin()->version;
+    const eversion_t last_divergent_update = entries.rbegin()->version;
+    const bool object_not_in_store =
+      !missing.is_missing(hoid) &&
+      entries.rbegin()->is_delete();
+    ldpp_dout(dpp, 10) << __func__ << ": hoid " << hoid
+		       << " prior_version: " << prior_version
+		       << " first_divergent_update: " << first_divergent_update
+		       << " last_divergent_update: " << last_divergent_update
+		       << dendl;
+
+    ceph::unordered_map<hobject_t, pg_log_entry_t*>::const_iterator objiter =
+      log.objects.find(hoid);
+    if (objiter != log.objects.end() &&
+	objiter->second->version >= first_divergent_update) {
+      /// Case 1)
+      assert(objiter->second->version > last_divergent_update);
+
+      ldpp_dout(dpp, 10) << __func__ << ": more recent entry found: "
+			 << *objiter->second << ", already merged" << dendl;
+
+      // ensure missing has been updated appropriately
+      if (objiter->second->is_update()) {
+	assert(missing.is_missing(hoid) &&
+	       missing.get_items().at(hoid).need == objiter->second->version);
+      } else {
+	assert(!missing.is_missing(hoid));
+      }
+      missing.revise_have(hoid, eversion_t());
+      if (rollbacker && !object_not_in_store)
+	rollbacker->remove(hoid);
+      return;
+    }
+
+    ldpp_dout(dpp, 10) << __func__ << ": hoid " << hoid
+		       <<" has no more recent entries in log" << dendl;
+    if (prior_version == eversion_t() || entries.front().is_clone()) {
+      /// Case 2)
+      ldpp_dout(dpp, 10) << __func__ << ": hoid " << hoid
+			 << " prior_version or op type indicates creation,"
+			 << " deleting"
+			 << dendl;
+      if (missing.is_missing(hoid))
+	missing.rm(missing.get_items().find(hoid));
+      if (rollbacker && !object_not_in_store)
+	rollbacker->remove(hoid);
+      return;
+    }
+
+    if (missing.is_missing(hoid)) {
+      /// Case 3)
+      ldpp_dout(dpp, 10) << __func__ << ": hoid " << hoid
+			 << " missing, " << missing.get_items().at(hoid)
+			 << " adjusting" << dendl;
+
+      if (missing.get_items().at(hoid).have == prior_version) {
+	ldpp_dout(dpp, 10) << __func__ << ": hoid " << hoid
+			   << " missing.have is prior_version " << prior_version
+			   << " removing from missing" << dendl;
+	missing.rm(missing.get_items().find(hoid));
+      } else {
+	ldpp_dout(dpp, 10) << __func__ << ": hoid " << hoid
+			   << " missing.have is " << missing.get_items().at(hoid).have
+			   << ", adjusting" << dendl;
+	missing.revise_need(hoid, prior_version);
+	if (prior_version <= info.log_tail) {
+	  ldpp_dout(dpp, 10) << __func__ << ": hoid " << hoid
+			     << " prior_version " << prior_version
+			     << " <= info.log_tail "
+			     << info.log_tail << dendl;
+	}
+      }
+      return;
+    }
+
+    ldpp_dout(dpp, 10) << __func__ << ": hoid " << hoid
+		       << " must be rolled back or recovered,"
+		       << " attempting to rollback"
+		       << dendl;
+    bool can_rollback = true;
+    /// Distinguish between 4) and 5)
+    for (list<pg_log_entry_t>::const_reverse_iterator i = entries.rbegin();
+	 i != entries.rend();
+	 ++i) {
+      if (!i->mod_desc.can_rollback() || i->version <= olog_can_rollback_to) {
+	ldpp_dout(dpp, 10) << __func__ << ": hoid " << hoid << " cannot rollback "
+			   << *i << dendl;
+	can_rollback = false;
+	break;
+      }
+    }
+
+    if (can_rollback) {
+      /// Case 4)
+      for (list<pg_log_entry_t>::const_reverse_iterator i = entries.rbegin();
+	   i != entries.rend();
+	   ++i) {
+	assert(i->mod_desc.can_rollback() && i->version > olog_can_rollback_to);
+	ldpp_dout(dpp, 10) << __func__ << ": hoid " << hoid
+			   << " rolling back " << *i << dendl;
+	if (rollbacker)
+	  rollbacker->rollback(*i);
+      }
+      ldpp_dout(dpp, 10) << __func__ << ": hoid " << hoid
+			 << " rolled back" << dendl;
+      return;
+    } else {
+      /// Case 5)
+      ldpp_dout(dpp, 10) << __func__ << ": hoid " << hoid << " cannot roll back, "
+			 << "removing and adding to missing" << dendl;
+      if (rollbacker && !object_not_in_store)
+	rollbacker->remove(hoid);
+      missing.add(hoid, prior_version, eversion_t());
+      if (prior_version <= info.log_tail) {
+	ldpp_dout(dpp, 10) << __func__ << ": hoid " << hoid
+			   << " prior_version " << prior_version
+			   << " <= info.log_tail "
+			   << info.log_tail << dendl;
+      }
+    }
+  }
 
   /// Merge all entries using above
+  template <typename missing_type>
   static void _merge_divergent_entries(
     const IndexedLog &log,               ///< [in] log to merge against
     list<pg_log_entry_t> &entries,       ///< [in] entries to merge
     const pg_info_t &oinfo,              ///< [in] info for merging entries
     eversion_t olog_can_rollback_to,     ///< [in] rollback boundary
-    pg_missing_t &omissing,              ///< [in,out] missing to adjust, use
-    map<eversion_t, hobject_t> *priors,  ///< [out] target for new priors
+    missing_type &omissing,              ///< [in,out] missing to adjust, use
     LogEntryHandler *rollbacker,         ///< [in] optional rollbacker object
     const DoutPrefixProvider *dpp        ///< [in] logging provider
     ) {
@@ -721,7 +863,6 @@ protected:
     for (map<hobject_t, list<pg_log_entry_t>, hobject_t::BitwiseComparator>::iterator i = split.begin();
 	 i != split.end();
 	 ++i) {
-      boost::optional<pair<eversion_t, hobject_t> > new_divergent_prior;
       _merge_object_divergent_entries(
 	log,
 	i->first,
@@ -729,12 +870,8 @@ protected:
 	oinfo,
 	olog_can_rollback_to,
 	omissing,
-	&new_divergent_prior,
 	rollbacker,
 	dpp);
-      if (priors && new_divergent_prior) {
-	(*priors)[new_divergent_prior->first] = new_divergent_prior->second;
-      }
     }
   }
 
@@ -747,7 +884,6 @@ protected:
     const pg_log_entry_t& oe,
     const pg_info_t& info,
     LogEntryHandler *rollbacker) {
-    boost::optional<pair<eversion_t, hobject_t> > new_divergent_prior;
     list<pg_log_entry_t> entries;
     entries.push_back(oe);
     _merge_object_divergent_entries(
@@ -757,13 +893,8 @@ protected:
       info,
       log.can_rollback_to,
       missing,
-      &new_divergent_prior,
       rollbacker,
       this);
-    if (new_divergent_prior)
-      add_divergent_prior(
-	(*new_divergent_prior).first,
-	(*new_divergent_prior).second);
   }
 public:
   void rewind_divergent_log(ObjectStore::Transaction& t, eversion_t newhead,
@@ -775,20 +906,53 @@ public:
 		 pg_info_t &info, LogEntryHandler *rollbacker,
 		 bool &dirty_info, bool &dirty_big_info);
 
-  static void append_log_entries_update_missing(
+  template <typename missing_type>
+  static bool append_log_entries_update_missing(
     const hobject_t &last_backfill,
     bool last_backfill_bitwise,
     const list<pg_log_entry_t> &entries,
     IndexedLog *log,
-    pg_missing_t &missing,
+    missing_type &missing,
     LogEntryHandler *rollbacker,
-    const DoutPrefixProvider *dpp);
-  void append_new_log_entries(
+    const DoutPrefixProvider *dpp) {
+    bool invalidate_stats = false;
+    if (log && !entries.empty()) {
+      assert(log->head < entries.begin()->version);
+      log->head = entries.rbegin()->version;
+    }
+    for (list<pg_log_entry_t>::const_iterator p = entries.begin();
+	 p != entries.end();
+	 ++p) {
+      invalidate_stats = invalidate_stats || !p->is_error();
+      if (log) {
+	log->log.push_back(*p);
+	pg_log_entry_t &ne = log->log.back();
+	ldpp_dout(dpp, 20) << "update missing, append " << ne << dendl;
+	log->index(ne);
+      }
+      if (cmp(p->soid, last_backfill, last_backfill_bitwise) <= 0 &&
+	  !p->is_error()) {
+	missing.add_next_event(*p);
+	if (rollbacker) {
+	  // hack to match PG::mark_all_unfound_lost
+	  if (p->is_lost_delete() && p->mod_desc.can_rollback()) {
+	    rollbacker->try_stash(p->soid, p->version.version);
+	  } else if (p->is_delete()) {
+	    rollbacker->remove(p->soid);
+	  }
+	}
+      }
+    }
+    if (log)
+      log->reset_rollback_info_trimmed_to_riter();
+    return invalidate_stats;
+  }
+  bool append_new_log_entries(
     const hobject_t &last_backfill,
     bool last_backfill_bitwise,
     const list<pg_log_entry_t> &entries,
     LogEntryHandler *rollbacker) {
-    append_log_entries_update_missing(
+    bool invalidate_stats = append_log_entries_update_missing(
       last_backfill,
       last_backfill_bitwise,
       entries,
@@ -799,15 +963,16 @@ public:
     if (!entries.empty()) {
       mark_writeout_from(entries.begin()->version);
     }
+    return invalidate_stats;
   }
 
-  void write_log(ObjectStore::Transaction& t,
+  void write_log_and_missing(ObjectStore::Transaction& t,
 		 map<string,bufferlist> *km,
 		 const coll_t& coll,
 		 const ghobject_t &log_oid,
 		 bool require_rollback);
 
-  static void write_log(
+  static void write_log_and_missing_wo_missing(
     ObjectStore::Transaction& t,
     map<string,bufferlist>* km,
     pg_log_t &log,
@@ -815,7 +980,16 @@ public:
     const ghobject_t &log_oid, map<eversion_t, hobject_t> &divergent_priors,
     bool require_rollback);
 
-  static void _write_log(
+  static void write_log_and_missing(
+    ObjectStore::Transaction& t,
+    map<string,bufferlist>* km,
+    pg_log_t &log,
+    const coll_t& coll,
+    const ghobject_t &log_oid,
+    const pg_missing_tracker_t &missing,
+    bool require_rollback);
+
+  static void _write_log_and_missing_wo_missing(
     ObjectStore::Transaction& t,
     map<string,bufferlist>* km,
     pg_log_t &log,
@@ -831,26 +1005,234 @@ public:
     set<string> *log_keys_debug
     );
 
-  void read_log(ObjectStore *store, coll_t pg_coll,
-		coll_t log_coll, ghobject_t log_oid,
-		const pg_info_t &info, ostringstream &oss,
-		bool tolerate_divergent_missing_log) {
-    return read_log(
-      store, pg_coll, log_coll, log_oid, info, divergent_priors,
-      log, missing, oss, tolerate_divergent_missing_log,
+  static void _write_log_and_missing(
+    ObjectStore::Transaction& t,
+    map<string,bufferlist>* km,
+    pg_log_t &log,
+    const coll_t& coll, const ghobject_t &log_oid,
+    eversion_t dirty_to,
+    eversion_t dirty_from,
+    eversion_t writeout_from,
+    const set<eversion_t> &trimmed,
+    const pg_missing_tracker_t &missing,
+    bool touch_log,
+    bool require_rollback,
+    bool clear_divergent_priors,
+    set<string> *log_keys_debug
+    );
+
+  void read_log_and_missing(
+    ObjectStore *store, coll_t pg_coll,
+    coll_t log_coll, ghobject_t log_oid,
+    const pg_info_t &info,
+    ostringstream &oss,
+    bool debug_verify_stored_missing = false
+    ) {
+    return read_log_and_missing(
+      store, pg_coll, log_coll, log_oid, info,
+      log, missing, oss,
+      &clear_divergent_priors,
       this,
-      (pg_log_debug ? &log_keys_debug : 0));
+      (pg_log_debug ? &log_keys_debug : 0),
+      debug_verify_stored_missing);
   }
 
-  static void read_log(ObjectStore *store, coll_t pg_coll,
+  template <typename missing_type>
+  static void read_log_and_missing(ObjectStore *store, coll_t pg_coll,
     coll_t log_coll, ghobject_t log_oid,
-    const pg_info_t &info, map<eversion_t, hobject_t> &divergent_priors,
+    const pg_info_t &info,
     IndexedLog &log,
-    pg_missing_t &missing, ostringstream &oss,
-    bool tolerate_divergent_missing_log,
+    missing_type &missing, ostringstream &oss,
+    bool *clear_divergent_priors = NULL,
     const DoutPrefixProvider *dpp = NULL,
-    set<string> *log_keys_debug = 0
-    );
+    set<string> *log_keys_debug = 0,
+    bool debug_verify_stored_missing = false
+    ) {
+    ldpp_dout(dpp, 20) << "read_log_and_missing coll " << pg_coll
+		       << " log_oid " << log_oid << dendl;
+
+    // legacy?
+    struct stat st;
+    int r = store->stat(log_coll, log_oid, &st);
+    assert(r == 0);
+    assert(st.st_size == 0);
+
+    log.tail = info.log_tail;
+    // will get overridden below if it had been recorded
+    log.can_rollback_to = info.last_update;
+    log.rollback_info_trimmed_to = eversion_t();
+    ObjectMap::ObjectMapIterator p = store->get_omap_iterator(log_coll, log_oid);
+    map<eversion_t, hobject_t> divergent_priors;
+    bool has_divergent_priors = false;
+    if (p) {
+      for (p->seek_to_first(); p->valid() ; p->next(false)) {
+	// non-log pgmeta_oid keys are prefixed with _; skip those
+	if (p->key()[0] == '_')
+	  continue;
+	bufferlist bl = p->value();//Copy bufferlist before creating iterator
+	bufferlist::iterator bp = bl.begin();
+	if (p->key() == "divergent_priors") {
+	  ::decode(divergent_priors, bp);
+	  ldpp_dout(dpp, 20) << "read_log_and_missing " << divergent_priors.size()
+			     << " divergent_priors" << dendl;
+	  has_divergent_priors = true;
+	  debug_verify_stored_missing = false;
+	} else if (p->key() == "can_rollback_to") {
+	  ::decode(log.can_rollback_to, bp);
+	} else if (p->key() == "rollback_info_trimmed_to") {
+	  ::decode(log.rollback_info_trimmed_to, bp);
+	} else if (p->key().substr(0, 7) == string("missing")) {
+	  pair<hobject_t, pg_missing_item> p;
+	  ::decode(p, bp);
+	  missing.add(p.first, p.second.need, p.second.have);
+	} else {
+	  pg_log_entry_t e;
+	  e.decode_with_checksum(bp);
+	  ldpp_dout(dpp, 20) << "read_log_and_missing " << e << dendl;
+	  if (!log.log.empty()) {
+	    pg_log_entry_t last_e(log.log.back());
+	    assert(last_e.version.version < e.version.version);
+	    assert(last_e.version.epoch <= e.version.epoch);
+	  }
+	  log.log.push_back(e);
+	  log.head = e.version;
+	  if (log_keys_debug)
+	    log_keys_debug->insert(e.get_key_name());
+	}
+      }
+    }
+    log.head = info.last_update;
+    log.reset_rollback_info_trimmed_to_riter();
+
+    if (has_divergent_priors || debug_verify_stored_missing) {
+      // build missing
+      if (debug_verify_stored_missing || info.last_complete < info.last_update) {
+	ldpp_dout(dpp, 10) << "read_log_and_missing checking for missing items over interval ("
+			   << info.last_complete
+			   << "," << info.last_update << "]" << dendl;
+
+	set<hobject_t, hobject_t::BitwiseComparator> did;
+	set<hobject_t, hobject_t::BitwiseComparator> checked;
+	for (list<pg_log_entry_t>::reverse_iterator i = log.log.rbegin();
+	     i != log.log.rend();
+	     ++i) {
+	  if (!debug_verify_stored_missing && i->version <= info.last_complete) break;
+	  if (cmp(i->soid, info.last_backfill, info.last_backfill_bitwise) > 0)
+	    continue;
+	  if (i->is_error())
+	    continue;
+	  if (did.count(i->soid)) continue;
+	  did.insert(i->soid);
+
+	  if (i->is_delete()) continue;
+
+	  bufferlist bv;
+	  int r = store->getattr(
+	    pg_coll,
+	    ghobject_t(i->soid, ghobject_t::NO_GEN, info.pgid.shard),
+	    OI_ATTR,
+	    bv);
+	  if (r >= 0) {
+	    object_info_t oi(bv);
+	    if (oi.version < i->version) {
+	      ldpp_dout(dpp, 15) << "read_log_and_missing  missing " << *i
+				 << " (have " << oi.version << ")" << dendl;
+	      if (debug_verify_stored_missing) {
+		auto miter = missing.get_items().find(i->soid);
+		assert(miter != missing.get_items().end());
+		assert(miter->second.need == i->version);
+		assert(miter->second.have == oi.version);
+		checked.insert(i->soid);
+	      } else {
+		missing.add(i->soid, i->version, oi.version);
+	      }
+	    }
+	  } else {
+	    ldpp_dout(dpp, 15) << "read_log_and_missing  missing " << *i << dendl;
+	    if (debug_verify_stored_missing) {
+	      auto miter = missing.get_items().find(i->soid);
+	      assert(miter != missing.get_items().end());
+	      assert(miter->second.need == i->version);
+	      assert(miter->second.have == eversion_t());
+	      checked.insert(i->soid);
+	    } else {
+	      missing.add(i->soid, i->version, eversion_t());
+	    }
+	  }
+	}
+	if (debug_verify_stored_missing) {
+	  for (auto &&i: missing.get_items()) {
+	    if (checked.count(i.first))
+	      continue;
+	    if (i.second.need > log.tail ||
+	      cmp(i.first, info.last_backfill, info.last_backfill_bitwise) > 0) {
+	      derr << __func__ << ": invalid missing set entry found "
+		   << i.first
+		   << dendl;
+	      assert(0 == "invalid missing set entry found");
+	    }
+	    bufferlist bv;
+	    int r = store->getattr(
+	      pg_coll,
+	      ghobject_t(i.first, ghobject_t::NO_GEN, info.pgid.shard),
+	      OI_ATTR,
+	      bv);
+	    if (r >= 0) {
+	      object_info_t oi(bv);
+	      assert(oi.version == i.second.have);
+	    } else {
+	      assert(eversion_t() == i.second.have);
+	    }
+	  }
+	} else {
+	  assert(has_divergent_priors);
+	  for (map<eversion_t, hobject_t>::reverse_iterator i =
+		 divergent_priors.rbegin();
+	       i != divergent_priors.rend();
+	       ++i) {
+	    if (i->first <= info.last_complete) break;
+	    if (cmp(i->second, info.last_backfill, info.last_backfill_bitwise) > 0)
+	      continue;
+	    if (did.count(i->second)) continue;
+	    did.insert(i->second);
+	    bufferlist bv;
+	    int r = store->getattr(
+	      pg_coll,
+	      ghobject_t(i->second, ghobject_t::NO_GEN, info.pgid.shard),
+	      OI_ATTR,
+	      bv);
+	    if (r >= 0) {
+	      object_info_t oi(bv);
+	      /**
+		 * 1) we see this entry in the divergent priors mapping
+		 * 2) we didn't see an entry for this object in the log
+		 *
+		 * From 1 & 2 we know that either the object does not exist
+		 * or it is at the version specified in the divergent_priors
+		 * map since the object would have been deleted atomically
+		 * with the addition of the divergent_priors entry, an older
+		 * version would not have been recovered, and a newer version
+		 * would show up in the log above.
+		 */
+	      assert(oi.version == i->first);
+	    } else {
+	      ldpp_dout(dpp, 15) << "read_log_and_missing  missing " << *i << dendl;
+	      missing.add(i->second, i->first, eversion_t());
+	    }
+	  }
+	}
+	if (clear_divergent_priors)
+	  (*clear_divergent_priors) = true;
+      }
+    }
+
+    if (!has_divergent_priors) {
+      if (clear_divergent_priors)
+	(*clear_divergent_priors) = false;
+      missing.flush();
+    }
+    ldpp_dout(dpp, 10) << "read_log_and_missing done" << dendl;
+  }
 };
   
 #endif // CEPH_PG_LOG_H
