@@ -17,29 +17,14 @@
 #ifndef CEPH_REPLICATEDPG_H
 #define CEPH_REPLICATEDPG_H
 
-#include <boost/optional/optional_io.hpp>
 #include <boost/tuple/tuple.hpp>
-
 #include "include/assert.h" 
-#include "include/unordered_map.h"
-#include "common/cmdparse.h"
-
-#include "HitSet.h"
-#include "OSD.h"
 #include "PG.h"
 #include "Watch.h"
-#include "OpRequest.h"
 #include "TierAgentState.h"
-
-#include "messages/MOSDOp.h"
 #include "messages/MOSDOpReply.h"
-#include "messages/MOSDSubOp.h"
-
 #include "common/sharedptr_registry.hpp"
-
-#include "PGBackend.h"
 #include "ReplicatedBackend.h"
-#include "ECBackend.h"
 
 class MOSDSubOpReply;
 
@@ -48,6 +33,13 @@ class PromoteCallback;
 
 class ReplicatedPG;
 class PGLSFilter;
+class HitSet;
+struct TierAgentState;
+class MOSDOp;
+class MOSDOpReply;
+class MOSDSubOp;
+class OSDService;
+
 void intrusive_ptr_add_ref(ReplicatedPG *pg);
 void intrusive_ptr_release(ReplicatedPG *pg);
 uint64_t get_with_id(ReplicatedPG *pg);
@@ -180,21 +172,6 @@ public:
    * can associate itself with the correct copy operation.
    */
   typedef boost::tuple<int, CopyResults*> CopyCallbackResults;
-  class CopyCallback : public GenContext<CopyCallbackResults> {
-  protected:
-    CopyCallback() {}
-    /**
-     * results.get<0>() is the return code: 0 for success; -ECANCELLED if
-     * the operation was cancelled by the local OSD; -errno for other issues.
-     * results.get<1>() is a pointer to a CopyResults object, which you are
-     * responsible for deleting.
-     */
-    virtual void finish(CopyCallbackResults results_) = 0;
-
-  public:
-    /// Provide the final size of the copied object to the CopyCallback
-    virtual ~CopyCallback() {}
-  };
 
   friend class CopyFromCallback;
   friend class PromoteCallback;
@@ -257,7 +234,7 @@ public:
   typedef ceph::shared_ptr<FlushOp> FlushOpRef;
 
   boost::scoped_ptr<PGBackend> pgbackend;
-  PGBackend *get_pgbackend() {
+  PGBackend *get_pgbackend() override {
     return pgbackend.get();
   }
 
@@ -267,121 +244,88 @@ public:
     const ObjectRecoveryInfo &recovery_info,
     ObjectContextRef obc,
     ObjectStore::Transaction *t
-    );
+    ) override;
   void on_peer_recover(
     pg_shard_t peer,
     const hobject_t &oid,
     const ObjectRecoveryInfo &recovery_info,
     const object_stat_sum_t &stat
-    );
+    ) override;
   void begin_peer_recover(
     pg_shard_t peer,
-    const hobject_t oid);
+    const hobject_t oid) override;
   void on_global_recover(
     const hobject_t &oid,
-    const object_stat_sum_t &stat_diff);
-  void failed_push(const list<pg_shard_t> &from, const hobject_t &soid) override;
-  void cancel_pull(const hobject_t &soid);
-  void backfill_add_missing(const hobject_t &oid, eversion_t v) override;
+    const object_stat_sum_t &stat_diff) override;
+  void failed_push(pg_shard_t from, const hobject_t &soid) override;
+  void cancel_pull(const hobject_t &soid) override;
+
+  template<class T> class BlessedGenContext;
+  class BlessedContext;
+  Context *bless_context(Context *c) override;
 
-  template <typename T>
-  class BlessedGenContext : public GenContext<T> {
-    ReplicatedPGRef pg;
-    GenContext<T> *c;
-    epoch_t e;
-  public:
-    BlessedGenContext(ReplicatedPG *pg, GenContext<T> *c, epoch_t e)
-      : pg(pg), c(c), e(e) {}
-    void finish(T t) {
-      pg->lock();
-      if (pg->pg_has_reset_since(e))
-	delete c;
-      else
-	c->complete(t);
-      pg->unlock();
-    }
-  };
-  class BlessedContext : public Context {
-    ReplicatedPGRef pg;
-    Context *c;
-    epoch_t e;
-  public:
-    BlessedContext(ReplicatedPG *pg, Context *c, epoch_t e)
-      : pg(pg), c(c), e(e) {}
-    void finish(int r) {
-      pg->lock();
-      if (pg->pg_has_reset_since(e))
-	delete c;
-      else
-	c->complete(r);
-      pg->unlock();
-    }
-  };
-  Context *bless_context(Context *c) {
-    return new BlessedContext(this, c, get_osdmap()->get_epoch());
-  }
   GenContext<ThreadPool::TPHandle&> *bless_gencontext(
-    GenContext<ThreadPool::TPHandle&> *c) {
-    return new BlessedGenContext<ThreadPool::TPHandle&>(
-      this, c, get_osdmap()->get_epoch());
-  }
+    GenContext<ThreadPool::TPHandle&> *c) override;
     
-  void send_message(int to_osd, Message *m) {
+  void send_message(int to_osd, Message *m) override {
     osd->send_message_osd_cluster(to_osd, m, get_osdmap()->get_epoch());
   }
-  void queue_transaction(ObjectStore::Transaction&& t, OpRequestRef op) {
+  void queue_transaction(ObjectStore::Transaction&& t,
+			 OpRequestRef op) override {
     osd->store->queue_transaction(osr.get(), std::move(t), 0, 0, 0, op);
   }
-  void queue_transactions(vector<ObjectStore::Transaction>& tls, OpRequestRef op) {
+  void queue_transactions(vector<ObjectStore::Transaction>& tls,
+			  OpRequestRef op) override {
     osd->store->queue_transactions(osr.get(), tls, 0, 0, 0, op, NULL);
   }
-  epoch_t get_epoch() const {
+  epoch_t get_epoch() const override {
     return get_osdmap()->get_epoch();
   }
-  const set<pg_shard_t> &get_actingbackfill_shards() const {
+  const set<pg_shard_t> &get_actingbackfill_shards() const override {
     return actingbackfill;
   }
-  const set<pg_shard_t> &get_acting_shards() const {
+  const set<pg_shard_t> &get_acting_shards() const override {
     return actingset;
   }
-  const set<pg_shard_t> &get_backfill_shards() const {
+  const set<pg_shard_t> &get_backfill_shards() const override {
     return backfill_targets;
   }
 
-  std::string gen_dbg_prefix() const { return gen_prefix(); }
+  std::string gen_dbg_prefix() const override { return gen_prefix(); }
   
-  const map<hobject_t, set<pg_shard_t>, hobject_t::BitwiseComparator> &get_missing_loc_shards() const {
+  const map<hobject_t, set<pg_shard_t>, hobject_t::BitwiseComparator>
+    &get_missing_loc_shards() const override {
     return missing_loc.get_missing_locs();
   }
-  const map<pg_shard_t, pg_missing_t> &get_shard_missing() const {
+  const map<pg_shard_t, pg_missing_t> &get_shard_missing() const override {
     return peer_missing;
   }
   using PGBackend::Listener::get_shard_missing;
-  const map<pg_shard_t, pg_info_t> &get_shard_info() const {
+  const map<pg_shard_t, pg_info_t> &get_shard_info() const override {
     return peer_info;
   }
   using PGBackend::Listener::get_shard_info;  
-  const pg_missing_t &get_local_missing() const {
+  const pg_missing_tracker_t &get_local_missing() const override {
     return pg_log.get_missing();
   }
-  const PGLog &get_log() const {
+  const PGLog &get_log() const override {
     return pg_log;
   }
-  bool pgb_is_primary() const {
+  bool pgb_is_primary() const override {
     return is_primary();
   }
-  OSDMapRef pgb_get_osdmap() const {
+  OSDMapRef pgb_get_osdmap() const override {
     return get_osdmap();
   }
-  const pg_info_t &get_info() const {
+  const pg_info_t &get_info() const override {
     return info;
   }
-  const pg_pool_t &get_pool() const {
+  const pg_pool_t &get_pool() const override {
     return pool.info;
   }
   ObjectContextRef get_obc(
     const hobject_t &hoid,
-    map<string, bufferlist> &attrs) {
+    map<string, bufferlist> &attrs) override {
     return get_object_context(hoid, true, &attrs);
   }
 
@@ -391,7 +335,7 @@ public:
     const eversion_t &trim_to,
     const eversion_t &trim_rollback_to,
     bool transaction_applied,
-    ObjectStore::Transaction &t) {
+    ObjectStore::Transaction &t) override {
     if (hset_history) {
       info.hit_set = *hset_history;
       dirty_info = true;
@@ -399,28 +343,13 @@ public:
     append_log(logv, trim_to, trim_rollback_to, t, transaction_applied);
   }
 
-  struct C_OSD_OnApplied : Context {
-    ReplicatedPGRef pg;
-    epoch_t epoch;
-    eversion_t v;
-    C_OSD_OnApplied(
-      ReplicatedPGRef pg,
-      epoch_t epoch,
-      eversion_t v)
-      : pg(pg), epoch(epoch), v(v) {}
-    void finish(int) override {
-      pg->lock();
-      if (!pg->pg_has_reset_since(epoch))
-	pg->op_applied(v);
-      pg->unlock();
-    }
-  };
+  struct C_OSD_OnApplied;
   void op_applied(
-    const eversion_t &applied_version);
+    const eversion_t &applied_version) override;
 
   bool should_send_op(
     pg_shard_t peer,
-    const hobject_t &hoid) {
+    const hobject_t &hoid) override {
     if (peer == get_primary())
       return true;
     assert(peer_info.count(peer));
@@ -435,55 +364,55 @@ public:
   
   void update_peer_last_complete_ondisk(
     pg_shard_t fromosd,
-    eversion_t lcod) {
+    eversion_t lcod) override {
     peer_last_complete_ondisk[fromosd] = lcod;
   }
 
   void update_last_complete_ondisk(
-    eversion_t lcod) {
+    eversion_t lcod) override {
     last_complete_ondisk = lcod;
   }
 
   void update_stats(
-    const pg_stat_t &stat) {
+    const pg_stat_t &stat) override {
     info.stats = stat;
   }
 
   void schedule_recovery_work(
-    GenContext<ThreadPool::TPHandle&> *c);
+    GenContext<ThreadPool::TPHandle&> *c) override;
 
-  pg_shard_t whoami_shard() const {
+  pg_shard_t whoami_shard() const override {
     return pg_whoami;
   }
-  spg_t primary_spg_t() const {
+  spg_t primary_spg_t() const override {
     return spg_t(info.pgid.pgid, primary.shard);
   }
-  pg_shard_t primary_shard() const {
+  pg_shard_t primary_shard() const override {
     return primary;
   }
-  uint64_t min_peer_features() const {
+  uint64_t min_peer_features() const override {
     return get_min_peer_features();
   }
-  bool sort_bitwise() const {
+  bool sort_bitwise() const override {
     return get_sort_bitwise();
   }
 
   void send_message_osd_cluster(
-    int peer, Message *m, epoch_t from_epoch);
+    int peer, Message *m, epoch_t from_epoch) override;
   void send_message_osd_cluster(
-    Message *m, Connection *con);
+    Message *m, Connection *con) override;
   void send_message_osd_cluster(
-    Message *m, const ConnectionRef& con);
-  ConnectionRef get_con_osd_cluster(int peer, epoch_t from_epoch);
-  entity_name_t get_cluster_msgr_name() {
+    Message *m, const ConnectionRef& con) override;
+  ConnectionRef get_con_osd_cluster(int peer, epoch_t from_epoch) override;
+  entity_name_t get_cluster_msgr_name() override {
     return osd->get_cluster_msgr_name();
   }
 
-  PerfCounters *get_logger();
+  PerfCounters *get_logger() override;
 
-  ceph_tid_t get_tid() { return osd->get_tid(); }
+  ceph_tid_t get_tid() override { return osd->get_tid(); }
 
-  LogClientTemp clog_error() { return osd->clog->error(); }
+  LogClientTemp clog_error() override { return osd->clog->error(); }
 
   struct watch_disconnect_t {
     uint64_t cookie;
@@ -518,6 +447,7 @@ public:
     bool cache_evict;     ///< true if this is a cache eviction
     bool ignore_cache;    ///< true if IGNORE_CACHE flag is set
     bool ignore_log_op_stats;  // don't log op stats
+    bool update_log_only; ///< this is a write that returned an error - just record in pg log for dup detection
 
     // side effects
     list<pair<watch_info_t,bool> > watch_connects; ///< new watch + will_ping flag
@@ -580,19 +510,19 @@ public:
     list<std::function<void()>> on_success;
     template <typename F>
     void register_on_finish(F &&f) {
-      on_finish.emplace_back(std::move(f));
+      on_finish.emplace_back(std::forward<F>(f));
     }
     template <typename F>
     void register_on_success(F &&f) {
-      on_success.emplace_back(std::move(f));
+      on_success.emplace_back(std::forward<F>(f));
     }
     template <typename F>
     void register_on_applied(F &&f) {
-      on_applied.emplace_back(std::move(f));
+      on_applied.emplace_back(std::forward<F>(f));
     }
     template <typename F>
     void register_on_commit(F &&f) {
-      on_committed.emplace_back(std::move(f));
+      on_committed.emplace_back(std::forward<F>(f));
     }
 
     bool sent_ack;
@@ -625,7 +555,7 @@ public:
     list<pair<boost::tuple<uint64_t, uint64_t, unsigned>,
 	      pair<bufferlist*, Context*> > > pending_async_reads;
     int async_read_result;
-    unsigned inflightreads;
+    int inflightreads;
     friend struct OnReadComplete;
     void start_async_reads(ReplicatedPG *pg);
     void finish_read(ReplicatedPG *pg);
@@ -649,7 +579,7 @@ public:
       snapset(0),
       new_obs(obs->oi, obs->exists),
       modify(false), user_modify(false), undirty(false), cache_evict(false),
-      ignore_cache(false), ignore_log_op_stats(false),
+      ignore_cache(false), ignore_log_op_stats(false), update_log_only(false),
       bytes_written(0), bytes_read(0), user_at_version(0),
       current_osd_subop_num(0),
       obc(obc),
@@ -670,7 +600,7 @@ public:
               vector<OSDOp>& _ops, ReplicatedPG *_pg) :
       op(_op), reqid(_reqid), ops(_ops), obs(NULL), snapset(0),
       modify(false), user_modify(false), undirty(false), cache_evict(false),
-      ignore_cache(false), ignore_log_op_stats(false),
+      ignore_cache(false), ignore_log_op_stats(false), update_log_only(false),
       bytes_written(0), bytes_read(0), user_at_version(0),
       current_osd_subop_num(0),
       data_off(0), reply(NULL), pg(_pg),
@@ -699,6 +629,12 @@ public:
 	delete i->second.second;
       }
     }
+    uint64_t get_features() {
+      if (op && op->get_req()) {
+        return op->get_req()->get_connection()->get_features();
+      }
+      return -1ull;
+    }
   };
   using OpContextUPtr = std::unique_ptr<OpContext>;
   friend struct OpContext;
@@ -859,7 +795,7 @@ protected:
       &requeue_recovery,
       &requeue_snaptrim);
     if (requeue_recovery)
-      osd->recovery_wq.queue(this);
+      queue_recovery();
     if (requeue_snaptrim)
       queue_snap_trim();
 
@@ -905,7 +841,9 @@ protected:
 
   /**
    * Merge entries atomically into all actingbackfill osds
-   * adjusting missing and recovery state as necessary
+   * adjusting missing and recovery state as necessary.
+   *
+   * Also used to store error log entries for dup detection.
    */
   void submit_log_entries(
     const list<pg_log_entry_t> &entries,
@@ -915,6 +853,7 @@ protected:
     boost::intrusive_ptr<RepGather> repop;
     set<pg_shard_t> waiting_on;
   };
+  void cancel_log_updates();
   map<ceph_tid_t, LogUpdateCtx> log_entry_update_waiting_on;
 
 
@@ -941,16 +880,12 @@ protected:
   // agent
   boost::scoped_ptr<TierAgentState> agent_state;
 
-  friend struct C_AgentFlushStartStop;
-  friend struct C_AgentEvictStartStop;
-  friend struct C_HitSetFlushing;
-
   void agent_setup();       ///< initialize agent state
-  bool agent_work(int max) ///< entry point to do some agent work
+  bool agent_work(int max) override ///< entry point to do some agent work
   {
     return agent_work(max, max);
   }
-  bool agent_work(int max, int agent_flush_quota);
+  bool agent_work(int max, int agent_flush_quota) override;
   bool agent_maybe_flush(ObjectContextRef& obc);  ///< maybe flush
   bool agent_maybe_evict(ObjectContextRef& obc, bool after_flush);  ///< maybe evict
 
@@ -963,15 +898,15 @@ protected:
   void agent_estimate_temp(const hobject_t& oid, int *temperature);
 
   /// stop the agent
-  void agent_stop();
-  void agent_delay();
+  void agent_stop() override;
+  void agent_delay() override;
 
   /// clear agent state
-  void agent_clear();
+  void agent_clear() override;
 
   /// choose (new) agent mode(s), returns true if op is requeued
   bool agent_choose_mode(bool restart = false, OpRequestRef op = OpRequestRef());
-  void agent_choose_mode_restart();
+  void agent_choose_mode_restart() override;
 
   /// true if we can send an ondisk/commit for v
   bool already_complete(eversion_t v) {
@@ -1015,8 +950,8 @@ protected:
 
   void populate_obc_watchers(ObjectContextRef obc);
   void check_blacklisted_obc_watchers(ObjectContextRef obc);
-  void check_blacklisted_watchers();
-  void get_watchers(list<obj_watch_item_t> &pg_watchers);
+  void check_blacklisted_watchers() override;
+  void get_watchers(list<obj_watch_item_t> &pg_watchers) override;
   void get_obc_watchers(ObjectContextRef obc, list<obj_watch_item_t> &pg_watchers);
 public:
   void handle_watch_timeout(WatchRef watch);
@@ -1031,15 +966,7 @@ protected:
 
   void context_registry_on_change();
   void object_context_destructor_callback(ObjectContext *obc);
-  struct C_PG_ObjectContext : public Context {
-    ReplicatedPGRef pg;
-    ObjectContext *obc;
-    C_PG_ObjectContext(ReplicatedPG *p, ObjectContext *o) :
-      pg(p), obc(o) {}
-    void finish(int r) {
-      pg->object_context_destructor_callback(obc);
-    }
-  };
+  class C_PG_ObjectContext;
 
   int find_object_context(const hobject_t& oid,
 			  ObjectContextRef *pobc,
@@ -1051,7 +978,6 @@ protected:
 
   void get_src_oloc(const object_t& oid, const object_locator_t& oloc, object_locator_t& src_oloc);
 
-  SnapSetContext *create_snapset_context(const hobject_t& oid);
   SnapSetContext *get_snapset_context(
     const hobject_t& oid,
     bool can_create,
@@ -1091,7 +1017,7 @@ protected:
   set<hobject_t, hobject_t::Comparator> backfills_in_flight;
   map<hobject_t, pg_stat_t, hobject_t::Comparator> pending_backfill_updates;
 
-  void dump_recovery_info(Formatter *f) const {
+  void dump_recovery_info(Formatter *f) const override {
     f->open_array_section("backfill_targets");
     for (set<pg_shard_t>::const_iterator p = backfill_targets.begin();
         p != backfill_targets.end(); ++p)
@@ -1155,7 +1081,7 @@ protected:
   void finish_degraded_object(const hobject_t& oid);
 
   // Cancels/resets pulls from peer
-  void check_recovery_sources(const OSDMapRef map);
+  void check_recovery_sources(const OSDMapRef& map) override ;
 
   int recover_missing(
     const hobject_t& oid,
@@ -1194,7 +1120,6 @@ protected:
     BLOCKED_PROMOTE,
     HANDLED_PROXY,
     HANDLED_REDIRECT,
-    REPLIED_WITH_EAGAIN,
   };
   cache_result_t maybe_handle_cache_detail(OpRequestRef op,
 					   bool write_ordered,
@@ -1257,23 +1182,24 @@ protected:
   void complete_read_ctx(int result, OpContext *ctx);
   
   // pg on-disk content
-  void check_local();
+  void check_local() override;
 
-  void _clear_recovery_state();
+  void _clear_recovery_state() override;
 
   bool start_recovery_ops(
-    int max, ThreadPool::TPHandle &handle, int *started);
+    uint64_t max,
+    ThreadPool::TPHandle &handle, uint64_t *started) override;
 
-  int recover_primary(int max, ThreadPool::TPHandle &handle);
-  int recover_replicas(int max, ThreadPool::TPHandle &handle);
+  uint64_t recover_primary(uint64_t max, ThreadPool::TPHandle &handle);
+  uint64_t recover_replicas(uint64_t max, ThreadPool::TPHandle &handle);
   hobject_t earliest_peer_backfill() const;
   bool all_peer_done() const;
   /**
    * @param work_started will be set to true if recover_backfill got anywhere
    * @returns the number of operations started
    */
-  int recover_backfill(int max, ThreadPool::TPHandle &handle,
-                       bool *work_started);
+  uint64_t recover_backfill(uint64_t max, ThreadPool::TPHandle &handle,
+			    bool *work_started);
 
   /**
    * scan a (hash) range of objects in the current pg
@@ -1301,58 +1227,10 @@ protected:
   void send_remove_op(const hobject_t& oid, eversion_t v, pg_shard_t peer);
 
 
-  struct C_OSD_OndiskWriteUnlock : public Context {
-    ObjectContextRef obc, obc2, obc3;
-    C_OSD_OndiskWriteUnlock(
-      ObjectContextRef o,
-      ObjectContextRef o2 = ObjectContextRef(),
-      ObjectContextRef o3 = ObjectContextRef()) : obc(o), obc2(o2), obc3(o3) {}
-    void finish(int r) {
-      obc->ondisk_write_unlock();
-      if (obc2)
-	obc2->ondisk_write_unlock();
-      if (obc3)
-	obc3->ondisk_write_unlock();
-    }
-  };
-  struct C_OSD_OndiskWriteUnlockList : public Context {
-    list<ObjectContextRef> *pls;
-    explicit C_OSD_OndiskWriteUnlockList(list<ObjectContextRef> *l) : pls(l) {}
-    void finish(int r) {
-      for (list<ObjectContextRef>::iterator p = pls->begin(); p != pls->end(); ++p)
-	(*p)->ondisk_write_unlock();
-    }
-  };
-  struct C_OSD_AppliedRecoveredObject : public Context {
-    ReplicatedPGRef pg;
-    ObjectContextRef obc;
-    C_OSD_AppliedRecoveredObject(ReplicatedPG *p, ObjectContextRef o) :
-      pg(p), obc(o) {}
-    void finish(int r) {
-      pg->_applied_recovered_object(obc);
-    }
-  };
-  struct C_OSD_CommittedPushedObject : public Context {
-    ReplicatedPGRef pg;
-    epoch_t epoch;
-    eversion_t last_complete;
-    C_OSD_CommittedPushedObject(
-      ReplicatedPG *p, epoch_t epoch, eversion_t lc) :
-      pg(p), epoch(epoch), last_complete(lc) {
-    }
-    void finish(int r) {
-      pg->_committed_pushed_object(epoch, last_complete);
-    }
-  };
-  struct C_OSD_AppliedRecoveredObjectReplica : public Context {
-    ReplicatedPGRef pg;
-    explicit C_OSD_AppliedRecoveredObjectReplica(ReplicatedPG *p) :
-      pg(p) {}
-    void finish(int r) {
-      pg->_applied_recovered_object_replica();
-    }
-  };
-
+  class C_OSD_OndiskWriteUnlock;
+  class C_OSD_AppliedRecoveredObject;
+  class C_OSD_CommittedPushedObject;
+  class C_OSD_AppliedRecoveredObjectReplica;
   void sub_op_remove(OpRequestRef op);
 
   void _applied_recovered_object(ObjectContextRef obc);
@@ -1402,8 +1280,8 @@ protected:
                                       PGBackend::PGTransaction *t);
   void finish_copyfrom(OpContext *ctx);
   void finish_promote(int r, CopyResults *results, ObjectContextRef obc);
-  void cancel_copy(CopyOpRef cop, bool requeue, vector<ceph_tid_t> *tids);
-  void cancel_copy_ops(bool requeue, vector<ceph_tid_t> *tids);
+  void cancel_copy(CopyOpRef cop, bool requeue);
+  void cancel_copy_ops(bool requeue);
 
   friend struct C_Copyfrom;
 
@@ -1417,8 +1295,8 @@ protected:
     boost::optional<std::function<void()>> &&on_flush);
   void finish_flush(hobject_t oid, ceph_tid_t tid, int r);
   int try_flush_mark_clean(FlushOpRef fop);
-  void cancel_flush(FlushOpRef fop, bool requeue, vector<ceph_tid_t> *tids);
-  void cancel_flush_ops(bool requeue, vector<ceph_tid_t> *tids);
+  void cancel_flush(FlushOpRef fop, bool requeue);
+  void cancel_flush_ops(bool requeue);
 
   /// @return false if clone is has been evicted
   bool is_present_clone(hobject_t coid);
@@ -1427,35 +1305,38 @@ protected:
 
   // -- scrub --
   virtual bool _range_available_for_scrub(
-    const hobject_t &begin, const hobject_t &end);
+    const hobject_t &begin, const hobject_t &end) override;
   virtual void _scrub(
     ScrubMap &map,
-    const std::map<hobject_t, pair<uint32_t, uint32_t>, hobject_t::BitwiseComparator> &missing_digest);
-  virtual void _scrub_clear_state();
-  virtual void _scrub_finish();
+    const std::map<hobject_t, pair<uint32_t, uint32_t>,
+    hobject_t::BitwiseComparator> &missing_digest) override;
+  virtual void _scrub_clear_state() override;
+  virtual void _scrub_finish() override;
   object_stat_collection_t scrub_cstat;
-  friend class C_ScrubDigestUpdated;
 
-  virtual void _split_into(pg_t child_pgid, PG *child, unsigned split_bits);
+  virtual void _split_into(pg_t child_pgid, PG *child,
+			   unsigned split_bits) override;
   void apply_and_flush_repops(bool requeue);
 
-  void calc_trim_to();
+  void calc_trim_to() override;
   int do_xattr_cmp_u64(int op, __u64 v1, bufferlist& xattr);
   int do_xattr_cmp_str(int op, string& v1s, bufferlist& xattr);
 
+  int do_writesame(OpContext *ctx, OSDOp& osd_op);
+
   bool pgls_filter(PGLSFilter *filter, hobject_t& sobj, bufferlist& outdata);
   int get_pgls_filter(bufferlist::iterator& iter, PGLSFilter **pfilter);
 
   map<hobject_t, list<OpRequestRef>, hobject_t::BitwiseComparator> in_progress_proxy_ops;
   void kick_proxy_ops_blocked(hobject_t& soid);
-  void cancel_proxy_ops(bool requeue, vector<ceph_tid_t> *tids);
+  void cancel_proxy_ops(bool requeue);
 
   // -- proxyread --
   map<ceph_tid_t, ProxyReadOpRef> proxyread_ops;
 
   void do_proxy_read(OpRequestRef op);
   void finish_proxy_read(hobject_t oid, ceph_tid_t tid, int r);
-  void cancel_proxy_read(ProxyReadOpRef prdop, vector<ceph_tid_t> *tids);
+  void cancel_proxy_read(ProxyReadOpRef prdop);
 
   friend struct C_ProxyRead;
 
@@ -1464,9 +1345,8 @@ protected:
 
   void do_proxy_write(OpRequestRef op, const hobject_t& missing_oid);
   void finish_proxy_write(hobject_t oid, ceph_tid_t tid, int r);
-  void cancel_proxy_write(ProxyWriteOpRef pwop, vector<ceph_tid_t> *tids);
+  void cancel_proxy_write(ProxyWriteOpRef pwop);
 
-  friend struct C_ProxyWrite_Apply;
   friend struct C_ProxyWrite_Commit;
 
 public:
@@ -1484,19 +1364,21 @@ public:
 
   void do_request(
     OpRequestRef& op,
-    ThreadPool::TPHandle &handle);
-  void do_op(OpRequestRef& op);
+    ThreadPool::TPHandle &handle) override;
+  void do_op(OpRequestRef& op) override;
+  void record_write_error(OpRequestRef op, const hobject_t &soid,
+			  MOSDOpReply *orig_reply, int r);
   bool pg_op_must_wait(MOSDOp *op);
   void do_pg_op(OpRequestRef op);
-  void do_sub_op(OpRequestRef op);
-  void do_sub_op_reply(OpRequestRef op);
+  void do_sub_op(OpRequestRef op) override;
+  void do_sub_op_reply(OpRequestRef op) override;
   void do_scan(
     OpRequestRef op,
-    ThreadPool::TPHandle &handle);
-  void do_backfill(OpRequestRef op);
+    ThreadPool::TPHandle &handle) override;
+  void do_backfill(OpRequestRef op) override;
 
   OpContextUPtr trim_object(const hobject_t &coid);
-  void snap_trimmer(epoch_t e);
+  void snap_trimmer(epoch_t e) override;
   int do_osd_ops(OpContext *ctx, vector<OSDOp>& ops);
 
   int _get_tmap(OpContext *ctx, bufferlist *header, bufferlist *vals);
@@ -1513,7 +1395,8 @@ private:
   uint64_t temp_seq; ///< last id for naming temp objects
   hobject_t generate_temp_object();  ///< generate a new temp object name
   /// generate a new temp object name (for recovery)
-  hobject_t get_temp_recovery_object(eversion_t version, snapid_t snap);
+  hobject_t get_temp_recovery_object(eversion_t version,
+				     snapid_t snap) override;
   int get_recovery_op_priority() const {
       int pri = 0;
       pool.info.opts.get(pool_opts_t::RECOVERY_OP_PRIORITY, &pri);
@@ -1545,7 +1428,7 @@ public:
     int split_bits,
     int seed,
     const pg_pool_t *pool,
-    ObjectStore::Transaction *t) {
+    ObjectStore::Transaction *t) override {
     coll_t target = coll_t(child);
     PG::_create(*t, child, split_bits);
     t->split_collection(
@@ -1557,20 +1440,12 @@ public:
   }
 private:
   struct NotTrimming;
-  struct WaitReservation;
   struct SnapTrim : boost::statechart::event< SnapTrim > {
     SnapTrim() : boost::statechart::event < SnapTrim >() {}
   };
   struct Reset : boost::statechart::event< Reset > {
     Reset() : boost::statechart::event< Reset >() {}
   };
-  struct SnapTrimReserved : boost::statechart::event< SnapTrimReserved > {
-    SnapTrimReserved() : boost::statechart::event< SnapTrimReserved >() {}
-  };
-  struct SnapTrimTimerReady : boost::statechart::event< SnapTrimTimerReady > {
-    SnapTrimTimerReady() : boost::statechart::event< SnapTrimTimerReady >() {}
-  };
-
   struct SnapTrimmer : public boost::statechart::state_machine< SnapTrimmer, NotTrimming > {
     ReplicatedPG *pg;
     set<hobject_t, hobject_t::BitwiseComparator> in_flight;
@@ -1583,136 +1458,18 @@ private:
   } snap_trimmer_machine;
 
   /* SnapTrimmerStates */
-  struct Trimming : boost::statechart::state< Trimming,
-					      SnapTrimmer,
-					      WaitReservation >,
-			   NamedState {
-    typedef boost::mpl::list <
-      boost::statechart::custom_reaction< SnapTrim >,
-      boost::statechart::transition< Reset, NotTrimming >
-      > reactions;
-    explicit Trimming(my_context ctx);
-    void exit();
-    boost::statechart::result react(const SnapTrim&) { return discard_event(); }
-  };
-
-  struct TrimmingObjects : boost::statechart::state<TrimmingObjects, Trimming>, NamedState {
+  struct TrimmingObjects : boost::statechart::state< TrimmingObjects, SnapTrimmer >, NamedState {
     typedef boost::mpl::list <
       boost::statechart::custom_reaction< SnapTrim >,
       boost::statechart::transition< Reset, NotTrimming >
       > reactions;
     hobject_t pos;
     explicit TrimmingObjects(my_context ctx);
-    void exit() { context< SnapTrimmer >().log_exit(state_name, enter_time); }
+    void exit();
     boost::statechart::result react(const SnapTrim&);
   };
 
-  struct WaitReservation : boost::statechart::state< WaitReservation, Trimming >, NamedState {
-    /* WaitReservation is a sub-state of trimming simply so that exiting Trimming
-     * always cancels the reservation */
-    typedef boost::mpl::list <
-      boost::statechart::custom_reaction< SnapTrimReserved >
-      > reactions;
-    struct ReservationCB : public Context {
-      ReplicatedPGRef pg;
-      bool canceled;
-      ReservationCB(ReplicatedPG *pg) : pg(pg), canceled(false) {}
-      void finish(int) override {
-        pg->lock();
-        if (!canceled)
-          pg->snap_trimmer_machine.process_event(SnapTrimReserved());
-        pg->unlock();
-      }
-      void cancel() {
-        assert(pg->is_locked());
-        assert(!canceled);
-        canceled = true;
-      }
-    };
-    ReservationCB *pending = nullptr;
-
-    explicit WaitReservation(my_context ctx)
-      : my_base(ctx),
-        NamedState(context< SnapTrimmer >().pg->cct, "Trimming/WaitReservation") {
-      context< SnapTrimmer >().log_enter(state_name);
-      auto *pg = context< SnapTrimmer >().pg;
-      pending = new ReservationCB(pg);
-      pg->osd->snap_reserver.request_reservation(pg->get_pgid(), pending, 0);
-      pg->state_set(PG_STATE_SNAPTRIM_WAIT);
-      pg->publish_stats_to_osd();
-    }
-    boost::statechart::result react(const SnapTrimReserved&);
-    void exit() {
-      context< SnapTrimmer >().log_exit(state_name, enter_time);
-      if (pending)
-        pending->cancel();
-      pending = nullptr;
-      auto *pg = context< SnapTrimmer >().pg;
-      pg->state_clear(PG_STATE_SNAPTRIM_WAIT);
-      pg->publish_stats_to_osd();
-    }
-    boost::statechart::result react(const SnapTrim&) {
-      return discard_event();
-    }
-  };
-
-  struct WaitTrimTimer : boost::statechart::state< WaitTrimTimer, Trimming >, NamedState {
-    typedef boost::mpl::list <
-      boost::statechart::custom_reaction< SnapTrimTimerReady >
-      > reactions;
-    Context *wakeup = nullptr;
-    bool slept = false;
-    explicit WaitTrimTimer(my_context ctx)
-      : my_base(ctx),
-	NamedState(context< SnapTrimmer >().pg->cct, "Trimming/WaitTrimTimer") {
-      context< SnapTrimmer >().log_enter(state_name);
-      struct OnTimer : Context {
-	ReplicatedPGRef pg;
-	epoch_t epoch;
-	OnTimer(ReplicatedPGRef pg, epoch_t epoch) : pg(pg), epoch(epoch) {}
-	void finish(int) override {
-	  pg->lock();
-	  if (!pg->pg_has_reset_since(epoch))
-	    pg->snap_trimmer_machine.process_event(SnapTrimTimerReady());
-	  pg->unlock();
-	}
-      };
-      auto *pg = context< SnapTrimmer >().pg;
-      if (pg->cct->_conf->osd_snap_trim_sleep > 0) {
-	wakeup = new OnTimer{pg, pg->get_osdmap()->get_epoch()};
-	Mutex::Locker l(pg->osd->snap_sleep_lock);
-	pg->osd->snap_sleep_timer.add_event_after(
-	  pg->cct->_conf->osd_snap_trim_sleep, wakeup);
-	slept = true;
-      } else {
-	post_event(SnapTrimTimerReady());
-      }
-    }
-    void exit() {
-      context< SnapTrimmer >().log_exit(state_name, enter_time);
-      auto *pg = context< SnapTrimmer >().pg;
-      if (wakeup) {
-	Mutex::Locker l(pg->osd->snap_sleep_lock);
-	pg->osd->snap_sleep_timer.cancel_event(wakeup);
-	wakeup = nullptr;
-      }
-    }
-    boost::statechart::result react(const SnapTrimTimerReady &) {
-      wakeup = nullptr;
-      auto *pg = context< SnapTrimmer >().pg;
-      if (!pg->is_primary() || !pg->is_active() || !pg->is_clean() ||
-	  pg->scrubber.active) {
-	post_event(SnapTrim());
-	return transit< NotTrimming >();
-      } else {
-	if (slept)
-	  post_event(SnapTrim());
-	return transit< TrimmingObjects >();
-      }
-    }
-  };
-
-  struct WaitingOnReplicas : boost::statechart::state< WaitingOnReplicas, Trimming >, NamedState {
+  struct WaitingOnReplicas : boost::statechart::state< WaitingOnReplicas, SnapTrimmer >, NamedState {
     typedef boost::mpl::list <
       boost::statechart::custom_reaction< SnapTrim >,
       boost::statechart::transition< Reset, NotTrimming >
@@ -1767,9 +1524,6 @@ public:
     ConnectionRef con,
     ceph_tid_t tid);
   eversion_t pick_newest_available(const hobject_t& oid);
-  ObjectContextRef mark_object_lost(ObjectStore::Transaction *t,
-				  const hobject_t& oid, eversion_t version,
-				  utime_t mtime, int what);
 
   void do_update_log_missing(
     OpRequestRef &op);
@@ -1777,14 +1531,14 @@ public:
   void do_update_log_missing_reply(
     OpRequestRef &op);
 
-  void on_role_change();
-  void on_pool_change();
-  void _on_new_interval();
-  void on_change(ObjectStore::Transaction *t);
-  void on_activate();
-  void on_flushed();
-  void on_removal(ObjectStore::Transaction *t);
-  void on_shutdown();
+  void on_role_change() override;
+  void on_pool_change() override;
+  void _on_new_interval() override;
+  void on_change(ObjectStore::Transaction *t) override;
+  void on_activate() override;
+  void on_flushed() override;
+  void on_removal(ObjectStore::Transaction *t) override;
+  void on_shutdown() override;
 
   // attr cache handling
   void replace_cached_attrs(
@@ -1817,7 +1571,7 @@ public:
     bool user_only = false);
 };
 
-inline ostream& operator<<(ostream& out, ReplicatedPG::RepGather& repop)
+inline ostream& operator<<(ostream& out, const ReplicatedPG::RepGather& repop)
 {
   out << "repgather(" << &repop
       << " " << repop.v
@@ -1828,7 +1582,8 @@ inline ostream& operator<<(ostream& out, ReplicatedPG::RepGather& repop)
   return out;
 }
 
-inline ostream& operator<<(ostream& out, ReplicatedPG::ProxyWriteOpRef pwop)
+inline ostream& operator<<(ostream& out,
+			   const ReplicatedPG::ProxyWriteOpRef& pwop)
 {
   out << "proxywrite(" << &pwop
       << " " << pwop->user_version
