@@ -27,6 +27,60 @@ using std::chrono::seconds;
 
 
 
+class ObjectCacher::C_ReadFinish : public Context {
+  ObjectCacher *oc;
+  int64_t poolid;
+  sobject_t oid;
+  loff_t start;
+  uint64_t length;
+  xlist<C_ReadFinish*>::item set_item;
+  bool trust_enoent;
+  ceph_tid_t tid;
+
+public:
+  bufferlist bl;
+  C_ReadFinish(ObjectCacher *c, Object *ob, ceph_tid_t t, loff_t s,
+	       uint64_t l) :
+    oc(c), poolid(ob->oloc.pool), oid(ob->get_soid()), start(s), length(l),
+    set_item(this), trust_enoent(true),
+    tid(t) {
+    ob->reads.push_back(&set_item);
+  }
+
+  void finish(int r) {
+    oc->bh_read_finish(poolid, oid, tid, start, length, bl, r, trust_enoent);
+
+    // object destructor clears the list
+    if (set_item.is_on_list())
+      set_item.remove_myself();
+  }
+
+  void distrust_enoent() {
+    trust_enoent = false;
+  }
+};
+
+class ObjectCacher::C_RetryRead : public Context {
+  ObjectCacher *oc;
+  OSDRead *rd;
+  ObjectSet *oset;
+  Context *onfinish;
+public:
+  C_RetryRead(ObjectCacher *_oc, OSDRead *r, ObjectSet *os, Context *c)
+    : oc(_oc), rd(r), oset(os), onfinish(c) {}
+  void finish(int r) {
+    if (r < 0) {
+      if (onfinish)
+        onfinish->complete(r);
+      return;
+    }
+    int ret = oc->_readx(rd, oset, onfinish, false);
+    if (ret != 0 && onfinish) {
+      onfinish->complete(ret);
+    }
+  }
+};
+
 ObjectCacher::BufferHead *ObjectCacher::Object::split(BufferHead *left,
 						      loff_t off)
 {
@@ -93,6 +147,9 @@ ObjectCacher::BufferHead *ObjectCacher::Object::split(BufferHead *left,
 void ObjectCacher::Object::merge_left(BufferHead *left, BufferHead *right)
 {
   assert(oc->lock.is_locked());
+  assert(left->end() == right->start());
+  assert(left->get_state() == right->get_state());
+  assert(left->can_merge_journal(right));
 
   ldout(oc->cct, 10) << "merge_left " << *left << " + " << *right << dendl;
   if (left->get_journal_tid() == 0) {
@@ -129,17 +186,6 @@ void ObjectCacher::Object::merge_left(BufferHead *left, BufferHead *right)
   ldout(oc->cct, 10) << "merge_left result " << *left << dendl;
 }
 
-bool ObjectCacher::Object::can_merge_bh(BufferHead *left, BufferHead *right)
-{
-  if (left->end() != right->start() ||
-      left->get_state() != right->get_state() ||
-      !left->can_merge_journal(right))
-    return false;
-  if (left->is_tx() && left->last_write_tid != right->last_write_tid)
-    return false;
-  return true;
-}
-
 void ObjectCacher::Object::try_merge_bh(BufferHead *bh)
 {
   assert(oc->lock.is_locked());
@@ -154,7 +200,9 @@ void ObjectCacher::Object::try_merge_bh(BufferHead *bh)
   assert(p->second == bh);
   if (p != data.begin()) {
     --p;
-    if (can_merge_bh(p->second, bh)) {
+    if (p->second->end() == bh->start() &&
+	p->second->get_state() == bh->get_state() &&
+	p->second->can_merge_journal(bh)) {
       merge_left(p->second, bh);
       bh = p->second;
     } else {
@@ -164,17 +212,20 @@ void ObjectCacher::Object::try_merge_bh(BufferHead *bh)
   // to the right?
   assert(p->second == bh);
   ++p;
-  if (p != data.end() && can_merge_bh(bh, p->second))
+  if (p != data.end() &&
+      p->second->start() == bh->end() &&
+      p->second->get_state() == bh->get_state() &&
+      p->second->can_merge_journal(bh))
     merge_left(bh, p->second);
 }
 
 /*
  * count bytes we have cached in given range
  */
-bool ObjectCacher::Object::is_cached(loff_t cur, loff_t left)
+bool ObjectCacher::Object::is_cached(loff_t cur, loff_t left) const
 {
   assert(oc->lock.is_locked());
-  map<loff_t, BufferHead*>::iterator p = data_lower_bound(cur);
+  map<loff_t, BufferHead*>::const_iterator p = data_lower_bound(cur);
   while (left > 0) {
     if (p == data.end())
       return false;
@@ -230,7 +281,7 @@ int ObjectCacher::Object::map_read(ObjectExtent &ex,
   loff_t cur = ex.offset;
   loff_t left = ex.length;
 
-  map<loff_t, BufferHead*>::iterator p = data_lower_bound(ex.offset);
+  map<loff_t, BufferHead*>::const_iterator p = data_lower_bound(ex.offset);
   while (left > 0) {
     // at end?
     if (p == data.end()) {
@@ -356,7 +407,7 @@ ObjectCacher::BufferHead *ObjectCacher::Object::map_write(ObjectExtent &ex,
   loff_t cur = ex.offset;
   loff_t left = ex.length;
 
-  map<loff_t, BufferHead*>::iterator p = data_lower_bound(ex.offset);
+  map<loff_t, BufferHead*>::const_iterator p = data_lower_bound(ex.offset);
   while (left > 0) {
     loff_t max = left;
 
@@ -513,7 +564,7 @@ void ObjectCacher::Object::discard(loff_t off, loff_t len)
     complete = false;
   }
 
-  map<loff_t, BufferHead*>::iterator p = data_lower_bound(off);
+  map<loff_t, BufferHead*>::const_iterator p = data_lower_bound(off);
   while (p != data.end()) {
     BufferHead *bh = p->second;
     if (bh->start() >= off + len)
@@ -786,7 +837,7 @@ void ObjectCacher::bh_read_finish(int64_t poolid, sobject_t oid,
     // apply to bh's!
     loff_t opos = start;
     while (true) {
-      map<loff_t, BufferHead*>::iterator p = ob->data_lower_bound(opos);
+      map<loff_t, BufferHead*>::const_iterator p = ob->data_lower_bound(opos);
       if (p == ob->data.end())
 	break;
       if (opos >= start+(loff_t)length) {
@@ -923,6 +974,27 @@ void ObjectCacher::bh_write_adjacencies(BufferHead *bh, ceph::real_time cutoff,
   bh_write_scattered(blist);
 }
 
+class ObjectCacher::C_WriteCommit : public Context {
+  ObjectCacher *oc;
+  int64_t poolid;
+  sobject_t oid;
+  vector<pair<loff_t, uint64_t> > ranges;
+public:
+  ceph_tid_t tid;
+  C_WriteCommit(ObjectCacher *c, int64_t _poolid, sobject_t o, loff_t s,
+		uint64_t l) :
+    oc(c), poolid(_poolid), oid(o), tid(0) {
+      ranges.push_back(make_pair(s, l));
+    }
+  C_WriteCommit(ObjectCacher *c, int64_t _poolid, sobject_t o,
+		vector<pair<loff_t, uint64_t> >& _ranges) :
+    oc(c), poolid(_poolid), oid(o), tid(0) {
+      ranges.swap(_ranges);
+    }
+  void finish(int r) {
+    oc->bh_write_commit(poolid, oid, ranges, tid, r);
+  }
+};
 void ObjectCacher::bh_write_scattered(list<BufferHead*>& blist)
 {
   assert(lock.is_locked());
@@ -1042,15 +1114,22 @@ void ObjectCacher::bh_write_commit(int64_t poolid, sobject_t oid,
       }
     }
 
+    list <BufferHead*> hit;
     // apply to bh's!
-    for (map<loff_t, BufferHead*>::iterator p = ob->data_lower_bound(start);
+    for (map<loff_t, BufferHead*>::const_iterator p = ob->data_lower_bound(start);
 	 p != ob->data.end();
 	 ++p) {
       BufferHead *bh = p->second;
 
-      if (bh->start() >= start+(loff_t)length)
+      if (bh->start() > start+(loff_t)length)
 	break;
 
+      if (bh->start() < start &&
+	  bh->end() > start+(loff_t)length) {
+	ldout(cct, 20) << "bh_write_commit skipping " << *bh << dendl;
+	continue;
+      }
+
       // make sure bh is tx
       if (!bh->is_tx()) {
 	ldout(cct, 10) << "bh_write_commit skipping non-tx " << *bh << dendl;
@@ -1064,16 +1143,13 @@ void ObjectCacher::bh_write_commit(int64_t poolid, sobject_t oid,
 	continue;
       }
 
-      // we don't merge tx buffers. tx buffer should be within the range
-      assert(bh->start() >= start);
-      assert(bh->end() <= start+(loff_t)length);
-
       if (r >= 0) {
 	// ok!  mark bh clean and error-free
 	mark_clean(bh);
 	bh->set_journal_tid(0);
 	if (bh->get_nocache())
 	  bh_lru_rest.lru_bottouch(bh);
+	hit.push_back(bh);
 	ldout(cct, 10) << "bh_write_commit clean " << *bh << dendl;
       } else {
 	mark_dirty(bh);
@@ -1082,6 +1158,13 @@ void ObjectCacher::bh_write_commit(int64_t poolid, sobject_t oid,
 		       << dendl;
       }
     }
+
+    for (list<BufferHead*>::iterator bh = hit.begin();
+	bh != hit.end();
+	++bh) {
+      assert(*bh);
+      ob->try_merge_bh(*bh);
+    }
   }
 
   // update last_commit.
@@ -1152,7 +1235,7 @@ void ObjectCacher::trim()
       break;
 
     ldout(cct, 10) << "trim trimming " << *bh << dendl;
-    assert(bh->is_clean() || bh->is_zero() || bh->is_error());
+    assert(bh->is_clean() || bh->is_zero());
 
     Object *ob = bh->ob;
     bh_remove(ob, bh);
@@ -1622,6 +1705,17 @@ int ObjectCacher::writex(OSDWrite *wr, ObjectSet *oset, Context *onfreespace)
   return r;
 }
 
+class ObjectCacher::C_WaitForWrite : public Context {
+public:
+  C_WaitForWrite(ObjectCacher *oc, uint64_t len, Context *onfinish) :
+    m_oc(oc), m_len(len), m_onfinish(onfinish) {}
+  void finish(int r);
+private:
+  ObjectCacher *m_oc;
+  uint64_t m_len;
+  Context *m_onfinish;
+};
+
 void ObjectCacher::C_WaitForWrite::finish(int r)
 {
   Mutex::Locker l(m_oc->lock);
@@ -1709,7 +1803,6 @@ int ObjectCacher::_wait_for_write(OSDWrite *wr, uint64_t len, ObjectSet *oset,
 void ObjectCacher::flusher_entry()
 {
   ldout(cct, 10) << "flusher start" << dendl;
-  writeback_handler.get_client_lock();
   lock.Lock();
   while (!flusher_stop) {
     loff_t all = get_stat_tx() + get_stat_rx() + get_stat_clean() +
@@ -1751,8 +1844,6 @@ void ObjectCacher::flusher_entry()
       if (!max) {
 	// back off the lock to avoid starving other threads
 	lock.Unlock();
-	writeback_handler.put_client_lock();
-	writeback_handler.get_client_lock();
 	lock.Lock();
 	continue;
       }
@@ -1760,12 +1851,7 @@ void ObjectCacher::flusher_entry()
     if (flusher_stop)
       break;
 
-    writeback_handler.put_client_lock();
     flusher_cond.WaitInterval(cct, lock, seconds(1));
-    lock.Unlock();
-
-    writeback_handler.get_client_lock();
-    lock.Lock();
   }
 
   /* Wait for reads to finish. This is only possible if handling
@@ -1781,7 +1867,6 @@ void ObjectCacher::flusher_entry()
   }
 
   lock.Unlock();
-  writeback_handler.put_client_lock();
   ldout(cct, 10) << "flusher finish" << dendl;
 }
 
@@ -1865,7 +1950,7 @@ bool ObjectCacher::flush(Object *ob, loff_t offset, loff_t length)
   list<BufferHead*> blist;
   bool clean = true;
   ldout(cct, 10) << "flush " << *ob << " " << offset << "~" << length << dendl;
-  for (map<loff_t,BufferHead*>::iterator p = ob->data_lower_bound(offset);
+  for (map<loff_t,BufferHead*>::const_iterator p = ob->data_lower_bound(offset);
        p != ob->data.end();
        ++p) {
     BufferHead *bh = p->second;
@@ -1928,7 +2013,7 @@ bool ObjectCacher::flush_set(ObjectSet *oset, Context *onfinish)
 
   list<BufferHead*> blist;
   Object *last_ob = NULL;
-  set<BufferHead*, BufferHead::ptr_lt>::iterator it, p, q;
+  set<BufferHead*, BufferHead::ptr_lt>::const_iterator it, p, q;
 
   // Buffer heads in dirty_or_tx_bh are sorted in ObjectSet/Object/offset
   // order. But items in oset->objects are not sorted. So the iterator can
@@ -2516,7 +2601,5 @@ void ObjectCacher::bh_remove(Object *ob, BufferHead *bh)
     dirty_or_tx_bh.erase(bh);
   }
   bh_stat_sub(bh);
-  if (get_stat_dirty_waiting() > 0)
-    stat_cond.Signal();
 }
 
