@@ -86,43 +86,12 @@ struct ObjectOperation {
     ops.rbegin()->op.flags = flags;
   }
 
-  /**
-   * This is a more limited form of C_Contexts, but that requires
-   * a ceph_context which we don't have here.
-   */
-  class C_TwoContexts : public Context {
-    Context *first;
-    Context *second;
-  public:
-    C_TwoContexts(Context *first, Context *second) : first(first),
-						     second(second) {}
-    void finish(int r) {
-      first->complete(r);
-      second->complete(r);
-      first = NULL;
-      second = NULL;
-    }
-
-    virtual ~C_TwoContexts() {
-      delete first;
-      delete second;
-    }
-  };
-
+  class C_TwoContexts;
   /**
    * Add a callback to run when this operation completes,
    * after any other callbacks for it.
    */
-  void add_handler(Context *extra) {
-    size_t last = out_handler.size() - 1;
-    Context *orig = out_handler[last];
-    if (orig) {
-      Context *wrapper = new C_TwoContexts(orig, extra);
-      out_handler[last] = wrapper;
-    } else {
-      out_handler[last] = extra;
-    }
-  }
+  void add_handler(Context *extra);
 
   OSDOp& add_op(int op) {
     int s = ops.size();
@@ -142,6 +111,14 @@ struct ObjectOperation {
     osd_op.op.extent.length = len;
     osd_op.indata.claim_append(bl);
   }
+  void add_writesame(int op, uint64_t off, uint64_t write_len,
+		     bufferlist& bl) {
+    OSDOp& osd_op = add_op(op);
+    osd_op.op.writesame.offset = off;
+    osd_op.op.writesame.length = write_len;
+    osd_op.op.writesame.data_length = bl.length();
+    osd_op.indata.claim_append(bl);
+  }
   void add_clone_range(int op, uint64_t off, uint64_t len,
 		       const object_t& srcoid, uint64_t srcoff,
 		       snapid_t srcsnapid) {
@@ -207,10 +184,12 @@ struct ObjectOperation {
     ::encode(cookie, osd_op.indata);
   }
   void add_alloc_hint(int op, uint64_t expected_object_size,
-		      uint64_t expected_write_size) {
+                      uint64_t expected_write_size,
+		      uint32_t flags) {
     OSDOp& osd_op = add_op(op);
     osd_op.op.alloc_hint.expected_object_size = expected_object_size;
     osd_op.op.alloc_hint.expected_write_size = expected_write_size;
+    osd_op.op.alloc_hint.flags = flags;
   }
 
   // ------
@@ -335,20 +314,13 @@ struct ObjectOperation {
     void finish(int r) {
       bufferlist::iterator iter = bl.begin();
       if (r >= 0) {
-        // NOTE: it's possible the sub-op has not been executed but the result
-        // code remains zeroed. Avoid the costly exception handling on a
-        // potential IO path.
-        if (bl.length() > 0) {
-	  try {
-	    ::decode(*extents, iter);
-	    ::decode(*data_bl, iter);
-	  } catch (buffer::error& e) {
-	    if (prval)
-              *prval = -EIO;
-	  }
-        } else if (prval) {
-          *prval = -EIO;
-        }
+	try {
+	  ::decode(*extents, iter);
+	  ::decode(*data_bl, iter);
+	} catch (buffer::error& e) {
+	  if (prval)
+	    *prval = -EIO;
+	}
       }
     }
   };
@@ -377,6 +349,9 @@ struct ObjectOperation {
   void write_full(bufferlist& bl) {
     add_data(CEPH_OSD_OP_WRITEFULL, 0, bl.length(), bl);
   }
+  void writesame(uint64_t off, uint64_t write_len, bufferlist& bl) {
+    add_writesame(CEPH_OSD_OP_WRITESAME, off, write_len, bl);
+  }
   void append(bufferlist& bl) {
     add_data(CEPH_OSD_OP_APPEND, 0, bl.length(), bl);
   }
@@ -1085,9 +1060,10 @@ struct ObjectOperation {
   }
 
   void set_alloc_hint(uint64_t expected_object_size,
-		      uint64_t expected_write_size ) {
+                      uint64_t expected_write_size,
+		      uint32_t flags) {
     add_alloc_hint(CEPH_OSD_OP_SETALLOCHINT, expected_object_size,
-		   expected_write_size);
+		   expected_write_size, flags);
 
     // CEPH_OSD_OP_SETALLOCHINT op is advisory and therefore deemed
     // not worth a feature bit.  Set FAILOK per-op flag to make
@@ -1177,13 +1153,7 @@ private:
   void tick();
   void update_crush_location();
 
-  class RequestStateHook : public AdminSocketHook {
-    Objecter *m_objecter;
-  public:
-    explicit RequestStateHook(Objecter *objecter);
-    bool call(std::string command, cmdmap_t& cmdmap, std::string format,
-	      bufferlist& out);
-  };
+  class RequestStateHook;
 
   RequestStateHook *m_request_state_hook;
 
@@ -1975,7 +1945,8 @@ private:
     op_throttle_bytes(cct, "objecter_bytes",
 		      cct->_conf->objecter_inflight_op_bytes),
     op_throttle_ops(cct, "objecter_ops", cct->_conf->objecter_inflight_ops),
-    epoch_barrier(0)
+    epoch_barrier(0),
+    retry_writes_after_first_reply(cct->_conf->objecter_retry_writes_after_first_reply)
   { }
   ~Objecter();
 
@@ -2064,9 +2035,7 @@ private:
     }
   }
   void ms_fast_dispatch(Message *m) {
-    if (!ms_dispatch(m)) {
-      m->put();
-    }
+    ms_dispatch(m);
   }
 
   void handle_osd_op_reply(class MOSDOpReply *m);
@@ -2142,7 +2111,6 @@ private:
   int _op_cancel(ceph_tid_t tid, int r);
 public:
   int op_cancel(ceph_tid_t tid, int r);
-  int op_cancel(const vector<ceph_tid_t>& tidls, int r);
 
   /**
    * Any write op which is in progress at the start of this call shall no
@@ -2608,6 +2576,44 @@ public:
     op_submit(o, &tid);
     return tid;
   }
+  Op *prepare_writesame_op(
+    const object_t& oid, const object_locator_t& oloc,
+    uint64_t write_len, uint64_t off,
+    const SnapContext& snapc, const bufferlist &bl,
+    ceph::real_time mtime, int flags, Context *onack,
+    Context *oncommit, version_t *objver = NULL,
+    ObjectOperation *extra_ops = NULL, int op_flags = 0) {
+
+    vector<OSDOp> ops;
+    int i = init_ops(ops, 1, extra_ops);
+    ops[i].op.op = CEPH_OSD_OP_WRITESAME;
+    ops[i].op.writesame.offset = off;
+    ops[i].op.writesame.length = write_len;
+    ops[i].op.writesame.data_length = bl.length();
+    ops[i].indata = bl;
+    ops[i].op.flags = op_flags;
+    Op *o = new Op(oid, oloc, ops, flags | global_op_flags.read() |
+		   CEPH_OSD_FLAG_WRITE, onack, oncommit, objver);
+    o->mtime = mtime;
+    o->snapc = snapc;
+    return o;
+  }
+  ceph_tid_t writesame(
+    const object_t& oid, const object_locator_t& oloc,
+    uint64_t write_len, uint64_t off,
+    const SnapContext& snapc, const bufferlist &bl,
+    ceph::real_time mtime, int flags, Context *onack,
+    Context *oncommit, version_t *objver = NULL,
+    ObjectOperation *extra_ops = NULL, int op_flags = 0) {
+
+    Op *o = prepare_writesame_op(oid, oloc, write_len, off, snapc, bl,
+				 mtime, flags, onack, oncommit, objver,
+				 extra_ops, op_flags);
+
+    ceph_tid_t tid;
+    op_submit(o, &tid);
+    return tid;
+  }
   ceph_tid_t trunc(const object_t& oid, const object_locator_t& oloc,
 		   const SnapContext& snapc, ceph::real_time mtime, int flags,
 		   uint64_t trunc_size, __u32 trunc_seq, Context *onack,
@@ -2917,6 +2923,7 @@ public:
   void ms_handle_connect(Connection *con);
   bool ms_handle_reset(Connection *con);
   void ms_handle_remote_reset(Connection *con);
+  bool ms_handle_refused(Connection *con);
   bool ms_get_authorizer(int dest_type,
 			 AuthAuthorizer **authorizer,
 			 bool force_new);
@@ -2925,6 +2932,7 @@ public:
 
 private:
   epoch_t epoch_barrier;
+  bool retry_writes_after_first_reply;
 public:
   void set_epoch_barrier(epoch_t epoch);
 };
