@@ -9,6 +9,7 @@ Copyright (C) 2013 Inktank Storage, Inc.
 
 LGPL2.  See file COPYING.
 """
+from __future__ import print_function
 import copy
 import errno
 import json
@@ -261,7 +262,7 @@ class CephIPAddr(CephArgtype):
             type = 4
         if type == 4:
             port = s.find(':')
-            if (port != -1):
+            if port != -1:
                 a = s[:port]
                 p = s[port + 1:]
                 if int(p) > 65535:
@@ -355,6 +356,10 @@ class CephPgid(CephArgtype):
         if s.find('.') == -1:
             raise ArgumentFormat('pgid has no .')
         poolid, pgnum = s.split('.', 1)
+        try:
+            poolid = int(poolid)
+        except ValueError:
+            raise ArgumentFormat('pool {0} not integer'.format(poolid))
         if poolid < 0:
             raise ArgumentFormat('pool {0} < 0'.format(poolid))
         try:
@@ -383,11 +388,15 @@ class CephName(CephArgtype):
         if s == '*':
             self.val = s
             return
+        elif s == "mgr":
+            self.nametype = "mgr"
+            self.val = s
+            return
         if s.find('.') == -1:
             raise ArgumentFormat('CephName: no . in {0}'.format(s))
         else:
             t, i = s.split('.', 1)
-            if t not in ('osd', 'mon', 'client', 'mds'):
+            if t not in ('osd', 'mon', 'client', 'mds', 'mgr'):
                 raise ArgumentValid('unknown type ' + t)
             if t == 'osd':
                 if i != '*':
@@ -428,6 +437,8 @@ class CephOsdName(CephArgtype):
             i = int(i)
         except:
             raise ArgumentFormat('osd id ' + i + ' not integer')
+        if i < 0:
+            raise ArgumentFormat('osd id {0} is less than 0'.format(i))
         self.nametype = t
         self.nameid = i
         self.val = i
@@ -549,7 +560,7 @@ class CephPrefix(CephArgtype):
                 self.val = s
                 return
         else:
-            if (s == self.prefix):
+            if s == self.prefix:
                 self.val = s
                 return
 
@@ -657,12 +668,19 @@ def concise_sig(sig):
     return ' '.join([d.helpstr() for d in sig])
 
 
-def descsort(sh1, sh2):
+def descsort_key(sh):
     """
     sort descriptors by prefixes, defined as the concatenation of all simple
     strings in the descriptor; this works out to just the leading strings.
     """
-    return cmp(concise_sig(sh1['sig']), concise_sig(sh2['sig']))
+    return concise_sig(sh['sig'])
+
+
+def descsort(sh1, sh2):
+    """
+    Deprecated; use (key=descsort_key) instead of (cmp=descsort)
+    """
+    return cmp(descsort_key(sh1), descsort_key(sh2))
 
 
 def parse_funcsig(sig):
@@ -740,7 +758,7 @@ def parse_json_funcsigs(s, consumer):
     try:
         overall = json.loads(s)
     except Exception as e:
-        print >> sys.stderr, "Couldn't parse JSON {0}: {1}".format(s, e)
+        print("Couldn't parse JSON {0}: {1}".format(s, e), file=sys.stderr)
         raise e
     sigdict = {}
     for cmdtag, cmd in overall.items():
@@ -890,6 +908,8 @@ def validate(args, signature, partial=False):
     reqsiglen = len([desc for desc in mysig if desc.req])
     matchcnt = 0
     d = dict()
+    save_exception = None
+
     for desc in mysig:
         setattr(desc, 'numseen', 0)
         while desc.numseen < desc.n:
@@ -932,19 +952,20 @@ def validate(args, signature, partial=False):
                 valid = True
             except ArgumentError as e:
                 valid = False
+                exc = e
             if not valid:
                 # argument mismatch
                 if not desc.req:
                     # if not required, just push back; it might match
                     # the next arg
-                    save_exception = [ myarg, e ]
+                    save_exception = [ myarg, exc ]
                     myargs.insert(0, myarg)
                     break
                 else:
                     # hm, it was required, so time to return/raise
                     if partial:
                         return d
-                    raise e
+                    raise exc
 
             # Whew, valid arg acquired.  Store in dict
             matchcnt += 1
@@ -958,7 +979,7 @@ def validate(args, signature, partial=False):
 
     if myargs and not partial:
         if save_exception:
-            print >> sys.stderr, save_exception[0], 'not valid: ', str(save_exception[1])
+            print(save_exception[0], 'not valid: ', save_exception[1], file=sys.stderr)
         raise ArgumentError("unused arguments: " + str(myargs))
 
     # Finally, success
@@ -978,8 +999,7 @@ def validate_command(sigdict, args, verbose=False):
     validated against sigdict.
     """
     if verbose:
-        print >> sys.stderr, \
-            "validate_command: " + " ".join(args)
+        print("validate_command: " + " ".join(args), file=sys.stderr)
     found = []
     valid_dict = {}
     if args:
@@ -990,20 +1010,18 @@ def validate_command(sigdict, args, verbose=False):
         for cmdtag, cmd in sigdict.items():
             sig = cmd['sig']
             matched = matchnum(args, sig, partial=True)
-            if (matched > best_match_cnt):
+            if matched > best_match_cnt:
                 if verbose:
-                    print >> sys.stderr, \
-                        "better match: {0} > {1}: {2}:{3} ".\
-                        format(matched, best_match_cnt, cmdtag,
-                               concise_sig(sig))
+                    print("better match: {0} > {1}: {2}:{3} ".format(
+                        matched, best_match_cnt, cmdtag, concise_sig(sig)
+                    ), file=sys.stderr)
                 best_match_cnt = matched
                 bestcmds = [{cmdtag: cmd}]
             elif matched == best_match_cnt:
                 if verbose:
-                    print >> sys.stderr, \
-                        "equal match: {0} > {1}: {2}:{3} ".\
-                        format(matched, best_match_cnt, cmdtag,
-                               concise_sig(sig))
+                    print("equal match: {0} > {1}: {2}:{3} ".format(
+                        matched, best_match_cnt, cmdtag, concise_sig(sig)
+                    ), file=sys.stderr)
                 bestcmds.append({cmdtag: cmd})
 
         # Sort bestcmds by number of args so we can try shortest first
@@ -1011,7 +1029,7 @@ def validate_command(sigdict, args, verbose=False):
         bestcmds_sorted = sorted(bestcmds, key=cmdsiglen)
 
         if verbose:
-            print >> sys.stderr, "bestcmds_sorted: "
+            print("bestcmds_sorted: ", file=sys.stderr)
             pprint.PrettyPrinter(stream=sys.stderr).pprint(bestcmds_sorted)
 
         # for everything in bestcmds, look for a true match
@@ -1032,23 +1050,23 @@ def validate_command(sigdict, args, verbose=False):
                     # cmdsigs we'll fall out unfound; if we're not, maybe
                     # the next one matches completely.  Whine, but pass.
                     if verbose:
-                        print >> sys.stderr, 'Not enough args supplied for ', \
-                            concise_sig(sig)
+                        print('Not enough args supplied for ',
+                              concise_sig(sig), file=sys.stderr)
                 except ArgumentError as e:
                     # Solid mismatch on an arg (type, range, etc.)
                     # Stop now, because we have the right command but
                     # some other input is invalid
-                    print >> sys.stderr, "Invalid command: ", str(e)
-                    print >> sys.stderr, concise_sig(sig), ': ', cmd['help']
+                    print("Invalid command: ", e, file=sys.stderr)
+                    print(concise_sig(sig), ': ', cmd['help'], file=sys.stderr)
                     return {}
             if found:
                 break
 
         if not found:
-            print >> sys.stderr, 'no valid command found; 10 closest matches:'
+            print('no valid command found; 10 closest matches:', file=sys.stderr)
             for cmdsig in bestcmds[:10]:
-                for (cmdtag, cmd) in cmdsig.iteritems():
-                    print >> sys.stderr, concise_sig(cmd['sig'])
+                for (cmdtag, cmd) in cmdsig.items():
+                    print(concise_sig(cmd['sig']), file=sys.stderr)
             return None
 
         return valid_dict
@@ -1190,7 +1208,7 @@ def send_command_retry(*args, **kwargs):
             else:
                 raise
 
-def send_command(cluster, target=('mon', ''), cmd=None, inbuf='', timeout=0,
+def send_command(cluster, target=('mon', ''), cmd=None, inbuf=b'', timeout=0,
                  verbose=False):
     """
     Send a command to a daemon using librados's
@@ -1209,11 +1227,15 @@ def send_command(cluster, target=('mon', ''), cmd=None, inbuf='', timeout=0,
             osdid = target[1]
 
             if verbose:
-                print >> sys.stderr, 'submit {0} to osd.{1}'.\
-                    format(cmd, osdid)
+                print('submit {0} to osd.{1}'.format(cmd, osdid),
+                      file=sys.stderr)
             ret, outbuf, outs = run_in_thread(
                 cluster.osd_command, osdid, cmd, inbuf, timeout)
 
+        elif target[0] == 'mgr':
+            ret, outbuf, outs = run_in_thread(
+                cluster.mgr_command, cmd, inbuf, timeout)
+
         elif target[0] == 'pg':
             pgid = target[1]
             # pgid will already be in the command for the pg <pgid>
@@ -1225,15 +1247,15 @@ def send_command(cluster, target=('mon', ''), cmd=None, inbuf='', timeout=0,
                 cmddict = dict(pgid=pgid)
             cmd = [json.dumps(cmddict)]
             if verbose:
-                print >> sys.stderr, 'submit {0} for pgid {1}'.\
-                    format(cmd, pgid)
+                print('submit {0} for pgid {1}'.format(cmd, pgid),
+                      file=sys.stderr)
             ret, outbuf, outs = run_in_thread(
                 cluster.pg_command, pgid, cmd, inbuf, timeout)
 
         elif target[0] == 'mon':
             if verbose:
-                print >> sys.stderr, '{0} to {1}'.\
-                    format(cmd, target[0])
+                print('{0} to {1}'.format(cmd, target[0]),
+                      file=sys.stderr)
             if target[1] == '':
                 ret, outbuf, outs = run_in_thread(
                     cluster.mon_command, cmd, inbuf, timeout)
@@ -1244,15 +1266,17 @@ def send_command(cluster, target=('mon', ''), cmd=None, inbuf='', timeout=0,
             mds_spec = target[1]
 
             if verbose:
-                print >> sys.stderr, 'submit {0} to mds.{1}'.\
-                    format(cmd, mds_spec)
+                print('submit {0} to mds.{1}'.format(cmd, mds_spec),
+                      file=sys.stderr)
 
             try:
                 from cephfs import LibCephFS
             except ImportError:
                 raise RuntimeError("CephFS unavailable, have you installed libcephfs?")
 
-            filesystem = LibCephFS(rados_inst=cluster)
+            filesystem = LibCephFS(cluster.conf_defaults, cluster.conffile)
+            filesystem.conf_parse_argv(cluster.parsed_args)
+
             filesystem.init()
             ret, outbuf, outs = \
                 filesystem.mds_command(mds_spec, cmd, inbuf)
@@ -1270,7 +1294,7 @@ def send_command(cluster, target=('mon', ''), cmd=None, inbuf='', timeout=0,
 
 
 def json_command(cluster, target=('mon', ''), prefix=None, argdict=None,
-                 inbuf='', timeout=0, verbose=False):
+                 inbuf=b'', timeout=0, verbose=False):
     """
     Format up a JSON command and send it with send_command() above.
     Prefix may be supplied separately or in argdict.  Any bulk input
@@ -1312,4 +1336,3 @@ def json_command(cluster, target=('mon', ''), prefix=None, argdict=None,
             raise
 
     return ret, outbuf, outs
-
