@@ -181,7 +181,7 @@ class RankEvicter(threading.Thread):
     def run(self):
         try:
             self._evict()
-        except Exception, e:
+        except Exception as e:
             self.success = False
             self.exception = e
         else:
@@ -318,7 +318,7 @@ class CephFSVolumeClient(object):
 
         log.debug("Recovered from partial auth updates (if any).")
 
-    def _recover_auth_meta(self, auth_id, auth_meta):
+    def _recover_auth_meta(auth_id, auth_meta):
         """
         Call me after locking the auth meta file.
         """
@@ -329,7 +329,6 @@ class CephFSVolumeClient(object):
                 continue
 
             (group_id, volume_id) = volume.split('/')
-            group_id = group_id if group_id is not 'None' else None
             volume_path = VolumePath(group_id, volume_id)
             access_level = volume_data['access_level']
 
@@ -339,7 +338,7 @@ class CephFSVolumeClient(object):
                 # No VMeta update indicates that there was no auth update
                 # in Ceph either. So it's safe to remove corresponding
                 # partial update in AMeta.
-                if not vol_meta or auth_id not in vol_meta['auths']:
+                if auth_id not in vol_meta['auths']:
                     remove_volumes.append(volume)
                     continue
 
@@ -598,7 +597,7 @@ class CephFSVolumeClient(object):
             try:
                 self.fs.stat(subpath)
             except cephfs.ObjectNotFound:
-                self.fs.mkdir(subpath, 0755)
+                self.fs.mkdir(subpath, 0o755)
 
     def create_volume(self, volume_path, size=None, data_isolated=False):
         """
@@ -640,7 +639,7 @@ class CephFSVolumeClient(object):
         # Create a volume meta file, if it does not already exist, to store
         # data about auth ids having access to the volume
         fd = self.fs.open(self._volume_metadata_path(volume_path),
-                          os.O_CREAT, 0755)
+                          os.O_CREAT, 0o755)
         self.fs.close(fd)
 
         return {
@@ -790,7 +789,7 @@ class CephFSVolumeClient(object):
         @contextmanager
         def fn():
             while(1):
-                fd = self.fs.open(path, os.O_CREAT, 0755)
+                fd = self.fs.open(path, os.O_CREAT, 0o755)
                 self.fs.flock(fd, fcntl.LOCK_EX, self._id)
 
                 # The locked file will be cleaned up sometime. It could be
@@ -1067,9 +1066,6 @@ class CephFSVolumeClient(object):
                 # occurrence of wanted auth caps and no occurrence of
                 # conflicting auth caps.
 
-                if not orig:
-                    return want
-
                 cap_tokens = set(orig.split(","))
 
                 cap_tokens.discard(unwanted)
@@ -1087,7 +1083,7 @@ class CephFSVolumeClient(object):
                     'caps': [
                         'mds', mds_cap_str,
                         'osd', osd_cap_str,
-                        'mon', cap['caps'].get('mon', 'allow r')]
+                        'mon', cap['caps'].get('mon')]
                 })
             caps = self._rados_command(
                 'auth get',
@@ -1221,7 +1217,7 @@ class CephFSVolumeClient(object):
                         'caps': [
                             'mds', mds_cap_str,
                             'osd', osd_cap_str,
-                            'mon', cap['caps'].get('mon', 'allow r')]
+                            'mon', cap['caps'].get('mon')]
                     })
 
         # FIXME: rados raising Error instead of ObjectNotFound in auth get failure
@@ -1304,7 +1300,7 @@ class CephFSVolumeClient(object):
 
     def _snapshot_create(self, dir_path, snapshot_name):
         # TODO: raise intelligible exception for clusters where snaps are disabled
-        self.fs.mkdir(self._snapshot_path(dir_path, snapshot_name), 0755)
+        self.fs.mkdir(self._snapshot_path(dir_path, snapshot_name), 0o755)
 
     def _snapshot_destroy(self, dir_path, snapshot_name):
         """
