@@ -13,7 +13,6 @@
  */
 #include <sys/types.h>
 #include <string.h>
-#include <chrono>
 
 #include "include/types.h"
 #include "include/rados/librgw.h"
@@ -42,6 +41,7 @@
 #include "rgw_rest_user.h"
 #include "rgw_rest_s3.h"
 #include "rgw_os_lib.h"
+#include "rgw_auth.h"
 #include "rgw_auth_s3.h"
 #include "rgw_lib.h"
 #include "rgw_lib_frontend.h"
@@ -80,8 +80,6 @@ namespace rgw {
     m_tp.drain(&req_wq);
   }
 
-#define MIN_EXPIRE_S 120
-
   void RGWLibProcess::run()
   {
     /* write completion interval */
@@ -94,14 +92,6 @@ namespace rgw {
     /* gc loop */
     while (! shutdown) {
       lsubdout(cct, rgw, 5) << "RGWLibProcess GC" << dendl;
-
-      /* dirent invalidate timeout--basically, the upper-bound on
-       * inconsistency with the S3 namespace */
-      auto expire_s = cct->_conf->rgw_nfs_namespace_expire_secs;
-
-      /* delay between gc cycles */
-      auto delay_s = std::max(1, std::min(MIN_EXPIRE_S, expire_s/2));
-
       unique_lock uniq(mtx);
     restart:
       int cur_gen = gen;
@@ -116,7 +106,7 @@ namespace rgw {
 	  goto restart; /* invalidated */
       }
       uniq.unlock();
-      std::this_thread::sleep_for(std::chrono::seconds(delay_s));
+      std::this_thread::sleep_for(std::chrono::seconds(120));
     }
   }
 
@@ -247,6 +237,12 @@ namespace rgw {
       goto done;
     }
 
+    /* FIXME: remove this after switching all handlers to the new authentication
+     * infrastructure. */
+    if (! s->auth_identity) {
+      s->auth_identity = rgw_auth_transform_old_authinfo(s);
+    }
+
     req->log(s, "reading op permissions");
     ret = req->read_permissions(op);
     if (ret < 0) {
@@ -273,6 +269,8 @@ namespace rgw {
     if (ret < 0) {
       if (s->system_request) {
 	dout(2) << "overriding permissions due to system operation" << dendl;
+      } else if (s->auth_identity->is_admin_of(s->user->user_id)) {
+	dout(2) << "overriding permissions due to admin operation" << dendl;
       } else {
 	abort_req(s, op, ret);
 	goto done;
@@ -297,8 +295,7 @@ namespace rgw {
       dout(0) << "ERROR: io->complete_request() returned " << r << dendl;
     }
     if (should_log) {
-      rgw_log_op(store, nullptr /* !rest */, s,
-		 (op ? op->name() : "unknown"), olog);
+      rgw_log_op(store, s, (op ? op->name() : "unknown"), olog);
     }
 
     int http_ret = s->err.http_ret;
@@ -352,6 +349,12 @@ namespace rgw {
       goto done;
     }
 
+    /* FIXME: remove this after switching all handlers to the new authentication
+     * infrastructure. */
+    if (! s->auth_identity) {
+      s->auth_identity = rgw_auth_transform_old_authinfo(s);
+    }
+
     req->log(s, "reading op permissions");
     ret = req->read_permissions(op);
     if (ret < 0) {
@@ -378,6 +381,8 @@ namespace rgw {
     if (ret < 0) {
       if (s->system_request) {
 	dout(2) << "overriding permissions due to system operation" << dendl;
+      } else if (s->auth_identity->is_admin_of(s->user->user_id)) {
+	dout(2) << "overriding permissions due to admin operation" << dendl;
       } else {
 	abort_req(s, op, ret);
 	goto done;
@@ -440,10 +445,10 @@ namespace rgw {
     def_args.push_back("--keyring=$rgw_data/keyring");
     def_args.push_back("--log-file=/var/log/radosgw/$cluster-$name.log");
 
-    cct = global_init(&def_args, args,
-		      CEPH_ENTITY_TYPE_CLIENT,
-		      CODE_ENVIRONMENT_DAEMON,
-		      CINIT_FLAG_UNPRIVILEGED_DAEMON_DEFAULTS);
+    global_init(&def_args, args,
+		CEPH_ENTITY_TYPE_CLIENT,
+		CODE_ENVIRONMENT_DAEMON,
+		CINIT_FLAG_UNPRIVILEGED_DAEMON_DEFAULTS);
 
     Mutex mutex("main");
     SafeTimer init_timer(g_ceph_context, mutex);
@@ -460,6 +465,7 @@ namespace rgw {
 
     store = RGWStoreManager::get_storage(g_ceph_context,
 					 g_conf->rgw_enable_gc_threads,
+					 g_conf->rgw_enable_lc_threads,
 					 g_conf->rgw_enable_quota_threads,
 					 g_conf->rgw_run_sync_thread);
 
@@ -550,7 +556,7 @@ namespace rgw {
     rgw_perf_stop(g_ceph_context);
 
     dout(1) << "final shutdown" << dendl;
-    cct.reset();
+    g_ceph_context->put();
 
     ceph::crypto::shutdown();
 
@@ -629,18 +635,7 @@ int librgw_create(librgw_t* rgw, int argc, char **argv)
     std::lock_guard<std::mutex> lg(librgw_mtx);
     if (! g_ceph_context) {
       vector<const char*> args;
-      std::vector<std::string> spl_args;
-      // last non-0 argument will be split and consumed
-      if (argc > 1) {
-	const std::string spl_arg{argv[(--argc)]};
-	get_str_vec(spl_arg, " \t", spl_args);
-      }
       argv_to_vec(argc, const_cast<const char**>(argv), args);
-      // append split args, if any
-      for (const auto& elt : spl_args) {
-	args.push_back(elt.c_str());
-      }
-      env_to_vec(args);
       rc = rgwlib.init(args);
     }
   }
