@@ -5,10 +5,8 @@
 
 #include <string>
 #include <map>
-#include <sstream>
 
 #include <boost/utility/string_ref.hpp>
-#include <boost/format.hpp>
 
 #include "common/errno.h"
 #include "common/ceph_json.h"
@@ -114,9 +112,7 @@ int rgw_read_user_buckets(RGWRados * store,
   buckets.clear();
   string buckets_obj_id;
   rgw_get_buckets_obj(user_id, buckets_obj_id);
-  bufferlist bl;
   rgw_obj obj(store->get_zone_params().user_uid_pool, buckets_obj_id);
-  bufferlist header;
   list<cls_user_bucket_entry> entries;
 
   bool truncated = false;
@@ -136,8 +132,9 @@ int rgw_read_user_buckets(RGWRados * store,
     if (ret < 0)
       return ret;
 
-    for (const auto& entry : entries) {
-      buckets.add(RGWBucketEnt(user_id, entry));
+    for (list<cls_user_bucket_entry>::iterator q = entries.begin(); q != entries.end(); ++q) {
+      RGWBucketEnt e(*q);
+      buckets.add(e);
       total++;
     }
 
@@ -206,7 +203,8 @@ int rgw_link_bucket(RGWRados *store, const rgw_user& user_id, rgw_bucket& bucket
   if (update_entrypoint) {
     ret = store->get_bucket_entrypoint_info(obj_ctx, tenant_name, bucket_name, ep, &ot, NULL, &attrs);
     if (ret < 0 && ret != -ENOENT) {
-      ldout(store->ctx(), 0) << "ERROR: store->get_bucket_entrypoint_info() returned " << ret << dendl;
+      ldout(store->ctx(), 0) << "ERROR: store->get_bucket_entrypoint_info() returned: "
+                             << cpp_strerror(-ret) << dendl;
     }
   }
 
@@ -217,7 +215,7 @@ int rgw_link_bucket(RGWRados *store, const rgw_user& user_id, rgw_bucket& bucket
   ret = store->cls_user_add_bucket(obj, new_bucket);
   if (ret < 0) {
     ldout(store->ctx(), 0) << "ERROR: error adding bucket to directory: "
-        << cpp_strerror(-ret)<< dendl;
+                           << cpp_strerror(-ret) << dendl;
     goto done_err;
   }
 
@@ -226,7 +224,6 @@ int rgw_link_bucket(RGWRados *store, const rgw_user& user_id, rgw_bucket& bucket
 
   ep.linked = true;
   ep.owner = user_id;
-  ep.bucket = bucket;
   ret = store->put_bucket_entrypoint_info(tenant_name, bucket_name, ep, false, ot, real_time(), &attrs);
   if (ret < 0)
     goto done_err;
@@ -235,7 +232,8 @@ int rgw_link_bucket(RGWRados *store, const rgw_user& user_id, rgw_bucket& bucket
 done_err:
   int r = rgw_unlink_bucket(store, user_id, bucket.tenant, bucket.name);
   if (r < 0) {
-    ldout(store->ctx(), 0) << "ERROR: failed unlinking bucket on error cleanup: " << cpp_strerror(-r) << dendl;
+    ldout(store->ctx(), 0) << "ERROR: failed unlinking bucket on error cleanup: "
+                           << cpp_strerror(-r) << dendl;
   }
   return ret;
 }
@@ -244,8 +242,6 @@ int rgw_unlink_bucket(RGWRados *store, const rgw_user& user_id, const string& te
 {
   int ret;
 
-  bufferlist bl;
-
   string buckets_obj_id;
   rgw_get_buckets_obj(user_id, buckets_obj_id);
 
@@ -280,11 +276,7 @@ int rgw_unlink_bucket(RGWRados *store, const rgw_user& user_id, const string& te
   }
 
   ep.linked = false;
-  ret = store->put_bucket_entrypoint_info(tenant_name, bucket_name, ep, false, ot, real_time(), &attrs);
-  if (ret < 0)
-    return ret;
-
-  return ret;
+  return store->put_bucket_entrypoint_info(tenant_name, bucket_name, ep, false, ot, real_time(), &attrs);
 }
 
 int rgw_bucket_store_info(RGWRados *store, const string& bucket_name, bufferlist& bl, bool exclusive,
@@ -431,9 +423,16 @@ int rgw_bucket_set_attrs(RGWRados *store, RGWBucketInfo& bucket_info,
 static void dump_mulipart_index_results(list<rgw_obj_key>& objs_to_unlink,
         Formatter *f)
 {
-  for (const auto& o : objs_to_unlink) {
-    f->dump_string("object",  o.name);
+  // make sure that an appropiately titled header has been opened previously
+  list<rgw_obj_key>::iterator oiter = objs_to_unlink.begin();
+
+  f->open_array_section("invalid_multipart_entries");
+
+  for ( ; oiter != objs_to_unlink.end(); ++oiter) {
+    f->dump_string("object",  oiter->name);
   }
+
+  f->close_section();
 }
 
 void check_bad_user_bucket_mapping(RGWRados *store, const rgw_user& user_id,
@@ -516,9 +515,7 @@ int rgw_remove_object(RGWRados *store, RGWBucketInfo& bucket_info, rgw_bucket& b
 
   rgw_obj obj(bucket, key);
 
-  int ret = store->delete_obj(rctx, bucket_info, obj, bucket_info.versioning_status());
-
-  return ret;
+  return store->delete_obj(rctx, bucket_info, obj, bucket_info.versioning_status());
 }
 
 int rgw_remove_bucket(RGWRados *store, rgw_bucket& bucket, bool delete_children)
@@ -548,23 +545,23 @@ int rgw_remove_bucket(RGWRados *store, rgw_bucket& bucket, bool delete_children)
 
   if (delete_children) {
     int max = 1000;
-    ret = list_op.list_objects(max, &objs, &common_prefixes, NULL);
-    if (ret < 0)
-      return ret;
 
-    while (!objs.empty()) {
+    do {
+      objs.clear();
+
+      ret = list_op.list_objects(max, &objs, &common_prefixes, NULL);
+      if (ret < 0)
+        return ret;
+
       std::vector<RGWObjEnt>::iterator it = objs.begin();
       for (; it != objs.end(); ++it) {
         ret = rgw_remove_object(store, info, bucket, (*it).key);
         if (ret < 0)
           return ret;
       }
-      objs.clear();
 
-      ret = list_op.list_objects(max, &objs, &common_prefixes, NULL);
-      if (ret < 0)
-        return ret;
-    }
+    } while (!objs.empty());
+
   }
 
   ret = rgw_bucket_sync_user_stats(store, bucket.tenant, bucket.name);
@@ -591,7 +588,7 @@ int rgw_remove_bucket(RGWRados *store, rgw_bucket& bucket, bool delete_children)
 static int aio_wait(librados::AioCompletion *handle)
 {
   librados::AioCompletion *c = (librados::AioCompletion *)handle;
-  c->wait_for_safe();
+  c->wait_for_complete();
   int ret = c->get_return_value();
   c->release();
   return ret;
@@ -894,7 +891,7 @@ int RGWBucket::link(RGWBucketAdminOpState& op_state, std::string *err_msg)
     rgw_obj obj_bucket_instance(bucket_instance, no_oid);
     r = store->system_obj_set_attr(NULL, obj_bucket_instance, RGW_ATTR_ACL, aclbl, &objv_tracker);
 
-    r = rgw_link_bucket(store, user_info.user_id, bucket_info.bucket, real_time());
+    r = rgw_link_bucket(store, user_info.user_id, bucket, real_time());
     if (r < 0)
       return r;
   }
@@ -978,8 +975,10 @@ static void dump_bucket_usage(map<RGWObjCategory, RGWStorageStats>& stats, Forma
     RGWStorageStats& s = iter->second;
     const char *cat_name = rgw_obj_category_name(iter->first);
     formatter->open_object_section(cat_name);
-    formatter->dump_int("size_kb", s.num_kb);
-    formatter->dump_int("size_kb_actual", s.num_kb_rounded);
+    formatter->dump_int("size", s.size);
+    formatter->dump_int("size_actual", s.size_rounded);
+    formatter->dump_int("size_kb", rgw_rounded_kb(s.size));
+    formatter->dump_int("size_kb_actual", rgw_rounded_kb(s.size_rounded));
     formatter->dump_int("num_objects", s.num_objects);
     formatter->close_section();
   }
@@ -1001,17 +1000,16 @@ static void dump_index_check(map<RGWObjCategory, RGWStorageStats> existing_stats
 }
 
 int RGWBucket::check_bad_index_multipart(RGWBucketAdminOpState& op_state,
-					 RGWFormatterFlusher& flusher,
-					 std::string *err_msg)
+        list<rgw_obj_key>& objs_to_unlink, std::string *err_msg)
 {
   bool fix_index = op_state.will_fix_index();
   rgw_bucket bucket = op_state.get_bucket();
 
-  size_t max = 1000;
+  int max = 1000;
 
   map<string, bool> common_prefixes;
+  string ns = "";
 
-  string ns = "multipart";
   bool is_truncated;
   map<string, bool> meta_objs;
   map<rgw_obj_key, string> all_objs;
@@ -1028,7 +1026,6 @@ int RGWBucket::check_bad_index_multipart(RGWBucketAdminOpState& op_state,
   RGWRados::Bucket::List list_op(&target);
 
   list_op.params.list_versions = true;
-  list_op.params.ns = ns;
 
   do {
     vector<RGWObjEnt> result;
@@ -1071,11 +1068,6 @@ int RGWBucket::check_bad_index_multipart(RGWBucketAdminOpState& op_state,
 
   } while (is_truncated);
 
-  list<rgw_obj_key> objs_to_unlink;
-  Formatter *f =  flusher.get_formatter();
-
-  f->open_array_section("invalid_multipart_entries");
-
   map<rgw_obj_key, string>::iterator aiter;
   for (aiter = all_objs.begin(); aiter != all_objs.end(); ++aiter) {
     string& name = aiter->second;
@@ -1083,23 +1075,11 @@ int RGWBucket::check_bad_index_multipart(RGWBucketAdminOpState& op_state,
     if (meta_objs.find(name) == meta_objs.end()) {
       objs_to_unlink.push_back(aiter->first);
     }
-
-    if (objs_to_unlink.size() > max) {
-      if (fix_index) {
-	int r = store->remove_objs_from_index(bucket, objs_to_unlink);
-	if (r < 0) {
-	  set_err_msg(err_msg, "ERROR: remove_obj_from_index() returned error: "
-		      + cpp_strerror(-r));
-	  return r;
-	}
-      }
-
-      dump_mulipart_index_results(objs_to_unlink, flusher.get_formatter());
-      flusher.flush();
-      objs_to_unlink.clear();
-    }
   }
 
+  if (objs_to_unlink.empty())
+    return 0;
+
   if (fix_index) {
     int r = store->remove_objs_from_index(bucket, objs_to_unlink);
     if (r < 0) {
@@ -1110,10 +1090,6 @@ int RGWBucket::check_bad_index_multipart(RGWBucketAdminOpState& op_state,
     }
   }
 
-  dump_mulipart_index_results(objs_to_unlink, f);
-  f->close_section();
-  flusher.flush();
-
   return 0;
 }
 
@@ -1331,6 +1307,7 @@ int RGWBucketAdminOp::check_index(RGWRados *store, RGWBucketAdminOpState& op_sta
   map<string, RGWObjEnt> result;
   map<RGWObjCategory, RGWStorageStats> existing_stats;
   map<RGWObjCategory, RGWStorageStats> calculated_stats;
+  list<rgw_obj_key> objs_to_unlink;
 
   RGWBucket bucket;
 
@@ -1341,10 +1318,13 @@ int RGWBucketAdminOp::check_index(RGWRados *store, RGWBucketAdminOpState& op_sta
   Formatter *formatter = flusher.get_formatter();
   flusher.start(0);
 
-  ret = bucket.check_bad_index_multipart(op_state, flusher);
+  ret = bucket.check_bad_index_multipart(op_state, objs_to_unlink);
   if (ret < 0)
     return ret;
 
+  dump_mulipart_index_results(objs_to_unlink, formatter);
+  flusher.flush();
+
   ret = bucket.check_object_index(op_state, result);
   if (ret < 0)
     return ret;
@@ -1371,12 +1351,7 @@ int RGWBucketAdminOp::remove_bucket(RGWRados *store, RGWBucketAdminOpState& op_s
   if (ret < 0)
     return ret;
 
-  std::string err_msg;
-  ret = bucket.remove(op_state, bypass_gc, keep_index_consistent, &err_msg);
-  if (!err_msg.empty()) {
-    lderr(store->ctx()) << "ERROR: " << err_msg << dendl;
-  }
-  return ret;
+  return bucket.remove(op_state, bypass_gc, keep_index_consistent);
 }
 
 int RGWBucketAdminOp::remove_object(RGWRados *store, RGWBucketAdminOpState& op_state)
@@ -1432,127 +1407,6 @@ static int bucket_stats(RGWRados *store, const std::string& tenant_name, std::st
   return 0;
 }
 
-int RGWBucketAdminOp::limit_check(RGWRados *store,
-				  RGWBucketAdminOpState& op_state,
-				  const std::list<std::string>& user_ids,
-				  RGWFormatterFlusher& flusher,
-				  bool warnings_only)
-{
-  int ret = 0;
-  const size_t max_entries =
-    store->ctx()->_conf->rgw_list_buckets_max_chunk;
-
-  const size_t safe_max_objs_per_shard =
-    store->ctx()->_conf->rgw_safe_max_objects_per_shard;
-
-  uint16_t shard_warn_pct =
-    store->ctx()->_conf->rgw_shard_warning_threshold;
-  if (shard_warn_pct > 100)
-    shard_warn_pct = 90;
-
-  Formatter *formatter = flusher.get_formatter();
-  flusher.start(0);
-
-  formatter->open_array_section("users");
-
-  for (const auto& user_id : user_ids) {
-    formatter->open_object_section("user");
-    formatter->dump_string("user_id", user_id);
-    bool done;
-    formatter->open_array_section("buckets");
-    do {
-      RGWUserBuckets buckets;
-      string marker;
-      bool is_truncated;
-
-      ret = rgw_read_user_buckets(store, user_id, buckets,
-				  marker, string(), max_entries, false,
-				  &is_truncated);
-      if (ret < 0)
-        return ret;
-
-      map<string, RGWBucketEnt>& m_buckets = buckets.get_buckets();
-
-      for (auto& iter : m_buckets) {
-	auto& bucket = iter.second.bucket;
-	uint32_t num_shards = 1;
-	uint64_t num_objects = 0;
-
-	/* need info for num_shards */
-	RGWBucketInfo info;
-	RGWObjectCtx obj_ctx(store);
-
-	marker = bucket.name; /* Casey's location for marker update,
-			       * as we may now not reach the end of
-			       * the loop body */
-
-	ret = store->get_bucket_info(obj_ctx, bucket.tenant, bucket.name,
-				     info, nullptr);
-	if (ret < 0)
-	  continue;
-
-	/* need stats for num_entries */
-	string bucket_ver, master_ver;
-	std::map<RGWObjCategory, RGWStorageStats> stats;
-	ret = store->get_bucket_stats(bucket, RGW_NO_SHARD, &bucket_ver,
-				      &master_ver, stats, nullptr);
-
-	if (ret < 0)
-	  continue;
-
-	for (const auto& s : stats) {
-	    num_objects += s.second.num_objects;
-	}
-
-	num_shards = info.num_shards;
-	uint64_t objs_per_shard =
-	  (num_shards) ? num_objects/num_shards : num_objects;
-	{
-	  bool warn = false;
-	  stringstream ss;
-	  if (objs_per_shard > safe_max_objs_per_shard) {
-	    double over =
-	      100 - (safe_max_objs_per_shard/objs_per_shard * 100);
-	      ss << boost::format("OVER %4f%%") % over;
-	      warn = true;
-	  } else {
-	    double fill_pct =
-	      objs_per_shard / safe_max_objs_per_shard * 100;
-	    if (fill_pct >= shard_warn_pct) {
-	      ss << boost::format("WARN %4f%%") % fill_pct;
-	      warn = true;
-	    } else {
-	      ss << "OK";
-	    }
-	  }
-
-	  if (warn || (! warnings_only)) {
-	    formatter->open_object_section("bucket");
-	    formatter->dump_string("bucket", bucket.name);
-	    formatter->dump_string("tenant", bucket.tenant);
-	    formatter->dump_int("num_objects", num_objects);
-	    formatter->dump_int("num_shards", num_shards);
-	    formatter->dump_int("objects_per_shard", objs_per_shard);
-	    formatter->dump_string("fill_status", ss.str());
-	    formatter->close_section();
-	  }
-	}
-      }
-
-      done = (m_buckets.size() < max_entries);
-    } while (!done); /* foreach: bucket */
-
-    formatter->close_section();
-    formatter->close_section();
-    formatter->flush(cout);
-
-  } /* foreach: user_id */
-
-  formatter->close_section();
-  formatter->flush(cout);
-
-  return ret;
-} /* RGWBucketAdminOp::limit_check */
 
 int RGWBucketAdminOp::info(RGWRados *store, RGWBucketAdminOpState& op_state,
                   RGWFormatterFlusher& flusher)
@@ -1573,7 +1427,7 @@ int RGWBucketAdminOp::info(RGWRados *store, RGWBucketAdminOpState& op_state,
 
   CephContext *cct = store->ctx();
 
-  const size_t max_entries = cct->_conf->rgw_list_buckets_max_chunk;
+  size_t max_entries = cct->_conf->rgw_list_buckets_max_chunk;
 
   bool show_stats = op_state.will_fetch_stats();
   rgw_user user_id = op_state.get_user_id();
@@ -1885,8 +1739,6 @@ int RGWDataChangesLog::list_entries(int shard, const real_time& start_time, cons
 				    const string& marker,
 				    string *out_marker,
 				    bool *truncated) {
-  if (shard >= num_shards)
-    return -EINVAL;
 
   list<cls_log_entry> log_entries;
 
@@ -1943,7 +1795,7 @@ int RGWDataChangesLog::list_entries(const real_time& start_time, const real_time
 
 int RGWDataChangesLog::get_info(int shard_id, RGWDataChangesLogInfo *info)
 {
-  if (shard_id >= num_shards)
+  if (shard_id > num_shards)
     return -EINVAL;
 
   string oid = oids[shard_id];
@@ -1970,7 +1822,7 @@ int RGWDataChangesLog::trim_entries(int shard_id, const real_time& start_time, c
 
   ret = store->time_log_trim(oids[shard_id], start_time, end_time, start_marker, end_marker);
 
-  if (ret == -ENOENT || ret == -ENODATA)
+  if (ret == -ENOENT)
     ret = 0;
 
   return ret;
@@ -1981,7 +1833,7 @@ int RGWDataChangesLog::trim_entries(const real_time& start_time, const real_time
 {
   for (int shard = 0; shard < num_shards; shard++) {
     int ret = store->time_log_trim(oids[shard], start_time, end_time, start_marker, end_marker);
-    if (ret == -ENOENT || ret == -ENODATA) {
+    if (ret == -ENOENT) {
       continue;
     }
     if (ret < 0)
@@ -2151,7 +2003,7 @@ public:
       return ret;
 
     /*
-     * We're unlinking the bucket but we don't want to update the entrypoint here - we're removing
+     * We're unlinking the bucket but we don't want to update the entrypoint here — we're removing
      * it immediately and don't want to invalidate our cached objv_version or the bucket obj removal
      * will incorrectly fail.
      */
@@ -2290,6 +2142,7 @@ public:
       bci.info.bucket.data_pool = old_bci.info.bucket.data_pool;
       bci.info.bucket.index_pool = old_bci.info.bucket.index_pool;
       bci.info.bucket.data_extra_pool = old_bci.info.bucket.data_extra_pool;
+      bci.info.index_type = old_bci.info.index_type;
     }
 
     // are we actually going to perform this put, or is it too old?
