@@ -126,7 +126,7 @@ public:
   /**
    * Add a (created) bucket to the user's bucket list.
    */
-  void add(const RGWBucketEnt& bucket) {
+  void add(RGWBucketEnt& bucket) {
     buckets[bucket.bucket.name] = bucket;
   }
 
@@ -274,8 +274,7 @@ public:
   int init(RGWRados *storage, RGWBucketAdminOpState& op_state);
 
   int check_bad_index_multipart(RGWBucketAdminOpState& op_state,
-				RGWFormatterFlusher& flusher,
-				std::string *err_msg = NULL);
+          list<rgw_obj_key>& objs_to_unlink, std::string *err_msg = NULL);
 
   int check_object_index(RGWBucketAdminOpState& op_state,
           map<string, RGWObjEnt> result, std::string *err_msg = NULL);
@@ -315,10 +314,6 @@ public:
   static int remove_bucket(RGWRados *store, RGWBucketAdminOpState& op_state, bool bypass_gc = false, bool keep_index_consistent = true);
   static int remove_object(RGWRados *store, RGWBucketAdminOpState& op_state);
   static int info(RGWRados *store, RGWBucketAdminOpState& op_state, RGWFormatterFlusher& flusher);
-  static int limit_check(RGWRados *store, RGWBucketAdminOpState& op_state,
-			 const std::list<std::string>& user_ids,
-			 RGWFormatterFlusher& flusher,
-			 bool warnings_only = false);
 };
 
 
