@@ -7,7 +7,6 @@
 #endif
 
 #include <boost/asio.hpp>
-#include <boost/intrusive_ptr.hpp>
 
 #ifdef NEED_ASSERT_H
 #pragma pop_macro("_ASSERT_H")
@@ -31,8 +30,7 @@ class RGWAioCompletionNotifier;
 class RGWCompletionManager : public RefCountedObject {
   CephContext *cct;
   list<void *> complete_reqs;
-  using NotifierRef = boost::intrusive_ptr<RGWAioCompletionNotifier>;
-  set<NotifierRef> cns;
+  set<RGWAioCompletionNotifier *> cns;
 
   Mutex lock;
   Cond cond;
@@ -43,17 +41,7 @@ class RGWCompletionManager : public RefCountedObject {
 
   map<void *, void *> waiters;
 
-  class WaitContext : public Context {
-    RGWCompletionManager *manager;
-    void *opaque;
-  public:
-    WaitContext(RGWCompletionManager *_cm, void *_opaque) : manager(_cm), opaque(_opaque) {}
-    void finish(int r) {
-      manager->_wakeup(opaque);
-    }
-  };
-
-  friend class WaitContext;
+  class WaitContext;
 
 protected:
   void _wakeup(void *opaque);
