@@ -11,6 +11,8 @@
 #include <errno.h>
 #include <signal.h>
 
+#include <curl/curl.h>
+
 #include <boost/intrusive_ptr.hpp>
 
 #include "acconfig.h"
@@ -44,7 +46,6 @@
 #include "rgw_rest_config.h"
 #include "rgw_rest_realm.h"
 #include "rgw_swift_auth.h"
-#include "rgw_swift.h"
 #include "rgw_log.h"
 #include "rgw_tools.h"
 #include "rgw_resolve.h"
@@ -52,7 +53,6 @@
 #include "rgw_request.h"
 #include "rgw_process.h"
 #include "rgw_frontend.h"
-#include "rgw_http_client_curl.h"
 
 #include <map>
 #include <string>
@@ -61,10 +61,6 @@
 #include "include/types.h"
 #include "common/BackTrace.h"
 
-#ifdef HAVE_SYS_PRCTL_H
-#include <sys/prctl.h>
-#endif
-
 #define dout_subsys ceph_subsys_rgw
 
 using namespace std;
@@ -82,9 +78,8 @@ void signal_shutdown()
     int val = 0;
     int ret = write(signal_fd[0], (char *)&val, sizeof(val));
     if (ret < 0) {
-      int err = -errno;
       derr << "ERROR: " << __func__ << ": write() returned "
-	   << cpp_strerror(-err) << dendl;
+           << cpp_strerror(errno) << dendl;
     }
   }
 }
@@ -134,6 +129,17 @@ static void godown_alarm(int signum)
   _exit(0);
 }
 
+#ifdef HAVE_CURL_MULTI_WAIT
+static void check_curl()
+{
+}
+#else
+static void check_curl()
+{
+  derr << "WARNING: libcurl doesn't support curl_multi_wait()" << dendl;
+  derr << "WARNING: cross zone / region transfer performance may be affected" << dendl;
+}
+#endif
 
 class C_InitTimeout : public Context {
 public:
@@ -166,6 +172,21 @@ static RGWRESTMgr *set_logging(RGWRESTMgr *mgr)
   return mgr;
 }
 
+void intrusive_ptr_add_ref(CephContext* cct) { cct->get(); }
+void intrusive_ptr_release(CephContext* cct) { cct->put(); }
+
+RGWRealmReloader *preloader = NULL;
+
+static void reloader_handler(int signum)
+{
+  if (preloader) {
+    bufferlist bl;
+    bufferlist::iterator p = bl.begin();
+    preloader->handle_notify(RGWRealmNotify::Reload, p);
+  }
+  sighup_handler(signum);
+}
+
 
 /*
  * start up the RADOS connection and then handle HTTP messages as they come in
@@ -178,7 +199,7 @@ int main(int argc, const char **argv)
   if (TEMP_FAILURE_RETRY(dup2(STDOUT_FILENO, STDERR_FILENO) < 0)) {
     int err = errno;
     cout << "failed to redirect stderr to stdout: " << cpp_strerror(err)
-	 << std::endl;
+         << std::endl;
     return ENOSYS;
   }
 
@@ -194,7 +215,7 @@ int main(int argc, const char **argv)
   // First, let's determine which frontends are configured.
   int flags = CINIT_FLAG_UNPRIVILEGED_DAEMON_DEFAULTS;
   global_pre_init(&def_args, args, CEPH_ENTITY_TYPE_CLIENT, CODE_ENVIRONMENT_DAEMON,
-		  flags);
+          flags);
 
   list<string> frontends;
   get_str_list(g_conf->rgw_frontends, ",", frontends);
@@ -211,11 +232,13 @@ int main(int argc, const char **argv)
       // dropping permissions by setting the appropriate flag.
       flags |= CINIT_FLAG_DEFER_DROP_PRIVILEGES;
       if (f.find("port") != string::npos) {
-	// check for the most common ws problems
-	if ((f.find("port=") == string::npos) ||
-	    (f.find("port= ") != string::npos)) {
-	  derr << "WARNING: civetweb frontend config found unexpected spacing around 'port' (ensure civetweb port parameter has the form 'port=80' with no spaces before or after '=')" << dendl;
-	}
+        // check for the most common ws problems
+        if ((f.find("port=") == string::npos) ||
+            (f.find("port= ") != string::npos)) {
+          derr << "WARNING: civetweb frontend config found unexpected spacing around 'port' "
+               << "(ensure civetweb port parameter has the form 'port=80' with no spaces "
+               << "before or after '=')" << dendl;
+        }
       }
     }
 
@@ -235,10 +258,8 @@ int main(int argc, const char **argv)
   // Now that we've determined which frontend(s) to use, continue with global
   // initialization. Passing false as the final argument ensures that
   // global_pre_init() is not invoked twice.
-  // claim the reference and release it after subsequent destructors have fired
-  auto cct = global_init(&def_args, args, CEPH_ENTITY_TYPE_CLIENT,
-			 CODE_ENVIRONMENT_DAEMON,
-			 flags, "rgw_data", false);
+  global_init(&def_args, args, CEPH_ENTITY_TYPE_CLIENT, CODE_ENVIRONMENT_DAEMON,
+          flags, "rgw_data", false);
 
   for (std::vector<const char*>::iterator i = args.begin(); i != args.end(); ++i) {
     if (ceph_argparse_flag(args, i, "-h", "--help", (char*)NULL)) {
@@ -247,19 +268,7 @@ int main(int argc, const char **argv)
     }
   }
 
-  // maintain existing region root pool for new multisite objects
-  if (!g_conf->rgw_region_root_pool.empty()) {
-    const char *root_pool = g_conf->rgw_region_root_pool.c_str();
-    if (g_conf->rgw_zonegroup_root_pool.empty()) {
-      g_conf->set_val_or_die("rgw_zonegroup_root_pool", root_pool);
-    }
-    if (g_conf->rgw_period_root_pool.empty()) {
-      g_conf->set_val_or_die("rgw_period_root_pool", root_pool);
-    }
-    if (g_conf->rgw_realm_root_pool.empty()) {
-      g_conf->set_val_or_die("rgw_realm_root_pool", root_pool);
-    }
-  }
+  check_curl();
 
   if (g_conf->daemonize) {
     global_init_daemonize(g_ceph_context);
@@ -276,17 +285,23 @@ int main(int argc, const char **argv)
 
   common_init_finish(g_ceph_context);
 
-  rgw_tools_init(g_ceph_context);
+  // claim the reference and release it after subsequent destructors have fired
+  boost::intrusive_ptr<CephContext> cct(g_ceph_context, false);
+
+  int r = rgw_tools_init(g_ceph_context);
+  if (r < 0) {
+    derr << "ERROR: unable to initialize rgw tools" << dendl;
+    return -r;
+  }
 
   rgw_init_resolver();
-
-  rgw::curl::setup_curl(fe_map);
-
+  
+  curl_global_init(CURL_GLOBAL_ALL);
+  
   FCGX_Init();
 
-  int r = 0;
   RGWRados *store = RGWStoreManager::get_storage(g_ceph_context,
-      g_conf->rgw_enable_gc_threads, g_conf->rgw_enable_quota_threads,
+      g_conf->rgw_enable_gc_threads, g_conf->rgw_enable_lc_threads, g_conf->rgw_enable_quota_threads,
       g_conf->rgw_run_sync_thread);
   if (!store) {
     mutex.Lock();
@@ -298,6 +313,10 @@ int main(int argc, const char **argv)
     return EIO;
   }
   r = rgw_perf_start(g_ceph_context);
+  if (r < 0) {
+    derr << "ERROR: failed starting rgw perf" << dendl;
+    return -r;
+  }
 
   rgw_rest_init(g_ceph_context, store, store->get_zonegroup());
 
@@ -306,9 +325,6 @@ int main(int argc, const char **argv)
   init_timer.shutdown();
   mutex.Unlock();
 
-  if (r) 
-    return 1;
-
   rgw_user_init(store);
   rgw_bucket_init(store->meta_mgr);
   rgw_log_usage_init(g_ceph_context, store);
@@ -316,7 +332,6 @@ int main(int argc, const char **argv)
   RGWREST rest;
 
   list<string> apis;
-  bool do_swift = false;
 
   get_str_list(g_conf->rgw_enable_apis, apis);
 
@@ -340,9 +355,6 @@ int main(int argc, const char **argv)
   }
 
   if (apis_map.count("swift") > 0) {
-    do_swift = true;
-    swift_init(g_ceph_context);
-
     RGWRESTMgr_SWIFT* const swift_resource = new RGWRESTMgr_SWIFT;
 
     if (! g_conf->rgw_cross_domain_policy.empty()) {
@@ -372,7 +384,7 @@ int main(int argc, const char **argv)
 
   if (apis_map.count("swift_auth") > 0) {
     rest.register_resource(g_conf->rgw_swift_auth_entry,
-			   set_logging(new RGWRESTMgr_SWIFT_Auth));
+               set_logging(new RGWRESTMgr_SWIFT_Auth));
   }
 
   if (apis_map.count("admin") > 0) {
@@ -391,9 +403,6 @@ int main(int argc, const char **argv)
     rest.register_resource(g_conf->rgw_admin_entry, admin_resource);
   }
 
-  /* Header custom behavior */
-  rest.register_x_headers(g_conf->rgw_log_http_headers);
-
   OpsLogSocket *olog = NULL;
 
   if (!g_conf->rgw_ops_log_socket_path.empty()) {
@@ -408,7 +417,7 @@ int main(int argc, const char **argv)
   }
 
   init_async_signal_handler();
-  register_async_signal_handler(SIGHUP, sighup_handler);
+  register_async_signal_handler(SIGHUP, reloader_handler);
   register_async_signal_handler(SIGTERM, handle_sigterm);
   register_async_signal_handler(SIGINT, handle_sigterm);
   register_async_signal_handler(SIGUSR1, handle_sigterm);
@@ -426,7 +435,10 @@ int main(int argc, const char **argv)
 
       fe = new RGWFCGXFrontend(fcgi_pe, config);
     } else if (framework == "civetweb" || framework == "mongoose") {
-      RGWProcessEnv env = { store, &rest, olog, 0 };
+      int port;
+      config->get_val("port", 80, &port);
+
+      RGWProcessEnv env = { store, &rest, olog, port };
 
       fe = new RGWMongooseFrontend(env, config);
     } else if (framework == "loadgen") {
@@ -460,16 +472,12 @@ int main(int argc, const char **argv)
   RGWFrontendPauser pauser(fes, &pusher);
   RGWRealmReloader reloader(store, &pauser);
 
+  preloader = &reloader;
+
   RGWRealmWatcher realm_watcher(g_ceph_context, store->realm);
   realm_watcher.add_watcher(RGWRealmNotify::Reload, reloader);
   realm_watcher.add_watcher(RGWRealmNotify::ZonesNeedPeriod, pusher);
 
-#if defined(HAVE_SYS_PRCTL_H)
-  if (prctl(PR_SET_DUMPABLE, 1) == -1) {
-    cerr << "warning: unable to set dumpable flag: " << cpp_strerror(errno) << std::endl;
-  }
-#endif
-
   wait_shutdown();
 
   derr << "shutting down" << dendl;
@@ -493,16 +501,12 @@ int main(int argc, const char **argv)
     delete fec;
   }
 
-  unregister_async_signal_handler(SIGHUP, sighup_handler);
+  unregister_async_signal_handler(SIGHUP, reloader_handler);
   unregister_async_signal_handler(SIGTERM, handle_sigterm);
   unregister_async_signal_handler(SIGINT, handle_sigterm);
   unregister_async_signal_handler(SIGUSR1, handle_sigterm);
   shutdown_async_signal_handler();
 
-  if (do_swift) {
-    swift_finalize();
-  }
-
   rgw_log_usage_finalize();
 
   delete olog;
@@ -511,7 +515,7 @@ int main(int argc, const char **argv)
 
   rgw_tools_cleanup();
   rgw_shutdown_resolver();
-  rgw::curl::cleanup_curl();
+  curl_global_cleanup();
 
   rgw_perf_stop(g_ceph_context);
 
