@@ -77,9 +77,6 @@ void RGWOp_MDLog_List::execute() {
       http_ret = -EINVAL;
       return;
     }
-    if (max_entries > LOG_CLASS_LIST_MAX_ENTRIES) {
-      max_entries = LOG_CLASS_LIST_MAX_ENTRIES;
-    }
   } 
 
   if (period.empty()) {
@@ -96,8 +93,15 @@ void RGWOp_MDLog_List::execute() {
 
   meta_log.init_list_entries(shard_id, ut_st, ut_et, marker, &handle);
 
-  http_ret = meta_log.list_entries(handle, max_entries, entries,
-                                   &last_marker, &truncated);
+  do {
+    http_ret = meta_log.list_entries(handle, max_entries, entries,
+				     &last_marker, &truncated);
+    if (http_ret < 0) 
+      break;
+
+    if (!max_entries_str.empty()) 
+      max_entries -= entries.size();
+  } while (truncated && (max_entries > 0));
 
   meta_log.complete_list_entries(handle);
 }
@@ -598,16 +602,20 @@ void RGWOp_DATALog_List::execute() {
       http_ret = -EINVAL;
       return;
     }
-    if (max_entries > LOG_CLASS_LIST_MAX_ENTRIES) {
-      max_entries = LOG_CLASS_LIST_MAX_ENTRIES;
-    }
-  }
+  } 
+  
+  do {
+    // Note that last_marker is updated to be the marker of the last
+    // entry listed
+    http_ret = store->data_log->list_entries(shard_id, ut_st, ut_et, 
+					     max_entries, entries, marker,
+					     &last_marker, &truncated);
+    if (http_ret < 0) 
+      break;
 
-  // Note that last_marker is updated to be the marker of the last
-  // entry listed
-  http_ret = store->data_log->list_entries(shard_id, ut_st, ut_et,
-                                           max_entries, entries, marker,
-                                           &last_marker, &truncated);
+    if (!max_entries_str.empty()) 
+      max_entries -= entries.size();
+  } while (truncated && (max_entries > 0));
 }
 
 void RGWOp_DATALog_List::send_response() {
@@ -854,7 +862,8 @@ void RGWOp_MDLog_Status::execute()
     http_ret = -ENOENT;
     return;
   }
-  http_ret = sync->read_sync_status(&status);
+  http_ret = sync->read_sync_status();
+  status = sync->get_sync_status();
 }
 
 void RGWOp_MDLog_Status::send_response()
