@@ -32,18 +32,23 @@ void RGWOp_User_Info::execute()
 
   std::string uid_str;
   bool fetch_stats;
-  bool sync_stats;
 
   RESTArgs::get_string(s, "uid", uid_str, &uid_str);
+
+  // if uid was not supplied in rest argument, error out now, otherwise we'll
+  // end up initializing anonymous user, for which keys.init will eventually
+  // return -EACESS
+  if (uid_str.empty()){
+    http_ret=-EINVAL;
+    return;
+  }
+
   rgw_user uid(uid_str);
 
   RESTArgs::get_bool(s, "stats", false, &fetch_stats);
 
-  RESTArgs::get_bool(s, "sync", false, &sync_stats);
-
   op_state.set_user_id(uid);
   op_state.set_fetch_stats(fetch_stats);
-  op_state.set_sync_stats(sync_stats);
 
   http_ret = RGWUserAdminOp_User::info(store, op_state, flusher);
 }
@@ -156,7 +161,7 @@ void RGWOp_User_Create::execute()
   }
 
   if (s->cct->_conf->rgw_bucket_default_quota_max_size >= 0) {
-    bucket_quota.max_size_kb = s->cct->_conf->rgw_bucket_default_quota_max_size;
+    bucket_quota.max_size = s->cct->_conf->rgw_bucket_default_quota_max_size;
     bucket_quota.enabled = true;
   }
 
@@ -166,7 +171,7 @@ void RGWOp_User_Create::execute()
   }
 
   if (s->cct->_conf->rgw_user_default_quota_max_size >= 0) {
-    user_quota.max_size_kb = s->cct->_conf->rgw_user_default_quota_max_size;
+    user_quota.max_size = s->cct->_conf->rgw_user_default_quota_max_size;
     user_quota.enabled = true;
   }
 
@@ -209,7 +214,6 @@ void RGWOp_User_Modify::execute()
   bool suspended;
   bool system;
 
-  bool quota_set;
   int32_t max_buckets;
 
   RGWUserAdminOpState op_state;
@@ -224,7 +228,7 @@ void RGWOp_User_Modify::execute()
   RESTArgs::get_string(s, "user-caps", caps, &caps);
   RESTArgs::get_bool(s, "generate-key", false, &gen_key);
   RESTArgs::get_bool(s, "suspended", false, &suspended);
-  RESTArgs::get_int32(s, "max-buckets", RGW_DEFAULT_MAX_BUCKETS, &max_buckets, &quota_set);
+  RESTArgs::get_int32(s, "max-buckets", RGW_DEFAULT_MAX_BUCKETS, &max_buckets);
   RESTArgs::get_string(s, "key-type", key_type_str, &key_type_str);
 
   RESTArgs::get_bool(s, "system", false, &system);
@@ -253,7 +257,7 @@ void RGWOp_User_Modify::execute()
   if (!secret_key.empty())
     op_state.set_secret_key(secret_key);
 
-  if (quota_set)
+  if (max_buckets != RGW_DEFAULT_MAX_BUCKETS)
     op_state.set_max_buckets(max_buckets);
 
   if (gen_key)
@@ -332,13 +336,11 @@ void RGWOp_Subuser_Create::execute()
   std::string uid_str;
   std::string subuser;
   std::string secret_key;
-  std::string access_key;
   std::string perm_str;
   std::string key_type_str;
 
   bool gen_subuser = false; // FIXME placeholder
   bool gen_secret;
-  bool gen_access;
 
   uint32_t perm_mask = 0;
   int32_t key_type = KEY_TYPE_SWIFT;
@@ -349,14 +351,12 @@ void RGWOp_Subuser_Create::execute()
   rgw_user uid(uid_str);
 
   RESTArgs::get_string(s, "subuser", subuser, &subuser);
-  RESTArgs::get_string(s, "access-key", access_key, &access_key);
   RESTArgs::get_string(s, "secret-key", secret_key, &secret_key);
   RESTArgs::get_string(s, "access", perm_str, &perm_str);
   RESTArgs::get_string(s, "key-type", key_type_str, &key_type_str);
   //RESTArgs::get_bool(s, "generate-subuser", false, &gen_subuser);
   RESTArgs::get_bool(s, "generate-secret", false, &gen_secret);
-  RESTArgs::get_bool(s, "gen-access-key", false, &gen_access);
-  
+
   perm_mask = rgw_str_to_perm(perm_str.c_str());
   op_state.set_perm(perm_mask);
 
@@ -367,17 +367,11 @@ void RGWOp_Subuser_Create::execute()
   if (!subuser.empty())
     op_state.set_subuser(subuser);
 
-  if (!access_key.empty())
-    op_state.set_access_key(access_key);
-  
   if (!secret_key.empty())
     op_state.set_secret_key(secret_key);
 
   op_state.set_generate_subuser(gen_subuser);
 
-  if (gen_access)
-    op_state.set_gen_access();
-
   if (gen_secret)
     op_state.set_gen_secret();
 
@@ -928,8 +922,11 @@ void RGWOp_Quota_Set::execute()
         old_quota = &info.bucket_quota;
       }
 
+      int64_t old_max_size_kb = rgw_rounded_kb(old_quota->max_size);
+      int64_t max_size_kb;
       RESTArgs::get_int64(s, "max-objects", old_quota->max_objects, &quota.max_objects);
-      RESTArgs::get_int64(s, "max-size-kb", old_quota->max_size_kb, &quota.max_size_kb);
+      RESTArgs::get_int64(s, "max-size-kb", old_max_size_kb, &max_size_kb);
+      quota.max_size = max_size_kb * 1024;
       RESTArgs::get_bool(s, "enabled", old_quota->enabled, &quota.enabled);
     }
 
