@@ -40,18 +40,13 @@ int rgw_put_system_obj(RGWRados *rgwstore, rgw_bucket& bucket, const string& oid
 
 int rgw_get_system_obj(RGWRados *rgwstore, RGWObjectCtx& obj_ctx, rgw_bucket& bucket, const string& key, bufferlist& bl,
                        RGWObjVersionTracker *objv_tracker, real_time *pmtime, map<string, bufferlist> *pattrs,
-                       rgw_cache_entry_info *cache_info, boost::optional<obj_version> refresh_version)
+                       rgw_cache_entry_info *cache_info)
 {
   struct rgw_err err;
   bufferlist::iterator iter;
   int request_len = READ_CHUNK_LEN;
   rgw_obj obj(bucket, key);
 
-  obj_version original_readv;
-  if (objv_tracker && !objv_tracker->read_version.empty()) {
-    original_readv = objv_tracker->read_version;
-  }
-
   do {
     RGWRados::SystemObject source(rgwstore, obj_ctx, obj);
     RGWRados::SystemObject::Read rop(&source);
@@ -66,17 +61,9 @@ int rgw_get_system_obj(RGWRados *rgwstore, RGWObjectCtx& obj_ctx, rgw_bucket& bu
 
     rop.read_params.cache_info = cache_info;
 
-    ret = rop.read(0, request_len - 1, bl, objv_tracker, refresh_version);
+    ret = rop.read(0, request_len - 1, bl, objv_tracker);
     if (ret == -ECANCELED) {
       /* raced, restart */
-      if (!original_readv.empty()) {
-        /* we were asked to read a specific obj_version, failed */
-        return ret;
-      }
-      if (objv_tracker) {
-        objv_tracker->read_version.clear();
-      }
-      source.invalidate_state();
       continue;
     }
     if (ret < 0)
@@ -136,7 +123,8 @@ static int ext_mime_map_init(CephContext *cct, const char *ext_map)
   int ret;
   if (fd < 0) {
     ret = -errno;
-    ldout(cct, 0) << "ext_mime_map_init(): failed to open file=" << ext_map << " ret=" << ret << dendl;
+    ldout(cct, 0) << __func__ << " failed to open file=" << ext_map
+                  << " : " << cpp_strerror(-ret) << dendl;
     return ret;
   }
 
@@ -144,21 +132,22 @@ static int ext_mime_map_init(CephContext *cct, const char *ext_map)
   ret = fstat(fd, &st);
   if (ret < 0) {
     ret = -errno;
-    ldout(cct, 0) << "ext_mime_map_init(): failed to stat file=" << ext_map << " ret=" << ret << dendl;
+    ldout(cct, 0) << __func__ << " failed to stat file=" << ext_map
+                  << " : " << cpp_strerror(-ret) << dendl;
     goto done;
   }
 
   buf = (char *)malloc(st.st_size + 1);
   if (!buf) {
     ret = -ENOMEM;
-    ldout(cct, 0) << "ext_mime_map_init(): failed to allocate buf" << dendl;
+    ldout(cct, 0) << __func__ << " failed to allocate buf" << dendl;
     goto done;
   }
 
   ret = safe_read(fd, buf, st.st_size + 1);
   if (ret != st.st_size) {
     // huh? file size has changed?
-    ldout(cct, 0) << "ext_mime_map_init(): raced! will retry.." << dendl;
+    ldout(cct, 0) << __func__ << " raced! will retry.." << dendl;
     free(buf);
     close(fd);
     return ext_mime_map_init(cct, ext_map);
