@@ -41,11 +41,6 @@ void rgw_get_anon_user(RGWUserInfo& info)
   info.access_keys.clear();
 }
 
-bool rgw_user_is_authenticated(RGWUserInfo& info)
-{
-  return (info.user_id.id != RGW_USER_ANON_ID);
-}
-
 int rgw_user_sync_all_stats(RGWRados *store, const rgw_user& user_id)
 {
   CephContext *cct = store->ctx();
@@ -100,8 +95,6 @@ int rgw_store_user_info(RGWRados *store,
                         bool exclusive,
                         map<string, bufferlist> *pattrs)
 {
-  bufferlist bl;
-  info.encode(bl);
   int ret;
   RGWObjVersionTracker ot;
 
@@ -215,8 +208,12 @@ struct user_info_entry {
 
 static RGWChainedCacheImpl<user_info_entry> uinfo_cache;
 
-int rgw_get_user_info_from_index(RGWRados *store, string& key, rgw_bucket& bucket, RGWUserInfo& info,
-                                 RGWObjVersionTracker *objv_tracker, real_time *pmtime)
+int rgw_get_user_info_from_index(RGWRados * const store,
+                                 const string& key,
+                                 rgw_bucket& bucket,
+                                 RGWUserInfo& info,
+                                 RGWObjVersionTracker * const objv_tracker,
+                                 real_time * const pmtime)
 {
   user_info_entry e;
   if (uinfo_cache.find(key, &e)) {
@@ -271,10 +268,10 @@ int rgw_get_user_info_from_index(RGWRados *store, string& key, rgw_bucket& bucke
 int rgw_get_user_info_by_uid(RGWRados *store,
                              const rgw_user& uid,
                              RGWUserInfo& info,
-                             RGWObjVersionTracker *objv_tracker,
-                             real_time *pmtime,
-                             rgw_cache_entry_info *cache_info,
-                             map<string, bufferlist> *pattrs)
+                             RGWObjVersionTracker * const objv_tracker,
+                             real_time * const pmtime,
+                             rgw_cache_entry_info * const cache_info,
+                             map<string, bufferlist> * const pattrs)
 {
   bufferlist bl;
   RGWUID user_id;
@@ -318,10 +315,15 @@ int rgw_get_user_info_by_email(RGWRados *store, string& email, RGWUserInfo& info
  * Given an swift username, finds the user_info associated with it.
  * returns: 0 on success, -ERR# on failure (including nonexistence)
  */
-extern int rgw_get_user_info_by_swift(RGWRados *store, string& swift_name, RGWUserInfo& info,
-                                      RGWObjVersionTracker *objv_tracker, real_time *pmtime)
+extern int rgw_get_user_info_by_swift(RGWRados * const store,
+                                      const string& swift_name,
+                                      RGWUserInfo& info,        /* out */
+                                      RGWObjVersionTracker * const objv_tracker,
+                                      real_time * const pmtime)
 {
-  return rgw_get_user_info_from_index(store, swift_name, store->get_zone_params().user_swift_pool, info, objv_tracker, pmtime);
+  return rgw_get_user_info_from_index(store, swift_name,
+                                      store->get_zone_params().user_swift_pool,
+                                      info, objv_tracker, pmtime);
 }
 
 /**
@@ -373,11 +375,9 @@ int rgw_remove_uid_index(RGWRados *store, rgw_user& uid)
 
 int rgw_remove_email_index(RGWRados *store, string& email)
 {
-  if (email.empty()) {
-    return 0;
-  }
   rgw_obj obj(store->get_zone_params().user_email_pool, email);
-  return store->delete_system_obj(obj);
+  int ret = store->delete_system_obj(obj);
+  return ret;
 }
 
 int rgw_remove_swift_name_index(RGWRados *store, string& swift_name)
@@ -445,11 +445,11 @@ int rgw_delete_user(RGWRados *store, RGWUserInfo& info, RGWObjVersionTracker& ob
     }
   }
 
+  rgw_obj email_obj(store->get_zone_params().user_email_pool, info.user_email);
   ldout(store->ctx(), 10) << "removing email index: " << info.user_email << dendl;
-  ret = rgw_remove_email_index(store, info.user_email);
+  ret = store->delete_system_obj(email_obj);
   if (ret < 0 && ret != -ENOENT) {
-    ldout(store->ctx(), 0) << "ERROR: could not remove email index object for "
-        << info.user_email << ", should be fixed (err=" << ret << ")" << dendl;
+    ldout(store->ctx(), 0) << "ERROR: could not remove " << info.user_id << ":" << email_obj << ", should be fixed (err=" << ret << ")" << dendl;
     return ret;
   }
 
@@ -1391,7 +1391,6 @@ int RGWSubUserPool::add(RGWUserAdminOpState& op_state, std::string *err_msg, boo
 {
   std::string subprocess_msg;
   int ret;
-  int32_t key_type = op_state.get_key_type();
 
   ret = check_op(op_state, &subprocess_msg);
   if (ret < 0) {
@@ -1399,10 +1398,6 @@ int RGWSubUserPool::add(RGWUserAdminOpState& op_state, std::string *err_msg, boo
     return ret;
   }
 
-  if (key_type == KEY_TYPE_S3 && op_state.get_access_key().empty()) {
-    op_state.set_gen_access();
-  }
-  
   if (op_state.get_secret_key().empty()) {
     op_state.set_gen_secret();
   }
@@ -1720,7 +1715,7 @@ int RGWUser::init(RGWUserAdminOpState& op_state)
 {
   bool found = false;
   std::string swift_user;
-  user_id = op_state.get_user_id();
+  rgw_user& uid = op_state.get_user_id();
   std::string user_email = op_state.get_user_email();
   std::string access_key = op_state.get_access_key();
   std::string subuser = op_state.get_subuser();
@@ -1735,16 +1730,16 @@ int RGWUser::init(RGWUserAdminOpState& op_state)
 
   clear_populated();
 
-  if (user_id.empty() && !subuser.empty()) {
+  if (uid.empty() && !subuser.empty()) {
     size_t pos = subuser.find(':');
     if (pos != string::npos) {
-      user_id = subuser.substr(0, pos);
-      op_state.set_user_id(user_id);
+      uid = subuser.substr(0, pos);
+      op_state.set_user_id(uid);
     }
   }
 
-  if (!user_id.empty() && (user_id.compare(RGW_USER_ANON_ID) != 0)) {
-    found = (rgw_get_user_info_by_uid(store, user_id, user_info, &op_state.objv) >= 0);
+  if (!uid.empty() && (uid.compare(RGW_USER_ANON_ID) != 0)) {
+    found = (rgw_get_user_info_by_uid(store, uid, user_info, &op_state.objv) >= 0);
     op_state.found_by_uid = found;
   }
   if (!user_email.empty() && !found) {
@@ -1769,9 +1764,7 @@ int RGWUser::init(RGWUserAdminOpState& op_state)
     set_populated();
   }
 
-  if (user_id.empty()) {
-    user_id = user_info.user_id;
-  }
+  user_id = user_info.user_id;
   op_state.set_initialized();
 
   // this may have been called by a helper object
@@ -1924,6 +1917,7 @@ int RGWUser::execute_add(RGWUserAdminOpState& op_state, std::string *err_msg)
   user_id = uid;
   user_info.user_id = user_id;
   user_info.display_name = display_name;
+  user_info.type = TYPE_RGW;
 
   if (!user_email.empty())
     user_info.user_email = user_email;
@@ -1936,6 +1930,7 @@ int RGWUser::execute_add(RGWUserAdminOpState& op_state, std::string *err_msg)
   }
 
   user_info.suspended = op_state.get_suspension_status();
+  user_info.admin = op_state.admin;
   user_info.system = op_state.system;
 
   if (op_state.op_mask_specified)
@@ -1949,7 +1944,7 @@ int RGWUser::execute_add(RGWUserAdminOpState& op_state, std::string *err_msg)
       user_info.bucket_quota.enabled = true;
     }
     if (cct->_conf->rgw_bucket_default_quota_max_size >= 0) {
-      user_info.bucket_quota.max_size_kb = cct->_conf->rgw_bucket_default_quota_max_size;
+      user_info.bucket_quota.max_size = cct->_conf->rgw_bucket_default_quota_max_size;
       user_info.bucket_quota.enabled = true;
     }
   }
@@ -1970,7 +1965,7 @@ int RGWUser::execute_add(RGWUserAdminOpState& op_state, std::string *err_msg)
       user_info.user_quota.enabled = true;
     }
     if (cct->_conf->rgw_user_default_quota_max_size >= 0) {
-      user_info.user_quota.max_size_kb = cct->_conf->rgw_user_default_quota_max_size;
+      user_info.user_quota.max_size = cct->_conf->rgw_user_default_quota_max_size;
       user_info.user_quota.enabled = true;
     }
   }
@@ -2173,6 +2168,9 @@ int RGWUser::execute_modify(RGWUserAdminOpState& op_state, std::string *err_msg)
   if (op_state.max_buckets_specified)
     user_info.max_buckets = op_state.get_max_buckets();
 
+  if (op_state.admin_specified)
+    user_info.admin = op_state.admin;
+
   if (op_state.system_specified)
     user_info.system = op_state.system;
 
@@ -2319,13 +2317,6 @@ int RGWUserAdminOp_User::info(RGWRados *store, RGWUserAdminOpState& op_state,
   if (ret < 0)
     return ret;
 
-  if (op_state.sync_stats) {
-    ret = rgw_user_sync_all_stats(store, info.user_id);
-    if (ret < 0) {
-      return ret;
-    }
-  }
-
   RGWStorageStats stats;
   RGWStorageStats *arg_stats = NULL;
   if (op_state.fetch_stats) {
