@@ -55,10 +55,6 @@ extern int rgw_user_sync_all_stats(RGWRados *store, const rgw_user& user_id);
  */
 extern void rgw_get_anon_user(RGWUserInfo& info);
 
-/**
- * verify that user is an actual user, and not the anonymous user
- */
-extern bool rgw_user_is_authenticated(RGWUserInfo& info);
 /**
  * Save the given user information to storage.
  * Returns: 0 on success, -ERR# on failure.
@@ -92,8 +88,11 @@ extern int rgw_get_user_info_by_email(RGWRados *store, string& email, RGWUserInf
  * Given an swift username, finds the user info associated with it.
  * returns: 0 on success, -ERR# on failure (including nonexistence)
  */
-extern int rgw_get_user_info_by_swift(RGWRados *store, string& swift_name, RGWUserInfo& info,
-                                      RGWObjVersionTracker *objv_tracker = NULL, real_time *pmtime = NULL);
+extern int rgw_get_user_info_by_swift(RGWRados *store,
+                                      const string& swift_name,
+                                      RGWUserInfo& info,        /* out */
+                                      RGWObjVersionTracker *objv_tracker = nullptr,
+                                      real_time *pmtime = nullptr);
 /**
  * Given an access key, finds the user info associated with it.
  * returns: 0 on success, -ERR# on failure (including nonexistence)
@@ -159,10 +158,10 @@ struct RGWUserAdminOpState {
   std::string display_name;
   int32_t max_buckets;
   __u8 suspended;
+  __u8 admin;
   __u8 system;
   __u8 exclusive;
   __u8 fetch_stats;
-  __u8 sync_stats;
   std::string caps;
   RGWObjVersionTracker objv;
   uint32_t op_mask;
@@ -199,6 +198,7 @@ struct RGWUserAdminOpState {
   bool op_mask_specified;
   bool caps_specified;
   bool suspension_op;
+  bool admin_specified;
   bool system_specified;
   bool key_op;
   bool temp_url_key_specified;
@@ -318,6 +318,11 @@ struct RGWUserAdminOpState {
     suspension_op = true;
   }
 
+  void set_admin(__u8 is_admin) {
+    admin = is_admin;
+    admin_specified = true;
+  }
+
   void set_system(__u8 is_system) {
     system = is_system;
     system_specified = true;
@@ -331,10 +336,6 @@ struct RGWUserAdminOpState {
     fetch_stats = is_fetch_stats;
   }
 
-  void set_sync_stats(__u8 is_sync_stats) {
-    sync_stats = is_sync_stats;
-  }
-
   void set_user_info(RGWUserInfo& user_info) {
     user_id = user_info.user_id;
     info = user_info;
@@ -479,6 +480,7 @@ struct RGWUserAdminOpState {
     key_type = -1;
     perm_mask = RGW_PERM_NONE;
     suspended = 0;
+    admin = 0;
     system = 0;
     exclusive = 0;
     fetch_stats = 0;
