@@ -8,10 +8,8 @@
 // found in the LICENSE file. See the AUTHORS file for names of contributors.
 
 #include "db/db_test_util.h"
-#include "options/options_helper.h"
-#include "port/port.h"
 #include "port/stack_trace.h"
-#include "util/fault_injection_test_env.h"
+#include "util/options_helper.h"
 #include "util/sync_point.h"
 
 namespace rocksdb {
@@ -51,7 +49,7 @@ TEST_F(DBWALTest, WAL) {
     // again both values should be present.
     ASSERT_EQ("v3", Get(1, "foo"));
     ASSERT_EQ("v3", Get(1, "bar"));
-  } while (ChangeWalOptions());
+  } while (ChangeCompactOptions());
 }
 
 TEST_F(DBWALTest, RollLog) {
@@ -68,7 +66,7 @@ TEST_F(DBWALTest, RollLog) {
     for (int i = 0; i < 10; i++) {
       ReopenWithColumnFamilies({"default", "pikachu"}, CurrentOptions());
     }
-  } while (ChangeWalOptions());
+  } while (ChangeOptions());
 }
 
 TEST_F(DBWALTest, SyncWALNotBlockWrite) {
@@ -87,7 +85,7 @@ TEST_F(DBWALTest, SyncWALNotBlockWrite) {
   });
   rocksdb::SyncPoint::GetInstance()->EnableProcessing();
 
-  rocksdb::port::Thread thread([&]() { ASSERT_OK(db_->SyncWAL()); });
+  std::thread thread([&]() { ASSERT_OK(db_->SyncWAL()); });
 
   TEST_SYNC_POINT("DBWALTest::SyncWALNotBlockWrite:1");
   ASSERT_OK(Put("foo2", "bar2"));
@@ -119,7 +117,7 @@ TEST_F(DBWALTest, SyncWALNotWaitWrite) {
   });
   rocksdb::SyncPoint::GetInstance()->EnableProcessing();
 
-  rocksdb::port::Thread thread([&]() { ASSERT_OK(Put("foo2", "bar2")); });
+  std::thread thread([&]() { ASSERT_OK(Put("foo2", "bar2")); });
   TEST_SYNC_POINT("DBWALTest::SyncWALNotWaitWrite:1");
   ASSERT_OK(db_->SyncWAL());
   TEST_SYNC_POINT("DBWALTest::SyncWALNotWaitWrite:2");
@@ -151,7 +149,7 @@ TEST_F(DBWALTest, Recover) {
     ASSERT_EQ("v4", Get(1, "foo"));
     ASSERT_EQ("v2", Get(1, "bar"));
     ASSERT_EQ("v5", Get(1, "baz"));
-  } while (ChangeWalOptions());
+  } while (ChangeOptions());
 }
 
 TEST_F(DBWALTest, RecoverWithTableHandle) {
@@ -188,22 +186,23 @@ TEST_F(DBWALTest, RecoverWithTableHandle) {
         }
       }
     }
-  } while (ChangeWalOptions());
+  } while (ChangeOptions());
 }
 
 TEST_F(DBWALTest, IgnoreRecoveredLog) {
   std::string backup_logs = dbname_ + "/backup_logs";
 
-  do {
-    // delete old files in backup_logs directory
-    env_->CreateDirIfMissing(backup_logs);
-    std::vector<std::string> old_files;
-    env_->GetChildren(backup_logs, &old_files);
-    for (auto& file : old_files) {
-      if (file != "." && file != "..") {
-        env_->DeleteFile(backup_logs + "/" + file);
-      }
+  // delete old files in backup_logs directory
+  env_->CreateDirIfMissing(backup_logs);
+  std::vector<std::string> old_files;
+  env_->GetChildren(backup_logs, &old_files);
+  for (auto& file : old_files) {
+    if (file != "." && file != "..") {
+      env_->DeleteFile(backup_logs + "/" + file);
     }
+  }
+
+  do {
     Options options = CurrentOptions();
     options.create_if_missing = true;
     options.merge_operator = MergeOperators::CreateUInt64AddOperator();
@@ -276,8 +275,7 @@ TEST_F(DBWALTest, IgnoreRecoveredLog) {
     }
     Status s = TryReopen(options);
     ASSERT_TRUE(!s.ok());
-    Destroy(options);
-  } while (ChangeWalOptions());
+  } while (ChangeOptions(kSkipHashCuckoo));
 }
 
 TEST_F(DBWALTest, RecoveryWithEmptyLog) {
@@ -290,173 +288,10 @@ TEST_F(DBWALTest, RecoveryWithEmptyLog) {
     ASSERT_OK(Put(1, "foo", "v3"));
     ReopenWithColumnFamilies({"default", "pikachu"}, CurrentOptions());
     ASSERT_EQ("v3", Get(1, "foo"));
-  } while (ChangeWalOptions());
+  } while (ChangeOptions());
 }
 
-#if !(defined NDEBUG) || !defined(OS_WIN)
-TEST_F(DBWALTest, PreallocateBlock) {
-  Options options = CurrentOptions();
-  options.write_buffer_size = 10 * 1000 * 1000;
-  options.max_total_wal_size = 0;
-
-  size_t expected_preallocation_size = static_cast<size_t>(
-      options.write_buffer_size + options.write_buffer_size / 10);
-
-  DestroyAndReopen(options);
-
-  std::atomic<int> called(0);
-  rocksdb::SyncPoint::GetInstance()->SetCallBack(
-      "DBTestWalFile.GetPreallocationStatus", [&](void* arg) {
-        ASSERT_TRUE(arg != nullptr);
-        size_t preallocation_size = *(static_cast<size_t*>(arg));
-        ASSERT_EQ(expected_preallocation_size, preallocation_size);
-        called.fetch_add(1);
-      });
-  rocksdb::SyncPoint::GetInstance()->EnableProcessing();
-  Put("", "");
-  Flush();
-  Put("", "");
-  Close();
-  rocksdb::SyncPoint::GetInstance()->DisableProcessing();
-  ASSERT_EQ(2, called.load());
-
-  options.max_total_wal_size = 1000 * 1000;
-  expected_preallocation_size = static_cast<size_t>(options.max_total_wal_size);
-  Reopen(options);
-  called.store(0);
-  rocksdb::SyncPoint::GetInstance()->SetCallBack(
-      "DBTestWalFile.GetPreallocationStatus", [&](void* arg) {
-        ASSERT_TRUE(arg != nullptr);
-        size_t preallocation_size = *(static_cast<size_t*>(arg));
-        ASSERT_EQ(expected_preallocation_size, preallocation_size);
-        called.fetch_add(1);
-      });
-  rocksdb::SyncPoint::GetInstance()->EnableProcessing();
-  Put("", "");
-  Flush();
-  Put("", "");
-  Close();
-  rocksdb::SyncPoint::GetInstance()->DisableProcessing();
-  ASSERT_EQ(2, called.load());
-
-  options.db_write_buffer_size = 800 * 1000;
-  expected_preallocation_size =
-      static_cast<size_t>(options.db_write_buffer_size);
-  Reopen(options);
-  called.store(0);
-  rocksdb::SyncPoint::GetInstance()->SetCallBack(
-      "DBTestWalFile.GetPreallocationStatus", [&](void* arg) {
-        ASSERT_TRUE(arg != nullptr);
-        size_t preallocation_size = *(static_cast<size_t*>(arg));
-        ASSERT_EQ(expected_preallocation_size, preallocation_size);
-        called.fetch_add(1);
-      });
-  rocksdb::SyncPoint::GetInstance()->EnableProcessing();
-  Put("", "");
-  Flush();
-  Put("", "");
-  Close();
-  rocksdb::SyncPoint::GetInstance()->DisableProcessing();
-  ASSERT_EQ(2, called.load());
-
-  expected_preallocation_size = 700 * 1000;
-  std::shared_ptr<WriteBufferManager> write_buffer_manager =
-      std::make_shared<WriteBufferManager>(static_cast<uint64_t>(700 * 1000));
-  options.write_buffer_manager = write_buffer_manager;
-  Reopen(options);
-  called.store(0);
-  rocksdb::SyncPoint::GetInstance()->SetCallBack(
-      "DBTestWalFile.GetPreallocationStatus", [&](void* arg) {
-        ASSERT_TRUE(arg != nullptr);
-        size_t preallocation_size = *(static_cast<size_t*>(arg));
-        ASSERT_EQ(expected_preallocation_size, preallocation_size);
-        called.fetch_add(1);
-      });
-  rocksdb::SyncPoint::GetInstance()->EnableProcessing();
-  Put("", "");
-  Flush();
-  Put("", "");
-  Close();
-  rocksdb::SyncPoint::GetInstance()->DisableProcessing();
-  ASSERT_EQ(2, called.load());
-}
-#endif  // !(defined NDEBUG) || !defined(OS_WIN)
-
 #ifndef ROCKSDB_LITE
-TEST_F(DBWALTest, FullPurgePreservesRecycledLog) {
-  // For github issue #1303
-  for (int i = 0; i < 2; ++i) {
-    Options options = CurrentOptions();
-    options.create_if_missing = true;
-    options.recycle_log_file_num = 2;
-    if (i != 0) {
-      options.wal_dir = alternative_wal_dir_;
-    }
-
-    DestroyAndReopen(options);
-    ASSERT_OK(Put("foo", "v1"));
-    VectorLogPtr log_files;
-    ASSERT_OK(dbfull()->GetSortedWalFiles(log_files));
-    ASSERT_GT(log_files.size(), 0);
-    ASSERT_OK(Flush());
-
-    // Now the original WAL is in log_files[0] and should be marked for
-    // recycling.
-    // Verify full purge cannot remove this file.
-    JobContext job_context(0);
-    dbfull()->TEST_LockMutex();
-    dbfull()->FindObsoleteFiles(&job_context, true /* force */);
-    dbfull()->TEST_UnlockMutex();
-    dbfull()->PurgeObsoleteFiles(job_context);
-
-    if (i == 0) {
-      ASSERT_OK(
-          env_->FileExists(LogFileName(dbname_, log_files[0]->LogNumber())));
-    } else {
-      ASSERT_OK(env_->FileExists(
-          LogFileName(alternative_wal_dir_, log_files[0]->LogNumber())));
-    }
-  }
-}
-
-TEST_F(DBWALTest, GetSortedWalFiles) {
-  do {
-    CreateAndReopenWithCF({"pikachu"}, CurrentOptions());
-    VectorLogPtr log_files;
-    ASSERT_OK(dbfull()->GetSortedWalFiles(log_files));
-    ASSERT_EQ(0, log_files.size());
-
-    ASSERT_OK(Put(1, "foo", "v1"));
-    ASSERT_OK(dbfull()->GetSortedWalFiles(log_files));
-    ASSERT_EQ(1, log_files.size());
-  } while (ChangeWalOptions());
-}
-
-TEST_F(DBWALTest, RecoveryWithLogDataForSomeCFs) {
-  // Test for regression of WAL cleanup missing files that don't contain data
-  // for every column family.
-  do {
-    CreateAndReopenWithCF({"pikachu"}, CurrentOptions());
-    ASSERT_OK(Put(1, "foo", "v1"));
-    ASSERT_OK(Put(1, "foo", "v2"));
-    uint64_t earliest_log_nums[2];
-    for (int i = 0; i < 2; ++i) {
-      if (i > 0) {
-        ReopenWithColumnFamilies({"default", "pikachu"}, CurrentOptions());
-      }
-      VectorLogPtr log_files;
-      ASSERT_OK(dbfull()->GetSortedWalFiles(log_files));
-      if (log_files.size() > 0) {
-        earliest_log_nums[i] = log_files[0]->LogNumber();
-      } else {
-        earliest_log_nums[i] = port::kMaxUint64;
-      }
-    }
-    // Check at least the first WAL was cleaned up during the recovery.
-    ASSERT_LT(earliest_log_nums[0], earliest_log_nums[1]);
-  } while (ChangeWalOptions());
-}
-
 TEST_F(DBWALTest, RecoverWithLargeLog) {
   do {
     {
@@ -481,7 +316,7 @@ TEST_F(DBWALTest, RecoverWithLargeLog) {
     ASSERT_EQ(std::string(10, '3'), Get(1, "small3"));
     ASSERT_EQ(std::string(10, '4'), Get(1, "small4"));
     ASSERT_GT(NumTableFilesAtLevel(0, 1), 1);
-  } while (ChangeWalOptions());
+  } while (ChangeCompactOptions());
 }
 
 // In https://reviews.facebook.net/D20661 we change
@@ -636,39 +471,6 @@ TEST_F(DBWALTest, SyncMultipleLogs) {
   ASSERT_OK(dbfull()->SyncWAL());
 }
 
-// Github issue 1339. Prior the fix we read sequence id from the first log to
-// a local variable, then keep increase the variable as we replay logs,
-// ignoring actual sequence id of the records. This is incorrect if some writes
-// come with WAL disabled.
-TEST_F(DBWALTest, PartOfWritesWithWALDisabled) {
-  std::unique_ptr<FaultInjectionTestEnv> fault_env(
-      new FaultInjectionTestEnv(env_));
-  Options options = CurrentOptions();
-  options.env = fault_env.get();
-  options.disable_auto_compactions = true;
-  WriteOptions wal_on, wal_off;
-  wal_on.sync = true;
-  wal_on.disableWAL = false;
-  wal_off.disableWAL = true;
-  CreateAndReopenWithCF({"dummy"}, options);
-  ASSERT_OK(Put(1, "dummy", "d1", wal_on));  // seq id 1
-  ASSERT_OK(Put(1, "dummy", "d2", wal_off));
-  ASSERT_OK(Put(1, "dummy", "d3", wal_off));
-  ASSERT_OK(Put(0, "key", "v4", wal_on));  // seq id 4
-  ASSERT_OK(Flush(0));
-  ASSERT_OK(Put(0, "key", "v5", wal_on));  // seq id 5
-  ASSERT_EQ("v5", Get(0, "key"));
-  // Simulate a crash.
-  fault_env->SetFilesystemActive(false);
-  Close();
-  fault_env->ResetState();
-  ReopenWithColumnFamilies({"default", "dummy"}, options);
-  // Prior to the fix, we may incorrectly recover "v5" with sequence id = 3.
-  ASSERT_EQ("v5", Get(0, "key"));
-  // Destroy DB before destruct fault_env.
-  Destroy(options);
-}
-
 //
 // Test WAL recovery for the various modes available
 //
@@ -679,18 +481,18 @@ class RecoveryTestHelper {
   // Starting number for the WAL file name like 00010.log
   static const int kWALFileOffset = 10;
   // Keys to be written per WAL file
-  static const int kKeysPerWALFile = 133;
+  static const int kKeysPerWALFile = 1024;
   // Size of the value
-  static const int kValueSize = 96;
+  static const int kValueSize = 10;
 
   // Create WAL files with values filled in
   static void FillData(DBWALTest* test, const Options& options,
                        const size_t wal_count, size_t* count) {
-    const ImmutableDBOptions db_options(options);
+    const DBOptions& db_options = options;
 
     *count = 0;
 
-    shared_ptr<Cache> table_cache = NewLRUCache(50, 0);
+    shared_ptr<Cache> table_cache = NewLRUCache(50000, 16);
     EnvOptions env_options;
     WriteBufferManager write_buffer_manager(db_options.db_write_buffer_size);
 
@@ -717,13 +519,12 @@ class RecoveryTestHelper {
           new log::Writer(std::move(file_writer), current_log_number,
                           db_options.recycle_log_file_num > 0));
 
-      WriteBatch batch;
       for (int i = 0; i < kKeysPerWALFile; i++) {
         std::string key = "key" + ToString((*count)++);
         std::string value = test->DummyString(kValueSize);
         assert(current_log_writer.get() != nullptr);
         uint64_t seq = versions->LastSequence() + 1;
-        batch.Clear();
+        WriteBatch batch;
         batch.Put(key, value);
         WriteBatchInternal::SetSequence(&batch, seq);
         current_log_writer->AddRecord(WriteBatchInternal::Contents(&batch));
@@ -774,7 +575,7 @@ class RecoveryTestHelper {
     if (trunc) {
       ASSERT_EQ(0, truncate(fname.c_str(), static_cast<int64_t>(size * off)));
     } else {
-      InduceCorruption(fname, static_cast<size_t>(size * off + 8),
+      InduceCorruption(fname, static_cast<size_t>(size * off),
                        static_cast<size_t>(size * len));
     }
   }
@@ -793,7 +594,7 @@ class RecoveryTestHelper {
     ASSERT_EQ(offset, lseek(fd, static_cast<long>(offset), SEEK_SET));
 
     void* buf = alloca(len);
-    memset(buf, 'b', len);
+    memset(buf, 'a', len);
     ASSERT_EQ(len, write(fd, buf, static_cast<unsigned int>(len)));
 
     close(fd);
@@ -809,7 +610,7 @@ TEST_F(DBWALTest, kTolerateCorruptedTailRecords) {
   const int jend = jstart + RecoveryTestHelper::kWALFilesCount;
 
   for (auto trunc : {true, false}) {        /* Corruption style */
-    for (int i = 0; i < 3; i++) {           /* Corruption offset position */
+    for (int i = 0; i < 4; i++) {           /* Corruption offset position */
       for (int j = jstart; j < jend; j++) { /* WAL file */
         // Fill data for testing
         Options options = CurrentOptions();
@@ -1006,32 +807,6 @@ TEST_F(DBWALTest, AvoidFlushDuringRecovery) {
   ASSERT_EQ(2, TotalTableFiles());
 }
 
-TEST_F(DBWALTest, WalCleanupAfterAvoidFlushDuringRecovery) {
-  // Verifies WAL files that were present during recovery, but not flushed due
-  // to avoid_flush_during_recovery, will be considered for deletion at a later
-  // stage. We check at least one such file is deleted during Flush().
-  Options options = CurrentOptions();
-  options.disable_auto_compactions = true;
-  options.avoid_flush_during_recovery = true;
-  Reopen(options);
-
-  ASSERT_OK(Put("foo", "v1"));
-  Reopen(options);
-  for (int i = 0; i < 2; ++i) {
-    if (i > 0) {
-      // Flush() triggers deletion of obsolete tracked files
-      Flush();
-    }
-    VectorLogPtr log_files;
-    ASSERT_OK(dbfull()->GetSortedWalFiles(log_files));
-    if (i == 0) {
-      ASSERT_GT(log_files.size(), 0);
-    } else {
-      ASSERT_EQ(0, log_files.size());
-    }
-  }
-}
-
 TEST_F(DBWALTest, RecoverWithoutFlush) {
   Options options = CurrentOptions();
   options.avoid_flush_during_recovery = true;
@@ -1192,29 +967,6 @@ TEST_F(DBWALTest, RecoverFromCorruptedWALWithoutFlush) {
 
 #endif  // ROCKSDB_LITE
 
-TEST_F(DBWALTest, WalTermTest) {
-  Options options = CurrentOptions();
-  options.env = env_;
-  CreateAndReopenWithCF({"pikachu"}, options);
-
-  ASSERT_OK(Put(1, "foo", "bar"));
-
-  WriteOptions wo;
-  wo.sync = true;
-  wo.disableWAL = false;
-
-  WriteBatch batch;
-  batch.Put("foo", "bar");
-  batch.MarkWalTerminationPoint();
-  batch.Put("foo2", "bar2");
-
-  ASSERT_OK(dbfull()->Write(wo, &batch));
-
-  // make sure we can re-open it.
-  ASSERT_OK(TryReopenWithColumnFamilies({"default", "pikachu"}, options));
-  ASSERT_EQ("bar", Get(1, "foo"));
-  ASSERT_EQ("NOT_FOUND", Get(1, "foo2"));
-}
 }  // namespace rocksdb
 
 int main(int argc, char** argv) {
