@@ -1,4 +1,4 @@
-//  Copyright (c) 2013, Facebook, Inc.  All rights reserved.
+//  Copyright (c) 2011-present, Facebook, Inc.  All rights reserved.
 //  This source code is licensed under the BSD-style license found in the
 //  LICENSE file in the root directory of this source tree. An additional grant
 //  of patent rights can be found in the PATENTS file in the same directory.
@@ -10,6 +10,7 @@
 #pragma once
 #include <stdio.h>
 #include <string>
+#include <utility>
 #include "rocksdb/comparator.h"
 #include "rocksdb/db.h"
 #include "rocksdb/filter_policy.h"
@@ -39,6 +40,13 @@ enum ValueType : unsigned char {
   kTypeColumnFamilyMerge = 0x6,     // WAL only.
   kTypeSingleDeletion = 0x7,
   kTypeColumnFamilySingleDeletion = 0x8,  // WAL only.
+  kTypeBeginPrepareXID = 0x9,             // WAL only.
+  kTypeEndPrepareXID = 0xA,               // WAL only.
+  kTypeCommitXID = 0xB,                   // WAL only.
+  kTypeRollbackXID = 0xC,                 // WAL only.
+  kTypeNoop = 0xD,                        // WAL only.
+  kTypeColumnFamilyRangeDeletion = 0xE,   // WAL only.
+  kTypeRangeDeletion = 0xF,               // meta block
   kMaxValue = 0x7F                        // Not used for storing records.
 };
 
@@ -50,12 +58,18 @@ enum ValueType : unsigned char {
 // ValueType, not the lowest).
 static const ValueType kValueTypeForSeek = kTypeSingleDeletion;
 
-// Checks whether a type is a value type (i.e. a type used in memtables and sst
-// files).
+// Checks whether a type is an inline value type
+// (i.e. a type used in memtable skiplist and sst file datablock).
 inline bool IsValueType(ValueType t) {
   return t <= kTypeMerge || t == kTypeSingleDeletion;
 }
 
+// Checks whether a type is from user operation
+// kTypeRangeDeletion is in meta block so this API is separated from above
+inline bool IsExtendedValueType(ValueType t) {
+  return IsValueType(t) || t == kTypeRangeDeletion;
+}
+
 // We leave eight bits empty at the bottom so a type and sequence#
 // can be packed together into 64-bits.
 static const SequenceNumber kMaxSequenceNumber =
@@ -174,6 +188,10 @@ class InternalKey {
   Slice user_key() const { return ExtractUserKey(rep_); }
   size_t size() { return rep_.size(); }
 
+  void Set(const Slice& _user_key, SequenceNumber s, ValueType t) {
+    SetFrom(ParsedInternalKey(_user_key, s, t));
+  }
+
   void SetFrom(const ParsedInternalKey& p) {
     rep_.clear();
     AppendInternalKey(&rep_, p);
@@ -199,7 +217,7 @@ inline bool ParseInternalKey(const Slice& internal_key,
   result->type = static_cast<ValueType>(c);
   assert(result->type <= ValueType::kMaxValue);
   result->user_key = Slice(internal_key.data(), n - 8);
-  return IsValueType(result->type);
+  return IsExtendedValueType(result->type);
 }
 
 // Update the sequence number in the internal key.
@@ -271,7 +289,8 @@ inline LookupKey::~LookupKey() {
 
 class IterKey {
  public:
-  IterKey() : key_(space_), buf_size_(sizeof(space_)), key_size_(0) {}
+  IterKey()
+      : buf_(space_), buf_size_(sizeof(space_)), key_(buf_), key_size_(0) {}
 
   ~IterKey() { ResetBuffer(); }
 
@@ -293,31 +312,41 @@ class IterKey {
   void TrimAppend(const size_t shared_len, const char* non_shared_data,
                   const size_t non_shared_len) {
     assert(shared_len <= key_size_);
-
     size_t total_size = shared_len + non_shared_len;
-    if (total_size <= buf_size_) {
-      key_size_ = total_size;
-    } else {
+
+    if (IsKeyPinned() /* key is not in buf_ */) {
+      // Copy the key from external memory to buf_ (copy shared_len bytes)
+      EnlargeBufferIfNeeded(total_size);
+      memcpy(buf_, key_, shared_len);
+    } else if (total_size > buf_size_) {
       // Need to allocate space, delete previous space
       char* p = new char[total_size];
       memcpy(p, key_, shared_len);
 
-      if (key_ != space_) {
-        delete[] key_;
+      if (buf_ != space_) {
+        delete[] buf_;
       }
 
-      key_ = p;
-      key_size_ = total_size;
+      buf_ = p;
       buf_size_ = total_size;
     }
 
-    memcpy(key_ + shared_len, non_shared_data, non_shared_len);
+    memcpy(buf_ + shared_len, non_shared_data, non_shared_len);
+    key_ = buf_;
+    key_size_ = total_size;
   }
 
-  Slice SetKey(const Slice& key) {
+  Slice SetKey(const Slice& key, bool copy = true) {
     size_t size = key.size();
-    EnlargeBufferIfNeeded(size);
-    memcpy(key_, key.data(), size);
+    if (copy) {
+      // Copy key to buf_
+      EnlargeBufferIfNeeded(size);
+      memcpy(buf_, key.data(), size);
+      key_ = buf_;
+    } else {
+      // Update key_ to point to external memory
+      key_ = key.data();
+    }
     key_size_ = size;
     return Slice(key_, key_size_);
   }
@@ -335,11 +364,14 @@ class IterKey {
   // Update the sequence number in the internal key.  Guarantees not to
   // invalidate slices to the key (and the user key).
   void UpdateInternalKey(uint64_t seq, ValueType t) {
+    assert(!IsKeyPinned());
     assert(key_size_ >= 8);
     uint64_t newval = (seq << 8) | t;
-    EncodeFixed64(&key_[key_size_ - 8], newval);
+    EncodeFixed64(&buf_[key_size_ - 8], newval);
   }
 
+  bool IsKeyPinned() const { return (key_ != buf_); }
+
   void SetInternalKey(const Slice& key_prefix, const Slice& user_key,
                       SequenceNumber s,
                       ValueType value_type = kValueTypeForSeek) {
@@ -347,10 +379,12 @@ class IterKey {
     size_t usize = user_key.size();
     EnlargeBufferIfNeeded(psize + usize + sizeof(uint64_t));
     if (psize > 0) {
-      memcpy(key_, key_prefix.data(), psize);
+      memcpy(buf_, key_prefix.data(), psize);
     }
-    memcpy(key_ + psize, user_key.data(), usize);
-    EncodeFixed64(key_ + usize + psize, PackSequenceAndType(s, value_type));
+    memcpy(buf_ + psize, user_key.data(), usize);
+    EncodeFixed64(buf_ + usize + psize, PackSequenceAndType(s, value_type));
+
+    key_ = buf_;
     key_size_ = psize + usize + sizeof(uint64_t);
   }
 
@@ -377,20 +411,22 @@ class IterKey {
   void EncodeLengthPrefixedKey(const Slice& key) {
     auto size = key.size();
     EnlargeBufferIfNeeded(size + static_cast<size_t>(VarintLength(size)));
-    char* ptr = EncodeVarint32(key_, static_cast<uint32_t>(size));
+    char* ptr = EncodeVarint32(buf_, static_cast<uint32_t>(size));
     memcpy(ptr, key.data(), size);
+    key_ = buf_;
   }
 
  private:
-  char* key_;
+  char* buf_;
   size_t buf_size_;
+  const char* key_;
   size_t key_size_;
   char space_[32];  // Avoid allocation for short keys
 
   void ResetBuffer() {
-    if (key_ != space_) {
-      delete[] key_;
-      key_ = space_;
+    if (buf_ != space_) {
+      delete[] buf_;
+      buf_ = space_;
     }
     buf_size_ = sizeof(space_);
     key_size_ = 0;
@@ -407,7 +443,7 @@ class IterKey {
     if (key_size > buf_size_) {
       // Need to enlarge the buffer.
       ResetBuffer();
-      key_ = new char[key_size];
+      buf_ = new char[key_size];
       buf_size_ = key_size;
     }
   }
@@ -447,6 +483,12 @@ class InternalKeySliceTransform : public SliceTransform {
   const SliceTransform* const transform_;
 };
 
+// Read the key of a record from a write batch.
+// if this record represent the default column family then cf_record
+// must be passed as false, otherwise it must be passed as true.
+extern bool ReadKeyFromWriteBatchEntry(Slice* input, Slice* key,
+                                       bool cf_record);
+
 // Read record from a write batch piece from input.
 // tag, column_family, key, value and blob are return values. Callers own the
 // Slice they point to.
@@ -454,5 +496,38 @@ class InternalKeySliceTransform : public SliceTransform {
 // input will be advanced to after the record.
 extern Status ReadRecordFromWriteBatch(Slice* input, char* tag,
                                        uint32_t* column_family, Slice* key,
-                                       Slice* value, Slice* blob);
+                                       Slice* value, Slice* blob, Slice* xid);
+
+// When user call DeleteRange() to delete a range of keys,
+// we will store a serialized RangeTombstone in MemTable and SST.
+// the struct here is a easy-understood form
+// start/end_key_ is the start/end user key of the range to be deleted
+struct RangeTombstone {
+  Slice start_key_;
+  Slice end_key_;
+  SequenceNumber seq_;
+  explicit RangeTombstone(Slice sk, Slice ek, SequenceNumber sn)
+      : start_key_(sk), end_key_(ek), seq_(sn) {}
+
+  explicit RangeTombstone(Slice internal_key, Slice value) {
+    ParsedInternalKey parsed_key;
+    if (ParseInternalKey(internal_key, &parsed_key)) {
+      start_key_ = parsed_key.user_key;
+      seq_ = parsed_key.sequence;
+      end_key_ = value;
+    }
+  }
+
+  // be careful to use Serialize(); InternalKey() allocates new memory
+  std::pair<InternalKey, Slice> Serialize() {
+    auto key = InternalKey(start_key_, seq_, kTypeRangeDeletion);
+    Slice value = end_key_;
+    return std::make_pair(std::move(key), std::move(value));
+  }
+
+  InternalKey SerializeKey() {
+    return InternalKey(start_key_, seq_, kTypeRangeDeletion);
+  }
+};
+
 }  // namespace rocksdb
