@@ -31,8 +31,9 @@ static std::string PrintContents(WriteBatch* b) {
   options.memtable_factory = factory;
   ImmutableCFOptions ioptions(options);
   WriteBufferManager wb(options.db_write_buffer_size);
-  MemTable* mem = new MemTable(cmp, ioptions, MutableCFOptions(options), &wb,
-                               kMaxSequenceNumber);
+  MemTable* mem =
+      new MemTable(cmp, ioptions, MutableCFOptions(options, ioptions), &wb,
+                   kMaxSequenceNumber);
   mem->Ref();
   std::string state;
   ColumnFamilyMemTablesDefault cf_mems_default(mem);
@@ -43,74 +44,60 @@ static std::string PrintContents(WriteBatch* b) {
   int single_delete_count = 0;
   int delete_range_count = 0;
   int merge_count = 0;
-  for (int i = 0; i < 2; ++i) {
-    Arena arena;
-    ScopedArenaIterator arena_iter_guard;
-    std::unique_ptr<InternalIterator> iter_guard;
-    InternalIterator* iter;
-    if (i == 0) {
-      iter = mem->NewIterator(ReadOptions(), &arena);
-      arena_iter_guard.set(iter);
-    } else {
-      iter = mem->NewRangeTombstoneIterator(ReadOptions());
-      iter_guard.reset(iter);
-    }
-    if (iter == nullptr) {
-      continue;
-    }
-    for (iter->SeekToFirst(); iter->Valid(); iter->Next()) {
-      ParsedInternalKey ikey;
-      memset((void*)&ikey, 0, sizeof(ikey));
-      EXPECT_TRUE(ParseInternalKey(iter->key(), &ikey));
-      switch (ikey.type) {
-        case kTypeValue:
-          state.append("Put(");
-          state.append(ikey.user_key.ToString());
-          state.append(", ");
-          state.append(iter->value().ToString());
-          state.append(")");
-          count++;
-          put_count++;
-          break;
-        case kTypeDeletion:
-          state.append("Delete(");
-          state.append(ikey.user_key.ToString());
-          state.append(")");
-          count++;
-          delete_count++;
-          break;
-        case kTypeSingleDeletion:
-          state.append("SingleDelete(");
-          state.append(ikey.user_key.ToString());
-          state.append(")");
-          count++;
-          single_delete_count++;
-          break;
-        case kTypeRangeDeletion:
-          state.append("DeleteRange(");
-          state.append(ikey.user_key.ToString());
-          state.append(", ");
-          state.append(iter->value().ToString());
-          state.append(")");
-          count++;
-          delete_range_count++;
-          break;
-        case kTypeMerge:
-          state.append("Merge(");
-          state.append(ikey.user_key.ToString());
-          state.append(", ");
-          state.append(iter->value().ToString());
-          state.append(")");
-          count++;
-          merge_count++;
-          break;
-        default:
-          assert(false);
-          break;
-      }
-      state.append("@");
-      state.append(NumberToString(ikey.sequence));
+  Arena arena;
+  ScopedArenaIterator iter(mem->NewIterator(ReadOptions(), &arena));
+  for (iter->SeekToFirst(); iter->Valid(); iter->Next()) {
+    ParsedInternalKey ikey;
+    memset((void *)&ikey, 0, sizeof(ikey));
+    EXPECT_TRUE(ParseInternalKey(iter->key(), &ikey));
+    switch (ikey.type) {
+      case kTypeValue:
+        state.append("Put(");
+        state.append(ikey.user_key.ToString());
+        state.append(", ");
+        state.append(iter->value().ToString());
+        state.append(")");
+        count++;
+        put_count++;
+        break;
+      case kTypeDeletion:
+        state.append("Delete(");
+        state.append(ikey.user_key.ToString());
+        state.append(")");
+        count++;
+        delete_count++;
+        break;
+      case kTypeSingleDeletion:
+        state.append("SingleDelete(");
+        state.append(ikey.user_key.ToString());
+        state.append(")");
+        count++;
+        single_delete_count++;
+        break;
+      case kTypeRangeDeletion:
+        state.append("DeleteRange(");
+        state.append(ikey.user_key.ToString());
+        state.append(", ");
+        state.append(iter->value().ToString());
+        state.append(")");
+        count++;
+        delete_range_count++;
+        break;
+      case kTypeMerge:
+        state.append("Merge(");
+        state.append(ikey.user_key.ToString());
+        state.append(", ");
+        state.append(iter->value().ToString());
+        state.append(")");
+        count++;
+        merge_count++;
+        break;
+      default:
+        assert(false);
+        break;
     }
+    state.append("@");
+    state.append(NumberToString(ikey.sequence));
   }
   EXPECT_EQ(b->HasPut(), put_count > 0);
   EXPECT_EQ(b->HasDelete(), delete_count > 0);
@@ -145,10 +132,10 @@ TEST_F(WriteBatchTest, Multiple) {
   ASSERT_EQ(100U, WriteBatchInternal::Sequence(&batch));
   ASSERT_EQ(4, WriteBatchInternal::Count(&batch));
   ASSERT_EQ(
+      "DeleteRange(bar, foo)@102"
       "Put(baz, boo)@103"
       "Delete(box)@101"
-      "Put(foo, bar)@100"
-      "DeleteRange(bar, foo)@102",
+      "Put(foo, bar)@100",
       PrintContents(&batch));
   ASSERT_EQ(4, batch.Count());
 }
@@ -194,27 +181,6 @@ TEST_F(WriteBatchTest, Append) {
             "Delete(foo)@203",
             PrintContents(&b1));
   ASSERT_EQ(4, b1.Count());
-  b2.Clear();
-  b2.Put("c", "cc");
-  b2.Put("d", "dd");
-  b2.MarkWalTerminationPoint();
-  b2.Put("e", "ee");
-  WriteBatchInternal::Append(&b1, &b2, /*wal only*/ true);
-  ASSERT_EQ(
-      "Put(a, va)@200"
-      "Put(b, vb)@202"
-      "Put(b, vb)@201"
-      "Put(c, cc)@204"
-      "Put(d, dd)@205"
-      "Delete(foo)@203",
-      PrintContents(&b1));
-  ASSERT_EQ(6, b1.Count());
-  ASSERT_EQ(
-      "Put(c, cc)@0"
-      "Put(d, dd)@1"
-      "Put(e, ee)@2",
-      PrintContents(&b2));
-  ASSERT_EQ(3, b2.Count());
 }
 
 TEST_F(WriteBatchTest, SingleDeletion) {
@@ -478,7 +444,7 @@ TEST_F(WriteBatchTest, DISABLED_LargeKeyValue) {
   // Insert key and value of 3GB and push total batch size to 12GB.
   static const size_t kKeyValueSize = 3221225472u;
   std::string raw(kKeyValueSize, 'A');
-  WriteBatch batch(size_t(12884901888ull + 1024u));
+  WriteBatch batch(12884901888u + 1024u);
   for (char i = 0; i < 2; i++) {
     raw[0] = 'A' + i;
     raw[raw.length() - 1] = 'A' - i;
@@ -861,18 +827,6 @@ TEST_F(WriteBatchTest, SavePointTest) {
   ASSERT_EQ("", PrintContents(&batch2));
 }
 
-TEST_F(WriteBatchTest, MemoryLimitTest) {
-  Status s;
-  // The header size is 12 bytes. The two Puts take 8 bytes which gives total
-  // of 12 + 8 * 2 = 28 bytes.
-  WriteBatch batch(0, 28);
-
-  ASSERT_OK(batch.Put("a", "...."));
-  ASSERT_OK(batch.Put("b", "...."));
-  s = batch.Put("c", "....");
-  ASSERT_TRUE(s.IsMemoryLimit());
-}
-
 }  // namespace rocksdb
 
 int main(int argc, char** argv) {
