@@ -5,7 +5,6 @@
 
 #ifndef ROCKSDB_LITE
 
-#include <functional>
 #include <string>
 #include <utility>
 #include <vector>
@@ -14,9 +13,7 @@
 #include "db/write_callback.h"
 #include "rocksdb/db.h"
 #include "rocksdb/write_batch.h"
-#include "port/port.h"
 #include "util/logging.h"
-#include "util/random.h"
 #include "util/sync_point.h"
 #include "util/testharness.h"
 
@@ -243,7 +240,7 @@ TEST_F(WriteCallbackTest, WriteWithCallbackTest) {
           rocksdb::SyncPoint::GetInstance()->EnableProcessing();
 
           // do all the writes
-          std::vector<port::Thread> threads;
+          std::vector<std::thread> threads;
           for (uint32_t i = 0; i < write_group.size(); i++) {
             threads.emplace_back(write_with_callback_func);
           }
