@@ -19,7 +19,6 @@
 
 #include <stdint.h>
 #include <cstdarg>
-#include <functional>
 #include <limits>
 #include <memory>
 #include <string>
@@ -41,10 +40,8 @@ class RandomAccessFile;
 class SequentialFile;
 class Slice;
 class WritableFile;
-class RandomRWFile;
 class Directory;
 struct DBOptions;
-struct ImmutableDBOptions;
 class RateLimiter;
 class ThreadStatusUpdater;
 struct ThreadStatus;
@@ -52,7 +49,6 @@ struct ThreadStatus;
 using std::unique_ptr;
 using std::shared_ptr;
 
-const size_t kDefaultPageSize = 4 * 1024;
 
 // Options while opening a file to read/write
 struct EnvOptions {
@@ -63,6 +59,9 @@ struct EnvOptions {
   // construct from Options
   explicit EnvOptions(const DBOptions& options);
 
+  // If true, then allow caching of data in environment buffers
+  bool use_os_buffer = true;
+
    // If true, then use mmap to read data
   bool use_mmap_reads = false;
 
@@ -163,37 +162,12 @@ class Env {
                                  unique_ptr<WritableFile>* result,
                                  const EnvOptions& options) = 0;
 
-  // Create an object that writes to a new file with the specified
-  // name.  Deletes any existing file with the same name and creates a
-  // new file.  On success, stores a pointer to the new file in
-  // *result and returns OK.  On failure stores nullptr in *result and
-  // returns non-OK.
-  //
-  // The returned file will only be accessed by one thread at a time.
-  virtual Status ReopenWritableFile(const std::string& fname,
-                                    unique_ptr<WritableFile>* result,
-                                    const EnvOptions& options) {
-    Status s;
-    return s;
-  }
-
   // Reuse an existing file by renaming it and opening it as writable.
   virtual Status ReuseWritableFile(const std::string& fname,
                                    const std::string& old_fname,
                                    unique_ptr<WritableFile>* result,
                                    const EnvOptions& options);
 
-  // Open `fname` for random read and write, if file dont exist the file
-  // will be created.  On success, stores a pointer to the new file in
-  // *result and returns OK.  On failure returns non-OK.
-  //
-  // The returned file will only be accessed by one thread at a time.
-  virtual Status NewRandomRWFile(const std::string& fname,
-                                 unique_ptr<RandomRWFile>* result,
-                                 const EnvOptions& options) {
-    return Status::NotSupported("RandomRWFile is not implemented in this Env");
-  }
-
   // Create an object that represents a directory. Will fail if directory
   // doesn't exist. If the directory exists, it will open the directory
   // and create a new Directory object.
@@ -214,10 +188,6 @@ class Env {
   // Store in *result the names of the children of the specified directory.
   // The names are relative to "dir".
   // Original contents of *results are dropped.
-  // Returns OK if "dir" exists and "*result" contains its children.
-  //         NotFound if "dir" does not exist, the calling process does not have
-  //                  permission to access "dir", or if "dir" is invalid.
-  //         IOError if an IO Error was encountered
   virtual Status GetChildren(const std::string& dir,
                              std::vector<std::string>* result) = 0;
 
@@ -227,10 +197,6 @@ class Env {
   // result.
   // The name attributes are relative to "dir".
   // Original contents of *results are dropped.
-  // Returns OK if "dir" exists and "*result" contains its children.
-  //         NotFound if "dir" does not exist, the calling process does not have
-  //                  permission to access "dir", or if "dir" is invalid.
-  //         IOError if an IO Error was encountered
   virtual Status GetChildrenFileAttributes(const std::string& dir,
                                            std::vector<FileAttributes>* result);
 
@@ -333,16 +299,15 @@ class Env {
   virtual Status NewLogger(const std::string& fname,
                            shared_ptr<Logger>* result) = 0;
 
-  // Returns the number of micro-seconds since some fixed point in time.
-  // It is often used as system time such as in GenericRateLimiter
+  // Returns the number of micro-seconds since some fixed point in time. Only
+  // useful for computing deltas of time.
+  // However, it is often used as system time such as in GenericRateLimiter
   // and other places so a port needs to return system time in order to work.
   virtual uint64_t NowMicros() = 0;
 
   // Returns the number of nano-seconds since some fixed point in time. Only
   // useful for computing deltas of time in one run.
-  // Default implementation simply relies on NowMicros.
-  // In platform-specific implementations, NowNanos() should return time points
-  // that are MONOTONIC.
+  // Default implementation simply relies on NowMicros
   virtual uint64_t NowNanos() {
     return NowMicros() * 1000;
   }
@@ -387,22 +352,8 @@ class Env {
   // OptimizeForManifestWrite will create a new EnvOptions object that is a copy
   // of the EnvOptions in the parameters, but is optimized for writing manifest
   // files. Default implementation returns the copy of the same object.
-  virtual EnvOptions OptimizeForManifestWrite(
-      const EnvOptions& env_options) const;
-
-  // OptimizeForCompactionTableWrite will create a new EnvOptions object that is a copy
-  // of the EnvOptions in the parameters, but is optimized for writing table
-  // files. Default implementation returns the copy of the same object.
-  virtual EnvOptions OptimizeForCompactionTableWrite(
-      const EnvOptions& env_options,
-      const ImmutableDBOptions& db_options) const;
-
-  // OptimizeForCompactionTableWrite will create a new EnvOptions object that is a copy
-  // of the EnvOptions in the parameters, but is optimized for reading table
-  // files. Default implementation returns the copy of the same object.
-  virtual EnvOptions OptimizeForCompactionTableRead(
-      const EnvOptions& env_options,
-      const ImmutableDBOptions& db_options) const;
+  virtual EnvOptions OptimizeForManifestWrite(const EnvOptions& env_options)
+      const;
 
   // Returns the status of all threads that belong to the current Env.
   virtual Status GetThreadList(std::vector<ThreadStatus>* thread_list) {
@@ -460,35 +411,17 @@ class SequentialFile {
   // REQUIRES: External synchronization
   virtual Status Skip(uint64_t n) = 0;
 
-  // Indicates the upper layers if the current SequentialFile implementation
-  // uses direct IO.
-  virtual bool use_direct_io() const { return false; }
-
-  // Use the returned alignment value to allocate
-  // aligned buffer for Direct I/O
-  virtual size_t GetRequiredBufferAlignment() const { return kDefaultPageSize; }
-
-  virtual void Rewind() {}
-
   // Remove any kind of caching of data from the offset to offset+length
   // of this file. If the length is 0, then it refers to the end of file.
   // If the system is not caching the file contents, then this is a noop.
   virtual Status InvalidateCache(size_t offset, size_t length) {
     return Status::NotSupported("InvalidateCache not supported.");
   }
-
-  // Positioned Read for direct I/O
-  // If Direct I/O enabled, offset, n, and scratch should be properly aligned
-  virtual Status PositionedRead(uint64_t offset, size_t n, Slice* result,
-                                char* scratch) {
-    return Status::NotSupported();
-  }
 };
 
 // A file abstraction for randomly reading the contents of a file.
 class RandomAccessFile {
  public:
-
   RandomAccessFile() { }
   virtual ~RandomAccessFile();
 
@@ -501,15 +434,9 @@ class RandomAccessFile {
   // status.
   //
   // Safe for concurrent use by multiple threads.
-  // If Direct I/O enabled, offset, n, and scratch should be aligned properly.
   virtual Status Read(uint64_t offset, size_t n, Slice* result,
                       char* scratch) const = 0;
 
-  // Readahead the file starting from offset by n bytes for caching.
-  virtual Status Prefetch(uint64_t offset, size_t n) {
-    return Status::OK();
-  }
-
   // Used by the file_reader_writer to decide if the ReadAhead wrapper
   // should simply forward the call and do not enact buffering or locking.
   virtual bool ShouldForwardRawRequest() const {
@@ -544,14 +471,6 @@ class RandomAccessFile {
 
   virtual void Hint(AccessPattern pattern) {}
 
-  // Indicates the upper layers if the current RandomAccessFile implementation
-  // uses direct IO.
-  virtual bool use_direct_io() const { return false; }
-
-  // Use the returned alignment value to allocate
-  // aligned buffer for Direct I/O
-  virtual size_t GetRequiredBufferAlignment() const { return kDefaultPageSize; }
-
   // Remove any kind of caching of data from the offset to offset+length
   // of this file. If the length is 0, then it refers to the end of file.
   // If the system is not caching the file contents, then this is a noop.
@@ -572,31 +491,24 @@ class WritableFile {
   }
   virtual ~WritableFile();
 
-  // Append data to the end of the file
-  // Note: A WriteabelFile object must support either Append or
-  // PositionedAppend, so the users cannot mix the two.
+  // Indicates if the class makes use of unbuffered I/O
+  virtual bool UseOSBuffer() const {
+    return true;
+  }
+
+  const size_t c_DefaultPageSize = 4 * 1024;
+
+  // This is needed when you want to allocate
+  // AlignedBuffer for use with file I/O classes
+  // Used for unbuffered file I/O when UseOSBuffer() returns false
+  virtual size_t GetRequiredBufferAlignment() const {
+    return c_DefaultPageSize;
+  }
+
   virtual Status Append(const Slice& data) = 0;
 
-  // PositionedAppend data to the specified offset. The new EOF after append
-  // must be larger than the previous EOF. This is to be used when writes are
-  // not backed by OS buffers and hence has to always start from the start of
-  // the sector. The implementation thus needs to also rewrite the last
-  // partial sector.
-  // Note: PositionAppend does not guarantee moving the file offset after the
-  // write. A WritableFile object must support either Append or
-  // PositionedAppend, so the users cannot mix the two.
-  //
-  // PositionedAppend() can only happen on the page/sector boundaries. For that
-  // reason, if the last write was an incomplete sector we still need to rewind
-  // back to the nearest sector/page and rewrite the portion of it with whatever
-  // we need to add. We need to keep where we stop writing.
-  //
-  // PositionedAppend() can only write whole sectors. For that reason we have to
-  // pad with zeros for the last write and trim the file when closing according
-  // to the position we keep in the previous step.
-  //
-  // PositionedAppend() requires aligned buffer to be passed in. The alignment
-  // required is queried via GetRequiredBufferAlignment()
+  // Positioned write for unbuffered access default forward
+  // to simple append as most of the tests are buffered by default
   virtual Status PositionedAppend(const Slice& /* data */, uint64_t /* offset */) {
     return Status::NotSupported();
   }
@@ -630,11 +542,8 @@ class WritableFile {
 
   // Indicates the upper layers if the current WritableFile implementation
   // uses direct IO.
-  virtual bool use_direct_io() const { return false; }
+  virtual bool UseDirectIO() const { return false; }
 
-  // Use the returned alignment value to allocate
-  // aligned buffer for Direct I/O
-  virtual size_t GetRequiredBufferAlignment() const { return kDefaultPageSize; }
   /*
    * Change the priority in rate limiter if rate limiting is enabled.
    * If rate limiting is not enabled, this call has no effect.
@@ -737,54 +646,6 @@ class WritableFile {
   Env::IOPriority io_priority_;
 };
 
-// A file abstraction for random reading and writing.
-class RandomRWFile {
- public:
-  RandomRWFile() {}
-  virtual ~RandomRWFile() {}
-
-  // Indicates if the class makes use of direct I/O
-  // If false you must pass aligned buffer to Write()
-  virtual bool use_direct_io() const { return false; }
-
-  // Use the returned alignment value to allocate
-  // aligned buffer for Direct I/O
-  virtual size_t GetRequiredBufferAlignment() const { return kDefaultPageSize; }
-
-  // Used by the file_reader_writer to decide if the ReadAhead wrapper
-  // should simply forward the call and do not enact read_ahead buffering or locking.
-  // The implementation below takes care of reading ahead
-  virtual bool ShouldForwardRawRequest() const {
-    return false;
-  }
-
-  // For cases when read-ahead is implemented in the platform dependent
-  // layer. This is when ShouldForwardRawRequest() returns true.
-  virtual void EnableReadAhead() {}
-
-  // Write bytes in `data` at  offset `offset`, Returns Status::OK() on success.
-  // Pass aligned buffer when use_direct_io() returns true.
-  virtual Status Write(uint64_t offset, const Slice& data) = 0;
-
-  // Read up to `n` bytes starting from offset `offset` and store them in
-  // result, provided `scratch` size should be at least `n`.
-  // Returns Status::OK() on success.
-  virtual Status Read(uint64_t offset, size_t n, Slice* result,
-                      char* scratch) const = 0;
-
-  virtual Status Flush() = 0;
-
-  virtual Status Sync() = 0;
-
-  virtual Status Fsync() { return Sync(); }
-
-  virtual Status Close() = 0;
-
-  // No copying allowed
-  RandomRWFile(const RandomRWFile&) = delete;
-  RandomRWFile& operator=(const RandomRWFile&) = delete;
-};
-
 // Directory object represents collection of files and implements
 // filesystem operations that can be executed on directories.
 class Directory {
@@ -934,22 +795,12 @@ class EnvWrapper : public Env {
                          const EnvOptions& options) override {
     return target_->NewWritableFile(f, r, options);
   }
-  Status ReopenWritableFile(const std::string& fname,
-                            unique_ptr<WritableFile>* result,
-                            const EnvOptions& options) override {
-    return target_->ReopenWritableFile(fname, result, options);
-  }
   Status ReuseWritableFile(const std::string& fname,
                            const std::string& old_fname,
                            unique_ptr<WritableFile>* r,
                            const EnvOptions& options) override {
     return target_->ReuseWritableFile(fname, old_fname, r, options);
   }
-  Status NewRandomRWFile(const std::string& fname,
-                         unique_ptr<RandomRWFile>* result,
-                         const EnvOptions& options) override {
-    return target_->NewRandomRWFile(fname, result, options);
-  }
   virtual Status NewDirectory(const std::string& name,
                               unique_ptr<Directory>* result) override {
     return target_->NewDirectory(name, result);
@@ -1025,7 +876,6 @@ class EnvWrapper : public Env {
     return target_->NewLogger(fname, result);
   }
   uint64_t NowMicros() override { return target_->NowMicros(); }
-
   void SleepForMicroseconds(int micros) override {
     target_->SleepForMicroseconds(micros);
   }
@@ -1135,11 +985,6 @@ Env* NewMemEnv(Env* base_env);
 // This is a factory method for HdfsEnv declared in hdfs/env_hdfs.h
 Status NewHdfsEnv(Env** hdfs_env, const std::string& fsname);
 
-// Returns a new environment that measures function call times for filesystem
-// operations, reporting results to variables in PerfContext.
-// This is a factory method for TimedEnv defined in utilities/env_timed.cc.
-Env* NewTimedEnv(Env* base_env);
-
 }  // namespace rocksdb
 
 #endif  // STORAGE_ROCKSDB_INCLUDE_ENV_H_
