@@ -5,9 +5,7 @@
 
 package org.rocksdb;
 
-import java.nio.file.Paths;
 import java.util.ArrayList;
-import java.util.Collections;
 import java.util.List;
 import java.util.Random;
 
@@ -137,9 +135,9 @@ public class OptionsTest {
   @Test
   public void maxBytesForLevelMultiplier() {
     try (final Options opt = new Options()) {
-      final double doubleValue = rand.nextDouble();
-      opt.setMaxBytesForLevelMultiplier(doubleValue);
-      assertThat(opt.maxBytesForLevelMultiplier()).isEqualTo(doubleValue);
+      final int intValue = rand.nextInt();
+      opt.setMaxBytesForLevelMultiplier(intValue);
+      assertThat(opt.maxBytesForLevelMultiplier()).isEqualTo(intValue);
     }
   }
 
@@ -163,6 +161,15 @@ public class OptionsTest {
     }
   }
 
+  @Test
+  public void softRateLimit() {
+    try (final Options opt = new Options()) {
+      final double doubleValue = rand.nextDouble();
+      opt.setSoftRateLimit(doubleValue);
+      assertThat(opt.softRateLimit()).isEqualTo(doubleValue);
+    }
+  }
+
   @Test
   public void softPendingCompactionBytesLimit() {
     try (final Options opt = new Options()) {
@@ -172,6 +179,15 @@ public class OptionsTest {
     }
   }
 
+  @Test
+  public void hardRateLimit() {
+    try (final Options opt = new Options()) {
+      final double doubleValue = rand.nextDouble();
+      opt.setHardRateLimit(doubleValue);
+      assertThat(opt.hardRateLimit()).isEqualTo(doubleValue);
+    }
+  }
+
   @Test
   public void hardPendingCompactionBytesLimit() {
     try (final Options opt = new Options()) {
@@ -208,6 +224,15 @@ public class OptionsTest {
     }
   }
 
+  @Test
+  public void rateLimitDelayMaxMilliseconds() {
+    try (final Options opt = new Options()) {
+      final int intValue = rand.nextInt();
+      opt.setRateLimitDelayMaxMilliseconds(intValue);
+      assertThat(opt.rateLimitDelayMaxMilliseconds()).isEqualTo(intValue);
+    }
+  }
+
   @Test
   public void arenaBlockSize() throws RocksDBException {
     try (final Options opt = new Options()) {
@@ -226,6 +251,24 @@ public class OptionsTest {
     }
   }
 
+  @Test
+  public void purgeRedundantKvsWhileFlush() {
+    try (final Options opt = new Options()) {
+      final boolean boolValue = rand.nextBoolean();
+      opt.setPurgeRedundantKvsWhileFlush(boolValue);
+      assertThat(opt.purgeRedundantKvsWhileFlush()).isEqualTo(boolValue);
+    }
+  }
+
+  @Test
+  public void verifyChecksumsInCompaction() {
+    try (final Options opt = new Options()) {
+      final boolean boolValue = rand.nextBoolean();
+      opt.setVerifyChecksumsInCompaction(boolValue);
+      assertThat(opt.verifyChecksumsInCompaction()).isEqualTo(boolValue);
+    }
+  }
+
   @Test
   public void maxSequentialSkipInIterations() {
     try (final Options opt = new Options()) {
@@ -289,6 +332,15 @@ public class OptionsTest {
     }
   }
 
+  @Test
+  public void minPartialMergeOperands() {
+    try (final Options opt = new Options()) {
+      final int intValue = rand.nextInt();
+      opt.setMinPartialMergeOperands(intValue);
+      assertThat(opt.minPartialMergeOperands()).isEqualTo(intValue);
+    }
+  }
+
   @Test
   public void optimizeFiltersForHits() {
     try (final Options opt = new Options()) {
@@ -357,11 +409,12 @@ public class OptionsTest {
   }
 
   @Test
-  public void maxFileOpeningThreads() {
+  public void disableDataSync() {
     try (final Options opt = new Options()) {
-      final int intValue = rand.nextInt();
-      opt.setMaxFileOpeningThreads(intValue);
-      assertThat(opt.maxFileOpeningThreads()).isEqualTo(intValue);
+      final boolean boolValue = rand.nextBoolean();
+      opt.setDisableDataSync(boolValue);
+      assertThat(opt.disableDataSync()).
+          isEqualTo(boolValue);
     }
   }
 
@@ -374,22 +427,6 @@ public class OptionsTest {
     }
   }
 
-  @Test
-  public void dbPaths() {
-    final List<DbPath> dbPaths = new ArrayList<>();
-    dbPaths.add(new DbPath(Paths.get("/a"), 10));
-    dbPaths.add(new DbPath(Paths.get("/b"), 100));
-    dbPaths.add(new DbPath(Paths.get("/c"), 1000));
-
-    try (final Options opt = new Options()) {
-      assertThat(opt.dbPaths()).isEqualTo(Collections.emptyList());
-
-      opt.setDbPaths(dbPaths);
-
-      assertThat(opt.dbPaths()).isEqualTo(dbPaths);
-    }
-  }
-
   @Test
   public void dbLogDir() {
     try (final Options opt = new Options()) {
@@ -486,15 +523,6 @@ public class OptionsTest {
     }
   }
 
-  @Test
-  public void recycleLogFileNum() throws RocksDBException {
-    try (final Options opt = new Options()) {
-      final long longValue = rand.nextLong();
-      opt.setRecycleLogFileNum(longValue);
-      assertThat(opt.recycleLogFileNum()).isEqualTo(longValue);
-    }
-  }
-
   @Test
   public void maxManifestFileSize() {
     try (final Options opt = new Options()) {
@@ -544,29 +572,11 @@ public class OptionsTest {
   }
 
   @Test
-  public void useDirectReads() {
-    try(final Options opt = new Options()) {
-      final boolean boolValue = rand.nextBoolean();
-      opt.setUseDirectReads(boolValue);
-      assertThat(opt.useDirectReads()).isEqualTo(boolValue);
-    }
-  }
-
-  @Test
-  public void useDirectIoForFlushAndCompaction() {
-    try(final Options opt = new Options()) {
-      final boolean boolValue = rand.nextBoolean();
-      opt.setUseDirectIoForFlushAndCompaction(boolValue);
-      assertThat(opt.useDirectIoForFlushAndCompaction()).isEqualTo(boolValue);
-    }
-  }
-
-  @Test
-  public void allowFAllocate() {
+  public void allowOsBuffer() {
     try (final Options opt = new Options()) {
       final boolean boolValue = rand.nextBoolean();
-      opt.setAllowFAllocate(boolValue);
-      assertThat(opt.allowFAllocate()).isEqualTo(boolValue);
+      opt.setAllowOsBuffer(boolValue);
+      assertThat(opt.allowOsBuffer()).isEqualTo(boolValue);
     }
   }
 
@@ -615,60 +625,6 @@ public class OptionsTest {
     }
   }
 
-  @Test
-  public void dbWriteBufferSize() {
-    try (final Options opt = new Options()) {
-      final long longValue = rand.nextLong();
-      opt.setDbWriteBufferSize(longValue);
-      assertThat(opt.dbWriteBufferSize()).isEqualTo(longValue);
-    }
-  }
-
-  @Test
-  public void accessHintOnCompactionStart() {
-    try (final Options opt = new Options()) {
-      final AccessHint accessHint = AccessHint.SEQUENTIAL;
-      opt.setAccessHintOnCompactionStart(accessHint);
-      assertThat(opt.accessHintOnCompactionStart()).isEqualTo(accessHint);
-    }
-  }
-
-  @Test
-  public void newTableReaderForCompactionInputs() {
-    try (final Options opt = new Options()) {
-      final boolean boolValue = rand.nextBoolean();
-      opt.setNewTableReaderForCompactionInputs(boolValue);
-      assertThat(opt.newTableReaderForCompactionInputs()).isEqualTo(boolValue);
-    }
-  }
-
-  @Test
-  public void compactionReadaheadSize() {
-    try (final Options opt = new Options()) {
-      final long longValue = rand.nextLong();
-      opt.setCompactionReadaheadSize(longValue);
-      assertThat(opt.compactionReadaheadSize()).isEqualTo(longValue);
-    }
-  }
-
-  @Test
-  public void randomAccessMaxBufferSize() {
-    try (final Options opt = new Options()) {
-      final long longValue = rand.nextLong();
-      opt.setRandomAccessMaxBufferSize(longValue);
-      assertThat(opt.randomAccessMaxBufferSize()).isEqualTo(longValue);
-    }
-  }
-
-  @Test
-  public void writableFileMaxBufferSize() {
-    try (final Options opt = new Options()) {
-      final long longValue = rand.nextLong();
-      opt.setWritableFileMaxBufferSize(longValue);
-      assertThat(opt.writableFileMaxBufferSize()).isEqualTo(longValue);
-    }
-  }
-
   @Test
   public void useAdaptiveMutex() {
     try (final Options opt = new Options()) {
@@ -687,33 +643,6 @@ public class OptionsTest {
     }
   }
 
-  @Test
-  public void walBytesPerSync() {
-    try (final Options opt = new Options()) {
-      final long longValue = rand.nextLong();
-      opt.setWalBytesPerSync(longValue);
-      assertThat(opt.walBytesPerSync()).isEqualTo(longValue);
-    }
-  }
-
-  @Test
-  public void enableThreadTracking() {
-    try (final Options opt = new Options()) {
-      final boolean boolValue = rand.nextBoolean();
-      opt.setEnableThreadTracking(boolValue);
-      assertThat(opt.enableThreadTracking()).isEqualTo(boolValue);
-    }
-  }
-
-  @Test
-  public void delayedWriteRate() {
-    try (final Options opt = new Options()) {
-      final long longValue = rand.nextLong();
-      opt.setDelayedWriteRate(longValue);
-      assertThat(opt.delayedWriteRate()).isEqualTo(longValue);
-    }
-  }
-
   @Test
   public void allowConcurrentMemtableWrite() {
     try (final Options opt = new Options()) {
@@ -750,87 +679,6 @@ public class OptionsTest {
     }
   }
 
-  @Test
-  public void skipStatsUpdateOnDbOpen() {
-    try (final Options opt = new Options()) {
-      final boolean boolValue = rand.nextBoolean();
-      opt.setSkipStatsUpdateOnDbOpen(boolValue);
-      assertThat(opt.skipStatsUpdateOnDbOpen()).isEqualTo(boolValue);
-    }
-  }
-
-  @Test
-  public void walRecoveryMode() {
-    try (final Options opt = new Options()) {
-      for (final WALRecoveryMode walRecoveryMode : WALRecoveryMode.values()) {
-        opt.setWalRecoveryMode(walRecoveryMode);
-        assertThat(opt.walRecoveryMode()).isEqualTo(walRecoveryMode);
-      }
-    }
-  }
-
-  @Test
-  public void allow2pc() {
-    try (final Options opt = new Options()) {
-      final boolean boolValue = rand.nextBoolean();
-      opt.setAllow2pc(boolValue);
-      assertThat(opt.allow2pc()).isEqualTo(boolValue);
-    }
-  }
-
-  @Test
-  public void rowCache() {
-    try (final Options opt = new Options()) {
-      assertThat(opt.rowCache()).isNull();
-
-      try(final Cache lruCache = new LRUCache(1000)) {
-        opt.setRowCache(lruCache);
-        assertThat(opt.rowCache()).isEqualTo(lruCache);
-      }
-
-      try(final Cache clockCache = new ClockCache(1000)) {
-        opt.setRowCache(clockCache);
-        assertThat(opt.rowCache()).isEqualTo(clockCache);
-      }
-    }
-  }
-
-  @Test
-  public void failIfOptionsFileError() {
-    try (final Options opt = new Options()) {
-      final boolean boolValue = rand.nextBoolean();
-      opt.setFailIfOptionsFileError(boolValue);
-      assertThat(opt.failIfOptionsFileError()).isEqualTo(boolValue);
-    }
-  }
-
-  @Test
-  public void dumpMallocStats() {
-    try (final Options opt = new Options()) {
-      final boolean boolValue = rand.nextBoolean();
-      opt.setDumpMallocStats(boolValue);
-      assertThat(opt.dumpMallocStats()).isEqualTo(boolValue);
-    }
-  }
-
-  @Test
-  public void avoidFlushDuringRecovery() {
-    try (final Options opt = new Options()) {
-      final boolean boolValue = rand.nextBoolean();
-      opt.setAvoidFlushDuringRecovery(boolValue);
-      assertThat(opt.avoidFlushDuringRecovery()).isEqualTo(boolValue);
-    }
-  }
-
-  @Test
-  public void avoidFlushDuringShutdown() {
-    try (final Options opt = new Options()) {
-      final boolean boolValue = rand.nextBoolean();
-      opt.setAvoidFlushDuringShutdown(boolValue);
-      assertThat(opt.avoidFlushDuringShutdown()).isEqualTo(boolValue);
-    }
-  }
-
   @Test
   public void env() {
     try (final Options options = new Options();
@@ -848,7 +696,6 @@ public class OptionsTest {
       options.optimizeLevelStyleCompaction();
       options.optimizeLevelStyleCompaction(3000);
       options.optimizeForPointLookup(10);
-      options.optimizeForSmallDb();
       options.prepareForBulkLoad();
     }
   }
@@ -869,15 +716,16 @@ public class OptionsTest {
 
   @Test
   public void compressionPerLevel() {
-    try (final Options options = new Options()) {
-      assertThat(options.compressionPerLevel()).isEmpty();
+    try (final ColumnFamilyOptions columnFamilyOptions =
+             new ColumnFamilyOptions()) {
+      assertThat(columnFamilyOptions.compressionPerLevel()).isEmpty();
       List<CompressionType> compressionTypeList =
           new ArrayList<>();
-      for (int i = 0; i < options.numLevels(); i++) {
+      for (int i = 0; i < columnFamilyOptions.numLevels(); i++) {
         compressionTypeList.add(CompressionType.NO_COMPRESSION);
       }
-      options.setCompressionPerLevel(compressionTypeList);
-      compressionTypeList = options.compressionPerLevel();
+      columnFamilyOptions.setCompressionPerLevel(compressionTypeList);
+      compressionTypeList = columnFamilyOptions.compressionPerLevel();
       for (final CompressionType compressionType : compressionTypeList) {
         assertThat(compressionType).isEqualTo(
             CompressionType.NO_COMPRESSION);
@@ -887,18 +735,19 @@ public class OptionsTest {
 
   @Test
   public void differentCompressionsPerLevel() {
-    try (final Options options = new Options()) {
-      options.setNumLevels(3);
+    try (final ColumnFamilyOptions columnFamilyOptions =
+             new ColumnFamilyOptions()) {
+      columnFamilyOptions.setNumLevels(3);
 
-      assertThat(options.compressionPerLevel()).isEmpty();
+      assertThat(columnFamilyOptions.compressionPerLevel()).isEmpty();
       List<CompressionType> compressionTypeList = new ArrayList<>();
 
       compressionTypeList.add(CompressionType.BZLIB2_COMPRESSION);
       compressionTypeList.add(CompressionType.SNAPPY_COMPRESSION);
       compressionTypeList.add(CompressionType.LZ4_COMPRESSION);
 
-      options.setCompressionPerLevel(compressionTypeList);
-      compressionTypeList = options.compressionPerLevel();
+      columnFamilyOptions.setCompressionPerLevel(compressionTypeList);
+      compressionTypeList = columnFamilyOptions.compressionPerLevel();
 
       assertThat(compressionTypeList.size()).isEqualTo(3);
       assertThat(compressionTypeList).
@@ -910,34 +759,6 @@ public class OptionsTest {
     }
   }
 
-  @Test
-  public void bottommostCompressionType() {
-    try (final Options options = new Options()) {
-      assertThat(options.bottommostCompressionType())
-          .isEqualTo(CompressionType.DISABLE_COMPRESSION_OPTION);
-
-      for (final CompressionType compressionType : CompressionType.values()) {
-        options.setBottommostCompressionType(compressionType);
-        assertThat(options.bottommostCompressionType())
-            .isEqualTo(compressionType);
-      }
-    }
-  }
-
-  @Test
-  public void compressionOptions() {
-    try (final Options options = new Options();
-         final CompressionOptions compressionOptions = new CompressionOptions()
-             .setMaxDictBytes(123)) {
-
-      options.setCompressionOptions(compressionOptions);
-      assertThat(options.compressionOptions())
-          .isEqualTo(compressionOptions);
-      assertThat(options.compressionOptions().maxDictBytes())
-          .isEqualTo(123);
-    }
-  }
-
   @Test
   public void compactionStyles() {
     try (final Options options = new Options()) {
@@ -966,15 +787,16 @@ public class OptionsTest {
   }
 
   @Test
-  public void rateLimiter() {
+  public void rateLimiterConfig() {
     try (final Options options = new Options();
-         final Options anotherOptions = new Options();
-         final RateLimiter rateLimiter =
-             new RateLimiter(1000, 100 * 1000, 1)) {
-      options.setRateLimiter(rateLimiter);
+         final Options anotherOptions = new Options()) {
+      final RateLimiterConfig rateLimiterConfig =
+          new GenericRateLimiterConfig(1000, 100 * 1000, 1);
+      options.setRateLimiterConfig(rateLimiterConfig);
       // Test with parameter initialization
-      anotherOptions.setRateLimiter(
-          new RateLimiter(1000));
+
+      anotherOptions.setRateLimiterConfig(
+          new GenericRateLimiterConfig(1000));
     }
   }
 
@@ -994,6 +816,7 @@ public class OptionsTest {
     }
   }
 
+
   @Test
   public void shouldTestMemTableFactoryName()
       throws RocksDBException {
@@ -1020,75 +843,4 @@ public class OptionsTest {
       }
     }
   }
-
-  @Test
-  public void maxWriteBufferNumberToMaintain() {
-    try (final Options options = new Options()) {
-      int intValue = rand.nextInt();
-      // Size has to be positive
-      intValue = (intValue < 0) ? -intValue : intValue;
-      intValue = (intValue == 0) ? intValue + 1 : intValue;
-      options.setMaxWriteBufferNumberToMaintain(intValue);
-      assertThat(options.maxWriteBufferNumberToMaintain()).
-          isEqualTo(intValue);
-    }
-  }
-
-  @Test
-  public void compactionPriorities() {
-    try (final Options options = new Options()) {
-      for (final CompactionPriority compactionPriority :
-          CompactionPriority.values()) {
-        options.setCompactionPriority(compactionPriority);
-        assertThat(options.compactionPriority()).
-            isEqualTo(compactionPriority);
-      }
-    }
-  }
-
-  @Test
-  public void reportBgIoStats() {
-    try (final Options options = new Options()) {
-      final boolean booleanValue = true;
-      options.setReportBgIoStats(booleanValue);
-      assertThat(options.reportBgIoStats()).
-          isEqualTo(booleanValue);
-    }
-  }
-
-  @Test
-  public void compactionOptionsUniversal() {
-    try (final Options options = new Options();
-         final CompactionOptionsUniversal optUni = new CompactionOptionsUniversal()
-             .setCompressionSizePercent(7)) {
-      options.setCompactionOptionsUniversal(optUni);
-      assertThat(options.compactionOptionsUniversal()).
-          isEqualTo(optUni);
-      assertThat(options.compactionOptionsUniversal().compressionSizePercent())
-          .isEqualTo(7);
-    }
-  }
-
-  @Test
-  public void compactionOptionsFIFO() {
-    try (final Options options = new Options();
-         final CompactionOptionsFIFO optFifo = new CompactionOptionsFIFO()
-             .setMaxTableFilesSize(2000)) {
-      options.setCompactionOptionsFIFO(optFifo);
-      assertThat(options.compactionOptionsFIFO()).
-          isEqualTo(optFifo);
-      assertThat(options.compactionOptionsFIFO().maxTableFilesSize())
-          .isEqualTo(2000);
-    }
-  }
-
-  @Test
-  public void forceConsistencyChecks() {
-    try (final Options options = new Options()) {
-      final boolean booleanValue = true;
-      options.setForceConsistencyChecks(booleanValue);
-      assertThat(options.forceConsistencyChecks()).
-          isEqualTo(booleanValue);
-    }
-  }
 }
