@@ -7,7 +7,6 @@
 #pragma once
 
 #include <string>
-#include "rocksdb/comparator.h"
 #include "rocksdb/iterator.h"
 #include "rocksdb/status.h"
 
@@ -37,11 +36,6 @@ class InternalIterator : public Cleanable {
   // an entry that comes at or past target.
   virtual void Seek(const Slice& target) = 0;
 
-  // Position at the first key in the source that at or before target
-  // The iterator is Valid() after this call iff the source contains
-  // an entry that comes at or before target.
-  virtual void SeekForPrev(const Slice& target) = 0;
-
   // Moves to the next entry in the source.  After this call, Valid() is
   // true iff the iterator was not positioned at the last entry in the source.
   // REQUIRES: Valid()
@@ -95,17 +89,6 @@ class InternalIterator : public Cleanable {
     return Status::NotSupported("");
   }
 
- protected:
-  void SeekForPrevImpl(const Slice& target, const Comparator* cmp) {
-    Seek(target);
-    if (!Valid()) {
-      SeekToLast();
-    }
-    while (Valid() && cmp->Compare(target, key()) < 0) {
-      Prev();
-    }
-  }
-
  private:
   // No copying allowed
   InternalIterator(const InternalIterator&) = delete;
