@@ -19,11 +19,12 @@ import argparse
 default_params = {
     "block_size": 16384,
     "cache_size": 1048576,
+    "use_clock_cache": "false",
     "delpercent": 5,
     "destroy_db_initially": 0,
     "disable_data_sync": 0,
     "disable_wal": 0,
-    "filter_deletes": lambda: random.randint(0, 1),
+    "allow_concurrent_memtable_write": 0,
     "iterpercent": 10,
     "max_background_compactions": 20,
     "max_bytes_for_level_base": 10485760,
@@ -44,6 +45,9 @@ default_params = {
     "verify_checksum": 1,
     "write_buffer_size": 4 * 1024 * 1024,
     "writepercent": 35,
+    "subcompactions": lambda: random.randint(1, 4),
+    "use_merge": lambda: random.randint(0, 1),
+    "use_full_merge_v1": lambda: random.randint(0, 1),
 }
 
 
@@ -74,17 +78,20 @@ whitebox_default_params = {
     "ops_per_thread": 200000,
     "test_batches_snapshots": lambda: random.randint(0, 1),
     "write_buffer_size": 4 * 1024 * 1024,
+    "subcompactions": lambda: random.randint(1, 4),
+    "random_kill_odd": 888887,
 }
 
 simple_default_params = {
     "block_size": 16384,
     "cache_size": 1048576,
+    "use_clock_cache": "false",
     "column_families": 1,
     "delpercent": 5,
     "destroy_db_initially": 0,
     "disable_data_sync": 0,
     "disable_wal": 0,
-    "filter_deletes": lambda: random.randint(0, 1),
+    "allow_concurrent_memtable_write": lambda: random.randint(0, 1),
     "iterpercent": 10,
     "max_background_compactions": 1,
     "max_bytes_for_level_base": 67108864,
@@ -105,6 +112,7 @@ simple_default_params = {
     "verify_checksum": 1,
     "write_buffer_size": 32 * 1024 * 1024,
     "writepercent": 35,
+    "subcompactions": lambda: random.randint(1, 4),
 }
 
 blackbox_simple_default_params = {
@@ -123,9 +131,18 @@ whitebox_simple_default_params = {
     "open_files": 500000,
     "ops_per_thread": 200000,
     "write_buffer_size": 32 * 1024 * 1024,
+    "subcompactions": lambda: random.randint(1, 4),
 }
 
 
+def finalize_and_sanitize(src_params):
+    dest_params = dict([(k,  v() if callable(v) else v)
+                        for (k, v) in src_params.items()])
+    if dest_params.get("allow_concurrent_memtable_write", 1) == 1:
+        dest_params["memtablerep"] = "skip_list"
+    return dest_params
+
+
 def gen_cmd_params(args):
     params = {}
 
@@ -151,9 +168,10 @@ def gen_cmd_params(args):
 
 def gen_cmd(params):
     cmd = './db_stress ' + ' '.join(
-        '--{0}={1}'.format(k, v() if callable(v) else v)
-        for k, v in params.items()
-        if k not in set(['test_type', 'simple', 'duration', 'interval'])
+        '--{0}={1}'.format(k, v)
+        for k, v in finalize_and_sanitize(params).items()
+        if k not in set(['test_type', 'simple', 'duration', 'interval',
+                         'random_kill_odd'])
         and v is not None)
     return cmd
 
@@ -170,7 +188,8 @@ def blackbox_crash_main(args):
           + "total-duration=" + str(cmd_params['duration']) + "\n"
           + "threads=" + str(cmd_params['threads']) + "\n"
           + "ops_per_thread=" + str(cmd_params['ops_per_thread']) + "\n"
-          + "write_buffer_size=" + str(cmd_params['write_buffer_size']) + "\n")
+          + "write_buffer_size=" + str(cmd_params['write_buffer_size']) + "\n"
+          + "subcompactions=" + str(cmd_params['subcompactions']) + "\n")
 
     while time.time() < exit_time:
         run_had_errors = False
@@ -232,11 +251,12 @@ def whitebox_crash_main(args):
           + "total-duration=" + str(cmd_params['duration']) + "\n"
           + "threads=" + str(cmd_params['threads']) + "\n"
           + "ops_per_thread=" + str(cmd_params['ops_per_thread']) + "\n"
-          + "write_buffer_size=" + str(cmd_params['write_buffer_size']) + "\n")
+          + "write_buffer_size=" + str(cmd_params['write_buffer_size']) + "\n"
+          + "subcompactions=" + str(cmd_params['subcompactions']) + "\n")
 
     total_check_mode = 4
     check_mode = 0
-    kill_random_test = 97
+    kill_random_test = cmd_params['random_kill_odd']
     kill_mode = 0
 
     while time.time() < exit_time:
@@ -255,13 +275,15 @@ def whitebox_crash_main(args):
                 })
             elif kill_mode == 1:
                 additional_opts.update({
-                    "kill_random_test": (kill_random_test / 2 + 1),
+                    "kill_random_test": (kill_random_test / 10 + 1),
                     "kill_prefix_blacklist": "WritableFileWriter::Append,"
                     + "WritableFileWriter::WriteBuffered",
                 })
             elif kill_mode == 2:
+                # TODO: May need to adjust random odds if kill_random_test
+                # is too small.
                 additional_opts.update({
-                    "kill_random_test": (kill_random_test / 4 + 1),
+                    "kill_random_test": (kill_random_test / 5000 + 1),
                     "kill_prefix_blacklist": "WritableFileWriter::Append,"
                     "WritableFileWriter::WriteBuffered,"
                     "PosixMmapFile::Allocate,WritableFileWriter::Flush",
