@@ -14,35 +14,32 @@
 
 #include "db/db_impl.h"
 #include "db/dbformat.h"
+#include "db/filename.h"
 #include "db/log_reader.h"
 #include "db/write_batch_internal.h"
 #include "port/dirent.h"
 #include "rocksdb/cache.h"
 #include "rocksdb/table_properties.h"
 #include "rocksdb/utilities/backupable_db.h"
-#include "rocksdb/utilities/checkpoint.h"
-#include "rocksdb/utilities/object_registry.h"
-#include "rocksdb/utilities/options_util.h"
+#include "rocksdb/utilities/env_registry.h"
 #include "rocksdb/write_batch.h"
 #include "rocksdb/write_buffer_manager.h"
 #include "table/scoped_arena_iterator.h"
 #include "tools/ldb_cmd_impl.h"
 #include "tools/sst_dump_tool_imp.h"
 #include "util/coding.h"
-#include "util/filename.h"
 #include "util/stderr_logger.h"
 #include "util/string_util.h"
 #include "utilities/ttl/db_ttl_impl.h"
 
 #include <cstdlib>
 #include <ctime>
-#include <fstream>
-#include <functional>
 #include <iostream>
 #include <limits>
 #include <sstream>
 #include <stdexcept>
 #include <string>
+#include <fstream>
 
 namespace rocksdb {
 
@@ -56,7 +53,6 @@ const std::string LDBCommand::ARG_TTL = "ttl";
 const std::string LDBCommand::ARG_TTL_START = "start_time";
 const std::string LDBCommand::ARG_TTL_END = "end_time";
 const std::string LDBCommand::ARG_TIMESTAMP = "timestamp";
-const std::string LDBCommand::ARG_TRY_LOAD_OPTIONS = "try_load_options";
 const std::string LDBCommand::ARG_FROM = "from";
 const std::string LDBCommand::ARG_TO = "to";
 const std::string LDBCommand::ARG_MAX_KEYS = "max_keys";
@@ -171,10 +167,6 @@ LDBCommand* LDBCommand::SelectCommand(const ParsedParams& parsed_params) {
   } else if (parsed_params.cmd == DeleteCommand::Name()) {
     return new DeleteCommand(parsed_params.cmd_params, parsed_params.option_map,
                              parsed_params.flags);
-  } else if (parsed_params.cmd == DeleteRangeCommand::Name()) {
-    return new DeleteRangeCommand(parsed_params.cmd_params,
-                                  parsed_params.option_map,
-                                  parsed_params.flags);
   } else if (parsed_params.cmd == ApproxSizeCommand::Name()) {
     return new ApproxSizeCommand(parsed_params.cmd_params,
                                  parsed_params.option_map, parsed_params.flags);
@@ -225,10 +217,6 @@ LDBCommand* LDBCommand::SelectCommand(const ParsedParams& parsed_params) {
     return new CheckConsistencyCommand(parsed_params.cmd_params,
                                        parsed_params.option_map,
                                        parsed_params.flags);
-  } else if (parsed_params.cmd == CheckPointCommand::Name()) {
-    return new CheckPointCommand(parsed_params.cmd_params,
-                                 parsed_params.option_map,
-                                 parsed_params.flags);
   } else if (parsed_params.cmd == RepairCommand::Name()) {
     return new RepairCommand(parsed_params.cmd_params, parsed_params.option_map,
                              parsed_params.flags);
@@ -250,12 +238,6 @@ void LDBCommand::Run() {
 
   if (db_ == nullptr && !NoDBOpen()) {
     OpenDB();
-    if (exec_state_.IsFailed() && try_load_options_) {
-      // We don't always return if there is a failure because a WAL file or
-      // manifest file can be given to "dump" command so we should continue.
-      // --try_load_options is not valid in those cases.
-      return;
-    }
   }
 
   // We'll intentionally proceed even if the DB can't be opened because users
@@ -280,8 +262,6 @@ LDBCommand::LDBCommand(const std::map<std::string, std::string>& options,
       is_value_hex_(false),
       is_db_ttl_(false),
       timestamp_(false),
-      try_load_options_(false),
-      create_if_missing_(false),
       option_map_(options),
       flags_(flags),
       valid_cmd_line_options_(valid_cmd_line_options) {
@@ -301,28 +281,10 @@ LDBCommand::LDBCommand(const std::map<std::string, std::string>& options,
   is_value_hex_ = IsValueHex(options, flags);
   is_db_ttl_ = IsFlagPresent(flags, ARG_TTL);
   timestamp_ = IsFlagPresent(flags, ARG_TIMESTAMP);
-  try_load_options_ = IsFlagPresent(flags, ARG_TRY_LOAD_OPTIONS);
 }
 
 void LDBCommand::OpenDB() {
-  Options opt;
-  bool opt_set = false;
-  if (!create_if_missing_ && try_load_options_) {
-    Status s =
-        LoadLatestOptions(db_path_, Env::Default(), &opt, &column_families_);
-    if (s.ok()) {
-      opt_set = true;
-    } else if (!s.IsNotFound()) {
-      // Option file exists but load option file error.
-      std::string msg = s.ToString();
-      exec_state_ = LDBCommandExecuteResult::Failed(msg);
-      db_ = nullptr;
-      return;
-    }
-  }
-  if (!opt_set) {
-    opt = PrepareOptionsForOpenDB();
-  }
+  Options opt = PrepareOptionsForOpenDB();
   if (!exec_state_.IsNotStarted()) {
     return;
   }
@@ -342,7 +304,7 @@ void LDBCommand::OpenDB() {
     }
     db_ = db_ttl_;
   } else {
-    if (!opt_set && column_families_.empty()) {
+    if (column_families_.empty()) {
       // Try to figure out column family lists
       std::vector<std::string> cf_list;
       st = DB::ListColumnFamilies(DBOptions(), db_path_, &cf_list);
@@ -436,7 +398,6 @@ std::vector<std::string> LDBCommand::BuildCmdLineOptions(
                                   ARG_WRITE_BUFFER_SIZE,
                                   ARG_FILE_SIZE,
                                   ARG_FIX_PREFIX_LEN,
-                                  ARG_TRY_LOAD_OPTIONS,
                                   ARG_CF_NAME};
   ret.insert(ret.end(), options.begin(), options.end());
   return ret;
@@ -814,7 +775,7 @@ void CompactorCommand::DoCommand() {
   CompactRangeOptions cro;
   cro.bottommost_level_compaction = BottommostLevelCompaction::kForce;
 
-  db_->CompactRange(cro, GetCfHandle(), begin, end);
+  db_->CompactRange(cro, begin, end);
   exec_state_ = LDBCommandExecuteResult::Succeed("");
 
   delete begin;
@@ -836,6 +797,7 @@ DBLoaderCommand::DBLoaderCommand(
           BuildCmdLineOptions({ARG_HEX, ARG_KEY_HEX, ARG_VALUE_HEX, ARG_FROM,
                                ARG_TO, ARG_CREATE_IF_MISSING, ARG_DISABLE_WAL,
                                ARG_BULK_LOAD, ARG_COMPACT})),
+      create_if_missing_(false),
       disable_wal_(false),
       bulk_load_(false),
       compact_(false) {
@@ -919,8 +881,7 @@ void DumpManifestFile(std::string file, bool verbose, bool hex, bool json) {
   options.num_levels = 64;
   WriteController wc(options.delayed_write_rate);
   WriteBufferManager wb(options.db_write_buffer_size);
-  ImmutableDBOptions immutable_db_options(options);
-  VersionSet versions(dbname, &immutable_db_options, sopt, tc.get(), &wb, &wc);
+  VersionSet versions(dbname, &options, sopt, tc.get(), &wb, &wc);
   Status s = versions.DumpManifest(options, file, verbose, hex, json);
   if (!s.ok()) {
     printf("Error in processing file %s %s\n", file.c_str(),
@@ -1221,9 +1182,7 @@ void InternalDumpCommand::DoCommand() {
   uint64_t s1=0,s2=0;
   // Setup internal key iterator
   Arena arena;
-  auto icmp = InternalKeyComparator(options_.comparator);
-  RangeDelAggregator range_del_agg(icmp, {} /* snapshots */);
-  ScopedArenaIterator iter(idb->NewInternalIterator(&arena, &range_del_agg));
+  ScopedArenaIterator iter(idb->NewInternalIterator(&arena));
   Status st = iter->status();
   if (!st.ok()) {
     exec_state_ =
@@ -1626,14 +1585,13 @@ Options ReduceDBLevelsCommand::PrepareOptionsForOpenDB() {
 
 Status ReduceDBLevelsCommand::GetOldNumOfLevels(Options& opt,
     int* levels) {
-  ImmutableDBOptions db_options(opt);
   EnvOptions soptions;
   std::shared_ptr<Cache> tc(
       NewLRUCache(opt.max_open_files - 10, opt.table_cache_numshardbits));
   const InternalKeyComparator cmp(opt.comparator);
   WriteController wc(opt.delayed_write_rate);
   WriteBufferManager wb(opt.db_write_buffer_size);
-  VersionSet versions(db_path_, &db_options, soptions, tc.get(), &wb, &wc);
+  VersionSet versions(db_path_, &opt, soptions, tc.get(), &wb, &wc);
   std::vector<ColumnFamilyDescriptor> dummy;
   ColumnFamilyDescriptor dummy_descriptor(kDefaultColumnFamilyName,
                                           ColumnFamilyOptions(opt));
@@ -1684,7 +1642,7 @@ void ReduceDBLevelsCommand::DoCommand() {
   old_levels_ = old_level_num;
 
   OpenDB();
-  if (exec_state_.IsFailed()) {
+  if (!db_) {
     return;
   }
   // Compact the whole DB to put all files to the highest level.
@@ -1803,7 +1761,7 @@ void ChangeCompactionStyleCommand::DoCommand() {
   // verify compaction result
   files_per_level = "";
   int num_files = 0;
-  for (int i = 0; i < db_->NumberLevels(GetCfHandle()); i++) {
+  for (int i = 0; i < db_->NumberLevels(); i++) {
     db_->GetProperty(GetCfHandle(),
                      "rocksdb.num-files-at-level" + NumberToString(i),
                      &property);
@@ -1892,14 +1850,6 @@ class InMemoryHandler : public WriteBatch::Handler {
     return Status::OK();
   }
 
-  virtual Status DeleteRangeCF(uint32_t cf, const Slice& begin_key,
-                               const Slice& end_key) override {
-    row_ << "DELETE_RANGE(" << cf << ") : ";
-    row_ << LDBCommand::StringToHex(begin_key.ToString()) << " ";
-    row_ << LDBCommand::StringToHex(end_key.ToString()) << " ";
-    return Status::OK();
-  }
-
   virtual Status MarkBeginPrepare() override {
     row_ << "BEGIN_PREARE ";
     return Status::OK();
@@ -2168,7 +2118,6 @@ BatchPutCommand::BatchPutCommand(
           is_value_hex_ ? HexToString(value) : value));
     }
   }
-  create_if_missing_ = IsFlagPresent(flags_, ARG_CREATE_IF_MISSING);
 }
 
 void BatchPutCommand::Help(std::string& ret) {
@@ -2201,7 +2150,7 @@ void BatchPutCommand::DoCommand() {
 
 Options BatchPutCommand::PrepareOptionsForOpenDB() {
   Options opt = LDBCommand::PrepareOptionsForOpenDB();
-  opt.create_if_missing = create_if_missing_;
+  opt.create_if_missing = IsFlagPresent(flags_, ARG_CREATE_IF_MISSING);
   return opt;
 }
 
@@ -2393,45 +2342,6 @@ void DeleteCommand::DoCommand() {
   }
 }
 
-DeleteRangeCommand::DeleteRangeCommand(
-    const std::vector<std::string>& params,
-    const std::map<std::string, std::string>& options,
-    const std::vector<std::string>& flags)
-    : LDBCommand(options, flags, false,
-                 BuildCmdLineOptions({ARG_HEX, ARG_KEY_HEX, ARG_VALUE_HEX})) {
-  if (params.size() != 2) {
-    exec_state_ = LDBCommandExecuteResult::Failed(
-        "begin and end keys must be specified for the delete command");
-  } else {
-    begin_key_ = params.at(0);
-    end_key_ = params.at(1);
-    if (is_key_hex_) {
-      begin_key_ = HexToString(begin_key_);
-      end_key_ = HexToString(end_key_);
-    }
-  }
-}
-
-void DeleteRangeCommand::Help(std::string& ret) {
-  ret.append("  ");
-  ret.append(DeleteRangeCommand::Name() + " <begin key> <end key>");
-  ret.append("\n");
-}
-
-void DeleteRangeCommand::DoCommand() {
-  if (!db_) {
-    assert(GetExecuteState().IsFailed());
-    return;
-  }
-  Status st =
-      db_->DeleteRange(WriteOptions(), GetCfHandle(), begin_key_, end_key_);
-  if (st.ok()) {
-    fprintf(stdout, "OK\n");
-  } else {
-    exec_state_ = LDBCommandExecuteResult::Failed(st.ToString());
-  }
-}
-
 PutCommand::PutCommand(const std::vector<std::string>& params,
                        const std::map<std::string, std::string>& options,
                        const std::vector<std::string>& flags)
@@ -2453,7 +2363,6 @@ PutCommand::PutCommand(const std::vector<std::string>& params,
   if (is_value_hex_) {
     value_ = HexToString(value_);
   }
-  create_if_missing_ = IsFlagPresent(flags_, ARG_CREATE_IF_MISSING);
 }
 
 void PutCommand::Help(std::string& ret) {
@@ -2479,7 +2388,7 @@ void PutCommand::DoCommand() {
 
 Options PutCommand::PrepareOptionsForOpenDB() {
   Options opt = LDBCommand::PrepareOptionsForOpenDB();
-  opt.create_if_missing = create_if_missing_;
+  opt.create_if_missing = IsFlagPresent(flags_, ARG_CREATE_IF_MISSING);
   return opt;
 }
 
@@ -2599,47 +2508,6 @@ void CheckConsistencyCommand::DoCommand() {
 
 // ----------------------------------------------------------------------------
 
-const std::string CheckPointCommand::ARG_CHECKPOINT_DIR = "checkpoint_dir";
-
-CheckPointCommand::CheckPointCommand(
-    const std::vector<std::string>& params,
-    const std::map<std::string, std::string>& options,
-    const std::vector<std::string>& flags)
-    : LDBCommand(options, flags, false /* is_read_only */,
-                 BuildCmdLineOptions({ARG_CHECKPOINT_DIR})) {
-  auto itr = options.find(ARG_CHECKPOINT_DIR);
-  if (itr == options.end()) {
-    exec_state_ = LDBCommandExecuteResult::Failed(
-        "--" + ARG_CHECKPOINT_DIR + ": missing checkpoint directory");
-  } else {
-    checkpoint_dir_ = itr->second;
-  }
-}
-
-void CheckPointCommand::Help(std::string& ret) {
-  ret.append("  ");
-  ret.append(CheckPointCommand::Name());
-  ret.append(" [--" + ARG_CHECKPOINT_DIR + "] ");
-  ret.append("\n");
-}
-
-void CheckPointCommand::DoCommand() {
-  if (!db_) {
-    assert(GetExecuteState().IsFailed());
-    return;
-  }
-  Checkpoint* checkpoint;
-  Status status = Checkpoint::Create(db_, &checkpoint);
-  status = checkpoint->CreateCheckpoint(checkpoint_dir_);
-  if (status.ok()) {
-    printf("OK\n");
-  } else {
-    exec_state_ = LDBCommandExecuteResult::Failed(status.ToString());
-  }
-}
-
-// ----------------------------------------------------------------------------
-
 RepairCommand::RepairCommand(const std::vector<std::string>& params,
                              const std::map<std::string, std::string>& options,
                              const std::vector<std::string>& flags)
@@ -2664,69 +2532,41 @@ void RepairCommand::DoCommand() {
 
 // ----------------------------------------------------------------------------
 
-const std::string BackupableCommand::ARG_NUM_THREADS = "num_threads";
-const std::string BackupableCommand::ARG_BACKUP_ENV_URI = "backup_env_uri";
-const std::string BackupableCommand::ARG_BACKUP_DIR = "backup_dir";
-const std::string BackupableCommand::ARG_STDERR_LOG_LEVEL = "stderr_log_level";
+const std::string BackupCommand::ARG_THREAD = "num_threads";
+const std::string BackupCommand::ARG_BACKUP_ENV = "backup_env_uri";
+const std::string BackupCommand::ARG_BACKUP_DIR = "backup_dir";
 
-BackupableCommand::BackupableCommand(
-    const std::vector<std::string>& params,
-    const std::map<std::string, std::string>& options,
-    const std::vector<std::string>& flags)
-    : LDBCommand(options, flags, false /* is_read_only */,
-                 BuildCmdLineOptions({ARG_BACKUP_ENV_URI, ARG_BACKUP_DIR,
-                                      ARG_NUM_THREADS, ARG_STDERR_LOG_LEVEL})),
-      num_threads_(1) {
-  auto itr = options.find(ARG_NUM_THREADS);
+BackupCommand::BackupCommand(const std::vector<std::string>& params,
+                             const std::map<std::string, std::string>& options,
+                             const std::vector<std::string>& flags)
+    : LDBCommand(
+          options, flags, false,
+          BuildCmdLineOptions({ARG_BACKUP_ENV, ARG_BACKUP_DIR, ARG_THREAD})),
+      thread_num_(1) {
+  auto itr = options.find(ARG_THREAD);
   if (itr != options.end()) {
-    num_threads_ = std::stoi(itr->second);
+    thread_num_ = std::stoi(itr->second);
   }
-  itr = options.find(ARG_BACKUP_ENV_URI);
+  itr = options.find(ARG_BACKUP_ENV);
   if (itr != options.end()) {
-    backup_env_uri_ = itr->second;
+    test_cluster_ = itr->second;
   }
   itr = options.find(ARG_BACKUP_DIR);
   if (itr == options.end()) {
     exec_state_ = LDBCommandExecuteResult::Failed("--" + ARG_BACKUP_DIR +
                                                   ": missing backup directory");
   } else {
-    backup_dir_ = itr->second;
-  }
-
-  itr = options.find(ARG_STDERR_LOG_LEVEL);
-  if (itr != options.end()) {
-    int stderr_log_level = std::stoi(itr->second);
-    if (stderr_log_level < 0 ||
-        stderr_log_level >= InfoLogLevel::NUM_INFO_LOG_LEVELS) {
-      exec_state_ = LDBCommandExecuteResult::Failed(
-          ARG_STDERR_LOG_LEVEL + " must be >= 0 and < " +
-          std::to_string(InfoLogLevel::NUM_INFO_LOG_LEVELS) + ".");
-    } else {
-      logger_.reset(
-          new StderrLogger(static_cast<InfoLogLevel>(stderr_log_level)));
-    }
+    test_path_ = itr->second;
   }
 }
 
-void BackupableCommand::Help(const std::string& name, std::string& ret) {
-  ret.append("  ");
-  ret.append(name);
-  ret.append(" [--" + ARG_BACKUP_ENV_URI + "] ");
-  ret.append(" [--" + ARG_BACKUP_DIR + "] ");
-  ret.append(" [--" + ARG_NUM_THREADS + "] ");
-  ret.append(" [--" + ARG_STDERR_LOG_LEVEL + "=<int (InfoLogLevel)>] ");
-  ret.append("\n");
-}
-
-// ----------------------------------------------------------------------------
-
-BackupCommand::BackupCommand(const std::vector<std::string>& params,
-                             const std::map<std::string, std::string>& options,
-                             const std::vector<std::string>& flags)
-    : BackupableCommand(params, options, flags) {}
-
 void BackupCommand::Help(std::string& ret) {
-  BackupableCommand::Help(Name(), ret);
+  ret.append("  ");
+  ret.append(BackupCommand::Name());
+  ret.append(" [--" + ARG_BACKUP_ENV + "] ");
+  ret.append(" [--" + ARG_BACKUP_DIR + "] ");
+  ret.append(" [--" + ARG_THREAD + "] ");
+  ret.append("\n");
 }
 
 void BackupCommand::DoCommand() {
@@ -2738,11 +2578,10 @@ void BackupCommand::DoCommand() {
   }
   printf("open db OK\n");
   std::unique_ptr<Env> custom_env_guard;
-  Env* custom_env = NewCustomObject<Env>(backup_env_uri_, &custom_env_guard);
+  Env* custom_env = NewEnvFromUri(test_cluster_, &custom_env_guard);
   BackupableDBOptions backup_options =
-      BackupableDBOptions(backup_dir_, custom_env);
-  backup_options.info_log = logger_.get();
-  backup_options.max_background_operations = num_threads_;
+      BackupableDBOptions(test_path_, custom_env);
+  backup_options.max_background_operations = thread_num_;
   status = BackupEngine::Open(Env::Default(), backup_options, &backup_engine);
   if (status.ok()) {
     printf("open backup engine OK\n");
@@ -2761,24 +2600,51 @@ void BackupCommand::DoCommand() {
 
 // ----------------------------------------------------------------------------
 
+const std::string RestoreCommand::ARG_BACKUP_ENV_URI = "backup_env_uri";
+const std::string RestoreCommand::ARG_BACKUP_DIR = "backup_dir";
+const std::string RestoreCommand::ARG_NUM_THREADS = "num_threads";
+
 RestoreCommand::RestoreCommand(
     const std::vector<std::string>& params,
     const std::map<std::string, std::string>& options,
     const std::vector<std::string>& flags)
-    : BackupableCommand(params, options, flags) {}
+    : LDBCommand(options, flags, false /* is_read_only */,
+                 BuildCmdLineOptions(
+                     {ARG_BACKUP_ENV_URI, ARG_BACKUP_DIR, ARG_NUM_THREADS})),
+      num_threads_(1) {
+  auto itr = options.find(ARG_NUM_THREADS);
+  if (itr != options.end()) {
+    num_threads_ = std::stoi(itr->second);
+  }
+  itr = options.find(ARG_BACKUP_ENV_URI);
+  if (itr != options.end()) {
+    backup_env_uri_ = itr->second;
+  }
+  itr = options.find(ARG_BACKUP_DIR);
+  if (itr == options.end()) {
+    exec_state_ = LDBCommandExecuteResult::Failed("--" + ARG_BACKUP_DIR +
+                                                  ": missing backup directory");
+  } else {
+    backup_dir_ = itr->second;
+  }
+}
 
 void RestoreCommand::Help(std::string& ret) {
-  BackupableCommand::Help(Name(), ret);
+  ret.append("  ");
+  ret.append(RestoreCommand::Name());
+  ret.append(" [--" + ARG_BACKUP_ENV_URI + "] ");
+  ret.append(" [--" + ARG_BACKUP_DIR + "] ");
+  ret.append(" [--" + ARG_NUM_THREADS + "] ");
+  ret.append("\n");
 }
 
 void RestoreCommand::DoCommand() {
   std::unique_ptr<Env> custom_env_guard;
-  Env* custom_env = NewCustomObject<Env>(backup_env_uri_, &custom_env_guard);
+  Env* custom_env = NewEnvFromUri(backup_env_uri_, &custom_env_guard);
   std::unique_ptr<BackupEngineReadOnly> restore_engine;
   Status status;
   {
     BackupableDBOptions opts(backup_dir_, custom_env);
-    opts.info_log = logger_.get();
     opts.max_background_operations = num_threads_;
     BackupEngineReadOnly* raw_restore_engine_ptr;
     status = BackupEngineReadOnly::Open(Env::Default(), opts,
