@@ -12,9 +12,9 @@
 #include <algorithm>
 #include <mutex>
 
-#include "monitoring/histogram.h"
-#include "monitoring/iostats_context_imp.h"
 #include "port/port.h"
+#include "util/histogram.h"
+#include "util/iostats_context_imp.h"
 #include "util/random.h"
 #include "util/rate_limiter.h"
 #include "util/sync_point.h"
@@ -22,44 +22,12 @@
 namespace rocksdb {
 
 Status SequentialFileReader::Read(size_t n, Slice* result, char* scratch) {
-  Status s;
-  if (use_direct_io()) {
-#ifndef ROCKSDB_LITE
-    size_t offset = offset_.fetch_add(n);
-    size_t alignment = file_->GetRequiredBufferAlignment();
-    size_t aligned_offset = TruncateToPageBoundary(alignment, offset);
-    size_t offset_advance = offset - aligned_offset;
-    size_t size = Roundup(offset + n, alignment) - aligned_offset;
-    size_t r = 0;
-    AlignedBuffer buf;
-    buf.Alignment(alignment);
-    buf.AllocateNewBuffer(size);
-    Slice tmp;
-    s = file_->PositionedRead(aligned_offset, size, &tmp, buf.BufferStart());
-    if (s.ok() && offset_advance < tmp.size()) {
-      buf.Size(tmp.size());
-      r = buf.Read(scratch, offset_advance,
-                   std::min(tmp.size() - offset_advance, n));
-    }
-    *result = Slice(scratch, r);
-#endif  // !ROCKSDB_LITE
-  } else {
-    s = file_->Read(n, result, scratch);
-  }
+  Status s = file_->Read(n, result, scratch);
   IOSTATS_ADD(bytes_read, result->size());
   return s;
 }
 
-
-Status SequentialFileReader::Skip(uint64_t n) {
-#ifndef ROCKSDB_LITE
-  if (use_direct_io()) {
-    offset_ += n;
-    return Status::OK();
-  }
-#endif  // !ROCKSDB_LITE
-  return file_->Skip(n);
-}
+Status SequentialFileReader::Skip(uint64_t n) { return file_->Skip(n); }
 
 Status RandomAccessFileReader::Read(uint64_t offset, size_t n, Slice* result,
                                     char* scratch) const {
@@ -69,28 +37,7 @@ Status RandomAccessFileReader::Read(uint64_t offset, size_t n, Slice* result,
     StopWatch sw(env_, stats_, hist_type_,
                  (stats_ != nullptr) ? &elapsed : nullptr);
     IOSTATS_TIMER_GUARD(read_nanos);
-    if (use_direct_io()) {
-#ifndef ROCKSDB_LITE
-      size_t alignment = file_->GetRequiredBufferAlignment();
-      size_t aligned_offset = TruncateToPageBoundary(alignment, offset);
-      size_t offset_advance = offset - aligned_offset;
-      size_t size = Roundup(offset + n, alignment) - aligned_offset;
-      size_t r = 0;
-      AlignedBuffer buf;
-      buf.Alignment(alignment);
-      buf.AllocateNewBuffer(size);
-      Slice tmp;
-      s = file_->Read(aligned_offset, size, &tmp, buf.BufferStart());
-      if (s.ok() && offset_advance < tmp.size()) {
-        buf.Size(tmp.size());
-        r = buf.Read(scratch, offset_advance,
-                            std::min(tmp.size() - offset_advance, n));
-      }
-      *result = Slice(scratch, r);
-#endif  // !ROCKSDB_LITE
-    } else {
-      s = file_->Read(offset, n, result, scratch);
-    }
+    s = file_->Read(offset, n, result, scratch);
     IOSTATS_ADD_IF_POSITIVE(bytes_read, result->size());
   }
   if (stats_ != nullptr && file_read_hist_ != nullptr) {
@@ -114,8 +61,9 @@ Status WritableFileWriter::Append(const Slice& data) {
     writable_file_->PrepareWrite(static_cast<size_t>(GetFileSize()), left);
   }
 
-  // Flush only when buffered I/O
-  if (!use_direct_io() && (buf_.Capacity() - buf_.CurrentSize()) < left) {
+  // Flush only when I/O is buffered
+  if (use_os_buffer_ &&
+    (buf_.Capacity() - buf_.CurrentSize()) < left) {
     if (buf_.CurrentSize() > 0) {
       s = Flush();
       if (!s.ok()) {
@@ -131,10 +79,10 @@ Status WritableFileWriter::Append(const Slice& data) {
     assert(buf_.CurrentSize() == 0);
   }
 
-  // We never write directly to disk with direct I/O on.
+  // We never write directly to disk with unbuffered I/O on.
   // or we simply use it for its original purpose to accumulate many small
   // chunks
-  if (use_direct_io() || (buf_.Capacity() >= left)) {
+  if (!use_os_buffer_ || (buf_.Capacity() >= left)) {
     while (left > 0) {
       size_t appended = buf_.Append(src, left);
       left -= appended;
@@ -148,7 +96,7 @@ Status WritableFileWriter::Append(const Slice& data) {
 
         // We double the buffer here because
         // Flush calls do not keep up with the incoming bytes
-        // This is the only place when buffer is changed with direct I/O
+        // This is the only place when buffer is changed with unbuffered I/O
         if (buf_.Capacity() < max_buffer_size_) {
           size_t desiredCapacity = buf_.Capacity() * 2;
           desiredCapacity = std::min(desiredCapacity, max_buffer_size_);
@@ -184,14 +132,11 @@ Status WritableFileWriter::Close() {
 
   s = Flush();  // flush cache to OS
 
-  Status interim;
-  // In direct I/O mode we write whole pages so
+  // In unbuffered mode we write whole pages so
   // we need to let the file know where data ends.
-  if (use_direct_io()) {
-    interim = writable_file_->Truncate(filesize_);
-    if (!interim.ok() && s.ok()) {
-      s = interim;
-    }
+  Status interim = writable_file_->Truncate(filesize_);
+  if (!interim.ok() && s.ok()) {
+    s = interim;
   }
 
   TEST_KILL_RANDOM("WritableFileWriter::Close:0", rocksdb_kill_odds);
@@ -206,20 +151,17 @@ Status WritableFileWriter::Close() {
   return s;
 }
 
-// write out the cached data to the OS cache or storage if direct I/O
-// enabled
+// write out the cached data to the OS cache
 Status WritableFileWriter::Flush() {
   Status s;
   TEST_KILL_RANDOM("WritableFileWriter::Flush:0",
                    rocksdb_kill_odds * REDUCE_ODDS2);
 
   if (buf_.CurrentSize() > 0) {
-    if (use_direct_io()) {
-#ifndef ROCKSDB_LITE
-      s = WriteDirect();
-#endif  // !ROCKSDB_LITE
-    } else {
+    if (use_os_buffer_) {
       s = WriteBuffered(buf_.BufferStart(), buf_.CurrentSize());
+    } else {
+      s = WriteUnbuffered();
     }
     if (!s.ok()) {
       return s;
@@ -243,7 +185,7 @@ Status WritableFileWriter::Flush() {
   //     the page.
   // Xfs does neighbor page flushing outside of the specified ranges. We
   // need to make sure sync range is far from the write offset.
-  if (!use_direct_io() && bytes_per_sync_) {
+  if (!direct_io_ && bytes_per_sync_) {
     const uint64_t kBytesNotSyncRange = 1024 * 1024;  // recent 1MB is not synced.
     const uint64_t kBytesAlignWhenSync = 4 * 1024;    // Align 4KB.
     if (filesize_ > kBytesNotSyncRange) {
@@ -267,7 +209,7 @@ Status WritableFileWriter::Sync(bool use_fsync) {
     return s;
   }
   TEST_KILL_RANDOM("WritableFileWriter::Sync:0", rocksdb_kill_odds);
-  if (!use_direct_io() && pending_sync_) {
+  if (!direct_io_ && pending_sync_) {
     s = SyncInternal(use_fsync);
     if (!s.ok()) {
       return s;
@@ -317,12 +259,12 @@ size_t WritableFileWriter::RequestToken(size_t bytes, bool align) {
 
     if (align) {
       // Here we may actually require more than burst and block
-      // but we can not write less than one page at a time on direct I/O
-      // thus we may want not to use ratelimiter
+      // but we can not write less than one page at a time on unbuffered
+      // thus we may want not to use ratelimiter s
       size_t alignment = buf_.Alignment();
       bytes = std::max(alignment, TruncateToPageBoundary(alignment, bytes));
     }
-    rate_limiter_->Request(bytes, io_priority, stats_);
+    rate_limiter_->Request(bytes, io_priority);
   }
   return bytes;
 }
@@ -331,7 +273,7 @@ size_t WritableFileWriter::RequestToken(size_t bytes, bool align) {
 // limiter if available
 Status WritableFileWriter::WriteBuffered(const char* data, size_t size) {
   Status s;
-  assert(!use_direct_io());
+  assert(use_os_buffer_);
   const char* src = data;
   size_t left = size;
 
@@ -366,10 +308,10 @@ Status WritableFileWriter::WriteBuffered(const char* data, size_t size) {
 // whole number of pages to be written again on the next flush because we can
 // only write on aligned
 // offsets.
-#ifndef ROCKSDB_LITE
-Status WritableFileWriter::WriteDirect() {
-  assert(use_direct_io());
+Status WritableFileWriter::WriteUnbuffered() {
   Status s;
+
+  assert(!use_os_buffer_);
   const size_t alignment = buf_.Alignment();
   assert((next_write_offset_ % alignment) == 0);
 
@@ -397,7 +339,7 @@ Status WritableFileWriter::WriteDirect() {
     {
       IOSTATS_TIMER_GUARD(write_nanos);
       TEST_SYNC_POINT("WritableFileWriter::Flush:BeforeAppend");
-      // direct writes must be positional
+      // Unbuffered writes must be positional
       s = writable_file_->PositionedAppend(Slice(src, size), write_offset);
       if (!s.ok()) {
         buf_.Size(file_advance + leftover_tail);
@@ -425,7 +367,7 @@ Status WritableFileWriter::WriteDirect() {
   }
   return s;
 }
-#endif  // !ROCKSDB_LITE
+
 
 namespace {
 class ReadaheadRandomAccessFile : public RandomAccessFile {
@@ -433,15 +375,13 @@ class ReadaheadRandomAccessFile : public RandomAccessFile {
   ReadaheadRandomAccessFile(std::unique_ptr<RandomAccessFile>&& file,
                             size_t readahead_size)
       : file_(std::move(file)),
-        alignment_(file_->GetRequiredBufferAlignment()),
-        readahead_size_(Roundup(readahead_size, alignment_)),
+        readahead_size_(readahead_size),
         forward_calls_(file_->ShouldForwardRawRequest()),
         buffer_(),
         buffer_offset_(0),
         buffer_len_(0) {
     if (!forward_calls_) {
-      buffer_.Alignment(alignment_);
-      buffer_.AllocateNewBuffer(readahead_size_);
+      buffer_.reset(new char[readahead_size_]);
     } else if (readahead_size_ > 0) {
       file_->EnableReadAhead();
     }
@@ -453,7 +393,7 @@ class ReadaheadRandomAccessFile : public RandomAccessFile {
 
   virtual Status Read(uint64_t offset, size_t n, Slice* result,
                       char* scratch) const override {
-    if (n + alignment_ >= readahead_size_) {
+    if (n >= readahead_size_) {
       return file_->Read(offset, n, result, scratch);
     }
 
@@ -467,51 +407,37 @@ class ReadaheadRandomAccessFile : public RandomAccessFile {
 
     std::unique_lock<std::mutex> lk(lock_);
 
-    size_t cached_len = 0;
-    // Check if there is a cache hit, means that [offset, offset + n) is either
-    // complitely or partially in the buffer
-    // If it's completely cached, including end of file case when offset + n is
-    // greater than EOF, return
-    if (TryReadFromCache(offset, n, &cached_len, scratch) &&
-        (cached_len == n ||
-         // End of file
-         buffer_len_ < readahead_size_)) {
-      *result = Slice(scratch, cached_len);
-      return Status::OK();
-    }
-    size_t advanced_offset = offset + cached_len;
-    // In the case of cache hit advanced_offset is already aligned, means that
-    // chunk_offset equals to advanced_offset
-    size_t chunk_offset = TruncateToPageBoundary(alignment_, advanced_offset);
-    Slice readahead_result;
-
-    Status s = ReadIntoBuffer(chunk_offset, readahead_size_);
-    if (s.ok()) {
-      // In the case of cache miss, i.e. when cached_len equals 0, an offset can
-      // exceed the file end position, so the following check is required
-      if (advanced_offset < chunk_offset + buffer_len_) {
-        // In the case of cache miss, the first chunk_padding bytes in buffer_
-        // are
-        // stored for alignment only and must be skipped
-        size_t chunk_padding = advanced_offset - chunk_offset;
-        auto remaining_len =
-            std::min(buffer_len_ - chunk_padding, n - cached_len);
-        memcpy(scratch + cached_len, buffer_.BufferStart() + chunk_padding,
-               remaining_len);
-        *result = Slice(scratch, cached_len + remaining_len);
-      } else {
-        *result = Slice(scratch, cached_len);
+    size_t copied = 0;
+    // if offset between [buffer_offset_, buffer_offset_ + buffer_len>
+    if (offset >= buffer_offset_ && offset < buffer_len_ + buffer_offset_) {
+      uint64_t offset_in_buffer = offset - buffer_offset_;
+      copied = std::min(buffer_len_ - static_cast<size_t>(offset_in_buffer), n);
+      memcpy(scratch, buffer_.get() + offset_in_buffer, copied);
+      if (copied == n) {
+        // fully cached
+        *result = Slice(scratch, n);
+        return Status::OK();
       }
     }
-    return s;
-  }
-
-  virtual Status Prefetch(uint64_t offset, size_t n) override {
-    size_t prefetch_offset = TruncateToPageBoundary(alignment_, offset);
-    if (prefetch_offset == buffer_offset_) {
-      return Status::OK();
+    Slice readahead_result;
+    Status s = file_->Read(offset + copied, readahead_size_, &readahead_result,
+      buffer_.get());
+    if (!s.ok()) {
+      return s;
     }
-    return ReadIntoBuffer(prefetch_offset, offset - prefetch_offset + n);
+
+    auto left_to_copy = std::min(readahead_result.size(), n - copied);
+    memcpy(scratch + copied, readahead_result.data(), left_to_copy);
+    *result = Slice(scratch, copied + left_to_copy);
+
+    if (readahead_result.data() == buffer_.get()) {
+      buffer_offset_ = offset + copied;
+      buffer_len_ = readahead_result.size();
+    } else {
+      buffer_len_ = 0;
+    }
+
+    return Status::OK();
   }
 
   virtual size_t GetUniqueId(char* id, size_t max_size) const override {
@@ -524,46 +450,15 @@ class ReadaheadRandomAccessFile : public RandomAccessFile {
     return file_->InvalidateCache(offset, length);
   }
 
-  virtual bool use_direct_io() const override {
-    return file_->use_direct_io();
-  }
-
  private:
-  bool TryReadFromCache(uint64_t offset, size_t n, size_t* cached_len,
-                         char* scratch) const {
-    if (offset < buffer_offset_ || offset >= buffer_offset_ + buffer_len_) {
-      *cached_len = 0;
-      return false;
-    }
-    uint64_t offset_in_buffer = offset - buffer_offset_;
-    *cached_len =
-        std::min(buffer_len_ - static_cast<size_t>(offset_in_buffer), n);
-    memcpy(scratch, buffer_.BufferStart() + offset_in_buffer, *cached_len);
-    return true;
-  }
-
-  Status ReadIntoBuffer(uint64_t offset, size_t n) const {
-    if (n > buffer_.Capacity()) {
-      n = buffer_.Capacity();
-    }
-    Slice result;
-    Status s = file_->Read(offset, n, &result, buffer_.BufferStart());
-    if (s.ok()) {
-      buffer_offset_ = offset;
-      buffer_len_ = result.size();
-    }
-    return s;
-  }
-
   std::unique_ptr<RandomAccessFile> file_;
-  const size_t alignment_;
   size_t               readahead_size_;
   const bool           forward_calls_;
 
-  mutable std::mutex lock_;
-  mutable AlignedBuffer buffer_;
-  mutable uint64_t buffer_offset_;
-  mutable size_t buffer_len_;
+  mutable std::mutex   lock_;
+  mutable std::unique_ptr<char[]> buffer_;
+  mutable uint64_t     buffer_offset_;
+  mutable size_t       buffer_len_;
 };
 }  // namespace
 
