@@ -1,4 +1,4 @@
-// Copyright (c) 2014, Facebook, Inc.  All rights reserved.
+// Copyright (c) 2011-present, Facebook, Inc.  All rights reserved.
 // This source code is licensed under the BSD-style license found in the
 // LICENSE file in the root directory of this source tree. An additional grant
 // of patent rights can be found in the PATENTS file in the same directory.
@@ -12,7 +12,7 @@
 #include "rocksdb/options.h"
 #include "rocksdb/status.h"
 #include "rocksdb/table.h"
-#include "util/mutable_cf_options.h"
+#include "util/cf_options.h"
 
 #ifndef ROCKSDB_LITE
 namespace rocksdb {
@@ -53,6 +53,8 @@ std::string EscapeOptionString(const std::string& raw_string);
 // @return the raw string of the input "escaped_string"
 std::string UnescapeOptionString(const std::string& escaped_string);
 
+uint64_t ParseUint64(const std::string& value);
+
 Status GetMutableOptionsFromStrings(
     const MutableCFOptions& base_options,
     const std::unordered_map<std::string, std::string>& options_map,
@@ -93,18 +95,23 @@ enum class OptionType {
   kFlushBlockPolicyFactory,
   kChecksumType,
   kEncodingType,
+  kWALRecoveryMode,
+  kAccessHint,
+  kInfoLogLevel,
   kUnknown
 };
 
 enum class OptionVerificationType {
   kNormal,
-  kByName,     // The option is pointer typed so we can only verify
-               // based on it's name.
-  kDeprecated  // The option is no longer used in rocksdb. The RocksDB
-               // OptionsParser will still accept this option if it
-               // happen to exists in some Options file.  However, the
-               // parser will not include it in serialization and
-               // verification processes.
+  kByName,           // The option is pointer typed so we can only verify
+                     // based on it's name.
+  kByNameAllowNull,  // Same as kByName, but it also allows the case
+                     // where one of them is a nullptr.
+  kDeprecated        // The option is no longer used in rocksdb. The RocksDB
+                     // OptionsParser will still accept this option if it
+                     // happen to exists in some Options file.  However, the
+                     // parser will not include it in serialization and
+                     // verification processes.
 };
 
 // A struct for storing constant option information such as option name,
@@ -141,10 +148,7 @@ Status GetColumnFamilyOptionsFromMapInternal(
 static std::unordered_map<std::string, OptionTypeInfo> db_options_type_info = {
     /*
      // not yet supported
-      AccessHint access_hint_on_compaction_start;
       Env* env;
-      InfoLogLevel info_log_level;
-      WALRecoveryMode wal_recovery_mode;
       std::shared_ptr<Cache> row_cache;
       std::shared_ptr<DeleteScheduler> delete_scheduler;
       std::shared_ptr<Logger> info_log;
@@ -165,6 +169,9 @@ static std::unordered_map<std::string, OptionTypeInfo> db_options_type_info = {
     {"allow_mmap_writes",
      {offsetof(struct DBOptions, allow_mmap_writes), OptionType::kBoolean,
       OptionVerificationType::kNormal}},
+    {"allow_2pc",
+     {offsetof(struct DBOptions, allow_2pc), OptionType::kBoolean,
+      OptionVerificationType::kNormal}},
     {"allow_os_buffer",
      {offsetof(struct DBOptions, allow_os_buffer), OptionType::kBoolean,
       OptionVerificationType::kNormal}},
@@ -208,7 +215,7 @@ static std::unordered_map<std::string, OptionTypeInfo> db_options_type_info = {
      {offsetof(struct DBOptions, random_access_max_buffer_size),
       OptionType::kSizeT, OptionVerificationType::kNormal}},
     {"writable_file_max_buffer_size",
-     {offsetof(struct DBOptions, writable_file_max_buffer_size),  
+     {offsetof(struct DBOptions, writable_file_max_buffer_size),
       OptionType::kSizeT, OptionVerificationType::kNormal}},
     {"use_adaptive_mutex",
      {offsetof(struct DBOptions, use_adaptive_mutex), OptionType::kBoolean,
@@ -219,6 +226,9 @@ static std::unordered_map<std::string, OptionTypeInfo> db_options_type_info = {
     {"max_background_compactions",
      {offsetof(struct DBOptions, max_background_compactions), OptionType::kInt,
       OptionVerificationType::kNormal}},
+    {"base_background_compactions",
+     {offsetof(struct DBOptions, base_background_compactions), OptionType::kInt,
+      OptionVerificationType::kNormal}},
     {"max_background_flushes",
      {offsetof(struct DBOptions, max_background_flushes), OptionType::kInt,
       OptionVerificationType::kNormal}},
@@ -284,7 +294,37 @@ static std::unordered_map<std::string, OptionTypeInfo> db_options_type_info = {
       OptionVerificationType::kNormal}},
     {"stats_dump_period_sec",
      {offsetof(struct DBOptions, stats_dump_period_sec), OptionType::kUInt,
-      OptionVerificationType::kNormal}}};
+      OptionVerificationType::kNormal}},
+    {"fail_if_options_file_error",
+     {offsetof(struct DBOptions, fail_if_options_file_error),
+      OptionType::kBoolean, OptionVerificationType::kNormal}},
+    {"allow_concurrent_memtable_write",
+     {offsetof(struct DBOptions, allow_concurrent_memtable_write),
+      OptionType::kBoolean, OptionVerificationType::kNormal}},
+    {"wal_recovery_mode",
+     {offsetof(struct DBOptions, wal_recovery_mode),
+      OptionType::kWALRecoveryMode, OptionVerificationType::kNormal}},
+    {"enable_write_thread_adaptive_yield",
+     {offsetof(struct DBOptions, enable_write_thread_adaptive_yield),
+      OptionType::kBoolean, OptionVerificationType::kNormal}},
+    {"write_thread_slow_yield_usec",
+     {offsetof(struct DBOptions, write_thread_slow_yield_usec),
+      OptionType::kUInt64T, OptionVerificationType::kNormal}},
+    {"write_thread_max_yield_usec",
+     {offsetof(struct DBOptions, write_thread_max_yield_usec),
+      OptionType::kUInt64T, OptionVerificationType::kNormal}},
+    {"access_hint_on_compaction_start",
+     {offsetof(struct DBOptions, access_hint_on_compaction_start),
+      OptionType::kAccessHint, OptionVerificationType::kNormal}},
+    {"info_log_level",
+     {offsetof(struct DBOptions, info_log_level), OptionType::kInfoLogLevel,
+      OptionVerificationType::kNormal}},
+    {"dump_malloc_stats",
+     {offsetof(struct DBOptions, dump_malloc_stats), OptionType::kBoolean,
+      OptionVerificationType::kNormal}},
+    {"avoid_flush_during_recovery",
+     {offsetof(struct DBOptions, avoid_flush_during_recovery),
+      OptionType::kBoolean, OptionVerificationType::kNormal}}};
 
 static std::unordered_map<std::string, OptionTypeInfo> cf_options_type_info = {
     /* not yet supported
@@ -300,15 +340,17 @@ static std::unordered_map<std::string, OptionTypeInfo> cf_options_type_info = {
                                      std::string* merged_value);
     std::vector<int> max_bytes_for_level_multiplier_additional;
      */
-    {"compaction_measure_io_stats",
-     {offsetof(struct ColumnFamilyOptions, compaction_measure_io_stats),
+    {"report_bg_io_stats",
+     {offsetof(struct ColumnFamilyOptions, report_bg_io_stats),
       OptionType::kBoolean, OptionVerificationType::kNormal}},
+    {"compaction_measure_io_stats",
+     {offsetof(struct ColumnFamilyOptions, report_bg_io_stats),
+      OptionType::kBoolean, OptionVerificationType::kDeprecated}},
     {"disable_auto_compactions",
      {offsetof(struct ColumnFamilyOptions, disable_auto_compactions),
       OptionType::kBoolean, OptionVerificationType::kNormal}},
     {"filter_deletes",
-     {offsetof(struct ColumnFamilyOptions, filter_deletes),
-      OptionType::kBoolean, OptionVerificationType::kNormal}},
+     {0, OptionType::kBoolean, OptionVerificationType::kDeprecated}},
     {"inplace_update_support",
      {offsetof(struct ColumnFamilyOptions, inplace_update_support),
       OptionType::kBoolean, OptionVerificationType::kNormal}},
@@ -340,9 +382,11 @@ static std::unordered_map<std::string, OptionTypeInfo> cf_options_type_info = {
     {"soft_rate_limit",
      {offsetof(struct ColumnFamilyOptions, soft_rate_limit),
       OptionType::kDouble, OptionVerificationType::kNormal}},
+    {"max_compaction_bytes",
+     {offsetof(struct ColumnFamilyOptions, max_compaction_bytes),
+      OptionType::kUInt64T, OptionVerificationType::kNormal}},
     {"expanded_compaction_factor",
-     {offsetof(struct ColumnFamilyOptions, expanded_compaction_factor),
-      OptionType::kInt, OptionVerificationType::kNormal}},
+     {0, OptionType::kInt, OptionVerificationType::kDeprecated}},
     {"level0_file_num_compaction_trigger",
      {offsetof(struct ColumnFamilyOptions, level0_file_num_compaction_trigger),
       OptionType::kInt, OptionVerificationType::kNormal}},
@@ -356,8 +400,7 @@ static std::unordered_map<std::string, OptionTypeInfo> cf_options_type_info = {
      {offsetof(struct ColumnFamilyOptions, max_bytes_for_level_multiplier),
       OptionType::kInt, OptionVerificationType::kNormal}},
     {"max_grandparent_overlap_factor",
-     {offsetof(struct ColumnFamilyOptions, max_grandparent_overlap_factor),
-      OptionType::kInt, OptionVerificationType::kNormal}},
+     {0, OptionType::kInt, OptionVerificationType::kDeprecated}},
     {"max_mem_compaction_level",
      {offsetof(struct ColumnFamilyOptions, max_mem_compaction_level),
       OptionType::kInt, OptionVerificationType::kDeprecated}},
@@ -374,8 +417,7 @@ static std::unordered_map<std::string, OptionTypeInfo> cf_options_type_info = {
      {offsetof(struct ColumnFamilyOptions, num_levels), OptionType::kInt,
       OptionVerificationType::kNormal}},
     {"source_compaction_factor",
-     {offsetof(struct ColumnFamilyOptions, source_compaction_factor),
-      OptionType::kInt, OptionVerificationType::kNormal}},
+     {0, OptionType::kInt, OptionVerificationType::kDeprecated}},
     {"target_file_size_multiplier",
      {offsetof(struct ColumnFamilyOptions, target_file_size_multiplier),
       OptionType::kInt, OptionVerificationType::kNormal}},
@@ -388,10 +430,11 @@ static std::unordered_map<std::string, OptionTypeInfo> cf_options_type_info = {
     {"max_successive_merges",
      {offsetof(struct ColumnFamilyOptions, max_successive_merges),
       OptionType::kSizeT, OptionVerificationType::kNormal}},
-    {"memtable_prefix_bloom_huge_page_tlb_size",
-     {offsetof(struct ColumnFamilyOptions,
-               memtable_prefix_bloom_huge_page_tlb_size),
+    {"memtable_huge_page_size",
+     {offsetof(struct ColumnFamilyOptions, memtable_huge_page_size),
       OptionType::kSizeT, OptionVerificationType::kNormal}},
+    {"memtable_prefix_bloom_huge_page_tlb_size",
+     {0, OptionType::kSizeT, OptionVerificationType::kDeprecated}},
     {"write_buffer_size",
      {offsetof(struct ColumnFamilyOptions, write_buffer_size),
       OptionType::kSizeT, OptionVerificationType::kNormal}},
@@ -399,11 +442,12 @@ static std::unordered_map<std::string, OptionTypeInfo> cf_options_type_info = {
      {offsetof(struct ColumnFamilyOptions, bloom_locality),
       OptionType::kUInt32T, OptionVerificationType::kNormal}},
     {"memtable_prefix_bloom_bits",
-     {offsetof(struct ColumnFamilyOptions, memtable_prefix_bloom_bits),
-      OptionType::kUInt32T, OptionVerificationType::kNormal}},
+     {0, OptionType::kUInt32T, OptionVerificationType::kDeprecated}},
+    {"memtable_prefix_bloom_size_ratio",
+     {offsetof(struct ColumnFamilyOptions, memtable_prefix_bloom_size_ratio),
+      OptionType::kDouble, OptionVerificationType::kNormal}},
     {"memtable_prefix_bloom_probes",
-     {offsetof(struct ColumnFamilyOptions, memtable_prefix_bloom_probes),
-      OptionType::kUInt32T, OptionVerificationType::kNormal}},
+     {0, OptionType::kUInt32T, OptionVerificationType::kDeprecated}},
     {"min_partial_merge_operands",
      {offsetof(struct ColumnFamilyOptions, min_partial_merge_operands),
       OptionType::kUInt32T, OptionVerificationType::kNormal}},
@@ -425,12 +469,15 @@ static std::unordered_map<std::string, OptionTypeInfo> cf_options_type_info = {
     {"compression_per_level",
      {offsetof(struct ColumnFamilyOptions, compression_per_level),
       OptionType::kVectorCompressionType, OptionVerificationType::kNormal}},
+    {"bottommost_compression",
+     {offsetof(struct ColumnFamilyOptions, bottommost_compression),
+      OptionType::kCompressionType, OptionVerificationType::kNormal}},
     {"comparator",
      {offsetof(struct ColumnFamilyOptions, comparator), OptionType::kComparator,
       OptionVerificationType::kByName}},
     {"prefix_extractor",
      {offsetof(struct ColumnFamilyOptions, prefix_extractor),
-      OptionType::kSliceTransform, OptionVerificationType::kByName}},
+      OptionType::kSliceTransform, OptionVerificationType::kByNameAllowNull}},
     {"memtable_factory",
      {offsetof(struct ColumnFamilyOptions, memtable_factory),
       OptionType::kMemTableRepFactory, OptionVerificationType::kByName}},
@@ -450,51 +497,71 @@ static std::unordered_map<std::string, OptionTypeInfo> cf_options_type_info = {
      {offsetof(struct ColumnFamilyOptions, compaction_style),
       OptionType::kCompactionStyle, OptionVerificationType::kNormal}}};
 
-static std::unordered_map<std::string,
-                          OptionTypeInfo> block_based_table_type_info = {
-    /* currently not supported
-      std::shared_ptr<Cache> block_cache = nullptr;
-      std::shared_ptr<Cache> block_cache_compressed = nullptr;
-     */
-    {"flush_block_policy_factory",
-     {offsetof(struct BlockBasedTableOptions, flush_block_policy_factory),
-      OptionType::kFlushBlockPolicyFactory, OptionVerificationType::kByName}},
-    {"cache_index_and_filter_blocks",
-     {offsetof(struct BlockBasedTableOptions, cache_index_and_filter_blocks),
-      OptionType::kBoolean, OptionVerificationType::kNormal}},
-    {"index_type",
-     {offsetof(struct BlockBasedTableOptions, index_type),
-      OptionType::kBlockBasedTableIndexType, OptionVerificationType::kNormal}},
-    {"hash_index_allow_collision",
-     {offsetof(struct BlockBasedTableOptions, hash_index_allow_collision),
-      OptionType::kBoolean, OptionVerificationType::kNormal}},
-    {"checksum",
-     {offsetof(struct BlockBasedTableOptions, checksum),
-      OptionType::kChecksumType, OptionVerificationType::kNormal}},
-    {"no_block_cache",
-     {offsetof(struct BlockBasedTableOptions, no_block_cache),
-      OptionType::kBoolean, OptionVerificationType::kNormal}},
-    {"block_size",
-     {offsetof(struct BlockBasedTableOptions, block_size), OptionType::kSizeT,
-      OptionVerificationType::kNormal}},
-    {"block_size_deviation",
-     {offsetof(struct BlockBasedTableOptions, block_size_deviation),
-      OptionType::kInt, OptionVerificationType::kNormal}},
-    {"block_restart_interval",
-     {offsetof(struct BlockBasedTableOptions, block_restart_interval),
-      OptionType::kInt, OptionVerificationType::kNormal}},
-    {"filter_policy",
-     {offsetof(struct BlockBasedTableOptions, filter_policy),
-      OptionType::kFilterPolicy, OptionVerificationType::kByName}},
-    {"whole_key_filtering",
-     {offsetof(struct BlockBasedTableOptions, whole_key_filtering),
-      OptionType::kBoolean, OptionVerificationType::kNormal}},
-    {"skip_table_builder_flush",
-     {offsetof(struct BlockBasedTableOptions, skip_table_builder_flush),  
-      OptionType::kBoolean, OptionVerificationType::kNormal}},
-    {"format_version",
-     {offsetof(struct BlockBasedTableOptions, format_version),
-      OptionType::kUInt32T, OptionVerificationType::kNormal}}};
+static std::unordered_map<std::string, OptionTypeInfo>
+    block_based_table_type_info = {
+        /* currently not supported
+          std::shared_ptr<Cache> block_cache = nullptr;
+          std::shared_ptr<Cache> block_cache_compressed = nullptr;
+         */
+        {"flush_block_policy_factory",
+         {offsetof(struct BlockBasedTableOptions, flush_block_policy_factory),
+          OptionType::kFlushBlockPolicyFactory,
+          OptionVerificationType::kByName}},
+        {"cache_index_and_filter_blocks",
+         {offsetof(struct BlockBasedTableOptions,
+                   cache_index_and_filter_blocks),
+          OptionType::kBoolean, OptionVerificationType::kNormal}},
+        {"cache_index_and_filter_blocks_with_high_priority",
+         {offsetof(struct BlockBasedTableOptions,
+                   cache_index_and_filter_blocks_with_high_priority),
+          OptionType::kBoolean, OptionVerificationType::kNormal}},
+        {"pin_l0_filter_and_index_blocks_in_cache",
+         {offsetof(struct BlockBasedTableOptions,
+                   pin_l0_filter_and_index_blocks_in_cache),
+          OptionType::kBoolean, OptionVerificationType::kNormal}},
+        {"index_type",
+         {offsetof(struct BlockBasedTableOptions, index_type),
+          OptionType::kBlockBasedTableIndexType,
+          OptionVerificationType::kNormal}},
+        {"hash_index_allow_collision",
+         {offsetof(struct BlockBasedTableOptions, hash_index_allow_collision),
+          OptionType::kBoolean, OptionVerificationType::kNormal}},
+        {"checksum",
+         {offsetof(struct BlockBasedTableOptions, checksum),
+          OptionType::kChecksumType, OptionVerificationType::kNormal}},
+        {"no_block_cache",
+         {offsetof(struct BlockBasedTableOptions, no_block_cache),
+          OptionType::kBoolean, OptionVerificationType::kNormal}},
+        {"block_size",
+         {offsetof(struct BlockBasedTableOptions, block_size),
+          OptionType::kSizeT, OptionVerificationType::kNormal}},
+        {"block_size_deviation",
+         {offsetof(struct BlockBasedTableOptions, block_size_deviation),
+          OptionType::kInt, OptionVerificationType::kNormal}},
+        {"block_restart_interval",
+         {offsetof(struct BlockBasedTableOptions, block_restart_interval),
+          OptionType::kInt, OptionVerificationType::kNormal}},
+        {"index_block_restart_interval",
+         {offsetof(struct BlockBasedTableOptions, index_block_restart_interval),
+          OptionType::kInt, OptionVerificationType::kNormal}},
+        {"filter_policy",
+         {offsetof(struct BlockBasedTableOptions, filter_policy),
+          OptionType::kFilterPolicy, OptionVerificationType::kByName}},
+        {"whole_key_filtering",
+         {offsetof(struct BlockBasedTableOptions, whole_key_filtering),
+          OptionType::kBoolean, OptionVerificationType::kNormal}},
+        {"skip_table_builder_flush",
+         {offsetof(struct BlockBasedTableOptions, skip_table_builder_flush),
+          OptionType::kBoolean, OptionVerificationType::kNormal}},
+        {"format_version",
+         {offsetof(struct BlockBasedTableOptions, format_version),
+          OptionType::kUInt32T, OptionVerificationType::kNormal}},
+        {"verify_compression",
+         {offsetof(struct BlockBasedTableOptions, verify_compression),
+          OptionType::kBoolean, OptionVerificationType::kNormal}},
+        {"read_amp_bytes_per_bit",
+         {offsetof(struct BlockBasedTableOptions, read_amp_bytes_per_bit),
+          OptionType::kSizeT, OptionVerificationType::kNormal}}};
 
 static std::unordered_map<std::string, OptionTypeInfo> plain_table_type_info = {
     {"user_key_len",
@@ -530,7 +597,10 @@ static std::unordered_map<std::string, CompressionType>
         {"kBZip2Compression", kBZip2Compression},
         {"kLZ4Compression", kLZ4Compression},
         {"kLZ4HCCompression", kLZ4HCCompression},
-        {"kZSTDNotFinalCompression", kZSTDNotFinalCompression}};
+        {"kXpressCompression", kXpressCompression},
+        {"kZSTD", kZSTD},
+        {"kZSTDNotFinalCompression", kZSTDNotFinalCompression},
+        {"kDisableCompressionOption", kDisableCompressionOption}};
 
 static std::unordered_map<std::string, BlockBasedTableOptions::IndexType>
     block_base_table_index_type_string_map = {
@@ -550,6 +620,28 @@ static std::unordered_map<std::string, CompactionStyle>
         {"kCompactionStyleFIFO", kCompactionStyleFIFO},
         {"kCompactionStyleNone", kCompactionStyleNone}};
 
+static std::unordered_map<std::string,
+                          WALRecoveryMode> wal_recovery_mode_string_map = {
+    {"kTolerateCorruptedTailRecords",
+     WALRecoveryMode::kTolerateCorruptedTailRecords},
+    {"kAbsoluteConsistency", WALRecoveryMode::kAbsoluteConsistency},
+    {"kPointInTimeRecovery", WALRecoveryMode::kPointInTimeRecovery},
+    {"kSkipAnyCorruptedRecords", WALRecoveryMode::kSkipAnyCorruptedRecords}};
+
+static std::unordered_map<std::string, DBOptions::AccessHint>
+    access_hint_string_map = {{"NONE", DBOptions::AccessHint::NONE},
+                              {"NORMAL", DBOptions::AccessHint::NORMAL},
+                              {"SEQUENTIAL", DBOptions::AccessHint::SEQUENTIAL},
+                              {"WILLNEED", DBOptions::AccessHint::WILLNEED}};
+
+static std::unordered_map<std::string, InfoLogLevel> info_log_level_string_map =
+    {{"DEBUG_LEVEL", InfoLogLevel::DEBUG_LEVEL},
+     {"INFO_LEVEL", InfoLogLevel::INFO_LEVEL},
+     {"WARN_LEVEL", InfoLogLevel::WARN_LEVEL},
+     {"ERROR_LEVEL", InfoLogLevel::ERROR_LEVEL},
+     {"FATAL_LEVEL", InfoLogLevel::FATAL_LEVEL},
+     {"HEADER_LEVEL", InfoLogLevel::HEADER_LEVEL}};
+
 }  // namespace rocksdb
 
 #endif  // !ROCKSDB_LITE
