@@ -1,4 +1,4 @@
-//  Copyright (c) 2013, Facebook, Inc.  All rights reserved.
+//  Copyright (c) 2011-present, Facebook, Inc.  All rights reserved.
 //  This source code is licensed under the BSD-style license found in the
 //  LICENSE file in the root directory of this source tree. An additional grant
 //  of patent rights can be found in the PATENTS file in the same directory.
@@ -32,12 +32,36 @@ StatisticsImpl::StatisticsImpl(
 StatisticsImpl::~StatisticsImpl() {}
 
 uint64_t StatisticsImpl::getTickerCount(uint32_t tickerType) const {
+  MutexLock lock(&aggregate_lock_);
   assert(
     enable_internal_stats_ ?
       tickerType < INTERNAL_TICKER_ENUM_MAX :
       tickerType < TICKER_ENUM_MAX);
-  // Return its own ticker version
-  return tickers_[tickerType].value;
+  uint64_t thread_local_sum = 0;
+  tickers_[tickerType].thread_value->Fold(
+      [](void* curr_ptr, void* res) {
+        auto* sum_ptr = static_cast<uint64_t*>(res);
+        *sum_ptr += static_cast<std::atomic_uint_fast64_t*>(curr_ptr)->load(
+            std::memory_order_relaxed);
+      },
+      &thread_local_sum);
+  return thread_local_sum +
+         tickers_[tickerType].merged_sum.load(std::memory_order_relaxed);
+}
+
+std::unique_ptr<HistogramImpl>
+StatisticsImpl::HistogramInfo::getMergedHistogram() const {
+  MutexLock lock(&merge_lock);
+  std::unique_ptr<HistogramImpl> res_hist(new HistogramImpl());
+  res_hist->Merge(merged_hist);
+  thread_value->Fold(
+      [](void* curr_ptr, void* res) {
+        auto tmp_res_hist = static_cast<HistogramImpl*>(res);
+        auto curr_hist = static_cast<HistogramImpl*>(curr_ptr);
+        tmp_res_hist->Merge(*curr_hist);
+      },
+      res_hist.get());
+  return res_hist;
 }
 
 void StatisticsImpl::histogramData(uint32_t histogramType,
@@ -46,23 +70,53 @@ void StatisticsImpl::histogramData(uint32_t histogramType,
     enable_internal_stats_ ?
       histogramType < INTERNAL_HISTOGRAM_ENUM_MAX :
       histogramType < HISTOGRAM_ENUM_MAX);
-  // Return its own ticker version
-  histograms_[histogramType].Data(data);
+  histograms_[histogramType].getMergedHistogram()->Data(data);
 }
 
 std::string StatisticsImpl::getHistogramString(uint32_t histogramType) const {
   assert(enable_internal_stats_ ? histogramType < INTERNAL_HISTOGRAM_ENUM_MAX
                                 : histogramType < HISTOGRAM_ENUM_MAX);
-  return histograms_[histogramType].ToString();
+  return histograms_[histogramType].getMergedHistogram()->ToString();
+}
+
+StatisticsImpl::ThreadTickerInfo* StatisticsImpl::getThreadTickerInfo(
+    uint32_t tickerType) {
+  auto info_ptr =
+      static_cast<ThreadTickerInfo*>(tickers_[tickerType].thread_value->Get());
+  if (info_ptr == nullptr) {
+    info_ptr =
+        new ThreadTickerInfo(0 /* value */, &tickers_[tickerType].merged_sum);
+    tickers_[tickerType].thread_value->Reset(info_ptr);
+  }
+  return info_ptr;
+}
+
+StatisticsImpl::ThreadHistogramInfo* StatisticsImpl::getThreadHistogramInfo(
+    uint32_t histogram_type) {
+  auto info_ptr = static_cast<ThreadHistogramInfo*>(
+      histograms_[histogram_type].thread_value->Get());
+  if (info_ptr == nullptr) {
+    info_ptr = new ThreadHistogramInfo(&histograms_[histogram_type].merged_hist,
+                                       &histograms_[histogram_type].merge_lock);
+    histograms_[histogram_type].thread_value->Reset(info_ptr);
+  }
+  return info_ptr;
 }
 
 void StatisticsImpl::setTickerCount(uint32_t tickerType, uint64_t count) {
-  assert(
-    enable_internal_stats_ ?
-      tickerType < INTERNAL_TICKER_ENUM_MAX :
-      tickerType < TICKER_ENUM_MAX);
-  if (tickerType < TICKER_ENUM_MAX || enable_internal_stats_) {
-    tickers_[tickerType].value.store(count, std::memory_order_relaxed);
+  {
+    MutexLock lock(&aggregate_lock_);
+    assert(enable_internal_stats_ ? tickerType < INTERNAL_TICKER_ENUM_MAX
+                                  : tickerType < TICKER_ENUM_MAX);
+    if (tickerType < TICKER_ENUM_MAX || enable_internal_stats_) {
+      tickers_[tickerType].thread_value->Fold(
+          [](void* curr_ptr, void* res) {
+            static_cast<std::atomic<uint64_t>*>(curr_ptr)->store(
+                0, std::memory_order_relaxed);
+          },
+          nullptr /* res */);
+      tickers_[tickerType].merged_sum.store(count, std::memory_order_relaxed);
+    }
   }
   if (stats_ && tickerType < TICKER_ENUM_MAX) {
     stats_->setTickerCount(tickerType, count);
@@ -75,7 +129,8 @@ void StatisticsImpl::recordTick(uint32_t tickerType, uint64_t count) {
       tickerType < INTERNAL_TICKER_ENUM_MAX :
       tickerType < TICKER_ENUM_MAX);
   if (tickerType < TICKER_ENUM_MAX || enable_internal_stats_) {
-    tickers_[tickerType].value.fetch_add(count, std::memory_order_relaxed);
+    auto info_ptr = getThreadTickerInfo(tickerType);
+    info_ptr->value.fetch_add(count, std::memory_order_relaxed);
   }
   if (stats_ && tickerType < TICKER_ENUM_MAX) {
     stats_->recordTick(tickerType, count);
@@ -88,7 +143,7 @@ void StatisticsImpl::measureTime(uint32_t histogramType, uint64_t value) {
       histogramType < INTERNAL_HISTOGRAM_ENUM_MAX :
       histogramType < HISTOGRAM_ENUM_MAX);
   if (histogramType < HISTOGRAM_ENUM_MAX || enable_internal_stats_) {
-    histograms_[histogramType].Add(value);
+    getThreadHistogramInfo(histogramType)->value.Add(value);
   }
   if (stats_ && histogramType < HISTOGRAM_ENUM_MAX) {
     stats_->measureTime(histogramType, value);
