@@ -21,12 +21,12 @@
 #include "crush/CrushWrapper.h"
 #include "common/config.h"
 #include "include/stringify.h"
+#include "global/global_init.h"
 #include "erasure-code/lrc/ErasureCodeLrc.h"
+#include "common/ceph_argparse.h"
 #include "global/global_context.h"
 #include "common/config.h"
 #include "gtest/gtest.h"
-#include "test/unit.h"
-
 
 TEST(ErasureCodeLrc, parse_ruleset)
 {
@@ -403,7 +403,7 @@ TEST(ErasureCodeLrc, layers_init)
     ErasureCodeProfile profile;
 
     const char* env = getenv("CEPH_LIB");
-    string directory(env ? env : "lib");
+    string directory(env ? env : ".libs");
     string description_string = 
       "[ " 
       "  [ \"_cDDD_cDD_\", \"directory=" + directory + "\" ]," 
@@ -908,6 +908,22 @@ TEST(ErasureCodeLrc, encode_decode_2)
   }
 }
 
+int main(int argc, char **argv)
+{
+  vector<const char*> args;
+  argv_to_vec(argc, (const char **)argv, args);
+
+  global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT, CODE_ENVIRONMENT_UTILITY, 0);
+  common_init_finish(g_ceph_context);
+
+  const char* env = getenv("CEPH_LIB");
+  string directory(env ? env : ".libs");
+  g_conf->set_val("erasure_code_dir", directory, false, false);
+
+  ::testing::InitGoogleTest(&argc, argv);
+  return RUN_ALL_TESTS();
+}
+
 /*
  * Local Variables:
  * compile-command: "cd ../.. ;
