@@ -37,7 +37,6 @@ namespace po = boost::program_options;
 class ErasureCodeCommand {
   po::variables_map vm;
   ErasureCodeProfile profile;
-  boost::intrusive_ptr<CephContext> cct;
 public:
   int setup(int argc, char** argv);
   int run();
@@ -83,14 +82,14 @@ int ErasureCodeCommand::setup(int argc, char** argv) {
     ceph_options.push_back(i->c_str());
   }
 
-  cct = global_init(
+  global_init(
     &def_args, ceph_options, CEPH_ENTITY_TYPE_CLIENT,
     CODE_ENVIRONMENT_UTILITY,
     CINIT_FLAG_NO_DEFAULT_CONFIG_FILE);
   common_init_finish(g_ceph_context);
   g_ceph_context->_conf->apply_changes(NULL);
   const char* env = getenv("CEPH_LIB");
-  string directory(env ? env : "lib");
+  string directory(env ? env : ".libs");
   g_conf->set_val("erasure_code_dir", directory, false, false);
 
   if (vm.count("help")) {
