@@ -22,7 +22,7 @@ using namespace ceph;
 
 TEST(HeartbeatMap, Healthy) {
   HeartbeatMap hm(g_ceph_context);
-  heartbeat_handle_d *h = hm.add_worker("one");
+  heartbeat_handle_d *h = hm.add_worker("one", pthread_self());
 
   hm.reset_timeout(h, 9, 18);
   bool healthy = hm.is_healthy();
@@ -33,7 +33,7 @@ TEST(HeartbeatMap, Healthy) {
 
 TEST(HeartbeatMap, Unhealth) {
   HeartbeatMap hm(g_ceph_context);
-  heartbeat_handle_d *h = hm.add_worker("one");
+  heartbeat_handle_d *h = hm.add_worker("one", pthread_self());
 
   hm.reset_timeout(h, 1, 3);
   sleep(2);
