@@ -46,7 +46,7 @@ void TestClassHandler::open_all_classes() {
   assert(m_class_handles.empty());
 
   const char* env = getenv("CEPH_LIB");
-  std::string CEPH_LIB(env ? env : "lib");
+  std::string CEPH_LIB(env ? env : ".libs");
   DIR *dir = ::opendir(CEPH_LIB.c_str());
   if (dir == NULL) {
     assert(false);;
@@ -120,6 +120,18 @@ TestClassHandler::SharedMethodContext TestClassHandler::get_method_context(
   return ctx;
 }
 
+int TestClassHandler::create_filter(cls_handle_t hclass,
+				    const std::string& name,
+				    cls_cxx_filter_factory_t fn)
+{
+  Class *cls = reinterpret_cast<Class*>(hclass);
+  if (cls->filters.find(name) != cls->filters.end()) {
+    return -EEXIST;
+  }
+  cls->filters[name] = fn;
+  return 0;
+}
+
 TestClassHandler::MethodContext::~MethodContext() {
   io_ctx_impl->put();
 }
