@@ -20,7 +20,8 @@ struct MockOperations {
                                                 Context *on_finish));
   MOCK_METHOD2(execute_rename, void(const std::string &dstname,
                                     Context *on_finish));
-  MOCK_METHOD4(execute_resize, void(uint64_t size, ProgressContext &prog_ctx,
+  MOCK_METHOD5(execute_resize, void(uint64_t size, bool allow_shrink,
+                                    ProgressContext &prog_ctx,
                                     Context *on_finish,
                                     uint64_t journal_op_tid));
   MOCK_METHOD2(snap_create, void(const std::string &snap_name,
@@ -43,6 +44,16 @@ struct MockOperations {
                                           Context *on_finish));
   MOCK_METHOD2(execute_snap_unprotect, void(const std::string &snap_name,
                                             Context *on_finish));
+  MOCK_METHOD2(execute_snap_set_limit, void(uint64_t limit,
+					    Context *on_finish));
+  MOCK_METHOD4(execute_update_features, void(uint64_t features, bool enabled,
+                                             Context *on_finish,
+                                             uint64_t journal_op_tid));
+  MOCK_METHOD3(execute_metadata_set, void(const std::string &key,
+                                          const std::string &value,
+                                          Context *on_finish));
+  MOCK_METHOD2(execute_metadata_remove, void(const std::string &key,
+                                             Context *on_finish));
 };
 
 } // namespace librbd
