@@ -88,7 +88,7 @@ int main(int argc, const char **argv)
 	messenger = new XioMessenger(g_ceph_context,
 				     entity_name_t::MON(-1),
 				     "xio_server",
-				     0 /* nonce */, XIO_ALL_FEATURES,
+				     0 /* nonce */, XIO_ALL_FEATURES, 0 /* cflags */,
 				     dstrategy);
 
 	static_cast<XioMessenger*>(messenger)->set_magic(
