@@ -5,7 +5,10 @@
 #
 
 # Includes
-source ../qa/workunits/ceph-helpers.sh
+
+source $(dirname $0)/../detect-build-env-vars.sh 
+
+source $CEPH_ROOT/qa/workunits/ceph-helpers.sh
 
 function run() {
     local dir=$1
