@@ -175,6 +175,7 @@ void usage()
   cout << "      [--min-x x] [--max-x x] [--x x]\n";
   cout << "      [--min-rule r] [--max-rule r] [--rule r] [--ruleset rs]\n";
   cout << "      [--num-rep n]\n";
+  cout << "      [--pool-id n]      specifies pool id\n";
   cout << "      [--batches b]      split the CRUSH mapping into b > 1 rounds\n";
   cout << "      [--weight|-w devno weight]\n";
   cout << "                         where weight is 0 to 1.0\n";
@@ -270,16 +271,13 @@ int main(int argc, const char **argv)
   // only parse arguments from CEPH_ARGS, if in the environment
   vector<const char *> env_args;
   env_to_vec(env_args);
-  auto cct = global_init(NULL, env_args, CEPH_ENTITY_TYPE_CLIENT,
-			 CODE_ENVIRONMENT_UTILITY,
-			 CINIT_FLAG_NO_DEFAULT_CONFIG_FILE);
-  // crushtool times out occasionally when quits. so do not
-  // release the g_ceph_context.
-  cct->get();
+  global_init(NULL, env_args, CEPH_ENTITY_TYPE_CLIENT, CODE_ENVIRONMENT_UTILITY,
+	      CINIT_FLAG_NO_DEFAULT_CONFIG_FILE);
   common_init_finish(g_ceph_context);
 
   int x;
   float y;
+  long long z;
 
   std::string val;
   std::ostringstream err;
@@ -289,7 +287,7 @@ int main(int argc, const char **argv)
       break;
     } else if (ceph_argparse_flag(args, i, "-h", "--help", (char*)NULL)) {
       usage();
-      return EXIT_SUCCESS;
+      exit(0);
     } else if (ceph_argparse_witharg(args, i, &val, "-d", "--decompile", (char*)NULL)) {
       infn = val;
       decompile = true;
@@ -360,17 +358,17 @@ int main(int argc, const char **argv)
     } else if (ceph_argparse_witharg(args, i, &add_item, err, "--add_item", (char*)NULL)) {
       if (!err.str().empty()) {
 	cerr << err.str() << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       if (i == args.end()) {
 	cerr << "expecting additional argument to --add-item" << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       add_weight = atof(*i);
       i = args.erase(i);
       if (i == args.end()) {
 	cerr << "expecting additional argument to --add-item" << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       add_name.assign(*i);
       i = args.erase(i);
@@ -378,17 +376,17 @@ int main(int argc, const char **argv)
       update_item = true;
       if (!err.str().empty()) {
 	cerr << err.str() << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       if (i == args.end()) {
 	cerr << "expecting additional argument to --update-item" << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       add_weight = atof(*i);
       i = args.erase(i);
       if (i == args.end()) {
 	cerr << "expecting additional argument to --update-item" << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       add_name.assign(*i);
       i = args.erase(i);
@@ -396,7 +394,7 @@ int main(int argc, const char **argv)
       std::string type(val);
       if (i == args.end()) {
 	cerr << "expecting additional argument to --loc" << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       std::string name(*i);
       i = args.erase(i);
@@ -407,12 +405,12 @@ int main(int argc, const char **argv)
       tester.set_output_csv(true);
     } else if (ceph_argparse_flag(args, i, "--help-output", (char*)NULL)) {
       data_analysis_usage();
-      return EXIT_SUCCESS;
+      exit(0);
     } else if (ceph_argparse_witharg(args, i, &val, "--output-name", (char*)NULL)) {
       std::string name(val);
       if (i == args.end()) {
 	cerr << "expecting additional argument to --output-name" << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       else {
         tester.set_output_data_file_name(name + "-");
@@ -423,7 +421,7 @@ int main(int argc, const char **argv)
       reweight_name = val;
       if (i == args.end()) {
 	cerr << "expecting additional argument to --reweight-item" << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       reweight_weight = atof(*i);
       i = args.erase(i);
@@ -432,83 +430,89 @@ int main(int argc, const char **argv)
     } else if (ceph_argparse_witharg(args, i, &num_osds, err, "--num_osds", (char*)NULL)) {
       if (!err.str().empty()) {
 	cerr << err.str() << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
     } else if (ceph_argparse_witharg(args, i, &x, err, "--num_rep", (char*)NULL)) {
       if (!err.str().empty()) {
 	cerr << err.str() << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       tester.set_num_rep(x);
     } else if (ceph_argparse_witharg(args, i, &x, err, "--max_x", (char*)NULL)) {
       if (!err.str().empty()) {
 	cerr << err.str() << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       tester.set_max_x(x);
     } else if (ceph_argparse_witharg(args, i, &x, err, "--min_x", (char*)NULL)) {
       if (!err.str().empty()) {
 	cerr << err.str() << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       tester.set_min_x(x);
+    } else if (ceph_argparse_witharg(args, i, &z, err, "--pool_id", (char*)NULL)) {
+      if (!err.str().empty()) {
+  cerr << err.str() << std::endl;
+  exit(EXIT_FAILURE);
+      }
+      tester.set_pool_id(z);
     } else if (ceph_argparse_witharg(args, i, &x, err, "--x", (char*)NULL)) {
       if (!err.str().empty()) {
 	cerr << err.str() << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       tester.set_x(x);
     } else if (ceph_argparse_witharg(args, i, &x, err, "--max_rule", (char*)NULL)) {
       if (!err.str().empty()) {
 	cerr << err.str() << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       tester.set_max_rule(x);
     } else if (ceph_argparse_witharg(args, i, &x, err, "--min_rule", (char*)NULL)) {
       if (!err.str().empty()) {
 	cerr << err.str() << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       tester.set_min_rule(x);
     } else if (ceph_argparse_witharg(args, i, &x, err, "--rule", (char*)NULL)) {
       if (!err.str().empty()) {
 	cerr << err.str() << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       tester.set_rule(x);
     } else if (ceph_argparse_witharg(args, i, &x, err, "--ruleset", (char*)NULL)) {
       if (!err.str().empty()) {
 	cerr << err.str() << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       tester.set_ruleset(x);
     } else if (ceph_argparse_witharg(args, i, &x, err, "--batches", (char*)NULL)) {
       if (!err.str().empty()) {
 	cerr << err.str() << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       tester.set_batches(x);
     } else if (ceph_argparse_witharg(args, i, &y, err, "--mark-down-ratio", (char*)NULL)) {
       if (!err.str().empty()) {
         cerr << err.str() << std::endl;
-        return EXIT_FAILURE;
+        exit(EXIT_FAILURE);
       }
       tester.set_device_down_ratio(y);
     } else if (ceph_argparse_witharg(args, i, &y, err, "--mark-down-bucket-ratio", (char*)NULL)) {
       if (!err.str().empty()) {
         cerr << err.str() << std::endl;
-        return EXIT_FAILURE;
+        exit(EXIT_FAILURE);
       }
       tester.set_bucket_down_ratio(y);
     } else if (ceph_argparse_witharg(args, i, &tmp, err, "--weight", (char*)NULL)) {
       if (!err.str().empty()) {
 	cerr << err.str() << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       int dev = tmp;
       if (i == args.end()) {
 	cerr << "expecting additional argument to --weight" << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
       float f = atof(*i);
       i = args.erase(i);
@@ -525,24 +529,24 @@ int main(int argc, const char **argv)
 
   if (decompile + compile + build > 1) {
     cerr << "cannot specify more than one of compile, decompile, and build" << std::endl;
-    return EXIT_FAILURE;
+    exit(EXIT_FAILURE);
   }
   if (!check && !compile && !decompile && !build && !test && !reweight && !adjust && !tree &&
       add_item < 0 && full_location < 0 &&
       remove_name.empty() && reweight_name.empty()) {
     cerr << "no action specified; -h for help" << std::endl;
-    return EXIT_FAILURE;
+    exit(EXIT_FAILURE);
   }
   if ((!build) && (!args.empty())) {
     cerr << "unrecognized arguments: " << args << std::endl;
-    return EXIT_FAILURE;
+    exit(EXIT_FAILURE);
   }
   else {
     if ((args.size() % 3) != 0U) {
       cerr << "remaining args: " << args << std::endl;
       cerr << "layers must be specified with 3-tuples of (name, buckettype, size)"
     	   << std::endl;
-      return EXIT_FAILURE;
+      exit(EXIT_FAILURE);
     }
     for (size_t j = 0; j < args.size(); j += 3) {
       layer_t l;
@@ -571,19 +575,19 @@ int main(int argc, const char **argv)
     if (infn == "-") {
       if (isatty(STDIN_FILENO)) {
         cerr << "stdin must not be from a tty" << std::endl;
-        return EXIT_FAILURE;
+        exit(EXIT_FAILURE);
       }
       r = get_fd_data(STDIN_FILENO, bl);
       if (r < 0) {
         cerr << "error reading data from STDIN" << std::endl;
-        return EXIT_FAILURE;
+        exit(EXIT_FAILURE);
       }
     } else {
       r = bl.read_file(infn.c_str(), &error);
       if (r < 0) {
         cerr << me << ": error reading '" << infn << "': " 
              << error << std::endl;
-        return EXIT_FAILURE;
+        exit(1);
       }
     }
     bufferlist::iterator p = bl.begin();
@@ -591,7 +595,7 @@ int main(int argc, const char **argv)
       crush.decode(p);
     } catch(...) {
       cerr << me << ": unable to decode " << infn << std::endl;
-      return EXIT_FAILURE;
+      exit(EXIT_FAILURE);
     }
   }
 
@@ -610,7 +614,7 @@ int main(int argc, const char **argv)
       cc.enable_unsafe_tunables();
     int r = cc.compile(in, srcfn.c_str());
     if (r < 0) 
-      return EXIT_FAILURE;
+      exit(1);
 
     modified = true;
   }
@@ -618,7 +622,7 @@ int main(int argc, const char **argv)
   if (build) {
     if (layers.empty()) {
       cerr << me << ": must specify at least one layer" << std::endl;
-      return EXIT_FAILURE;
+      exit(1);
     }
 
     crush.create();
@@ -654,7 +658,7 @@ int main(int argc, const char **argv)
 	}
       if (buckettype < 0) {
 	cerr << "unknown bucket type '" << l.buckettype << "'" << std::endl;
-	return EXIT_FAILURE;
+	exit(EXIT_FAILURE);
       }
 
       // build items
@@ -719,18 +723,17 @@ int main(int argc, const char **argv)
     {
       set<int> roots;
       crush.find_roots(roots);
-      if (roots.size() > 1) {
-	cerr << "The crush rulesets will use the root " << root << "\n"
-	     << "and ignore the others.\n"
-	     << "There are " << roots.size() << " roots, they can be\n"
-	     << "grouped into a single root by appending something like:\n"
-	     << "  root straw 0\n"
-	     << std::endl;
-      }
+      if (roots.size() > 1)
+	dout(1)	<< "The crush rulesets will use the root " << root << "\n"
+		<< "and ignore the others.\n"
+		<< "There are " << roots.size() << " roots, they can be\n"
+		<< "grouped into a single root by appending something like:\n"
+		<< "  root straw 0\n"
+		<< dendl;
     }
     
     if (OSDMap::build_simple_crush_rulesets(g_ceph_context, crush, root, &cerr))
-      return EXIT_FAILURE;
+      exit(EXIT_FAILURE);
 
     modified = true;
   }
@@ -848,7 +851,7 @@ int main(int argc, const char **argv)
       o.open(outfn.c_str(), ios::out | ios::binary | ios::trunc);
       if (!o.is_open()) {
 	cerr << me << ": error writing '" << outfn << "'" << std::endl;
-	return EXIT_FAILURE;
+	exit(1);
       }
       cc.decompile(o);
       o.close();
@@ -861,7 +864,7 @@ int main(int argc, const char **argv)
     tester.check_overlapped_rules();
     if (max_id >= 0) {
       if (!tester.check_name_maps(max_id)) {
-	return EXIT_FAILURE;
+	exit(1);
       }
     }
   }
@@ -873,7 +876,7 @@ int main(int argc, const char **argv)
 
     int r = tester.test();
     if (r < 0)
-      return EXIT_FAILURE;
+      exit(1);
   }
 
   // output ---
@@ -884,11 +887,11 @@ int main(int argc, const char **argv)
       cout << me << " successfully built or modified map.  Use '-o <file>' to write it out." << std::endl;
     } else {
       bufferlist bl;
-      crush.encode(bl, CEPH_FEATURES_SUPPORTED_DEFAULT);
+      crush.encode(bl);
       int r = bl.write_file(outfn.c_str());
       if (r < 0) {
 	cerr << me << ": error writing '" << outfn << "': " << cpp_strerror(r) << std::endl;
-	return EXIT_FAILURE;
+	exit(1);
       }
       if (verbose)
 	cout << "wrote crush map to " << outfn << std::endl;
