@@ -111,12 +111,10 @@ static int do_show_journal_status(librados::IoCtx& io_ctx,
     f->open_object_section("status");
     f->dump_unsigned("minimum_set", minimum_set);
     f->dump_unsigned("active_set", active_set);
-    f->open_array_section("registered_clients");
+    f->open_object_section("registered_clients");
     for (std::set<cls::journal::Client>::iterator c =
           registered_clients.begin(); c != registered_clients.end(); ++c) {
-      f->open_object_section("client");
       c->dump(f);
-      f->close_section();
     }
     f->close_section();
     f->close_section();
@@ -574,7 +572,9 @@ static int do_export_journal(librados::IoCtx& io_ctx,
       std::cerr << "rbd: error creating " << path << std::endl;
       return r;
     }
+#ifdef HAVE_POSIX_FADVISE
     posix_fadvise(fd, 0, 0, POSIX_FADV_SEQUENTIAL);
+#endif
   }
 
   JournalExporter exporter(io_ctx, journal_id, fd, no_error, verbose);
@@ -782,7 +782,9 @@ static int do_import_journal(librados::IoCtx& io_ctx,
       std::cerr << "rbd: error opening " << path << std::endl;
       return r;
     }
+#ifdef HAVE_POSIX_FADVISE
     posix_fadvise(fd, 0, 0, POSIX_FADV_SEQUENTIAL);
+#endif
   }
 
   JournalImporter importer(io_ctx, journal_id, fd, no_error, verbose);
