@@ -27,7 +27,7 @@ namespace po = boost::program_options;
 
 namespace {
 
-std::map<std::string, std::string> map_options; // used for both map and unmap
+std::map<std::string, std::string> map_options;
 
 } // anonymous namespace
 
@@ -49,7 +49,7 @@ static std::string map_option_ip_cb(const char *value_char)
     return "";
   }
 
-  return stringify(a.addr);
+  return stringify(a.get_sockaddr());
 }
 
 static std::string map_option_int_cb(const char *value_char)
@@ -140,27 +140,6 @@ static int parse_map_options(char *options)
   return 0;
 }
 
-static int parse_unmap_options(char *options)
-{
-  for (char *this_char = strtok(options, ", ");
-       this_char != NULL;
-       this_char = strtok(NULL, ",")) {
-    char *value_char;
-
-    if ((value_char = strchr(this_char, '=')) != NULL)
-      *value_char++ = '\0';
-
-    if (!strcmp(this_char, "force")) {
-      put_map_option("force", this_char);
-    } else {
-      std::cerr << "rbd: unknown unmap option '" << this_char << "'" << std::endl;
-      return -EINVAL;
-    }
-  }
-
-  return 0;
-}
-
 static int do_kernel_showmapped(Formatter *f)
 {
   struct krbd_ctx *krbd;
@@ -233,7 +212,7 @@ static int do_kernel_map(const char *poolname, const char *imgname,
     // default and omit it even if it was specified by the user
     // (see ceph.git commit fb0f1986449b)
     if (it->first == "rw" && it->second == "rw") {
-      it = map_options.erase(it);
+      map_options.erase(it);
     } else {
       if (it != map_options.begin())
         oss << ",";
@@ -260,24 +239,16 @@ static int do_kernel_unmap(const char *dev, const char *poolname,
                            const char *imgname, const char *snapname)
 {
   struct krbd_ctx *krbd;
-  std::ostringstream oss;
   int r;
 
   r = krbd_create_from_context(g_ceph_context, &krbd);
   if (r < 0)
     return r;
 
-  for (auto it = map_options.cbegin(); it != map_options.cend(); ++it) {
-    if (it != map_options.cbegin())
-      oss << ",";
-    oss << it->second;
-  }
-
   if (dev)
-    r = krbd_unmap(krbd, dev, oss.str().c_str());
+    r = krbd_unmap(krbd, dev);
   else
-    r = krbd_unmap_by_spec(krbd, poolname, imgname, snapname,
-                           oss.str().c_str());
+    r = krbd_unmap_by_spec(krbd, poolname, imgname, snapname);
 
   krbd_destroy(krbd);
   return r;
@@ -310,8 +281,8 @@ void get_map_arguments(po::options_description *positional,
   at::add_image_or_snap_spec_options(positional, options,
                                      at::ARGUMENT_MODIFIER_NONE);
   options->add_options()
-    ("options,o", po::value<std::string>(), "map options")
-    ("read-only", po::bool_switch(), "map read-only");
+    ("options,o", po::value<std::string>(), "mapping options")
+    ("read-only", po::bool_switch(), "mount read-only");
 }
 
 int execute_map(const po::variables_map &vm) {
@@ -374,8 +345,6 @@ void get_unmap_arguments(po::options_description *positional,
   at::add_pool_option(options, at::ARGUMENT_MODIFIER_NONE);
   at::add_image_option(options, at::ARGUMENT_MODIFIER_NONE);
   at::add_snap_option(options, at::ARGUMENT_MODIFIER_NONE);
-  options->add_options()
-    ("options,o", po::value<std::string>(), "unmap options");
 }
 
 int execute_unmap(const po::variables_map &vm) {
@@ -405,18 +374,6 @@ int execute_unmap(const po::variables_map &vm) {
     return -EINVAL;
   }
 
-  if (vm.count("options")) {
-    char *cli_unmap_options = strdup(vm["options"].as<std::string>().c_str());
-    BOOST_SCOPE_EXIT( (cli_unmap_options) ) {
-      free(cli_unmap_options);
-    } BOOST_SCOPE_EXIT_END;
-
-    if (parse_unmap_options(cli_unmap_options)) {
-      std::cerr << "rbd: couldn't parse unmap options" << std::endl;
-      return -EINVAL;
-    }
-  }
-
   utils::init_context();
 
   r = do_kernel_unmap(device_name.empty() ? nullptr : device_name.c_str(),
