@@ -1,6 +1,7 @@
 // -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
 // vim: ts=8 sw=2 smarttab
 
+#include "include/compat.h"
 #include "common/Formatter.h"
 #include "common/debug.h"
 #include "common/errno.h"
@@ -397,7 +398,7 @@ void ImageReplayer<I>::bootstrap() {
     &m_local_image_ctx, m_local_image_name, m_remote_image_id,
     m_global_image_id, m_threads->work_queue, m_threads->timer,
     &m_threads->timer_lock, m_local_mirror_uuid, m_remote_mirror_uuid,
-    m_remote_journaler, &m_client_meta, ctx, &m_do_resync, &m_progress_cxt);
+    m_remote_journaler, &m_client_meta, ctx, &m_progress_cxt);
 
   {
     Mutex::Locker locker(m_lock);
@@ -428,9 +429,6 @@ void ImageReplayer<I>::handle_bootstrap(int r) {
     dout(5) << "remote image is non-primary or local image is primary" << dendl;
     on_start_fail(0, "remote image is non-primary or local image is primary");
     return;
-  } else if (r == -EEXIST) {
-    on_start_fail(r, "split-brain detected");
-    return;
   } else if (r < 0) {
     on_start_fail(r, "error bootstrapping replay");
     return;
@@ -438,6 +436,7 @@ void ImageReplayer<I>::handle_bootstrap(int r) {
     return;
   }
 
+
   assert(m_local_journal == nullptr);
   {
     RWLock::RLocker snap_locker(m_local_image_ctx->snap_lock);
@@ -454,8 +453,13 @@ void ImageReplayer<I>::handle_bootstrap(int r) {
 
   {
     Mutex::Locker locker(m_lock);
+    bool do_resync = false;
+    r = m_local_image_ctx->journal->is_resync_requested(&do_resync);
+    if (r < 0) {
+      derr << "failed to check if a resync was requested" << dendl;
+    }
 
-    if (m_do_resync) {
+    if (do_resync) {
       Context *on_finish = m_on_start_finish;
       m_stopping_for_resync = true;
       FunctionContext *ctx = new FunctionContext([this, on_finish](int r) {
@@ -561,7 +565,6 @@ void ImageReplayer<I>::handle_start_replay(int r) {
     Mutex::Locker locker(m_lock);
     assert(m_state == STATE_STARTING);
     m_state = STATE_REPLAYING;
-    m_state_desc.clear();
     std::swap(m_on_start_finish, on_finish);
   }
 
@@ -604,7 +607,6 @@ void ImageReplayer<I>::on_start_fail(int r, const std::string &desc)
   Context *ctx = new FunctionContext([this, r, desc](int _r) {
       {
         Mutex::Locker locker(m_lock);
-        assert(m_state == STATE_STARTING);
         m_state = STATE_STOPPING;
         if (r < 0 && r != -ECANCELED) {
           derr << "start failed: " << cpp_strerror(r) << dendl;
@@ -724,7 +726,6 @@ void ImageReplayer<I>::handle_replay_ready()
     return;
   }
 
-  m_event_replay_tracker.start_op();
   if (m_replay_tag_valid && m_replay_tag.tid == m_replay_tag_tid) {
     preprocess_entry();
     return;
@@ -877,20 +878,13 @@ template <typename I>
 void ImageReplayer<I>::replay_flush() {
   dout(20) << dendl;
 
-  bool interrupted = false;
   {
     Mutex::Locker locker(m_lock);
     if (m_state != STATE_REPLAYING) {
       dout(20) << "replay interrupted" << dendl;
-      interrupted = true;
-    } else {
-      m_state = STATE_REPLAY_FLUSHING;
+      return;
     }
-  }
-
-  if (interrupted) {
-    m_event_replay_tracker.finish_op();
-    return;
+    m_state = STATE_REPLAY_FLUSHING;
   }
 
   // shut down the replay to flush all IO and ops and create a new
@@ -923,11 +917,9 @@ void ImageReplayer<I>::handle_replay_flush(int r) {
 
   if (r < 0) {
     derr << "replay flush encountered an error: " << cpp_strerror(r) << dendl;
-    m_event_replay_tracker.finish_op();
     handle_replay_complete(r, "replay flush encountered an error");
     return;
   } else if (on_replay_interrupted()) {
-    m_event_replay_tracker.finish_op();
     return;
   }
 
@@ -959,7 +951,6 @@ void ImageReplayer<I>::handle_get_remote_tag(int r) {
   if (r < 0) {
     derr << "failed to retrieve remote tag " << m_replay_tag_tid << ": "
          << cpp_strerror(r) << dendl;
-    m_event_replay_tracker.finish_op();
     handle_replay_complete(r, "failed to retrieve remote tag");
     return;
   }
@@ -1007,7 +998,6 @@ void ImageReplayer<I>::handle_allocate_local_tag(int r) {
 
   if (r < 0) {
     derr << "failed to allocate journal tag: " << cpp_strerror(r) << dendl;
-    m_event_replay_tracker.finish_op();
     handle_replay_complete(r, "failed to allocate journal tag");
     return;
   }
@@ -1025,7 +1015,6 @@ void ImageReplayer<I>::preprocess_entry() {
   int r = m_local_replay->decode(&it, &m_event_entry);
   if (r < 0) {
     derr << "failed to decode journal event" << dendl;
-    m_event_replay_tracker.finish_op();
     handle_replay_complete(r, "failed to decode journal event");
     return;
   }
@@ -1046,7 +1035,6 @@ void ImageReplayer<I>::handle_preprocess_entry(int r) {
 
   if (r < 0) {
     derr << "failed to preprocess journal event" << dendl;
-    m_event_replay_tracker.finish_op();
     handle_replay_complete(r, "failed to preprocess journal event");
     return;
   }
@@ -1059,16 +1047,11 @@ void ImageReplayer<I>::process_entry() {
   dout(20) << "processing entry tid=" << m_replay_entry.get_commit_tid()
            << dendl;
 
-  // stop replaying events if stop has been requested
-  if (on_replay_interrupted()) {
-    m_event_replay_tracker.finish_op();
-    return;
-  }
-
   Context *on_ready = create_context_callback<
     ImageReplayer, &ImageReplayer<I>::handle_process_entry_ready>(this);
   Context *on_commit = new C_ReplayCommitted(this, std::move(m_replay_entry));
 
+  m_event_replay_tracker.start_op();
   m_local_replay->process(m_event_entry, on_ready, on_commit);
 }
 
