@@ -1,10 +1,10 @@
 // -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
 // vim: ts=8 sw=2 smarttab
 
+#include "include/compat.h"
 #include "BootstrapRequest.h"
 #include "CloseImageRequest.h"
 #include "CreateImageRequest.h"
-#include "IsPrimaryRequest.h"
 #include "OpenImageRequest.h"
 #include "OpenLocalImageRequest.h"
 #include "common/debug.h"
@@ -19,6 +19,7 @@
 #include "librbd/Journal.h"
 #include "librbd/Utils.h"
 #include "librbd/journal/Types.h"
+#include "tools/rbd_mirror/ImageSync.h"
 #include "tools/rbd_mirror/ProgressContext.h"
 #include "tools/rbd_mirror/ImageSyncThrottler.h"
 
@@ -51,7 +52,6 @@ BootstrapRequest<I>::BootstrapRequest(
         Journaler *journaler,
         MirrorPeerClientMeta *client_meta,
         Context *on_finish,
-        bool *do_resync,
         rbd::mirror::ProgressContext *progress_ctx)
   : BaseRequest("rbd::mirror::image_replayer::BootstrapRequest",
 		reinterpret_cast<CephContext*>(local_io_ctx.cct()), on_finish),
@@ -63,7 +63,6 @@ BootstrapRequest<I>::BootstrapRequest(
     m_local_mirror_uuid(local_mirror_uuid),
     m_remote_mirror_uuid(remote_mirror_uuid), m_journaler(journaler),
     m_client_meta(client_meta), m_progress_ctx(progress_ctx),
-    m_do_resync(do_resync),
     m_lock(unique_lock_name("BootstrapRequest::m_lock", this)) {
 }
 
@@ -74,8 +73,6 @@ BootstrapRequest<I>::~BootstrapRequest() {
 
 template <typename I>
 void BootstrapRequest<I>::send() {
-  *m_do_resync = false;
-
   get_local_image_id();
 }
 
@@ -256,6 +253,9 @@ void BootstrapRequest<I>::open_remote_image() {
 
 template <typename I>
 void BootstrapRequest<I>::handle_open_remote_image(int r) {
+  // deduce the class type for the journal to support unit tests
+  typedef typename std::decay<decltype(*I::journal)>::type Journal;
+
   dout(20) << ": r=" << r << dendl;
 
   if (r < 0) {
@@ -265,36 +265,18 @@ void BootstrapRequest<I>::handle_open_remote_image(int r) {
     return;
   }
 
-  is_primary();
-}
-
-template <typename I>
-void BootstrapRequest<I>::is_primary() {
-  dout(20) << dendl;
-
-  update_progress("OPEN_REMOTE_IMAGE");
-
-  Context *ctx = create_context_callback<
-    BootstrapRequest<I>, &BootstrapRequest<I>::handle_is_primary>(
-      this);
-  IsPrimaryRequest<I> *request = IsPrimaryRequest<I>::create(m_remote_image_ctx,
-                                                             &m_primary, ctx);
-  request->send();
-}
-
-template <typename I>
-void BootstrapRequest<I>::handle_is_primary(int r) {
-  dout(20) << ": r=" << r << dendl;
-
+  // TODO: make async
+  bool tag_owner;
+  r = Journal::is_tag_owner(m_remote_image_ctx, &tag_owner);
   if (r < 0) {
-    derr << ": error querying remote image primary status: " << cpp_strerror(r)
+    derr << ": failed to query remote image primary status: " << cpp_strerror(r)
          << dendl;
     m_ret_val = r;
     close_remote_image();
     return;
   }
 
-  if (!m_primary) {
+  if (!tag_owner) {
     dout(5) << ": remote image is not primary -- skipping image replay"
             << dendl;
     m_ret_val = -EREMOTEIO;
@@ -388,33 +370,7 @@ void BootstrapRequest<I>::handle_open_local_image(int r) {
     m_ret_val = r;
     close_remote_image();
     return;
-  }
-
-  I *local_image_ctx = (*m_local_image_ctx);
-  {
-    RWLock::RLocker snap_locker(local_image_ctx->snap_lock);
-    if (local_image_ctx->journal == nullptr) {
-      derr << ": local image does not support journaling" << dendl;
-      m_ret_val = -EINVAL;
-      close_local_image();
-      return;
-    }
-
-    r = (*m_local_image_ctx)->journal->is_resync_requested(m_do_resync);
-    if (r < 0) {
-      derr << ": failed to check if a resync was requested" << dendl;
-      m_ret_val = r;
-      close_local_image();
-      return;
-    }
-  }
-
-  if (*m_do_resync) {
-    close_remote_image();
-    return;
-  }
-
-  if (m_client.state == cls::journal::CLIENT_STATE_DISCONNECTED) {
+  } if (m_client.state == cls::journal::CLIENT_STATE_DISCONNECTED) {
     dout(10) << ": client flagged disconnected -- skipping bootstrap" << dendl;
     // The caller is expected to detect disconnect initializing remote journal.
     m_ret_val = 0;
@@ -452,6 +408,7 @@ void BootstrapRequest<I>::handle_create_local_image(int r) {
     return;
   }
 
+  m_created_local_image = true;
   open_local_image();
 }
 
@@ -470,8 +427,8 @@ void BootstrapRequest<I>::update_client_image() {
 
   dout(20) << dendl;
 
-  librbd::journal::MirrorPeerClientMeta client_meta{m_local_image_id};
-  client_meta.state = librbd::journal::MIRROR_PEER_STATE_SYNCING;
+  librbd::journal::MirrorPeerClientMeta client_meta;
+  client_meta.image_id = m_local_image_id;
 
   librbd::journal::ClientData client_data(client_meta);
   bufferlist data_bl;
@@ -501,8 +458,7 @@ void BootstrapRequest<I>::handle_update_client_image(int r) {
     return;
   }
 
-  *m_client_meta = {m_local_image_id};
-  m_client_meta->state = librbd::journal::MIRROR_PEER_STATE_SYNCING;
+  m_client_meta->image_id = m_local_image_id;
   get_remote_tags();
 }
 
@@ -512,7 +468,8 @@ void BootstrapRequest<I>::get_remote_tags() {
 
   update_progress("GET_REMOTE_TAGS");
 
-  if (m_client_meta->state == librbd::journal::MIRROR_PEER_STATE_SYNCING) {
+  if (m_created_local_image ||
+      m_client_meta->state == librbd::journal::MIRROR_PEER_STATE_SYNCING) {
     // optimization -- no need to compare remote tags if we just created
     // the image locally or sync was interrupted
     image_sync();
