@@ -12,22 +12,6 @@
 #define dout_prefix *_dout << "rbd::mirror::image_sync::ObjectCopyRequest: " \
                            << this << " " << __func__
 
-namespace librados {
-
-bool operator==(const clone_info_t& rhs, const clone_info_t& lhs) {
-  return (rhs.cloneid == lhs.cloneid &&
-          rhs.snaps == lhs.snaps &&
-          rhs.overlap == lhs.overlap &&
-          rhs.size == lhs.size);
-}
-
-bool operator==(const snap_set_t& rhs, const snap_set_t& lhs) {
-  return (rhs.clones == lhs.clones &&
-          rhs.seq == lhs.seq);
-}
-
-} // namespace librados
-
 namespace rbd {
 namespace mirror {
 namespace image_sync {
@@ -70,8 +54,6 @@ void ObjectCopyRequest<I>::send_list_snaps() {
     ObjectCopyRequest<I>, &ObjectCopyRequest<I>::handle_list_snaps>(this);
 
   librados::ObjectReadOperation op;
-  m_snap_set = {};
-  m_snap_ret = 0;
   op.list_snaps(&m_snap_set, &m_snap_ret);
 
   m_remote_io_ctx.snap_set_read(CEPH_SNAPDIR);
@@ -93,26 +75,12 @@ void ObjectCopyRequest<I>::handle_list_snaps(int r) {
     finish(0);
     return;
   }
-
   if (r < 0) {
     derr << ": failed to list snaps: " << cpp_strerror(r) << dendl;
     finish(r);
     return;
   }
 
-  if (m_retry_missing_read) {
-    if (m_snap_set == m_retry_snap_set) {
-      derr << ": read encountered missing object using up-to-date snap set"
-           << dendl;
-      finish(-ENOENT);
-      return;
-    }
-
-    dout(20) << ": retrying using updated snap set" << dendl;
-    m_retry_missing_read = false;
-    m_retry_snap_set = {};
-  }
-
   compute_diffs();
   send_read_object();
 }
@@ -129,26 +97,23 @@ void ObjectCopyRequest<I>::send_read_object() {
   auto &sync_ops = m_snap_sync_ops.begin()->second;
   assert(!sync_ops.empty());
 
+  // map the sync op start snap id back to the necessary read snap id
+  librados::snap_t remote_snap_seq = m_snap_sync_ops.begin()->first;
+  m_remote_io_ctx.snap_set_read(remote_snap_seq);
+
   bool read_required = false;
   librados::ObjectReadOperation op;
   for (auto &sync_op : sync_ops) {
-    switch (sync_op.type) {
+    switch (std::get<0>(sync_op)) {
     case SYNC_OP_TYPE_WRITE:
       if (!read_required) {
-        // map the sync op start snap id back to the necessary read snap id
-        librados::snap_t remote_snap_seq =
-          m_snap_sync_ops.begin()->first.second;
-        m_remote_io_ctx.snap_set_read(remote_snap_seq);
-
         dout(20) << ": remote_snap_seq=" << remote_snap_seq << dendl;
         read_required = true;
       }
-
-      dout(20) << ": read op: " << sync_op.offset << "~" << sync_op.length
-               << dendl;
-      op.read(sync_op.offset, sync_op.length, &sync_op.out_bl, nullptr);
-      op.set_op_flags2(LIBRADOS_OP_FLAG_FADVISE_SEQUENTIAL |
-                       LIBRADOS_OP_FLAG_FADVISE_NOCACHE);
+      dout(20) << ": read op: " << std::get<1>(sync_op) << "~"
+               << std::get<2>(sync_op) << dendl;
+      op.sparse_read(std::get<1>(sync_op), std::get<2>(sync_op), &std::get<4>(sync_op),
+                     &std::get<3>(sync_op), nullptr);
       break;
     default:
       break;
@@ -172,15 +137,6 @@ template <typename I>
 void ObjectCopyRequest<I>::handle_read_object(int r) {
   dout(20) << ": r=" << r << dendl;
 
-  if (r == -ENOENT) {
-    m_retry_snap_set = m_snap_set;
-    m_retry_missing_read = true;
-
-    dout(5) << ": object missing potentially due to removed snapshot" << dendl;
-    send_list_snaps();
-    return;
-  }
-
   if (r < 0) {
     derr << ": failed to read from remote object: " << cpp_strerror(r)
          << dendl;
@@ -196,7 +152,7 @@ void ObjectCopyRequest<I>::send_write_object() {
   // retrieve the local snap context for the op
   SnapIds local_snap_ids;
   librados::snap_t local_snap_seq = 0;
-  librados::snap_t remote_snap_seq = m_snap_sync_ops.begin()->first.first;
+  librados::snap_t remote_snap_seq = m_snap_sync_ops.begin()->first;
   if (remote_snap_seq != 0) {
     auto snap_map_it = m_snap_map->find(remote_snap_seq);
     assert(snap_map_it != m_snap_map->end());
@@ -216,20 +172,23 @@ void ObjectCopyRequest<I>::send_write_object() {
 
   auto &sync_ops = m_snap_sync_ops.begin()->second;
   assert(!sync_ops.empty());
-
+  uint64_t buffer_offset;
   librados::ObjectWriteOperation op;
   for (auto &sync_op : sync_ops) {
-    switch (sync_op.type) {
+    switch (std::get<0>(sync_op)) {
     case SYNC_OP_TYPE_WRITE:
-      dout(20) << ": write op: " << sync_op.offset << "~"
-               << sync_op.out_bl.length() << dendl;
-      op.write(sync_op.offset, sync_op.out_bl);
-      op.set_op_flags2(LIBRADOS_OP_FLAG_FADVISE_SEQUENTIAL |
-                       LIBRADOS_OP_FLAG_FADVISE_NOCACHE);
+      buffer_offset = 0;
+      for(auto it : std::get<4>(sync_op)){
+         bufferlist tmpbl;
+         tmpbl.substr_of(std::get<3>(sync_op), buffer_offset, it.second);
+         op.write(it.first, tmpbl);
+         buffer_offset += it.second;
+	 dout(20) << ": write op: " << it.first<< "~" << it.second << dendl;
+      }
       break;
     case SYNC_OP_TYPE_TRUNC:
-      dout(20) << ": trunc op: " << sync_op.offset << dendl;
-      op.truncate(sync_op.offset);
+      dout(20) << ": trunc op: " << std::get<1>(sync_op) << dendl;
+      op.truncate(std::get<1>(sync_op));
       break;
     case SYNC_OP_TYPE_REMOVE:
       dout(20) << ": remove op" << dendl;
@@ -273,6 +232,7 @@ void ObjectCopyRequest<I>::handle_write_object(int r) {
 
 template <typename I>
 void ObjectCopyRequest<I>::send_update_object_map() {
+
   m_local_image_ctx->snap_lock.get_read();
   if (!m_local_image_ctx->test_features(RBD_FEATURE_OBJECT_MAP,
                                         m_local_image_ctx->snap_lock) ||
@@ -300,11 +260,15 @@ void ObjectCopyRequest<I>::send_update_object_map() {
            << dendl;
 
   RWLock::WLocker object_map_locker(m_local_image_ctx->object_map_lock);
-  bool sent = m_local_image_ctx->object_map->template aio_update<
+  Context *ctx = create_context_callback<
     ObjectCopyRequest<I>, &ObjectCopyRequest<I>::handle_update_object_map>(
-      snap_object_state.first, m_object_number, snap_object_state.second, {},
       this);
-  assert(sent);
+
+  m_local_image_ctx->object_map->aio_update(snap_object_state.first,
+                                            m_object_number,
+                                            m_object_number + 1,
+                                            snap_object_state.second,
+                                            boost::none, ctx);
   m_local_image_ctx->snap_lock.put_read();
 }
 
@@ -324,10 +288,6 @@ template <typename I>
 void ObjectCopyRequest<I>::compute_diffs() {
   CephContext *cct = m_local_image_ctx->cct;
 
-  m_snap_sync_ops = {};
-  m_snap_object_states = {};
-
-  librados::snap_t remote_sync_pont_snap_id = m_snap_map->rbegin()->first;
   uint64_t prev_end_size = 0;
   bool prev_exists = false;
   librados::snap_t start_remote_snap_id = 0;
@@ -339,15 +299,12 @@ void ObjectCopyRequest<I>::compute_diffs() {
     interval_set<uint64_t> diff;
     uint64_t end_size;
     bool exists;
-    librados::snap_t clone_end_snap_id;
     calc_snap_set_diff(cct, m_snap_set, start_remote_snap_id,
-                       end_remote_snap_id, &diff, &end_size, &exists,
-                       &clone_end_snap_id);
+                       end_remote_snap_id, &diff, &end_size, &exists);
 
     dout(20) << ": "
              << "start_remote_snap=" << start_remote_snap_id << ", "
              << "end_remote_snap_id=" << end_remote_snap_id << ", "
-             << "clone_end_snap_id=" << clone_end_snap_id << ", "
              << "end_local_snap_id=" << end_local_snap_id << ", "
              << "diff=" << diff << ", "
              << "end_size=" << end_size << ", "
@@ -369,43 +326,36 @@ void ObjectCopyRequest<I>::compute_diffs() {
         uint8_t object_state = OBJECT_EXISTS;
         if (m_local_image_ctx->test_features(RBD_FEATURE_FAST_DIFF,
                                              m_local_image_ctx->snap_lock) &&
-            prev_exists && diff.empty() && end_size == prev_end_size) {
+            diff.empty() && end_size == prev_end_size) {
           object_state = OBJECT_EXISTS_CLEAN;
         }
         m_snap_object_states[end_local_snap_id] = object_state;
       }
 
-      // reads should be issued against the newest (existing) snapshot within
-      // the associated snapshot object clone. writes should be issued
-      // against the oldest snapshot in the snap_map.
-      assert(clone_end_snap_id >= end_remote_snap_id);
-      if (clone_end_snap_id > remote_sync_pont_snap_id) {
-        // do not read past the sync point snapshot
-        clone_end_snap_id = remote_sync_pont_snap_id;
-      }
-
       // object write/zero, or truncate
-      // NOTE: a single snapshot clone might represent multiple snapshots, but
-      // the write/zero and truncate ops will only be associated with the first
-      // snapshot encountered within the clone since the diff will be empty for
-      // subsequent snapshots and the size will remain constant for a clone.
       for (auto it = diff.begin(); it != diff.end(); ++it) {
         dout(20) << ": read/write op: " << it.get_start() << "~"
                  << it.get_len() << dendl;
-        m_snap_sync_ops[{end_remote_snap_id, clone_end_snap_id}].emplace_back(
-          SYNC_OP_TYPE_WRITE, it.get_start(), it.get_len());
+        m_snap_sync_ops[end_remote_snap_id].emplace_back(SYNC_OP_TYPE_WRITE,
+                                                         it.get_start(),
+                                                         it.get_len(),
+                                                         bufferlist(),
+                                                         std::map<uint64_t, uint64_t>());
       }
       if (end_size < prev_end_size) {
         dout(20) << ": trunc op: " << end_size << dendl;
-        m_snap_sync_ops[{end_remote_snap_id, clone_end_snap_id}].emplace_back(
-          SYNC_OP_TYPE_TRUNC, end_size, 0U);
+        m_snap_sync_ops[end_remote_snap_id].emplace_back(SYNC_OP_TYPE_TRUNC,
+                                                         end_size, 0U,
+                                                         bufferlist(),
+                                                         std::map<uint64_t, uint64_t>());
       }
     } else {
       if (prev_exists) {
         // object remove
         dout(20) << ": remove op" << dendl;
-        m_snap_sync_ops[{end_remote_snap_id, end_remote_snap_id}].emplace_back(
-          SYNC_OP_TYPE_REMOVE, 0U, 0U);
+        m_snap_sync_ops[end_remote_snap_id].emplace_back(SYNC_OP_TYPE_REMOVE,
+                                                         0U, 0U, bufferlist(),
+                                                         std::map<uint64_t, uint64_t>());
       }
     }
 
