@@ -11,6 +11,7 @@
 #include <list>
 #include <map>
 #include <string>
+#include <tuple>
 #include <vector>
 
 class Context;
@@ -54,11 +55,9 @@ private:
    * <start>
    *    |
    *    v
-   * LIST_SNAPS < * * *
-   *    |             * (-ENOENT and snap set stale)
-   *    |   * * * * * *
-   *    |   *
-   *    v   *
+   * LIST_SNAPS
+   *    |
+   *    v
    * READ_OBJECT <--------\
    *    |                 | (repeat for each snapshot)
    *    v                 |
@@ -82,21 +81,9 @@ private:
     SYNC_OP_TYPE_REMOVE
   };
 
-  struct SyncOp {
-    SyncOp(SyncOpType type, uint64_t offset, uint64_t length)
-      : type(type), offset(offset), length(length) {
-    }
-
-    SyncOpType type;
-    uint64_t offset;
-    uint64_t length;
-
-    bufferlist out_bl;
-  };
-
+  typedef std::tuple<SyncOpType, uint64_t, uint64_t, bufferlist, std::map<uint64_t, uint64_t> > SyncOp;
   typedef std::list<SyncOp> SyncOps;
-  typedef std::pair<librados::snap_t, librados::snap_t> WriteReadSnapIds;
-  typedef std::map<WriteReadSnapIds, SyncOps> SnapSyncOps;
+  typedef std::map<librados::snap_t, SyncOps> SnapSyncOps;
   typedef std::map<librados::snap_t, uint8_t> SnapObjectStates;
 
   ImageCtxT *m_local_image_ctx;
@@ -113,9 +100,6 @@ private:
   librados::snap_set_t m_snap_set;
   int m_snap_ret;
 
-  bool m_retry_missing_read = false;
-  librados::snap_set_t m_retry_snap_set;
-
   SnapSyncOps m_snap_sync_ops;
   SnapObjectStates m_snap_object_states;
 
