@@ -8,13 +8,8 @@ if [ -n "$VSTART_DEST" ]; then
   SRC_PATH=`(cd $SRC_PATH; pwd)`
 
   CEPH_DIR=$SRC_PATH
-  CEPH_BIN=$SRC_PATH
-  CEPH_LIB=$SRC_PATH/.libs
-
-  if [ -e CMakeCache.txt ]; then
-      CEPH_BIN=$VSTART_DEST/../../bin
-      CEPH_LIB=$VSTART_DEST/../../lib
-  fi
+  CEPH_BIN=${PWD}/bin
+  CEPH_LIB=${PWD}/lib
 
   CEPH_CONF_PATH=$VSTART_DEST
   CEPH_DEV_DIR=$VSTART_DEST/dev
@@ -24,17 +19,17 @@ fi
 # for running out of the CMake build directory
 if [ -e CMakeCache.txt ]; then
   # Out of tree build, learn source location from CMakeCache.txt
-  CEPH_ROOT=`grep Ceph_SOURCE_DIR CMakeCache.txt | cut -d "=" -f 2`
+  CEPH_ROOT=`grep ceph_SOURCE_DIR CMakeCache.txt | cut -d "=" -f 2`
   CEPH_BUILD_DIR=`pwd`
+  [ -z "$MGR_PYTHON_PATH" ] && MGR_PYTHON_PATH=$CEPH_ROOT/src/pybind/mgr
 fi
 
 # use CEPH_BUILD_ROOT to vstart from a 'make install' 
 if [ -n "$CEPH_BUILD_ROOT" ]; then
-        [ -z $CEPH_BIN ] && CEPH_BIN=$CEPH_BUILD_ROOT/bin
-        [ -z $CEPH_LIB ] && CEPH_LIB=$CEPH_BUILD_ROOT/lib
-        [ -z $EC_PATH ] && EC_PATH=$CEPH_LIB/erasure-code
-        [ -z $CS_PATH ] && CS_PATH=$CEPH_LIB/compressor
-        [ -z $OBJCLASS_PATH ] && OBJCLASS_PATH=$CEPH_LIB/rados-classes
+        [ -z "$CEPH_BIN" ] && CEPH_BIN=$CEPH_BUILD_ROOT/bin
+        [ -z "$CEPH_LIB" ] && CEPH_LIB=$CEPH_BUILD_ROOT/lib
+        [ -z "$EC_PATH" ] && EC_PATH=$CEPH_LIB/erasure-code
+        [ -z "$OBJCLASS_PATH" ] && OBJCLASS_PATH=$CEPH_LIB/rados-classes
 elif [ -n "$CEPH_ROOT" ]; then
         [ -z "$PYBIND" ] && PYBIND=$CEPH_ROOT/src/pybind
         [ -z "$CEPH_BIN" ] && CEPH_BIN=$CEPH_BUILD_DIR/bin
@@ -43,12 +38,6 @@ elif [ -n "$CEPH_ROOT" ]; then
         [ -z "$CEPH_LIB" ] && CEPH_LIB=$CEPH_BUILD_DIR/lib
         [ -z "$OBJCLASS_PATH" ] && OBJCLASS_PATH=$CEPH_LIB
         [ -z "$EC_PATH" ] && EC_PATH=$CEPH_LIB
-else
-        [ -z "$CEPH_BIN" ] && CEPH_BIN=.
-        [ -z "$CEPH_LIB" ] && CEPH_LIB=.libs
-        [ -z $EC_PATH ] && EC_PATH=$CEPH_LIB
-        [ -z $CS_PATH ] && CS_PATH=$CEPH_LIB
-        [ -z $OBJCLASS_PATH ] && OBJCLASS_PATH=$CEPH_LIB
 fi
 
 if [ -z "${CEPH_VSTART_WRAPPER}" ]; then
@@ -57,20 +46,23 @@ fi
 
 [ -z "$PYBIND" ] && PYBIND=./pybind
 
-export PYTHONPATH=$PYBIND:$PYTHONPATH
+export PYTHONPATH=$PYBIND:$CEPH_LIB/cython_modules/lib.2:$PYTHONPATH
 export LD_LIBRARY_PATH=$CEPH_LIB:$LD_LIBRARY_PATH
 export DYLD_LIBRARY_PATH=$CEPH_LIB:$DYLD_LIBRARY_PATH
 
 [ -z "$CEPH_NUM_MON" ] && CEPH_NUM_MON="$MON"
 [ -z "$CEPH_NUM_OSD" ] && CEPH_NUM_OSD="$OSD"
 [ -z "$CEPH_NUM_MDS" ] && CEPH_NUM_MDS="$MDS"
+[ -z "$CEPH_NUM_MGR" ] && CEPH_NUM_MGR="$MGR"
 [ -z "$CEPH_NUM_FS"  ] && CEPH_NUM_FS="$FS"
 [ -z "$CEPH_NUM_RGW" ] && CEPH_NUM_RGW="$RGW"
 
 [ -z "$CEPH_NUM_MON" ] && CEPH_NUM_MON=3
 [ -z "$CEPH_NUM_OSD" ] && CEPH_NUM_OSD=3
 [ -z "$CEPH_NUM_MDS" ] && CEPH_NUM_MDS=3
+[ -z "$CEPH_NUM_MGR" ] && CEPH_NUM_MGR=0
 [ -z "$CEPH_NUM_FS"  ] && CEPH_NUM_FS=1
+[ -z "$CEPH_MAX_MDS" ] && CEPH_MAX_MDS=1
 [ -z "$CEPH_NUM_RGW" ] && CEPH_NUM_RGW=1
 
 [ -z "$CEPH_DIR" ] && CEPH_DIR="$PWD"
@@ -99,6 +91,9 @@ cephx=1 #turn cephx on by default
 cache=""
 memstore=0
 bluestore=0
+lockdep=${LOCKDEP:-1}
+
+VSTART_SEC="client.vstart.sh"
 
 MON_ADDR=""
 
@@ -115,7 +110,7 @@ usage=$usage"\t-l, --localhost: use localhost instead of hostname\n"
 usage=$usage"\t-i <ip>: bind to specific ip\n"
 usage=$usage"\t-r start radosgw (needs ceph compiled with --radosgw)\n"
 usage=$usage"\t-n, --new\n"
-usage=$usage"\t--valgrind[_{osd,mds,mon}] 'toolname args...'\n"
+usage=$usage"\t--valgrind[_{osd,mds,mon,rgw}] 'toolname args...'\n"
 usage=$usage"\t--nodaemon: use ceph-run as wrapper for mon/osd/mds\n"
 usage=$usage"\t--smallmds: limit mds cache size\n"
 usage=$usage"\t-m ip:port\t\tspecify monitor address\n"
@@ -129,10 +124,12 @@ usage=$usage"\t--mon_num specify ceph monitor count\n"
 usage=$usage"\t--osd_num specify ceph osd count\n"
 usage=$usage"\t--mds_num specify ceph mds count\n"
 usage=$usage"\t--rgw_port specify ceph rgw http listen port\n"
-usage=$usage"\t--bluestore use bluestore as the osd objectstore backend\n"
+usage=$usage"\t-b, --bluestore use bluestore as the osd objectstore backend\n"
 usage=$usage"\t--memstore use memstore as the osd objectstore backend\n"
 usage=$usage"\t--cache <pool>: enable cache tiering on pool\n"
 usage=$usage"\t--short: short object names only; necessary for ext4 dev\n"
+usage=$usage"\t--nolockdep disable lockdep\n"
+usage=$usage"\t--multimds <count> allow multimds with maximum active count\n"
 
 usage_exit() {
 	printf "$usage"
@@ -187,6 +184,11 @@ case $1 in
 	    valgrind_mon=$2
 	    shift
 	    ;;
+    --valgrind_rgw )
+	    [ -z "$2" ] && usage_exit
+	    valgrind_rgw=$2
+	    shift
+	    ;;
     --nodaemon )
 	    nodaemon=1
 	    ;;
@@ -234,12 +236,16 @@ case $1 in
 	    cephx=0
 	    ;;
     -k )
+	    if [ ! -r $conf_fn ]; then
+	        echo "cannot use old configuration: $conf_fn not readable." >&2
+	        exit
+	    fi
 	    overwrite_conf=0
 	    ;;
     --memstore )
 	    memstore=1
 	    ;;
-    --bluestore )
+    -b | --bluestore )
 	    bluestore=1
 	    ;;
     --hitset )
@@ -260,6 +266,13 @@ case $1 in
 	    fi
 	    shift
 	    ;;
+    --nolockdep )
+            lockdep=0
+            ;;
+    --multimds)
+        CEPH_MAX_MDS="$2"
+        shift
+        ;;
     * )
 	    usage_exit
 esac
@@ -267,10 +280,24 @@ shift
 done
 
 if [ "$overwrite_conf" -eq 0 ]; then
-  CEPH_NUM_MON=`awk -F= '/CEPH_NUM_MON/{print $2}' $conf_fn`
-  CEPH_NUM_OSD=`awk -F= '/CEPH_NUM_OSD/{print $2}' $conf_fn`
-  CEPH_NUM_MDS=`awk -F= '/CEPH_NUM_MDS/{print $2}' $conf_fn`
-  CEPH_NUM_RGW=`awk -F= '/CEPH_NUM_RGW/{print $2}' $conf_fn`
+    MON=`$CEPH_BIN/ceph-conf -c $conf_fn --name $VSTART_SEC num_mon 2>/dev/null` && \
+        CEPH_NUM_MON="$MON"
+    OSD=`$CEPH_BIN/ceph-conf -c $conf_fn --name $VSTART_SEC num_osd 2>/dev/null` && \
+        CEPH_NUM_OSD="$OSD"
+    MDS=`$CEPH_BIN/ceph-conf -c $conf_fn --name $VSTART_SEC num_mds 2>/dev/null` && \
+        CEPH_NUM_MDS="$MDS"
+    MGR=`$CEPH_BIN/ceph-conf -c $conf_fn --name $VSTART_SEC num_mgr 2>/dev/null` && \
+        CEPH_NUM_MGR="$MGR"
+    RGW=`$CEPH_BIN/ceph-conf -c $conf_fn --name $VSTART_SEC num_rgw 2>/dev/null` && \
+        CEPH_NUM_RGW="$RGW"
+else
+    if [ "$new" -ne 0 ]; then
+        # only delete if -n
+        [ -e "$conf_fn" ] && rm -- "$conf_fn"
+    else
+        # -k is implied... (doesn't make sense otherwise)
+        overwrite_conf=0
+    fi
 fi
 
 if [ "$start_all" -eq 1 ]; then
@@ -281,6 +308,16 @@ fi
 
 ARGS="-c $conf_fn"
 
+prunb() {
+    echo "$* &"
+    "$@" &
+}
+
+prun() {
+    echo "$*"
+    "$@"
+}
+
 run() {
     type=$1
     shift
@@ -288,20 +325,23 @@ run() {
     [ -z "$valg" ] && valg="$valgrind"
 
     if [ -n "$valg" ]; then
-	echo "valgrind --tool=$valg $* -f &"
-	valgrind --tool=$valg $* -f &
-	sleep 1
+        prunb valgrind --tool="$valg" "$@" -f
+        sleep 1
     else
-	if [ "$nodaemon" -eq 0 ]; then
-	    echo "$*"
-	    $*
-	else
-	    echo "ceph-run $* -f &"
-	    ./ceph-run $* -f &
-	fi
+        if [ "$nodaemon" -eq 0 ]; then
+            prun "$@"
+        else
+            prunb ./ceph-run "$@" -f
+        fi
     fi
 }
 
+wconf() {
+	if [ "$overwrite_conf" -eq 1 ]; then
+		cat >> "$conf_fn"
+	fi
+}
+
 if [ "$debug" -eq 0 ]; then
     CMONDEBUG='
 	debug mon = 10
@@ -322,6 +362,7 @@ else
         debug osd = 25
         debug objecter = 20
         debug monc = 20
+        debug mgrc = 20
         debug journal = 20
         debug filestore = 20
         debug bluestore = 30
@@ -335,9 +376,14 @@ else
         debug mds = 20
         debug auth = 20
         debug monc = 20
+        debug mgrc = 20
         mds debug scatterstat = true
         mds verify scatter = true
         mds log max segments = 2'
+    CMGRDEBUG='
+        debug ms = 1
+        debug monc = 20
+        debug mgr = 20'
 fi
 
 if [ -n "$MON_ADDR" ]; then
@@ -355,9 +401,6 @@ if [ "$bluestore" -eq 1 ]; then
 	osd objectstore = bluestore'
 fi
 
-# lockdep everywhere?
-# export CEPH_ARGS="--lockdep 1"
-
 if [ -z "$CEPH_PORT" ]; then
     CEPH_PORT=6789
     [ -e ".ceph_port" ] && CEPH_PORT=`cat .ceph_port`
@@ -371,7 +414,7 @@ test -d $CEPH_DEV_DIR/osd0/. && test -e $CEPH_DEV_DIR/sudo && SUDO="sudo"
 if [ "$start_all" -eq 1 ]; then
     $SUDO $INIT_CEPH stop
 fi
-$SUDO rm -f core*
+prun $SUDO rm -f core*
 
 test -d $CEPH_OUT_DIR || mkdir $CEPH_OUT_DIR
 test -d $CEPH_DEV_DIR || mkdir $CEPH_DEV_DIR
@@ -387,23 +430,29 @@ if [ -n "$ip" ]; then
     IP="$ip"
 else
     echo hostname $HOSTNAME
+    if [ -x "$(which ip 2>/dev/null)" ]; then
+	IP_CMD="ip addr"
+    else
+	IP_CMD="ifconfig"
+    fi
     # filter out IPv6 and localhost addresses
-    IP="$(ifconfig | sed -En 's/127.0.0.1//;s/.*inet (addr:)?(([0-9]*\.){3}[0-9]*).*/\2/p' | head -n1)"
+    IP="$($IP_CMD | sed -En 's/127.0.0.1//;s/.*inet (addr:)?(([0-9]*\.){3}[0-9]*).*/\2/p' | head -n1)"
     # if nothing left, try using localhost address, it might work
     if [ -z "$IP" ]; then IP="127.0.0.1"; fi
-    echo ip $IP
 fi
 echo "ip $IP"
-echo "port $PORT"
+echo "port $CEPH_PORT"
 
 
 [ -z $CEPH_ADM ] && CEPH_ADM=$CEPH_BIN/ceph
 
-if [ "$cephx" -eq 1 ]; then
-    CEPH_ADM="$CEPH_ADM -c $conf_fn -k $keyring_fn"
-else
-    CEPH_ADM="$CEPH_ADM -c $conf_fn"
-fi
+ceph_adm() {
+    if [ "$cephx" -eq 1 ]; then
+        prun $SUDO "$CEPH_ADM" -c "$conf_fn" -k "$keyring_fn" "$@"
+    else
+        prun $SUDO "$CEPH_ADM" -c "$conf_fn" "$@"
+    fi
+}
 
 MONS=""
 count=0
@@ -429,15 +478,16 @@ DAEMONOPTS="
 
 
 if [ "$start_mon" -eq 1 ]; then
-
 	if [ "$new" -eq 1 ]; then
-		if [ $overwrite_conf -eq 1 ]; then
-		        cat <<EOF > $conf_fn
+        wconf <<EOF
 ; generated by vstart.sh on `date`
-; CEPH_NUM_MON=$CEPH_NUM_MON
-; CEPH_NUM_OSD=$CEPH_NUM_OSD
-; CEPH_NUM_MDS=$CEPH_NUM_MDS
-; CEPH_NUM_RGW=$CEPH_NUM_RGW
+[$VSTART_SEC]
+        num mon = $CEPH_NUM_MON
+        num osd = $CEPH_NUM_OSD
+        num mds = $CEPH_NUM_MDS
+        num mgr = $CEPH_NUM_MGR
+        num rgw = $CEPH_NUM_RGW
+
 [global]
         fsid = $(uuidgen)
         osd pg bits = 3
@@ -450,32 +500,35 @@ if [ "$start_mon" -eq 1 ]; then
         mon data avail warn = 10
         mon data avail crit = 1
         erasure code dir = $EC_PATH
-        plugin dir = $CS_PATH
+        plugin dir = $CEPH_LIB
         osd pool default erasure code profile = plugin=jerasure technique=reed_sol_van k=2 m=1 ruleset-failure-domain=osd
         rgw frontends = fastcgi, civetweb port=$CEPH_RGW_PORT
         rgw dns name = localhost
         filestore fd cache size = 32
         run dir = $CEPH_OUT_DIR
         enable experimental unrecoverable data corrupting features = *
+EOF
+		if [ "$lockdep" -eq 1 ] ; then
+			wconf <<EOF
         lockdep = true
 EOF
-if [ "$cephx" -eq 1 ] ; then
-cat <<EOF >> $conf_fn
+		fi
+		if [ "$cephx" -eq 1 ] ; then
+			wconf <<EOF
         auth supported = cephx
 EOF
-else
-cat <<EOF >> $conf_fn
+		else
+			wconf <<EOF
 	auth cluster required = none
 	auth service required = none
 	auth client required = none
 EOF
-fi
-if [ "$short" -eq 1 ]; then
-    COSDSHORT="        osd max object name len = 460
+		fi
+		if [ "$short" -eq 1 ]; then
+			COSDSHORT="        osd max object name len = 460
         osd max object namespace len = 64"
-fi
-			cat <<EOF >> $conf_fn
-
+		fi
+		wconf <<EOF
 [client]
         keyring = $keyring_fn
         log file = $CEPH_OUT_DIR/\$name.\$pid.log
@@ -491,13 +544,23 @@ $CMDSDEBUG
         mds root ino uid = `id -u`
         mds root ino gid = `id -g`
 $extra_conf
+[mgr]
+        mgr modules = rest fsstatus
+        mgr data = $CEPH_DEV_DIR/mgr.\$id
+        mgr module path = $MGR_PYTHON_PATH
+$DAEMONOPTS
+$CMGRDEBUG
+$extra_conf
 [osd]
 $DAEMONOPTS
+        osd_check_max_object_name_len_on_startup = false
         osd data = $CEPH_DEV_DIR/osd\$id
         osd journal = $CEPH_DEV_DIR/osd\$id/journal
         osd journal size = 100
         osd class tmp = out
         osd class dir = $OBJCLASS_PATH
+        osd class load list = *
+        osd class default list = *
         osd scrub load threshold = 2000.0
         osd debug op order = true
         filestore wbthrottle xfs ios start flusher = 10
@@ -529,8 +592,6 @@ $extra_conf
 [global]
 $extra_conf
 EOF
-		fi
-
 		if [ `echo $IP | grep '^127\\.'` ]
 		then
 			echo
@@ -540,48 +601,37 @@ EOF
 			echo
 		fi
 
-	        $SUDO $CEPH_BIN/ceph-authtool --create-keyring --gen-key --name=mon. $keyring_fn --cap mon 'allow *'
-	        $SUDO $CEPH_BIN/ceph-authtool --gen-key --name=client.admin --set-uid=0 \
-		    --cap mon 'allow *' \
-		    --cap osd 'allow *' \
-		    --cap mds 'allow *' \
-		    $keyring_fn
+		prun $SUDO "$CEPH_BIN/ceph-authtool" --create-keyring --gen-key --name=mon. "$keyring_fn" --cap mon 'allow *'
+		prun $SUDO "$CEPH_BIN/ceph-authtool" --gen-key --name=client.admin --set-uid=0 \
+			--cap mon 'allow *' \
+			--cap osd 'allow *' \
+			--cap mds 'allow *' \
+			"$keyring_fn"
 
 		# build a fresh fs monmap, mon fs
-		str="$CEPH_BIN/monmaptool --create --clobber"
+		str=""
 		count=0
 		for f in $MONS
 		do
-			str=$str" --add $f $IP:$(($CEPH_PORT+$count))"
-			if [ $overwrite_conf -eq 1 ]; then
-				cat <<EOF >> $conf_fn
+			str="$str --add $f $IP:$(($CEPH_PORT+$count))"
+			wconf <<EOF
 [mon.$f]
         host = $HOSTNAME
         mon data = $CEPH_DEV_DIR/mon.$f
         mon addr = $IP:$(($CEPH_PORT+$count))
 EOF
-			fi
 			count=$(($count + 1))
 		done
-		str=$str" --print $monmap_fn"
-		echo $str
-		$str
+		prun "$CEPH_BIN/monmaptool" --create --clobber $str --print "$monmap_fn"
 
 		for f in $MONS
 		do
-		    cmd="rm -rf $CEPH_DEV_DIR/mon.$f"
-		    echo $cmd
-		    $cmd
-                    cmd="mkdir -p $CEPH_DEV_DIR/mon.$f"
-                    echo $cmd
-                    $cmd
-		    cmd="$CEPH_BIN/ceph-mon --mkfs -c $conf_fn -i $f --monmap=$monmap_fn"
-		    cmd="$cmd --keyring=$keyring_fn"
-		    echo $cmd
-		    $cmd
+			prun rm -rf -- "$CEPH_DEV_DIR/mon.$f"
+			prun mkdir -p "$CEPH_DEV_DIR/mon.$f"
+			prun "$CEPH_BIN/ceph-mon" --mkfs -c "$conf_fn" -i "$f" --monmap="$monmap_fn" --keyring="$keyring_fn"
 		done
 
-		rm $monmap_fn
+		prun rm -- "$monmap_fn"
 	fi
 
 	# start monitors
@@ -598,12 +648,10 @@ if [ "$start_osd" -eq 1 ]; then
     for osd in `seq 0 $((CEPH_NUM_OSD-1))`
     do
 	if [ "$new" -eq 1 ]; then
-	    if [ $overwrite_conf -eq 1 ]; then
-		    cat <<EOF >> $conf_fn
+		wconf <<EOF
 [osd.$osd]
         host = $HOSTNAME
 EOF
-	    fi
 
 	    rm -rf $CEPH_DEV_DIR/osd$osd || true
 	    for f in $CEPH_DEV_DIR/osd$osd/*; do btrfs sub delete $f &> /dev/null || true; done
@@ -611,13 +659,13 @@ EOF
 
 	    uuid=`uuidgen`
 	    echo "add osd$osd $uuid"
-	    $SUDO $CEPH_ADM osd create $uuid
-	    $SUDO $CEPH_ADM osd crush add osd.$osd 1.0 host=$HOSTNAME root=default
+	    ceph_adm osd create $uuid
+	    ceph_adm osd crush add osd.$osd 1.0 host=$HOSTNAME root=default
 	    $SUDO $CEPH_BIN/ceph-osd -i $osd $ARGS --mkfs --mkkey --osd-uuid $uuid
 
 	    key_fn=$CEPH_DEV_DIR/osd$osd/keyring
 	    echo adding osd$osd key to auth repository
-	    $SUDO $CEPH_ADM -i $key_fn auth add osd.$osd osd "allow *" mon "allow profile osd"
+	    ceph_adm -i "$key_fn" auth add osd.$osd osd "allow *" mon "allow profile osd" mgr "allow"
 	fi
 	echo start osd$osd
 	run 'osd' $SUDO $CEPH_BIN/ceph-osd -i $osd $ARGS $COSD_ARGS
@@ -626,7 +674,7 @@ fi
 
 # mds
 if [ "$smallmds" -eq 1 ]; then
-    cat <<EOF >> $conf_fn
+    wconf <<EOF
 [mds]
 	mds log max segments = 2
 	mds cache size = 10000
@@ -635,20 +683,19 @@ fi
 
 if [ "$start_mds" -eq 1 -a "$CEPH_NUM_MDS" -gt 0 ]; then
     if [ "$CEPH_NUM_FS" -gt "1" ] ; then
-        $CEPH_ADM fs flag set enable_multiple true --yes-i-really-mean-it
+        ceph_adm fs flag set enable_multiple true --yes-i-really-mean-it
     fi
 
     fs=0
     for name in a b c d e f g h i j k l m n o p
     do
-        cmd="$CEPH_ADM osd pool create cephfs_data_${name} 8"
-        $cmd
-
-        cmd="$CEPH_ADM osd pool create cephfs_metadata_${name} 8"
-        $cmd
-
-        cmd="$CEPH_ADM fs new cephfs_${name} cephfs_metadata_${name} cephfs_data_${name}"
-        $cmd
+        ceph_adm osd pool create "cephfs_data_${name}" 8
+        ceph_adm osd pool create "cephfs_metadata_${name}" 8
+        ceph_adm fs new "cephfs_${name}" "cephfs_metadata_${name}" "cephfs_data_${name}"
+        if [ "$CEPH_MAX_MDS" -gt 1 ]; then
+            ceph_adm fs set "cephfs_${name}" allow_multimds true --yes-i-really-mean-it
+            ceph_adm fs set "cephfs_${name}" max_mds "$CEPH_MAX_MDS"
+        fi
         fs=$(($fs + 1))
         [ $fs -eq $CEPH_NUM_FS ] && break
     done
@@ -657,30 +704,28 @@ if [ "$start_mds" -eq 1 -a "$CEPH_NUM_MDS" -gt 0 ]; then
     for name in a b c d e f g h i j k l m n o p
     do
 	if [ "$new" -eq 1 ]; then
-	    mkdir -p $CEPH_DEV_DIR/mds.$name
+	    prun mkdir -p "$CEPH_DEV_DIR/mds.$name"
 	    key_fn=$CEPH_DEV_DIR/mds.$name/keyring
-	    if [ $overwrite_conf -eq 1 ]; then
-	    cat <<EOF >> $conf_fn
+	    wconf <<EOF
 [mds.$name]
         host = $HOSTNAME
 EOF
 		if [ "$standby" -eq 1 ]; then
 		    mkdir -p $CEPH_DEV_DIR/mds.${name}s
-		    cat <<EOF >> $conf_fn
+			wconf <<EOF
        mds standby for rank = $mds
 [mds.${name}s]
         mds standby replay = true
         mds standby for name = ${name}
 EOF
-		fi
 	    fi
-	    $SUDO $CEPH_BIN/ceph-authtool --create-keyring --gen-key --name=mds.$name $key_fn
-	    $SUDO $CEPH_ADM -i $key_fn auth add mds.$name mon 'allow profile mds' osd 'allow *' mds 'allow'
+	    prun $SUDO "$CEPH_BIN/ceph-authtool" --create-keyring --gen-key --name="mds.$name" "$key_fn"
+	    ceph_adm -i "$key_fn" auth add "mds.$name" mon 'allow profile mds' osd 'allow *' mds 'allow' mgr 'allow'
 	    if [ "$standby" -eq 1 ]; then
-		    $SUDO $CEPH_BIN/ceph-authtool --create-keyring --gen-key --name=mds.${name}s \
-			$CEPH_DEV_DIR/mds.${name}s/keyring
-                    $SUDO $CEPH_ADM -i $CEPH_DEV_DIR/mds.${name}s/keyring auth add mds.${name}s \
-			mon 'allow *' osd 'allow *' mds 'allow'
+			prun $SUDO "$CEPH_BIN/ceph-authtool" --create-keyring --gen-key --name="mds.${name}s" \
+				"$CEPH_DEV_DIR/mds.${name}s/keyring"
+			ceph_adm -i "$CEPH_DEV_DIR/mds.${name}s/keyring" auth add "mds.${name}s" \
+				mon 'allow *' osd 'allow *' mds 'allow' mgr 'allow'
 	    fi
 
 	fi
@@ -695,12 +740,36 @@ EOF
 
 #valgrind --tool=massif $CEPH_BIN/ceph-mds $ARGS --mds_log_max_segments 2 --mds_thrash_fragments 0 --mds_thrash_exports 0 > m  #--debug_ms 20
 #$CEPH_BIN/ceph-mds -d $ARGS --mds_thrash_fragments 0 --mds_thrash_exports 0 #--debug_ms 20
-#$CEPH_ADM mds set max_mds 2
+#ceph_adm mds set max_mds 2
+    done
+fi
+
+if [ "$CEPH_NUM_MGR" -gt 0 ]; then
+    mgr=0
+    for name in x y z a b c d e f g h i j k l m n o p
+    do
+        if [ "$new" -eq 1 ]; then
+            mkdir -p $CEPH_DEV_DIR/mgr.$name
+            key_fn=$CEPH_DEV_DIR/mgr.$name/keyring
+            $SUDO $CEPH_BIN/ceph-authtool --create-keyring --gen-key --name=mgr.$name $key_fn
+            ceph_adm -i $key_fn auth add mgr.$name mon 'allow *'
+        fi
+
+        cat <<EOF >> $conf_fn
+[mgr.$name]
+
+EOF
+
+        echo "Starting mgr.${name}"
+        run 'mgr' $CEPH_BIN/ceph-mgr -i $name
+
+        mgr=$(($mgr + 1))
+        [ $mgr -eq $CEPH_NUM_MGR ] && break
     done
 fi
 
 if [ "$ec" -eq 1 ]; then
-    $SUDO $CEPH_ADM <<EOF
+    ceph_adm <<EOF
 osd erasure-code-profile set ec-profile m=2 k=2
 osd pool create ec 8 8 erasure ec-profile
 EOF
@@ -711,7 +780,7 @@ do_cache() {
 	p="$1"
 	shift
 	echo "creating cache for pool $p ..."
-	$SUDO $CEPH_ADM <<EOF
+	ceph_adm <<EOF
 osd pool create ${p}-cache 8
 osd tier add $p ${p}-cache
 osd tier cache-mode ${p}-cache writeback
@@ -728,7 +797,7 @@ do_hitsets() {
 	shift
 	shift
 	echo "setting hit_set on pool $pool type $type ..."
-	$CEPH_ADM <<EOF
+	ceph_adm <<EOF
 osd pool set $pool hit_set_type $type
 osd pool set $pool hit_set_count 8
 osd pool set $pool hit_set_period 30
@@ -785,7 +854,7 @@ do_rgw()
 
     RGWSUDO=
     [ $CEPH_RGW_PORT -lt 1024 ] && RGWSUDO=sudo
-    $RGWSUDO $CEPH_BIN/radosgw -c $conf_fn --log-file=${CEPH_OUT_DIR}/rgw.log ${RGWDEBUG} --debug-ms=1
+    run 'rgw' $RGWSUDO $CEPH_BIN/radosgw -c $conf_fn --log-file=${CEPH_OUT_DIR}/rgw.log ${RGWDEBUG} --debug-ms=1
 }
 if [ "$start_rgw" -eq 1 ]; then
     do_rgw
